// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK31_H_
#define UNPACK31_H_


#include "unpack.h"


#define UNPACK_31BIT_MAX_COUNT 16

extern unpack_func_t unpack_31bit[UNPACK_31BIT_MAX_COUNT][8][2];

#endif /* UNPACK31_H_ */
