// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK21_H_
#define UNPACK21_H_


#include "unpack.h"


#define UNPACK_21BIT_MAX_COUNT 24

extern unpack_func_t unpack_21bit[UNPACK_21BIT_MAX_COUNT][8][2];

#endif /* UNPACK21_H_ */
