// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK7_H_
#define UNPACK7_H_


#include "unpack.h"


#define UNPACK_7BIT_MAX_COUNT 8

extern unpack_func_t unpack_7bit[UNPACK_7BIT_MAX_COUNT][8][2];

#endif /* UNPACK7_H_ */
