// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK11_H_
#define UNPACK11_H_


#include "unpack.h"


#define UNPACK_11BIT_MAX_COUNT 46

extern unpack_func_t unpack_11bit[UNPACK_11BIT_MAX_COUNT][8][2];

#endif /* UNPACK11_H_ */
