// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK27_H_
#define UNPACK27_H_


#include "unpack.h"


#define UNPACK_27BIT_MAX_COUNT 18

extern unpack_func_t unpack_27bit[UNPACK_27BIT_MAX_COUNT][8][2];

#endif /* UNPACK27_H_ */
