/*
 * Copyright (c) 2016, Suprock Technologies
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdlib.h>

#include "asphodel.h"


ASPHODEL_API int asphodel_supports_rf_power_commands(AsphodelDevice_t *device)
{
	if (device != NULL)
	{
		if ((device->protocol_type & ASPHODEL_PROTOCOL_TYPE_RF_POWER) != 0)
		{
			return 1;
		}
	}

	return 0;
}

ASPHODEL_API int asphodel_supports_radio_commands(AsphodelDevice_t *device)
{
	if (device != NULL)
	{
		if ((device->protocol_type & ASPHODEL_PROTOCOL_TYPE_RADIO) != 0)
		{
			return 1;
		}
	}

	return 0;
}

ASPHODEL_API int asphodel_supports_remote_commands(AsphodelDevice_t *device)
{
	if (device != NULL)
	{
		if ((device->protocol_type & ASPHODEL_PROTOCOL_TYPE_REMOTE) != 0)
		{
			return 1;
		}
	}

	return 0;
}

ASPHODEL_API int asphodel_supports_bootloader_commands(AsphodelDevice_t *device)
{
	if (device != NULL)
	{
		if ((device->protocol_type & ASPHODEL_PROTOCOL_TYPE_BOOTLOADER) != 0)
		{
			return 1;
		}
	}

	return 0;
}
