// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK2_H_
#define UNPACK2_H_


#include "unpack.h"


#define UNPACK_2BIT_MAX_COUNT 8

extern unpack_func_t unpack_2bit[UNPACK_2BIT_MAX_COUNT][8][2];

#endif /* UNPACK2_H_ */
