// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK15_H_
#define UNPACK15_H_


#include "unpack.h"


#define UNPACK_15BIT_MAX_COUNT 34

extern unpack_func_t unpack_15bit[UNPACK_15BIT_MAX_COUNT][8][2];

#endif /* UNPACK15_H_ */
