// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK20_H_
#define UNPACK20_H_


#include "unpack.h"


#define UNPACK_20BIT_MAX_COUNT 25

extern unpack_func_t unpack_20bit[UNPACK_20BIT_MAX_COUNT][8][2];

#endif /* UNPACK20_H_ */
