// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK22_H_
#define UNPACK22_H_


#include "unpack.h"


#define UNPACK_22BIT_MAX_COUNT 23

extern unpack_func_t unpack_22bit[UNPACK_22BIT_MAX_COUNT][8][2];

#endif /* UNPACK22_H_ */
