// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK10_H_
#define UNPACK10_H_


#include "unpack.h"


#define UNPACK_10BIT_MAX_COUNT 51

extern unpack_func_t unpack_10bit[UNPACK_10BIT_MAX_COUNT][8][2];

#endif /* UNPACK10_H_ */
