// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK3_H_
#define UNPACK3_H_


#include "unpack.h"


#define UNPACK_3BIT_MAX_COUNT 8

extern unpack_func_t unpack_3bit[UNPACK_3BIT_MAX_COUNT][8][2];

#endif /* UNPACK3_H_ */
