// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK26_H_
#define UNPACK26_H_


#include "unpack.h"


#define UNPACK_26BIT_MAX_COUNT 19

extern unpack_func_t unpack_26bit[UNPACK_26BIT_MAX_COUNT][8][2];

#endif /* UNPACK26_H_ */
