// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK24_H_
#define UNPACK24_H_


#include "unpack.h"


#define UNPACK_24BIT_MAX_COUNT 42

extern unpack_func_t unpack_24bit[UNPACK_24BIT_MAX_COUNT][8][2];

#endif /* UNPACK24_H_ */
