// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK16_H_
#define UNPACK16_H_


#include "unpack.h"


#define UNPACK_16BIT_MAX_COUNT 64

extern unpack_func_t unpack_16bit[UNPACK_16BIT_MAX_COUNT][8][2];

#endif /* UNPACK16_H_ */
