// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK8_H_
#define UNPACK8_H_


#include "unpack.h"


#define UNPACK_8BIT_MAX_COUNT 64

extern unpack_func_t unpack_8bit[UNPACK_8BIT_MAX_COUNT][8][2];

#endif /* UNPACK8_H_ */
