// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK14_H_
#define UNPACK14_H_


#include "unpack.h"


#define UNPACK_14BIT_MAX_COUNT 36

extern unpack_func_t unpack_14bit[UNPACK_14BIT_MAX_COUNT][8][2];

#endif /* UNPACK14_H_ */
