// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK17_H_
#define UNPACK17_H_


#include "unpack.h"


#define UNPACK_17BIT_MAX_COUNT 30

extern unpack_func_t unpack_17bit[UNPACK_17BIT_MAX_COUNT][8][2];

#endif /* UNPACK17_H_ */
