// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK18_H_
#define UNPACK18_H_


#include "unpack.h"


#define UNPACK_18BIT_MAX_COUNT 28

extern unpack_func_t unpack_18bit[UNPACK_18BIT_MAX_COUNT][8][2];

#endif /* UNPACK18_H_ */
