// AUTOGENERATED FILE. DO NOT MODIFY.

#include <stdlib.h>
#include <stdint.h>

#include "unpack0.h"


static void unpack_1_0bit(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 0-bit unsigned
	// Skip 0 bits, read 0 bits of data.
	// Input must be at least 0 bytes long

	(void)input; // suppress unused parameter warning

	output[0] = (double)0;
}
static void unpack_2_0bit(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 0-bit unsigned
	// Skip 0 bits, read 0 bits of data.
	// Input must be at least 0 bytes long

	(void)input; // suppress unused parameter warning

	output[0] = (double)0;

	output[1] = (double)0;
}
static void unpack_3_0bit(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 0-bit unsigned
	// Skip 0 bits, read 0 bits of data.
	// Input must be at least 0 bytes long

	(void)input; // suppress unused parameter warning

	output[0] = (double)0;

	output[1] = (double)0;

	output[2] = (double)0;
}
static void unpack_4_0bit(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 0-bit unsigned
	// Skip 0 bits, read 0 bits of data.
	// Input must be at least 0 bytes long

	(void)input; // suppress unused parameter warning

	output[0] = (double)0;

	output[1] = (double)0;

	output[2] = (double)0;

	output[3] = (double)0;
}
static void unpack_5_0bit(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 0-bit unsigned
	// Skip 0 bits, read 0 bits of data.
	// Input must be at least 0 bytes long

	(void)input; // suppress unused parameter warning

	output[0] = (double)0;

	output[1] = (double)0;

	output[2] = (double)0;

	output[3] = (double)0;

	output[4] = (double)0;
}
static void unpack_6_0bit(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 0-bit unsigned
	// Skip 0 bits, read 0 bits of data.
	// Input must be at least 0 bytes long

	(void)input; // suppress unused parameter warning

	output[0] = (double)0;

	output[1] = (double)0;

	output[2] = (double)0;

	output[3] = (double)0;

	output[4] = (double)0;

	output[5] = (double)0;
}
static void unpack_7_0bit(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 0-bit unsigned
	// Skip 0 bits, read 0 bits of data.
	// Input must be at least 0 bytes long

	(void)input; // suppress unused parameter warning

	output[0] = (double)0;

	output[1] = (double)0;

	output[2] = (double)0;

	output[3] = (double)0;

	output[4] = (double)0;

	output[5] = (double)0;

	output[6] = (double)0;
}
static void unpack_8_0bit(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 0-bit unsigned
	// Skip 0 bits, read 0 bits of data.
	// Input must be at least 0 bytes long

	(void)input; // suppress unused parameter warning

	output[0] = (double)0;

	output[1] = (double)0;

	output[2] = (double)0;

	output[3] = (double)0;

	output[4] = (double)0;

	output[5] = (double)0;

	output[6] = (double)0;

	output[7] = (double)0;
}


unpack_func_t unpack_0bit[UNPACK_0BIT_MAX_COUNT][8][2] = {
	{
		{
			unpack_1_0bit,
			unpack_1_0bit,
		},
		{
			unpack_1_0bit,
			unpack_1_0bit,
		},
		{
			unpack_1_0bit,
			unpack_1_0bit,
		},
		{
			unpack_1_0bit,
			unpack_1_0bit,
		},
		{
			unpack_1_0bit,
			unpack_1_0bit,
		},
		{
			unpack_1_0bit,
			unpack_1_0bit,
		},
		{
			unpack_1_0bit,
			unpack_1_0bit,
		},
		{
			unpack_1_0bit,
			unpack_1_0bit,
		},
	},
	{
		{
			unpack_2_0bit,
			unpack_2_0bit,
		},
		{
			unpack_2_0bit,
			unpack_2_0bit,
		},
		{
			unpack_2_0bit,
			unpack_2_0bit,
		},
		{
			unpack_2_0bit,
			unpack_2_0bit,
		},
		{
			unpack_2_0bit,
			unpack_2_0bit,
		},
		{
			unpack_2_0bit,
			unpack_2_0bit,
		},
		{
			unpack_2_0bit,
			unpack_2_0bit,
		},
		{
			unpack_2_0bit,
			unpack_2_0bit,
		},
	},
	{
		{
			unpack_3_0bit,
			unpack_3_0bit,
		},
		{
			unpack_3_0bit,
			unpack_3_0bit,
		},
		{
			unpack_3_0bit,
			unpack_3_0bit,
		},
		{
			unpack_3_0bit,
			unpack_3_0bit,
		},
		{
			unpack_3_0bit,
			unpack_3_0bit,
		},
		{
			unpack_3_0bit,
			unpack_3_0bit,
		},
		{
			unpack_3_0bit,
			unpack_3_0bit,
		},
		{
			unpack_3_0bit,
			unpack_3_0bit,
		},
	},
	{
		{
			unpack_4_0bit,
			unpack_4_0bit,
		},
		{
			unpack_4_0bit,
			unpack_4_0bit,
		},
		{
			unpack_4_0bit,
			unpack_4_0bit,
		},
		{
			unpack_4_0bit,
			unpack_4_0bit,
		},
		{
			unpack_4_0bit,
			unpack_4_0bit,
		},
		{
			unpack_4_0bit,
			unpack_4_0bit,
		},
		{
			unpack_4_0bit,
			unpack_4_0bit,
		},
		{
			unpack_4_0bit,
			unpack_4_0bit,
		},
	},
	{
		{
			unpack_5_0bit,
			unpack_5_0bit,
		},
		{
			unpack_5_0bit,
			unpack_5_0bit,
		},
		{
			unpack_5_0bit,
			unpack_5_0bit,
		},
		{
			unpack_5_0bit,
			unpack_5_0bit,
		},
		{
			unpack_5_0bit,
			unpack_5_0bit,
		},
		{
			unpack_5_0bit,
			unpack_5_0bit,
		},
		{
			unpack_5_0bit,
			unpack_5_0bit,
		},
		{
			unpack_5_0bit,
			unpack_5_0bit,
		},
	},
	{
		{
			unpack_6_0bit,
			unpack_6_0bit,
		},
		{
			unpack_6_0bit,
			unpack_6_0bit,
		},
		{
			unpack_6_0bit,
			unpack_6_0bit,
		},
		{
			unpack_6_0bit,
			unpack_6_0bit,
		},
		{
			unpack_6_0bit,
			unpack_6_0bit,
		},
		{
			unpack_6_0bit,
			unpack_6_0bit,
		},
		{
			unpack_6_0bit,
			unpack_6_0bit,
		},
		{
			unpack_6_0bit,
			unpack_6_0bit,
		},
	},
	{
		{
			unpack_7_0bit,
			unpack_7_0bit,
		},
		{
			unpack_7_0bit,
			unpack_7_0bit,
		},
		{
			unpack_7_0bit,
			unpack_7_0bit,
		},
		{
			unpack_7_0bit,
			unpack_7_0bit,
		},
		{
			unpack_7_0bit,
			unpack_7_0bit,
		},
		{
			unpack_7_0bit,
			unpack_7_0bit,
		},
		{
			unpack_7_0bit,
			unpack_7_0bit,
		},
		{
			unpack_7_0bit,
			unpack_7_0bit,
		},
	},
	{
		{
			unpack_8_0bit,
			unpack_8_0bit,
		},
		{
			unpack_8_0bit,
			unpack_8_0bit,
		},
		{
			unpack_8_0bit,
			unpack_8_0bit,
		},
		{
			unpack_8_0bit,
			unpack_8_0bit,
		},
		{
			unpack_8_0bit,
			unpack_8_0bit,
		},
		{
			unpack_8_0bit,
			unpack_8_0bit,
		},
		{
			unpack_8_0bit,
			unpack_8_0bit,
		},
		{
			unpack_8_0bit,
			unpack_8_0bit,
		},
	},
};
