// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK30_H_
#define UNPACK30_H_


#include "unpack.h"


#define UNPACK_30BIT_MAX_COUNT 17

extern unpack_func_t unpack_30bit[UNPACK_30BIT_MAX_COUNT][8][2];

#endif /* UNPACK30_H_ */
