// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK23_H_
#define UNPACK23_H_


#include "unpack.h"


#define UNPACK_23BIT_MAX_COUNT 22

extern unpack_func_t unpack_23bit[UNPACK_23BIT_MAX_COUNT][8][2];

#endif /* UNPACK23_H_ */
