// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK_ID_H_
#define UNPACK_ID_H_


#include "unpack.h"


#define UNPACK_ID_MIN_COUNT 0
#define UNPACK_ID_MAX_COUNT 8

extern unpack_id_func_t unpack_id[UNPACK_ID_MAX_COUNT - UNPACK_ID_MIN_COUNT + 1][8];


#endif /* UNPACK_ID_H_ */
