// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK29_H_
#define UNPACK29_H_


#include "unpack.h"


#define UNPACK_29BIT_MAX_COUNT 17

extern unpack_func_t unpack_29bit[UNPACK_29BIT_MAX_COUNT][8][2];

#endif /* UNPACK29_H_ */
