// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK5_H_
#define UNPACK5_H_


#include "unpack.h"


#define UNPACK_5BIT_MAX_COUNT 8

extern unpack_func_t unpack_5bit[UNPACK_5BIT_MAX_COUNT][8][2];

#endif /* UNPACK5_H_ */
