// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNWRAP_H_
#define UNWRAP_H_


#include "unpack.h"


#define UNWRAP_MIN_COUNT 8
#define UNWRAP_MAX_COUNT 32

extern unwrap_func_t unwrap[UNWRAP_MAX_COUNT - UNWRAP_MIN_COUNT + 1][8];


#endif /* UNWRAP_H_ */
