// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK9_H_
#define UNPACK9_H_


#include "unpack.h"


#define UNPACK_9BIT_MAX_COUNT 56

extern unpack_func_t unpack_9bit[UNPACK_9BIT_MAX_COUNT][8][2];

#endif /* UNPACK9_H_ */
