// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK19_H_
#define UNPACK19_H_


#include "unpack.h"


#define UNPACK_19BIT_MAX_COUNT 26

extern unpack_func_t unpack_19bit[UNPACK_19BIT_MAX_COUNT][8][2];

#endif /* UNPACK19_H_ */
