// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK0_H_
#define UNPACK0_H_


#include "unpack.h"


#define UNPACK_0BIT_MAX_COUNT 8

extern unpack_func_t unpack_0bit[UNPACK_0BIT_MAX_COUNT][8][2];

#endif /* UNPACK0_H_ */
