// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK6_H_
#define UNPACK6_H_


#include "unpack.h"


#define UNPACK_6BIT_MAX_COUNT 8

extern unpack_func_t unpack_6bit[UNPACK_6BIT_MAX_COUNT][8][2];

#endif /* UNPACK6_H_ */
