// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK1_H_
#define UNPACK1_H_


#include "unpack.h"


#define UNPACK_1BIT_MAX_COUNT 8

extern unpack_func_t unpack_1bit[UNPACK_1BIT_MAX_COUNT][8][2];

#endif /* UNPACK1_H_ */
