// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK25_H_
#define UNPACK25_H_


#include "unpack.h"


#define UNPACK_25BIT_MAX_COUNT 20

extern unpack_func_t unpack_25bit[UNPACK_25BIT_MAX_COUNT][8][2];

#endif /* UNPACK25_H_ */
