// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK4_H_
#define UNPACK4_H_


#include "unpack.h"


#define UNPACK_4BIT_MAX_COUNT 8

extern unpack_func_t unpack_4bit[UNPACK_4BIT_MAX_COUNT][8][2];

#endif /* UNPACK4_H_ */
