// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK12_H_
#define UNPACK12_H_


#include "unpack.h"


#define UNPACK_12BIT_MAX_COUNT 42

extern unpack_func_t unpack_12bit[UNPACK_12BIT_MAX_COUNT][8][2];

#endif /* UNPACK12_H_ */
