// AUTOGENERATED FILE. DO NOT MODIFY.

#ifndef UNPACK28_H_
#define UNPACK28_H_


#include "unpack.h"


#define UNPACK_28BIT_MAX_COUNT 18

extern unpack_func_t unpack_28bit[UNPACK_28BIT_MAX_COUNT][8][2];

#endif /* UNPACK28_H_ */
