// AUTOGENERATED FILE. DO NOT MODIFY.

#include <stdlib.h>
#include <stdint.h>

#include "unpack15.h"



static void unpack_1_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 15 bits of data.
	// Input must be at least 2 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);
}

static void unpack_1_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 15 bits of data.
	// Input must be at least 2 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);
}

static void unpack_1_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 15 bits of data.
	// Input must be at least 2 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);
}

static void unpack_1_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 15 bits of data.
	// Input must be at least 2 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);
}

static void unpack_1_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 15 bits of data.
	// Input must be at least 3 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);
}

static void unpack_1_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 15 bits of data.
	// Input must be at least 3 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);
}

static void unpack_1_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 15 bits of data.
	// Input must be at least 3 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);
}

static void unpack_1_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 15 bits of data.
	// Input must be at least 3 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);
}

static void unpack_1_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 15 bits of data.
	// Input must be at least 3 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);
}

static void unpack_1_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 15 bits of data.
	// Input must be at least 3 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);
}

static void unpack_1_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 15 bits of data.
	// Input must be at least 3 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);
}

static void unpack_1_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 15 bits of data.
	// Input must be at least 3 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);
}

static void unpack_1_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 15 bits of data.
	// Input must be at least 3 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);
}

static void unpack_1_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 15 bits of data.
	// Input must be at least 3 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);
}

static void unpack_1_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 15 bits of data.
	// Input must be at least 3 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);
}

static void unpack_1_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 15 bits of data.
	// Input must be at least 3 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);
}

static void unpack_2_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 30 bits of data.
	// Input must be at least 4 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);
}

static void unpack_2_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 30 bits of data.
	// Input must be at least 4 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);
}

static void unpack_2_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 30 bits of data.
	// Input must be at least 4 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);
}

static void unpack_2_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 30 bits of data.
	// Input must be at least 4 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);
}

static void unpack_2_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 30 bits of data.
	// Input must be at least 4 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);
}

static void unpack_2_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 30 bits of data.
	// Input must be at least 4 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);
}

static void unpack_2_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 30 bits of data.
	// Input must be at least 5 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);
}

static void unpack_2_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 30 bits of data.
	// Input must be at least 5 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);
}

static void unpack_2_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 30 bits of data.
	// Input must be at least 5 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);
}

static void unpack_2_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 30 bits of data.
	// Input must be at least 5 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);
}

static void unpack_2_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 30 bits of data.
	// Input must be at least 5 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);
}

static void unpack_2_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 30 bits of data.
	// Input must be at least 5 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);
}

static void unpack_2_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 30 bits of data.
	// Input must be at least 5 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);
}

static void unpack_2_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 30 bits of data.
	// Input must be at least 5 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);
}

static void unpack_2_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 30 bits of data.
	// Input must be at least 5 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);
}

static void unpack_2_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 30 bits of data.
	// Input must be at least 5 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);
}

static void unpack_3_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 45 bits of data.
	// Input must be at least 6 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);
}

static void unpack_3_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 45 bits of data.
	// Input must be at least 6 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);
}

static void unpack_3_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 45 bits of data.
	// Input must be at least 6 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);
}

static void unpack_3_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 45 bits of data.
	// Input must be at least 6 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);
}

static void unpack_3_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 45 bits of data.
	// Input must be at least 6 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);
}

static void unpack_3_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 45 bits of data.
	// Input must be at least 6 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);
}

static void unpack_3_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 45 bits of data.
	// Input must be at least 6 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);
}

static void unpack_3_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 45 bits of data.
	// Input must be at least 6 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);
}

static void unpack_3_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 45 bits of data.
	// Input must be at least 7 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);
}

static void unpack_3_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 45 bits of data.
	// Input must be at least 7 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);
}

static void unpack_3_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 45 bits of data.
	// Input must be at least 7 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);
}

static void unpack_3_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 45 bits of data.
	// Input must be at least 7 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);
}

static void unpack_3_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 45 bits of data.
	// Input must be at least 7 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);
}

static void unpack_3_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 45 bits of data.
	// Input must be at least 7 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);
}

static void unpack_3_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 45 bits of data.
	// Input must be at least 7 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);
}

static void unpack_3_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 45 bits of data.
	// Input must be at least 7 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);
}

static void unpack_4_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 60 bits of data.
	// Input must be at least 8 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);
}

static void unpack_4_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 60 bits of data.
	// Input must be at least 8 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);
}

static void unpack_4_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 60 bits of data.
	// Input must be at least 8 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);
}

static void unpack_4_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 60 bits of data.
	// Input must be at least 8 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);
}

static void unpack_4_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 60 bits of data.
	// Input must be at least 8 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);
}

static void unpack_4_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 60 bits of data.
	// Input must be at least 8 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);
}

static void unpack_4_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 60 bits of data.
	// Input must be at least 8 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);
}

static void unpack_4_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 60 bits of data.
	// Input must be at least 8 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);
}

static void unpack_4_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 60 bits of data.
	// Input must be at least 8 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);
}

static void unpack_4_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 60 bits of data.
	// Input must be at least 8 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);
}

static void unpack_4_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 60 bits of data.
	// Input must be at least 9 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);
}

static void unpack_4_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 60 bits of data.
	// Input must be at least 9 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);
}

static void unpack_4_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 60 bits of data.
	// Input must be at least 9 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);
}

static void unpack_4_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 60 bits of data.
	// Input must be at least 9 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);
}

static void unpack_4_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 60 bits of data.
	// Input must be at least 9 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);
}

static void unpack_4_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 60 bits of data.
	// Input must be at least 9 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);
}

static void unpack_5_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 75 bits of data.
	// Input must be at least 10 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);
}

static void unpack_5_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 75 bits of data.
	// Input must be at least 10 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);
}

static void unpack_5_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 75 bits of data.
	// Input must be at least 10 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);
}

static void unpack_5_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 75 bits of data.
	// Input must be at least 10 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);
}

static void unpack_5_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 75 bits of data.
	// Input must be at least 10 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);
}

static void unpack_5_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 75 bits of data.
	// Input must be at least 10 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);
}

static void unpack_5_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 75 bits of data.
	// Input must be at least 10 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);
}

static void unpack_5_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 75 bits of data.
	// Input must be at least 10 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);
}

static void unpack_5_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 75 bits of data.
	// Input must be at least 10 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);
}

static void unpack_5_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 75 bits of data.
	// Input must be at least 10 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);
}

static void unpack_5_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 75 bits of data.
	// Input must be at least 10 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);
}

static void unpack_5_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 75 bits of data.
	// Input must be at least 10 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);
}

static void unpack_5_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 75 bits of data.
	// Input must be at least 11 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);
}

static void unpack_5_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 75 bits of data.
	// Input must be at least 11 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);
}

static void unpack_5_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 75 bits of data.
	// Input must be at least 11 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);
}

static void unpack_5_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 75 bits of data.
	// Input must be at least 11 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);
}

static void unpack_6_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 90 bits of data.
	// Input must be at least 12 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);
}

static void unpack_6_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 90 bits of data.
	// Input must be at least 12 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);
}

static void unpack_6_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 90 bits of data.
	// Input must be at least 12 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);
}

static void unpack_6_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 90 bits of data.
	// Input must be at least 12 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);
}

static void unpack_6_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 90 bits of data.
	// Input must be at least 12 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);
}

static void unpack_6_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 90 bits of data.
	// Input must be at least 12 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);
}

static void unpack_6_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 90 bits of data.
	// Input must be at least 12 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);
}

static void unpack_6_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 90 bits of data.
	// Input must be at least 12 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);
}

static void unpack_6_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 90 bits of data.
	// Input must be at least 12 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);
}

static void unpack_6_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 90 bits of data.
	// Input must be at least 12 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);
}

static void unpack_6_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 90 bits of data.
	// Input must be at least 12 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);
}

static void unpack_6_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 90 bits of data.
	// Input must be at least 12 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);
}

static void unpack_6_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 90 bits of data.
	// Input must be at least 12 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);
}

static void unpack_6_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 90 bits of data.
	// Input must be at least 12 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);
}

static void unpack_6_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 90 bits of data.
	// Input must be at least 13 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);
}

static void unpack_6_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 90 bits of data.
	// Input must be at least 13 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);
}

static void unpack_7_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 105 bits of data.
	// Input must be at least 14 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);
}

static void unpack_7_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 105 bits of data.
	// Input must be at least 14 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);
}

static void unpack_7_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 105 bits of data.
	// Input must be at least 14 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);
}

static void unpack_7_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 105 bits of data.
	// Input must be at least 14 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);
}

static void unpack_7_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 105 bits of data.
	// Input must be at least 14 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);
}

static void unpack_7_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 105 bits of data.
	// Input must be at least 14 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);
}

static void unpack_7_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 105 bits of data.
	// Input must be at least 14 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);
}

static void unpack_7_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 105 bits of data.
	// Input must be at least 14 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);
}

static void unpack_7_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 105 bits of data.
	// Input must be at least 14 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);
}

static void unpack_7_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 105 bits of data.
	// Input must be at least 14 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);
}

static void unpack_7_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 105 bits of data.
	// Input must be at least 14 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);
}

static void unpack_7_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 105 bits of data.
	// Input must be at least 14 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);
}

static void unpack_7_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 105 bits of data.
	// Input must be at least 14 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);
}

static void unpack_7_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 105 bits of data.
	// Input must be at least 14 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);
}

static void unpack_7_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 105 bits of data.
	// Input must be at least 14 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);
}

static void unpack_7_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 105 bits of data.
	// Input must be at least 14 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);
}

static void unpack_8_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 120 bits of data.
	// Input must be at least 15 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);
}

static void unpack_8_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 120 bits of data.
	// Input must be at least 15 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);
}

static void unpack_8_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 120 bits of data.
	// Input must be at least 16 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);
}

static void unpack_8_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 120 bits of data.
	// Input must be at least 16 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);
}

static void unpack_8_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 120 bits of data.
	// Input must be at least 16 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);
}

static void unpack_8_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 120 bits of data.
	// Input must be at least 16 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);
}

static void unpack_8_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 120 bits of data.
	// Input must be at least 16 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);
}

static void unpack_8_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 120 bits of data.
	// Input must be at least 16 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);
}

static void unpack_8_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 120 bits of data.
	// Input must be at least 16 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);
}

static void unpack_8_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 120 bits of data.
	// Input must be at least 16 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);
}

static void unpack_8_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 120 bits of data.
	// Input must be at least 16 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);
}

static void unpack_8_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 120 bits of data.
	// Input must be at least 16 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);
}

static void unpack_8_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 120 bits of data.
	// Input must be at least 16 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);
}

static void unpack_8_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 120 bits of data.
	// Input must be at least 16 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);
}

static void unpack_8_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 120 bits of data.
	// Input must be at least 16 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);
}

static void unpack_8_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 120 bits of data.
	// Input must be at least 16 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);
}

static void unpack_9_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 135 bits of data.
	// Input must be at least 17 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);
}

static void unpack_9_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 135 bits of data.
	// Input must be at least 17 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);
}

static void unpack_9_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 135 bits of data.
	// Input must be at least 17 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);
}

static void unpack_9_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 135 bits of data.
	// Input must be at least 17 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);
}

static void unpack_9_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 135 bits of data.
	// Input must be at least 18 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);
}

static void unpack_9_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 135 bits of data.
	// Input must be at least 18 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);
}

static void unpack_9_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 135 bits of data.
	// Input must be at least 18 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);
}

static void unpack_9_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 135 bits of data.
	// Input must be at least 18 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);
}

static void unpack_9_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 135 bits of data.
	// Input must be at least 18 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);
}

static void unpack_9_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 135 bits of data.
	// Input must be at least 18 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);
}

static void unpack_9_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 135 bits of data.
	// Input must be at least 18 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);
}

static void unpack_9_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 135 bits of data.
	// Input must be at least 18 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);
}

static void unpack_9_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 135 bits of data.
	// Input must be at least 18 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);
}

static void unpack_9_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 135 bits of data.
	// Input must be at least 18 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);
}

static void unpack_9_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 135 bits of data.
	// Input must be at least 18 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);
}

static void unpack_9_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 135 bits of data.
	// Input must be at least 18 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);
}

static void unpack_10_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 150 bits of data.
	// Input must be at least 19 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);
}

static void unpack_10_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 150 bits of data.
	// Input must be at least 19 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);
}

static void unpack_10_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 150 bits of data.
	// Input must be at least 19 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);
}

static void unpack_10_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 150 bits of data.
	// Input must be at least 19 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);
}

static void unpack_10_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 150 bits of data.
	// Input must be at least 19 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);
}

static void unpack_10_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 150 bits of data.
	// Input must be at least 19 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);
}

static void unpack_10_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 150 bits of data.
	// Input must be at least 20 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);
}

static void unpack_10_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 150 bits of data.
	// Input must be at least 20 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);
}

static void unpack_10_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 150 bits of data.
	// Input must be at least 20 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);
}

static void unpack_10_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 150 bits of data.
	// Input must be at least 20 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);
}

static void unpack_10_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 150 bits of data.
	// Input must be at least 20 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);
}

static void unpack_10_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 150 bits of data.
	// Input must be at least 20 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);
}

static void unpack_10_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 150 bits of data.
	// Input must be at least 20 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);
}

static void unpack_10_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 150 bits of data.
	// Input must be at least 20 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);
}

static void unpack_10_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 150 bits of data.
	// Input must be at least 20 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);
}

static void unpack_10_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 150 bits of data.
	// Input must be at least 20 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);
}

static void unpack_11_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 165 bits of data.
	// Input must be at least 21 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);
}

static void unpack_11_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 165 bits of data.
	// Input must be at least 21 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);
}

static void unpack_11_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 165 bits of data.
	// Input must be at least 21 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);
}

static void unpack_11_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 165 bits of data.
	// Input must be at least 21 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);
}

static void unpack_11_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 165 bits of data.
	// Input must be at least 21 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);
}

static void unpack_11_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 165 bits of data.
	// Input must be at least 21 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);
}

static void unpack_11_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 165 bits of data.
	// Input must be at least 21 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);
}

static void unpack_11_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 165 bits of data.
	// Input must be at least 21 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);
}

static void unpack_11_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 165 bits of data.
	// Input must be at least 22 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);
}

static void unpack_11_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 165 bits of data.
	// Input must be at least 22 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);
}

static void unpack_11_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 165 bits of data.
	// Input must be at least 22 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);
}

static void unpack_11_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 165 bits of data.
	// Input must be at least 22 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);
}

static void unpack_11_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 165 bits of data.
	// Input must be at least 22 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);
}

static void unpack_11_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 165 bits of data.
	// Input must be at least 22 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);
}

static void unpack_11_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 165 bits of data.
	// Input must be at least 22 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);
}

static void unpack_11_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 165 bits of data.
	// Input must be at least 22 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);
}

static void unpack_12_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 180 bits of data.
	// Input must be at least 23 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);
}

static void unpack_12_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 180 bits of data.
	// Input must be at least 23 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);
}

static void unpack_12_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 180 bits of data.
	// Input must be at least 23 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);
}

static void unpack_12_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 180 bits of data.
	// Input must be at least 23 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);
}

static void unpack_12_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 180 bits of data.
	// Input must be at least 23 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);
}

static void unpack_12_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 180 bits of data.
	// Input must be at least 23 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);
}

static void unpack_12_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 180 bits of data.
	// Input must be at least 23 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);
}

static void unpack_12_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 180 bits of data.
	// Input must be at least 23 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);
}

static void unpack_12_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 180 bits of data.
	// Input must be at least 23 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);
}

static void unpack_12_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 180 bits of data.
	// Input must be at least 23 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);
}

static void unpack_12_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 180 bits of data.
	// Input must be at least 24 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);
}

static void unpack_12_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 180 bits of data.
	// Input must be at least 24 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);
}

static void unpack_12_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 180 bits of data.
	// Input must be at least 24 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);
}

static void unpack_12_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 180 bits of data.
	// Input must be at least 24 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);
}

static void unpack_12_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 180 bits of data.
	// Input must be at least 24 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);
}

static void unpack_12_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 180 bits of data.
	// Input must be at least 24 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);
}

static void unpack_13_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);
}

static void unpack_13_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);
}

static void unpack_13_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);
}

static void unpack_13_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);
}

static void unpack_13_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);
}

static void unpack_13_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);
}

static void unpack_13_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);
}

static void unpack_13_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);
}

static void unpack_13_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);
}

static void unpack_13_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);
}

static void unpack_13_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);
}

static void unpack_13_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);
}

static void unpack_13_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 195 bits of data.
	// Input must be at least 26 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);
}

static void unpack_13_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 195 bits of data.
	// Input must be at least 26 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);
}

static void unpack_13_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 195 bits of data.
	// Input must be at least 26 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);
}

static void unpack_13_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 195 bits of data.
	// Input must be at least 26 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);
}

static void unpack_14_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 210 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);
}

static void unpack_14_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 210 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);
}

static void unpack_14_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 210 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);
}

static void unpack_14_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 210 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);
}

static void unpack_14_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 210 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);
}

static void unpack_14_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 210 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);
}

static void unpack_14_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 210 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);
}

static void unpack_14_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 210 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);
}

static void unpack_14_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 210 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);
}

static void unpack_14_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 210 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);
}

static void unpack_14_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 210 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);
}

static void unpack_14_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 210 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);
}

static void unpack_14_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 210 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);
}

static void unpack_14_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 210 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);
}

static void unpack_14_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 210 bits of data.
	// Input must be at least 28 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);
}

static void unpack_14_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 210 bits of data.
	// Input must be at least 28 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);
}

static void unpack_15_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 225 bits of data.
	// Input must be at least 29 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);
}

static void unpack_15_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 225 bits of data.
	// Input must be at least 29 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);
}

static void unpack_15_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 225 bits of data.
	// Input must be at least 29 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);
}

static void unpack_15_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 225 bits of data.
	// Input must be at least 29 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);
}

static void unpack_15_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 225 bits of data.
	// Input must be at least 29 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);
}

static void unpack_15_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 225 bits of data.
	// Input must be at least 29 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);
}

static void unpack_15_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 225 bits of data.
	// Input must be at least 29 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);
}

static void unpack_15_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 225 bits of data.
	// Input must be at least 29 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);
}

static void unpack_15_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 225 bits of data.
	// Input must be at least 29 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);
}

static void unpack_15_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 225 bits of data.
	// Input must be at least 29 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);
}

static void unpack_15_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 225 bits of data.
	// Input must be at least 29 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);
}

static void unpack_15_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 225 bits of data.
	// Input must be at least 29 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);
}

static void unpack_15_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 225 bits of data.
	// Input must be at least 29 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);
}

static void unpack_15_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 225 bits of data.
	// Input must be at least 29 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);
}

static void unpack_15_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 225 bits of data.
	// Input must be at least 29 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);
}

static void unpack_15_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 225 bits of data.
	// Input must be at least 29 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);
}

static void unpack_16_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 240 bits of data.
	// Input must be at least 30 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);
}

static void unpack_16_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 240 bits of data.
	// Input must be at least 30 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);
}

static void unpack_16_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 240 bits of data.
	// Input must be at least 31 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);
}

static void unpack_16_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 240 bits of data.
	// Input must be at least 31 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);
}

static void unpack_16_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 240 bits of data.
	// Input must be at least 31 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);
}

static void unpack_16_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 240 bits of data.
	// Input must be at least 31 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);
}

static void unpack_16_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 240 bits of data.
	// Input must be at least 31 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);
}

static void unpack_16_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 240 bits of data.
	// Input must be at least 31 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);
}

static void unpack_16_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 240 bits of data.
	// Input must be at least 31 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);
}

static void unpack_16_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 240 bits of data.
	// Input must be at least 31 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);
}

static void unpack_16_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 240 bits of data.
	// Input must be at least 31 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);
}

static void unpack_16_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 240 bits of data.
	// Input must be at least 31 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);
}

static void unpack_16_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 240 bits of data.
	// Input must be at least 31 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);
}

static void unpack_16_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 240 bits of data.
	// Input must be at least 31 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);
}

static void unpack_16_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 240 bits of data.
	// Input must be at least 31 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);
}

static void unpack_16_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 240 bits of data.
	// Input must be at least 31 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);
}

static void unpack_17_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 255 bits of data.
	// Input must be at least 32 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);
}

static void unpack_17_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 255 bits of data.
	// Input must be at least 32 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);
}

static void unpack_17_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 255 bits of data.
	// Input must be at least 32 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);
}

static void unpack_17_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 255 bits of data.
	// Input must be at least 32 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);
}

static void unpack_17_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 255 bits of data.
	// Input must be at least 33 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);
}

static void unpack_17_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 255 bits of data.
	// Input must be at least 33 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);
}

static void unpack_17_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 255 bits of data.
	// Input must be at least 33 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);
}

static void unpack_17_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 255 bits of data.
	// Input must be at least 33 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);
}

static void unpack_17_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 255 bits of data.
	// Input must be at least 33 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);
}

static void unpack_17_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 255 bits of data.
	// Input must be at least 33 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);
}

static void unpack_17_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 255 bits of data.
	// Input must be at least 33 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);
}

static void unpack_17_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 255 bits of data.
	// Input must be at least 33 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);
}

static void unpack_17_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 255 bits of data.
	// Input must be at least 33 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);
}

static void unpack_17_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 255 bits of data.
	// Input must be at least 33 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);
}

static void unpack_17_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 255 bits of data.
	// Input must be at least 33 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);
}

static void unpack_17_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 255 bits of data.
	// Input must be at least 33 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);
}

static void unpack_18_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 270 bits of data.
	// Input must be at least 34 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);
}

static void unpack_18_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 270 bits of data.
	// Input must be at least 34 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);
}

static void unpack_18_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 270 bits of data.
	// Input must be at least 34 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);
}

static void unpack_18_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 270 bits of data.
	// Input must be at least 34 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);
}

static void unpack_18_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 270 bits of data.
	// Input must be at least 34 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);
}

static void unpack_18_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 270 bits of data.
	// Input must be at least 34 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);
}

static void unpack_18_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 270 bits of data.
	// Input must be at least 35 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);
}

static void unpack_18_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 270 bits of data.
	// Input must be at least 35 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);
}

static void unpack_18_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 270 bits of data.
	// Input must be at least 35 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);
}

static void unpack_18_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 270 bits of data.
	// Input must be at least 35 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);
}

static void unpack_18_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 270 bits of data.
	// Input must be at least 35 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);
}

static void unpack_18_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 270 bits of data.
	// Input must be at least 35 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);
}

static void unpack_18_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 270 bits of data.
	// Input must be at least 35 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);
}

static void unpack_18_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 270 bits of data.
	// Input must be at least 35 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);
}

static void unpack_18_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 270 bits of data.
	// Input must be at least 35 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);
}

static void unpack_18_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 270 bits of data.
	// Input must be at least 35 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);
}

static void unpack_19_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 285 bits of data.
	// Input must be at least 36 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);
}

static void unpack_19_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 285 bits of data.
	// Input must be at least 36 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);
}

static void unpack_19_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 285 bits of data.
	// Input must be at least 36 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);
}

static void unpack_19_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 285 bits of data.
	// Input must be at least 36 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);
}

static void unpack_19_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 285 bits of data.
	// Input must be at least 36 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);
}

static void unpack_19_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 285 bits of data.
	// Input must be at least 36 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);
}

static void unpack_19_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 285 bits of data.
	// Input must be at least 36 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);
}

static void unpack_19_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 285 bits of data.
	// Input must be at least 36 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);
}

static void unpack_19_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 285 bits of data.
	// Input must be at least 37 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);
}

static void unpack_19_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 285 bits of data.
	// Input must be at least 37 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);
}

static void unpack_19_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 285 bits of data.
	// Input must be at least 37 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);
}

static void unpack_19_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 285 bits of data.
	// Input must be at least 37 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);
}

static void unpack_19_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 285 bits of data.
	// Input must be at least 37 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);
}

static void unpack_19_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 285 bits of data.
	// Input must be at least 37 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);
}

static void unpack_19_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 285 bits of data.
	// Input must be at least 37 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);
}

static void unpack_19_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 285 bits of data.
	// Input must be at least 37 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);
}

static void unpack_20_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 300 bits of data.
	// Input must be at least 38 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);
}

static void unpack_20_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 300 bits of data.
	// Input must be at least 38 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);
}

static void unpack_20_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 300 bits of data.
	// Input must be at least 38 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);
}

static void unpack_20_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 300 bits of data.
	// Input must be at least 38 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);
}

static void unpack_20_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 300 bits of data.
	// Input must be at least 38 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);
}

static void unpack_20_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 300 bits of data.
	// Input must be at least 38 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);
}

static void unpack_20_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 300 bits of data.
	// Input must be at least 38 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);
}

static void unpack_20_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 300 bits of data.
	// Input must be at least 38 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);
}

static void unpack_20_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 300 bits of data.
	// Input must be at least 38 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);
}

static void unpack_20_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 300 bits of data.
	// Input must be at least 38 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);
}

static void unpack_20_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 300 bits of data.
	// Input must be at least 39 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);
}

static void unpack_20_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 300 bits of data.
	// Input must be at least 39 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);
}

static void unpack_20_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 300 bits of data.
	// Input must be at least 39 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);
}

static void unpack_20_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 300 bits of data.
	// Input must be at least 39 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);
}

static void unpack_20_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 300 bits of data.
	// Input must be at least 39 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);
}

static void unpack_20_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 300 bits of data.
	// Input must be at least 39 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);
}

static void unpack_21_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 315 bits of data.
	// Input must be at least 40 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);
}

static void unpack_21_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 315 bits of data.
	// Input must be at least 40 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);
}

static void unpack_21_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 315 bits of data.
	// Input must be at least 40 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);
}

static void unpack_21_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 315 bits of data.
	// Input must be at least 40 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);
}

static void unpack_21_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 315 bits of data.
	// Input must be at least 40 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);
}

static void unpack_21_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 315 bits of data.
	// Input must be at least 40 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);
}

static void unpack_21_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 315 bits of data.
	// Input must be at least 40 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);
}

static void unpack_21_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 315 bits of data.
	// Input must be at least 40 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);
}

static void unpack_21_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 315 bits of data.
	// Input must be at least 40 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);
}

static void unpack_21_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 315 bits of data.
	// Input must be at least 40 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);
}

static void unpack_21_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 315 bits of data.
	// Input must be at least 40 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);
}

static void unpack_21_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 315 bits of data.
	// Input must be at least 40 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);
}

static void unpack_21_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 315 bits of data.
	// Input must be at least 41 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);
}

static void unpack_21_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 315 bits of data.
	// Input must be at least 41 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);
}

static void unpack_21_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 315 bits of data.
	// Input must be at least 41 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);
}

static void unpack_21_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 315 bits of data.
	// Input must be at least 41 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);
}

static void unpack_22_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);
}

static void unpack_22_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);
}

static void unpack_22_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);
}

static void unpack_22_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);
}

static void unpack_22_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);
}

static void unpack_22_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);
}

static void unpack_22_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);
}

static void unpack_22_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);
}

static void unpack_22_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);
}

static void unpack_22_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);
}

static void unpack_22_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);
}

static void unpack_22_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);
}

static void unpack_22_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);
}

static void unpack_22_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);
}

static void unpack_22_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 330 bits of data.
	// Input must be at least 43 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);
}

static void unpack_22_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 330 bits of data.
	// Input must be at least 43 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);
}

static void unpack_23_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 345 bits of data.
	// Input must be at least 44 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);
}

static void unpack_23_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 345 bits of data.
	// Input must be at least 44 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);
}

static void unpack_23_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 345 bits of data.
	// Input must be at least 44 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);
}

static void unpack_23_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 345 bits of data.
	// Input must be at least 44 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);
}

static void unpack_23_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 345 bits of data.
	// Input must be at least 44 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);
}

static void unpack_23_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 345 bits of data.
	// Input must be at least 44 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);
}

static void unpack_23_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 345 bits of data.
	// Input must be at least 44 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);
}

static void unpack_23_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 345 bits of data.
	// Input must be at least 44 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);
}

static void unpack_23_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 345 bits of data.
	// Input must be at least 44 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);
}

static void unpack_23_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 345 bits of data.
	// Input must be at least 44 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);
}

static void unpack_23_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 345 bits of data.
	// Input must be at least 44 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);
}

static void unpack_23_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 345 bits of data.
	// Input must be at least 44 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);
}

static void unpack_23_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 345 bits of data.
	// Input must be at least 44 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);
}

static void unpack_23_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 345 bits of data.
	// Input must be at least 44 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);
}

static void unpack_23_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 345 bits of data.
	// Input must be at least 44 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);
}

static void unpack_23_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 345 bits of data.
	// Input must be at least 44 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);
}

static void unpack_24_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 360 bits of data.
	// Input must be at least 45 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);
}

static void unpack_24_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 360 bits of data.
	// Input must be at least 45 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);
}

static void unpack_24_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 360 bits of data.
	// Input must be at least 46 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);
}

static void unpack_24_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 360 bits of data.
	// Input must be at least 46 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);
}

static void unpack_24_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 360 bits of data.
	// Input must be at least 46 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);
}

static void unpack_24_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 360 bits of data.
	// Input must be at least 46 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);
}

static void unpack_24_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 360 bits of data.
	// Input must be at least 46 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);
}

static void unpack_24_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 360 bits of data.
	// Input must be at least 46 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);
}

static void unpack_24_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 360 bits of data.
	// Input must be at least 46 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);
}

static void unpack_24_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 360 bits of data.
	// Input must be at least 46 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);
}

static void unpack_24_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 360 bits of data.
	// Input must be at least 46 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);
}

static void unpack_24_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 360 bits of data.
	// Input must be at least 46 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);
}

static void unpack_24_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 360 bits of data.
	// Input must be at least 46 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);
}

static void unpack_24_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 360 bits of data.
	// Input must be at least 46 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);
}

static void unpack_24_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 360 bits of data.
	// Input must be at least 46 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);
}

static void unpack_24_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 360 bits of data.
	// Input must be at least 46 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);
}

static void unpack_25_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 375 bits of data.
	// Input must be at least 47 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 360 to 374
	value = ((uint32_t)input[45] << 24) |
			((uint32_t)(input[46] & 0xfe) << 16);
	output[24] = (double)(value >> 17);
}

static void unpack_25_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 375 bits of data.
	// Input must be at least 47 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 360 to 374
	value = ((uint32_t)input[45] << 24) |
			((uint32_t)(input[46] & 0xfe) << 16);
	output[24] = (double)(value >> 17);
}

static void unpack_25_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 375 bits of data.
	// Input must be at least 47 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 361 to 375
	value = ((uint32_t)(input[45] & 0x7f) << 25) |
			((uint32_t)input[46] << 17);
	output[24] = (double)(value >> 17);
}

static void unpack_25_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 375 bits of data.
	// Input must be at least 47 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 361 to 375
	value = ((uint32_t)(input[45] & 0x7f) << 25) |
			((uint32_t)input[46] << 17);
	output[24] = (double)(value >> 17);
}

static void unpack_25_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 375 bits of data.
	// Input must be at least 48 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 362 to 376
	value = ((uint32_t)(input[45] & 0x3f) << 26) |
			((uint32_t)input[46] << 18) |
			((uint32_t)(input[47] & 0x80) << 10);
	output[24] = (double)(value >> 17);
}

static void unpack_25_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 375 bits of data.
	// Input must be at least 48 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 362 to 376
	value = ((uint32_t)(input[45] & 0x3f) << 26) |
			((uint32_t)input[46] << 18) |
			((uint32_t)(input[47] & 0x80) << 10);
	output[24] = (double)(value >> 17);
}

static void unpack_25_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 375 bits of data.
	// Input must be at least 48 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 363 to 377
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)input[46] << 19) |
			((uint32_t)(input[47] & 0xc0) << 11);
	output[24] = (double)(value >> 17);
}

static void unpack_25_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 375 bits of data.
	// Input must be at least 48 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 363 to 377
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)input[46] << 19) |
			((uint32_t)(input[47] & 0xc0) << 11);
	output[24] = (double)(value >> 17);
}

static void unpack_25_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 375 bits of data.
	// Input must be at least 48 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 364 to 378
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0xe0) << 12);
	output[24] = (double)(value >> 17);
}

static void unpack_25_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 375 bits of data.
	// Input must be at least 48 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 364 to 378
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0xe0) << 12);
	output[24] = (double)(value >> 17);
}

static void unpack_25_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 375 bits of data.
	// Input must be at least 48 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 365 to 379
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xf0) << 13);
	output[24] = (double)(value >> 17);
}

static void unpack_25_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 375 bits of data.
	// Input must be at least 48 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 365 to 379
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xf0) << 13);
	output[24] = (double)(value >> 17);
}

static void unpack_25_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 375 bits of data.
	// Input must be at least 48 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 366 to 380
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xf8) << 14);
	output[24] = (double)(value >> 17);
}

static void unpack_25_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 375 bits of data.
	// Input must be at least 48 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 366 to 380
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xf8) << 14);
	output[24] = (double)(value >> 17);
}

static void unpack_25_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 375 bits of data.
	// Input must be at least 48 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 367 to 381
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xfc) << 15);
	output[24] = (double)(value >> 17);
}

static void unpack_25_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 375 bits of data.
	// Input must be at least 48 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 367 to 381
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xfc) << 15);
	output[24] = (double)(value >> 17);
}

static void unpack_26_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 390 bits of data.
	// Input must be at least 49 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 360 to 374
	value = ((uint32_t)input[45] << 24) |
			((uint32_t)(input[46] & 0xfe) << 16);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 375 to 389
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xfc) << 15);
	output[25] = (double)(value >> 17);
}

static void unpack_26_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 390 bits of data.
	// Input must be at least 49 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 360 to 374
	value = ((uint32_t)input[45] << 24) |
			((uint32_t)(input[46] & 0xfe) << 16);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 375 to 389
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xfc) << 15);
	output[25] = (double)(value >> 17);
}

static void unpack_26_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 390 bits of data.
	// Input must be at least 49 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 361 to 375
	value = ((uint32_t)(input[45] & 0x7f) << 25) |
			((uint32_t)input[46] << 17);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 376 to 390
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xfe) << 16);
	output[25] = (double)(value >> 17);
}

static void unpack_26_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 390 bits of data.
	// Input must be at least 49 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 361 to 375
	value = ((uint32_t)(input[45] & 0x7f) << 25) |
			((uint32_t)input[46] << 17);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 376 to 390
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xfe) << 16);
	output[25] = (double)(value >> 17);
}

static void unpack_26_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 390 bits of data.
	// Input must be at least 49 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 362 to 376
	value = ((uint32_t)(input[45] & 0x3f) << 26) |
			((uint32_t)input[46] << 18) |
			((uint32_t)(input[47] & 0x80) << 10);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 377 to 391
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)input[48] << 17);
	output[25] = (double)(value >> 17);
}

static void unpack_26_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 390 bits of data.
	// Input must be at least 49 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 362 to 376
	value = ((uint32_t)(input[45] & 0x3f) << 26) |
			((uint32_t)input[46] << 18) |
			((uint32_t)(input[47] & 0x80) << 10);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 377 to 391
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)input[48] << 17);
	output[25] = (double)(value >> 17);
}

static void unpack_26_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 390 bits of data.
	// Input must be at least 50 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 363 to 377
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)input[46] << 19) |
			((uint32_t)(input[47] & 0xc0) << 11);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 378 to 392
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)input[48] << 18) |
			((uint32_t)(input[49] & 0x80) << 10);
	output[25] = (double)(value >> 17);
}

static void unpack_26_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 390 bits of data.
	// Input must be at least 50 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 363 to 377
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)input[46] << 19) |
			((uint32_t)(input[47] & 0xc0) << 11);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 378 to 392
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)input[48] << 18) |
			((uint32_t)(input[49] & 0x80) << 10);
	output[25] = (double)(value >> 17);
}

static void unpack_26_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 390 bits of data.
	// Input must be at least 50 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 364 to 378
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0xe0) << 12);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 379 to 393
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19) |
			((uint32_t)(input[49] & 0xc0) << 11);
	output[25] = (double)(value >> 17);
}

static void unpack_26_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 390 bits of data.
	// Input must be at least 50 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 364 to 378
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0xe0) << 12);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 379 to 393
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19) |
			((uint32_t)(input[49] & 0xc0) << 11);
	output[25] = (double)(value >> 17);
}

static void unpack_26_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 390 bits of data.
	// Input must be at least 50 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 365 to 379
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xf0) << 13);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 380 to 394
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0xe0) << 12);
	output[25] = (double)(value >> 17);
}

static void unpack_26_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 390 bits of data.
	// Input must be at least 50 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 365 to 379
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xf0) << 13);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 380 to 394
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0xe0) << 12);
	output[25] = (double)(value >> 17);
}

static void unpack_26_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 390 bits of data.
	// Input must be at least 50 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 366 to 380
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xf8) << 14);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 381 to 395
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xf0) << 13);
	output[25] = (double)(value >> 17);
}

static void unpack_26_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 390 bits of data.
	// Input must be at least 50 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 366 to 380
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xf8) << 14);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 381 to 395
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xf0) << 13);
	output[25] = (double)(value >> 17);
}

static void unpack_26_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 390 bits of data.
	// Input must be at least 50 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 367 to 381
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xfc) << 15);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 382 to 396
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xf8) << 14);
	output[25] = (double)(value >> 17);
}

static void unpack_26_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 390 bits of data.
	// Input must be at least 50 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 367 to 381
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xfc) << 15);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 382 to 396
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xf8) << 14);
	output[25] = (double)(value >> 17);
}

static void unpack_27_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 405 bits of data.
	// Input must be at least 51 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 360 to 374
	value = ((uint32_t)input[45] << 24) |
			((uint32_t)(input[46] & 0xfe) << 16);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 375 to 389
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xfc) << 15);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 390 to 404
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xf8) << 14);
	output[26] = (double)(value >> 17);
}

static void unpack_27_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 405 bits of data.
	// Input must be at least 51 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 360 to 374
	value = ((uint32_t)input[45] << 24) |
			((uint32_t)(input[46] & 0xfe) << 16);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 375 to 389
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xfc) << 15);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 390 to 404
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xf8) << 14);
	output[26] = (double)(value >> 17);
}

static void unpack_27_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 405 bits of data.
	// Input must be at least 51 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 361 to 375
	value = ((uint32_t)(input[45] & 0x7f) << 25) |
			((uint32_t)input[46] << 17);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 376 to 390
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xfe) << 16);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 391 to 405
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xfc) << 15);
	output[26] = (double)(value >> 17);
}

static void unpack_27_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 405 bits of data.
	// Input must be at least 51 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 361 to 375
	value = ((uint32_t)(input[45] & 0x7f) << 25) |
			((uint32_t)input[46] << 17);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 376 to 390
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xfe) << 16);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 391 to 405
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xfc) << 15);
	output[26] = (double)(value >> 17);
}

static void unpack_27_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 405 bits of data.
	// Input must be at least 51 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 362 to 376
	value = ((uint32_t)(input[45] & 0x3f) << 26) |
			((uint32_t)input[46] << 18) |
			((uint32_t)(input[47] & 0x80) << 10);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 377 to 391
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)input[48] << 17);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 392 to 406
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xfe) << 16);
	output[26] = (double)(value >> 17);
}

static void unpack_27_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 405 bits of data.
	// Input must be at least 51 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 362 to 376
	value = ((uint32_t)(input[45] & 0x3f) << 26) |
			((uint32_t)input[46] << 18) |
			((uint32_t)(input[47] & 0x80) << 10);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 377 to 391
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)input[48] << 17);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 392 to 406
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xfe) << 16);
	output[26] = (double)(value >> 17);
}

static void unpack_27_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 405 bits of data.
	// Input must be at least 51 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 363 to 377
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)input[46] << 19) |
			((uint32_t)(input[47] & 0xc0) << 11);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 378 to 392
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)input[48] << 18) |
			((uint32_t)(input[49] & 0x80) << 10);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 393 to 407
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)input[50] << 17);
	output[26] = (double)(value >> 17);
}

static void unpack_27_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 405 bits of data.
	// Input must be at least 51 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 363 to 377
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)input[46] << 19) |
			((uint32_t)(input[47] & 0xc0) << 11);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 378 to 392
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)input[48] << 18) |
			((uint32_t)(input[49] & 0x80) << 10);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 393 to 407
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)input[50] << 17);
	output[26] = (double)(value >> 17);
}

static void unpack_27_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 405 bits of data.
	// Input must be at least 52 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 364 to 378
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0xe0) << 12);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 379 to 393
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19) |
			((uint32_t)(input[49] & 0xc0) << 11);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 394 to 408
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)input[50] << 18) |
			((uint32_t)(input[51] & 0x80) << 10);
	output[26] = (double)(value >> 17);
}

static void unpack_27_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 405 bits of data.
	// Input must be at least 52 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 364 to 378
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0xe0) << 12);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 379 to 393
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19) |
			((uint32_t)(input[49] & 0xc0) << 11);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 394 to 408
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)input[50] << 18) |
			((uint32_t)(input[51] & 0x80) << 10);
	output[26] = (double)(value >> 17);
}

static void unpack_27_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 405 bits of data.
	// Input must be at least 52 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 365 to 379
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xf0) << 13);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 380 to 394
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0xe0) << 12);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 395 to 409
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19) |
			((uint32_t)(input[51] & 0xc0) << 11);
	output[26] = (double)(value >> 17);
}

static void unpack_27_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 405 bits of data.
	// Input must be at least 52 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 365 to 379
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xf0) << 13);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 380 to 394
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0xe0) << 12);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 395 to 409
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19) |
			((uint32_t)(input[51] & 0xc0) << 11);
	output[26] = (double)(value >> 17);
}

static void unpack_27_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 405 bits of data.
	// Input must be at least 52 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 366 to 380
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xf8) << 14);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 381 to 395
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xf0) << 13);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 396 to 410
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0xe0) << 12);
	output[26] = (double)(value >> 17);
}

static void unpack_27_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 405 bits of data.
	// Input must be at least 52 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 366 to 380
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xf8) << 14);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 381 to 395
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xf0) << 13);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 396 to 410
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0xe0) << 12);
	output[26] = (double)(value >> 17);
}

static void unpack_27_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 405 bits of data.
	// Input must be at least 52 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 367 to 381
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xfc) << 15);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 382 to 396
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xf8) << 14);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 397 to 411
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xf0) << 13);
	output[26] = (double)(value >> 17);
}

static void unpack_27_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 405 bits of data.
	// Input must be at least 52 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 367 to 381
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xfc) << 15);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 382 to 396
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xf8) << 14);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 397 to 411
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xf0) << 13);
	output[26] = (double)(value >> 17);
}

static void unpack_28_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 420 bits of data.
	// Input must be at least 53 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 360 to 374
	value = ((uint32_t)input[45] << 24) |
			((uint32_t)(input[46] & 0xfe) << 16);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 375 to 389
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xfc) << 15);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 390 to 404
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xf8) << 14);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 405 to 419
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xf0) << 13);
	output[27] = (double)(value >> 17);
}

static void unpack_28_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 420 bits of data.
	// Input must be at least 53 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 360 to 374
	value = ((uint32_t)input[45] << 24) |
			((uint32_t)(input[46] & 0xfe) << 16);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 375 to 389
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xfc) << 15);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 390 to 404
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xf8) << 14);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 405 to 419
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xf0) << 13);
	output[27] = (double)(value >> 17);
}

static void unpack_28_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 420 bits of data.
	// Input must be at least 53 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 361 to 375
	value = ((uint32_t)(input[45] & 0x7f) << 25) |
			((uint32_t)input[46] << 17);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 376 to 390
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xfe) << 16);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 391 to 405
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xfc) << 15);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 406 to 420
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xf8) << 14);
	output[27] = (double)(value >> 17);
}

static void unpack_28_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 420 bits of data.
	// Input must be at least 53 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 361 to 375
	value = ((uint32_t)(input[45] & 0x7f) << 25) |
			((uint32_t)input[46] << 17);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 376 to 390
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xfe) << 16);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 391 to 405
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xfc) << 15);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 406 to 420
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xf8) << 14);
	output[27] = (double)(value >> 17);
}

static void unpack_28_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 420 bits of data.
	// Input must be at least 53 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 362 to 376
	value = ((uint32_t)(input[45] & 0x3f) << 26) |
			((uint32_t)input[46] << 18) |
			((uint32_t)(input[47] & 0x80) << 10);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 377 to 391
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)input[48] << 17);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 392 to 406
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xfe) << 16);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 407 to 421
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xfc) << 15);
	output[27] = (double)(value >> 17);
}

static void unpack_28_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 420 bits of data.
	// Input must be at least 53 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 362 to 376
	value = ((uint32_t)(input[45] & 0x3f) << 26) |
			((uint32_t)input[46] << 18) |
			((uint32_t)(input[47] & 0x80) << 10);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 377 to 391
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)input[48] << 17);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 392 to 406
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xfe) << 16);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 407 to 421
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xfc) << 15);
	output[27] = (double)(value >> 17);
}

static void unpack_28_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 420 bits of data.
	// Input must be at least 53 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 363 to 377
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)input[46] << 19) |
			((uint32_t)(input[47] & 0xc0) << 11);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 378 to 392
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)input[48] << 18) |
			((uint32_t)(input[49] & 0x80) << 10);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 393 to 407
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)input[50] << 17);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 408 to 422
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xfe) << 16);
	output[27] = (double)(value >> 17);
}

static void unpack_28_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 420 bits of data.
	// Input must be at least 53 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 363 to 377
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)input[46] << 19) |
			((uint32_t)(input[47] & 0xc0) << 11);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 378 to 392
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)input[48] << 18) |
			((uint32_t)(input[49] & 0x80) << 10);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 393 to 407
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)input[50] << 17);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 408 to 422
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xfe) << 16);
	output[27] = (double)(value >> 17);
}

static void unpack_28_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 420 bits of data.
	// Input must be at least 53 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 364 to 378
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0xe0) << 12);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 379 to 393
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19) |
			((uint32_t)(input[49] & 0xc0) << 11);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 394 to 408
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)input[50] << 18) |
			((uint32_t)(input[51] & 0x80) << 10);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 409 to 423
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)input[52] << 17);
	output[27] = (double)(value >> 17);
}

static void unpack_28_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 420 bits of data.
	// Input must be at least 53 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 364 to 378
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0xe0) << 12);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 379 to 393
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19) |
			((uint32_t)(input[49] & 0xc0) << 11);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 394 to 408
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)input[50] << 18) |
			((uint32_t)(input[51] & 0x80) << 10);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 409 to 423
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)input[52] << 17);
	output[27] = (double)(value >> 17);
}

static void unpack_28_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 420 bits of data.
	// Input must be at least 54 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 365 to 379
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xf0) << 13);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 380 to 394
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0xe0) << 12);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 395 to 409
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19) |
			((uint32_t)(input[51] & 0xc0) << 11);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 410 to 424
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)input[52] << 18) |
			((uint32_t)(input[53] & 0x80) << 10);
	output[27] = (double)(value >> 17);
}

static void unpack_28_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 420 bits of data.
	// Input must be at least 54 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 365 to 379
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xf0) << 13);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 380 to 394
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0xe0) << 12);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 395 to 409
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19) |
			((uint32_t)(input[51] & 0xc0) << 11);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 410 to 424
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)input[52] << 18) |
			((uint32_t)(input[53] & 0x80) << 10);
	output[27] = (double)(value >> 17);
}

static void unpack_28_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 420 bits of data.
	// Input must be at least 54 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 366 to 380
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xf8) << 14);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 381 to 395
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xf0) << 13);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 396 to 410
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0xe0) << 12);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 411 to 425
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)input[52] << 19) |
			((uint32_t)(input[53] & 0xc0) << 11);
	output[27] = (double)(value >> 17);
}

static void unpack_28_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 420 bits of data.
	// Input must be at least 54 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 366 to 380
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xf8) << 14);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 381 to 395
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xf0) << 13);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 396 to 410
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0xe0) << 12);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 411 to 425
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)input[52] << 19) |
			((uint32_t)(input[53] & 0xc0) << 11);
	output[27] = (double)(value >> 17);
}

static void unpack_28_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 420 bits of data.
	// Input must be at least 54 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 367 to 381
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xfc) << 15);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 382 to 396
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xf8) << 14);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 397 to 411
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xf0) << 13);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 412 to 426
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)input[52] << 20) |
			((uint32_t)(input[53] & 0xe0) << 12);
	output[27] = (double)(value >> 17);
}

static void unpack_28_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 420 bits of data.
	// Input must be at least 54 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 367 to 381
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xfc) << 15);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 382 to 396
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xf8) << 14);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 397 to 411
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xf0) << 13);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 412 to 426
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)input[52] << 20) |
			((uint32_t)(input[53] & 0xe0) << 12);
	output[27] = (double)(value >> 17);
}

static void unpack_29_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 435 bits of data.
	// Input must be at least 55 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 360 to 374
	value = ((uint32_t)input[45] << 24) |
			((uint32_t)(input[46] & 0xfe) << 16);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 375 to 389
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xfc) << 15);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 390 to 404
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xf8) << 14);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 405 to 419
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xf0) << 13);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 420 to 434
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0xe0) << 12);
	output[28] = (double)(value >> 17);
}

static void unpack_29_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 435 bits of data.
	// Input must be at least 55 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 360 to 374
	value = ((uint32_t)input[45] << 24) |
			((uint32_t)(input[46] & 0xfe) << 16);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 375 to 389
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xfc) << 15);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 390 to 404
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xf8) << 14);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 405 to 419
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xf0) << 13);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 420 to 434
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0xe0) << 12);
	output[28] = (double)(value >> 17);
}

static void unpack_29_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 435 bits of data.
	// Input must be at least 55 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 361 to 375
	value = ((uint32_t)(input[45] & 0x7f) << 25) |
			((uint32_t)input[46] << 17);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 376 to 390
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xfe) << 16);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 391 to 405
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xfc) << 15);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 406 to 420
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xf8) << 14);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 421 to 435
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xf0) << 13);
	output[28] = (double)(value >> 17);
}

static void unpack_29_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 435 bits of data.
	// Input must be at least 55 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 361 to 375
	value = ((uint32_t)(input[45] & 0x7f) << 25) |
			((uint32_t)input[46] << 17);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 376 to 390
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xfe) << 16);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 391 to 405
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xfc) << 15);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 406 to 420
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xf8) << 14);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 421 to 435
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xf0) << 13);
	output[28] = (double)(value >> 17);
}

static void unpack_29_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 435 bits of data.
	// Input must be at least 55 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 362 to 376
	value = ((uint32_t)(input[45] & 0x3f) << 26) |
			((uint32_t)input[46] << 18) |
			((uint32_t)(input[47] & 0x80) << 10);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 377 to 391
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)input[48] << 17);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 392 to 406
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xfe) << 16);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 407 to 421
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xfc) << 15);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 422 to 436
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xf8) << 14);
	output[28] = (double)(value >> 17);
}

static void unpack_29_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 435 bits of data.
	// Input must be at least 55 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 362 to 376
	value = ((uint32_t)(input[45] & 0x3f) << 26) |
			((uint32_t)input[46] << 18) |
			((uint32_t)(input[47] & 0x80) << 10);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 377 to 391
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)input[48] << 17);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 392 to 406
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xfe) << 16);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 407 to 421
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xfc) << 15);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 422 to 436
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xf8) << 14);
	output[28] = (double)(value >> 17);
}

static void unpack_29_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 435 bits of data.
	// Input must be at least 55 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 363 to 377
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)input[46] << 19) |
			((uint32_t)(input[47] & 0xc0) << 11);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 378 to 392
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)input[48] << 18) |
			((uint32_t)(input[49] & 0x80) << 10);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 393 to 407
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)input[50] << 17);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 408 to 422
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xfe) << 16);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 423 to 437
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xfc) << 15);
	output[28] = (double)(value >> 17);
}

static void unpack_29_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 435 bits of data.
	// Input must be at least 55 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 363 to 377
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)input[46] << 19) |
			((uint32_t)(input[47] & 0xc0) << 11);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 378 to 392
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)input[48] << 18) |
			((uint32_t)(input[49] & 0x80) << 10);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 393 to 407
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)input[50] << 17);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 408 to 422
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xfe) << 16);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 423 to 437
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xfc) << 15);
	output[28] = (double)(value >> 17);
}

static void unpack_29_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 435 bits of data.
	// Input must be at least 55 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 364 to 378
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0xe0) << 12);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 379 to 393
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19) |
			((uint32_t)(input[49] & 0xc0) << 11);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 394 to 408
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)input[50] << 18) |
			((uint32_t)(input[51] & 0x80) << 10);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 409 to 423
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)input[52] << 17);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 424 to 438
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xfe) << 16);
	output[28] = (double)(value >> 17);
}

static void unpack_29_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 435 bits of data.
	// Input must be at least 55 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 364 to 378
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0xe0) << 12);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 379 to 393
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19) |
			((uint32_t)(input[49] & 0xc0) << 11);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 394 to 408
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)input[50] << 18) |
			((uint32_t)(input[51] & 0x80) << 10);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 409 to 423
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)input[52] << 17);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 424 to 438
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xfe) << 16);
	output[28] = (double)(value >> 17);
}

static void unpack_29_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 435 bits of data.
	// Input must be at least 55 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 365 to 379
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xf0) << 13);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 380 to 394
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0xe0) << 12);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 395 to 409
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19) |
			((uint32_t)(input[51] & 0xc0) << 11);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 410 to 424
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)input[52] << 18) |
			((uint32_t)(input[53] & 0x80) << 10);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 425 to 439
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)input[54] << 17);
	output[28] = (double)(value >> 17);
}

static void unpack_29_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 435 bits of data.
	// Input must be at least 55 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 365 to 379
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xf0) << 13);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 380 to 394
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0xe0) << 12);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 395 to 409
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19) |
			((uint32_t)(input[51] & 0xc0) << 11);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 410 to 424
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)input[52] << 18) |
			((uint32_t)(input[53] & 0x80) << 10);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 425 to 439
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)input[54] << 17);
	output[28] = (double)(value >> 17);
}

static void unpack_29_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 435 bits of data.
	// Input must be at least 56 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 366 to 380
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xf8) << 14);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 381 to 395
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xf0) << 13);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 396 to 410
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0xe0) << 12);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 411 to 425
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)input[52] << 19) |
			((uint32_t)(input[53] & 0xc0) << 11);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 426 to 440
	value = ((uint32_t)(input[53] & 0x3f) << 26) |
			((uint32_t)input[54] << 18) |
			((uint32_t)(input[55] & 0x80) << 10);
	output[28] = (double)(value >> 17);
}

static void unpack_29_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 435 bits of data.
	// Input must be at least 56 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 366 to 380
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xf8) << 14);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 381 to 395
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xf0) << 13);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 396 to 410
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0xe0) << 12);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 411 to 425
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)input[52] << 19) |
			((uint32_t)(input[53] & 0xc0) << 11);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 426 to 440
	value = ((uint32_t)(input[53] & 0x3f) << 26) |
			((uint32_t)input[54] << 18) |
			((uint32_t)(input[55] & 0x80) << 10);
	output[28] = (double)(value >> 17);
}

static void unpack_29_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 435 bits of data.
	// Input must be at least 56 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 367 to 381
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xfc) << 15);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 382 to 396
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xf8) << 14);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 397 to 411
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xf0) << 13);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 412 to 426
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)input[52] << 20) |
			((uint32_t)(input[53] & 0xe0) << 12);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 427 to 441
	value = ((uint32_t)(input[53] & 0x1f) << 27) |
			((uint32_t)input[54] << 19) |
			((uint32_t)(input[55] & 0xc0) << 11);
	output[28] = (double)(value >> 17);
}

static void unpack_29_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 435 bits of data.
	// Input must be at least 56 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 367 to 381
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xfc) << 15);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 382 to 396
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xf8) << 14);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 397 to 411
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xf0) << 13);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 412 to 426
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)input[52] << 20) |
			((uint32_t)(input[53] & 0xe0) << 12);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 427 to 441
	value = ((uint32_t)(input[53] & 0x1f) << 27) |
			((uint32_t)input[54] << 19) |
			((uint32_t)(input[55] & 0xc0) << 11);
	output[28] = (double)(value >> 17);
}

static void unpack_30_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 450 bits of data.
	// Input must be at least 57 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 360 to 374
	value = ((uint32_t)input[45] << 24) |
			((uint32_t)(input[46] & 0xfe) << 16);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 375 to 389
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xfc) << 15);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 390 to 404
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xf8) << 14);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 405 to 419
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xf0) << 13);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 420 to 434
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0xe0) << 12);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 435 to 449
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19) |
			((uint32_t)(input[56] & 0xc0) << 11);
	output[29] = (double)(value >> 17);
}

static void unpack_30_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 450 bits of data.
	// Input must be at least 57 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 360 to 374
	value = ((uint32_t)input[45] << 24) |
			((uint32_t)(input[46] & 0xfe) << 16);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 375 to 389
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xfc) << 15);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 390 to 404
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xf8) << 14);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 405 to 419
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xf0) << 13);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 420 to 434
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0xe0) << 12);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 435 to 449
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19) |
			((uint32_t)(input[56] & 0xc0) << 11);
	output[29] = (double)(value >> 17);
}

static void unpack_30_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 450 bits of data.
	// Input must be at least 57 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 361 to 375
	value = ((uint32_t)(input[45] & 0x7f) << 25) |
			((uint32_t)input[46] << 17);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 376 to 390
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xfe) << 16);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 391 to 405
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xfc) << 15);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 406 to 420
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xf8) << 14);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 421 to 435
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xf0) << 13);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 436 to 450
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0xe0) << 12);
	output[29] = (double)(value >> 17);
}

static void unpack_30_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 450 bits of data.
	// Input must be at least 57 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 361 to 375
	value = ((uint32_t)(input[45] & 0x7f) << 25) |
			((uint32_t)input[46] << 17);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 376 to 390
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xfe) << 16);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 391 to 405
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xfc) << 15);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 406 to 420
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xf8) << 14);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 421 to 435
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xf0) << 13);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 436 to 450
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0xe0) << 12);
	output[29] = (double)(value >> 17);
}

static void unpack_30_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 450 bits of data.
	// Input must be at least 57 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 362 to 376
	value = ((uint32_t)(input[45] & 0x3f) << 26) |
			((uint32_t)input[46] << 18) |
			((uint32_t)(input[47] & 0x80) << 10);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 377 to 391
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)input[48] << 17);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 392 to 406
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xfe) << 16);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 407 to 421
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xfc) << 15);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 422 to 436
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xf8) << 14);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 437 to 451
	value = ((uint32_t)(input[54] & 0x07) << 29) |
			((uint32_t)input[55] << 21) |
			((uint32_t)(input[56] & 0xf0) << 13);
	output[29] = (double)(value >> 17);
}

static void unpack_30_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 450 bits of data.
	// Input must be at least 57 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 362 to 376
	value = ((uint32_t)(input[45] & 0x3f) << 26) |
			((uint32_t)input[46] << 18) |
			((uint32_t)(input[47] & 0x80) << 10);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 377 to 391
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)input[48] << 17);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 392 to 406
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xfe) << 16);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 407 to 421
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xfc) << 15);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 422 to 436
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xf8) << 14);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 437 to 451
	value = ((uint32_t)(input[54] & 0x07) << 29) |
			((uint32_t)input[55] << 21) |
			((uint32_t)(input[56] & 0xf0) << 13);
	output[29] = (double)(value >> 17);
}

static void unpack_30_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 450 bits of data.
	// Input must be at least 57 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 363 to 377
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)input[46] << 19) |
			((uint32_t)(input[47] & 0xc0) << 11);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 378 to 392
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)input[48] << 18) |
			((uint32_t)(input[49] & 0x80) << 10);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 393 to 407
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)input[50] << 17);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 408 to 422
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xfe) << 16);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 423 to 437
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xfc) << 15);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 438 to 452
	value = ((uint32_t)(input[54] & 0x03) << 30) |
			((uint32_t)input[55] << 22) |
			((uint32_t)(input[56] & 0xf8) << 14);
	output[29] = (double)(value >> 17);
}

static void unpack_30_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 450 bits of data.
	// Input must be at least 57 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 363 to 377
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)input[46] << 19) |
			((uint32_t)(input[47] & 0xc0) << 11);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 378 to 392
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)input[48] << 18) |
			((uint32_t)(input[49] & 0x80) << 10);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 393 to 407
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)input[50] << 17);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 408 to 422
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xfe) << 16);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 423 to 437
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xfc) << 15);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 438 to 452
	value = ((uint32_t)(input[54] & 0x03) << 30) |
			((uint32_t)input[55] << 22) |
			((uint32_t)(input[56] & 0xf8) << 14);
	output[29] = (double)(value >> 17);
}

static void unpack_30_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 450 bits of data.
	// Input must be at least 57 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 364 to 378
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0xe0) << 12);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 379 to 393
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19) |
			((uint32_t)(input[49] & 0xc0) << 11);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 394 to 408
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)input[50] << 18) |
			((uint32_t)(input[51] & 0x80) << 10);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 409 to 423
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)input[52] << 17);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 424 to 438
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xfe) << 16);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 439 to 453
	value = ((uint32_t)(input[54] & 0x01) << 31) |
			((uint32_t)input[55] << 23) |
			((uint32_t)(input[56] & 0xfc) << 15);
	output[29] = (double)(value >> 17);
}

static void unpack_30_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 450 bits of data.
	// Input must be at least 57 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 364 to 378
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0xe0) << 12);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 379 to 393
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19) |
			((uint32_t)(input[49] & 0xc0) << 11);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 394 to 408
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)input[50] << 18) |
			((uint32_t)(input[51] & 0x80) << 10);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 409 to 423
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)input[52] << 17);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 424 to 438
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xfe) << 16);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 439 to 453
	value = ((uint32_t)(input[54] & 0x01) << 31) |
			((uint32_t)input[55] << 23) |
			((uint32_t)(input[56] & 0xfc) << 15);
	output[29] = (double)(value >> 17);
}

static void unpack_30_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 450 bits of data.
	// Input must be at least 57 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 365 to 379
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xf0) << 13);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 380 to 394
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0xe0) << 12);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 395 to 409
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19) |
			((uint32_t)(input[51] & 0xc0) << 11);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 410 to 424
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)input[52] << 18) |
			((uint32_t)(input[53] & 0x80) << 10);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 425 to 439
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)input[54] << 17);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 440 to 454
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xfe) << 16);
	output[29] = (double)(value >> 17);
}

static void unpack_30_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 450 bits of data.
	// Input must be at least 57 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 365 to 379
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xf0) << 13);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 380 to 394
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0xe0) << 12);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 395 to 409
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19) |
			((uint32_t)(input[51] & 0xc0) << 11);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 410 to 424
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)input[52] << 18) |
			((uint32_t)(input[53] & 0x80) << 10);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 425 to 439
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)input[54] << 17);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 440 to 454
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xfe) << 16);
	output[29] = (double)(value >> 17);
}

static void unpack_30_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 450 bits of data.
	// Input must be at least 57 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 366 to 380
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xf8) << 14);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 381 to 395
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xf0) << 13);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 396 to 410
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0xe0) << 12);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 411 to 425
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)input[52] << 19) |
			((uint32_t)(input[53] & 0xc0) << 11);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 426 to 440
	value = ((uint32_t)(input[53] & 0x3f) << 26) |
			((uint32_t)input[54] << 18) |
			((uint32_t)(input[55] & 0x80) << 10);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 441 to 455
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)input[56] << 17);
	output[29] = (double)(value >> 17);
}

static void unpack_30_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 450 bits of data.
	// Input must be at least 57 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 366 to 380
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xf8) << 14);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 381 to 395
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xf0) << 13);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 396 to 410
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0xe0) << 12);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 411 to 425
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)input[52] << 19) |
			((uint32_t)(input[53] & 0xc0) << 11);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 426 to 440
	value = ((uint32_t)(input[53] & 0x3f) << 26) |
			((uint32_t)input[54] << 18) |
			((uint32_t)(input[55] & 0x80) << 10);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 441 to 455
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)input[56] << 17);
	output[29] = (double)(value >> 17);
}

static void unpack_30_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 450 bits of data.
	// Input must be at least 58 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 367 to 381
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xfc) << 15);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 382 to 396
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xf8) << 14);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 397 to 411
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xf0) << 13);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 412 to 426
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)input[52] << 20) |
			((uint32_t)(input[53] & 0xe0) << 12);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 427 to 441
	value = ((uint32_t)(input[53] & 0x1f) << 27) |
			((uint32_t)input[54] << 19) |
			((uint32_t)(input[55] & 0xc0) << 11);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 442 to 456
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)input[56] << 18) |
			((uint32_t)(input[57] & 0x80) << 10);
	output[29] = (double)(value >> 17);
}

static void unpack_30_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 450 bits of data.
	// Input must be at least 58 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 367 to 381
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xfc) << 15);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 382 to 396
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xf8) << 14);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 397 to 411
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xf0) << 13);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 412 to 426
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)input[52] << 20) |
			((uint32_t)(input[53] & 0xe0) << 12);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 427 to 441
	value = ((uint32_t)(input[53] & 0x1f) << 27) |
			((uint32_t)input[54] << 19) |
			((uint32_t)(input[55] & 0xc0) << 11);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 442 to 456
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)input[56] << 18) |
			((uint32_t)(input[57] & 0x80) << 10);
	output[29] = (double)(value >> 17);
}

static void unpack_31_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 465 bits of data.
	// Input must be at least 59 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 360 to 374
	value = ((uint32_t)input[45] << 24) |
			((uint32_t)(input[46] & 0xfe) << 16);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 375 to 389
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xfc) << 15);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 390 to 404
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xf8) << 14);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 405 to 419
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xf0) << 13);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 420 to 434
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0xe0) << 12);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 435 to 449
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19) |
			((uint32_t)(input[56] & 0xc0) << 11);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 450 to 464
	value = ((uint32_t)(input[56] & 0x3f) << 26) |
			((uint32_t)input[57] << 18) |
			((uint32_t)(input[58] & 0x80) << 10);
	output[30] = (double)(value >> 17);
}

static void unpack_31_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 465 bits of data.
	// Input must be at least 59 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 360 to 374
	value = ((uint32_t)input[45] << 24) |
			((uint32_t)(input[46] & 0xfe) << 16);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 375 to 389
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xfc) << 15);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 390 to 404
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xf8) << 14);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 405 to 419
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xf0) << 13);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 420 to 434
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0xe0) << 12);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 435 to 449
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19) |
			((uint32_t)(input[56] & 0xc0) << 11);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 450 to 464
	value = ((uint32_t)(input[56] & 0x3f) << 26) |
			((uint32_t)input[57] << 18) |
			((uint32_t)(input[58] & 0x80) << 10);
	output[30] = (double)(value >> 17);
}

static void unpack_31_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 465 bits of data.
	// Input must be at least 59 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 361 to 375
	value = ((uint32_t)(input[45] & 0x7f) << 25) |
			((uint32_t)input[46] << 17);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 376 to 390
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xfe) << 16);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 391 to 405
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xfc) << 15);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 406 to 420
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xf8) << 14);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 421 to 435
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xf0) << 13);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 436 to 450
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0xe0) << 12);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 451 to 465
	value = ((uint32_t)(input[56] & 0x1f) << 27) |
			((uint32_t)input[57] << 19) |
			((uint32_t)(input[58] & 0xc0) << 11);
	output[30] = (double)(value >> 17);
}

static void unpack_31_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 465 bits of data.
	// Input must be at least 59 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 361 to 375
	value = ((uint32_t)(input[45] & 0x7f) << 25) |
			((uint32_t)input[46] << 17);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 376 to 390
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xfe) << 16);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 391 to 405
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xfc) << 15);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 406 to 420
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xf8) << 14);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 421 to 435
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xf0) << 13);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 436 to 450
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0xe0) << 12);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 451 to 465
	value = ((uint32_t)(input[56] & 0x1f) << 27) |
			((uint32_t)input[57] << 19) |
			((uint32_t)(input[58] & 0xc0) << 11);
	output[30] = (double)(value >> 17);
}

static void unpack_31_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 465 bits of data.
	// Input must be at least 59 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 362 to 376
	value = ((uint32_t)(input[45] & 0x3f) << 26) |
			((uint32_t)input[46] << 18) |
			((uint32_t)(input[47] & 0x80) << 10);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 377 to 391
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)input[48] << 17);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 392 to 406
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xfe) << 16);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 407 to 421
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xfc) << 15);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 422 to 436
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xf8) << 14);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 437 to 451
	value = ((uint32_t)(input[54] & 0x07) << 29) |
			((uint32_t)input[55] << 21) |
			((uint32_t)(input[56] & 0xf0) << 13);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 452 to 466
	value = ((uint32_t)(input[56] & 0x0f) << 28) |
			((uint32_t)input[57] << 20) |
			((uint32_t)(input[58] & 0xe0) << 12);
	output[30] = (double)(value >> 17);
}

static void unpack_31_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 465 bits of data.
	// Input must be at least 59 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 362 to 376
	value = ((uint32_t)(input[45] & 0x3f) << 26) |
			((uint32_t)input[46] << 18) |
			((uint32_t)(input[47] & 0x80) << 10);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 377 to 391
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)input[48] << 17);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 392 to 406
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xfe) << 16);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 407 to 421
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xfc) << 15);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 422 to 436
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xf8) << 14);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 437 to 451
	value = ((uint32_t)(input[54] & 0x07) << 29) |
			((uint32_t)input[55] << 21) |
			((uint32_t)(input[56] & 0xf0) << 13);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 452 to 466
	value = ((uint32_t)(input[56] & 0x0f) << 28) |
			((uint32_t)input[57] << 20) |
			((uint32_t)(input[58] & 0xe0) << 12);
	output[30] = (double)(value >> 17);
}

static void unpack_31_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 465 bits of data.
	// Input must be at least 59 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 363 to 377
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)input[46] << 19) |
			((uint32_t)(input[47] & 0xc0) << 11);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 378 to 392
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)input[48] << 18) |
			((uint32_t)(input[49] & 0x80) << 10);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 393 to 407
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)input[50] << 17);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 408 to 422
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xfe) << 16);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 423 to 437
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xfc) << 15);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 438 to 452
	value = ((uint32_t)(input[54] & 0x03) << 30) |
			((uint32_t)input[55] << 22) |
			((uint32_t)(input[56] & 0xf8) << 14);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 453 to 467
	value = ((uint32_t)(input[56] & 0x07) << 29) |
			((uint32_t)input[57] << 21) |
			((uint32_t)(input[58] & 0xf0) << 13);
	output[30] = (double)(value >> 17);
}

static void unpack_31_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 465 bits of data.
	// Input must be at least 59 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 363 to 377
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)input[46] << 19) |
			((uint32_t)(input[47] & 0xc0) << 11);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 378 to 392
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)input[48] << 18) |
			((uint32_t)(input[49] & 0x80) << 10);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 393 to 407
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)input[50] << 17);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 408 to 422
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xfe) << 16);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 423 to 437
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xfc) << 15);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 438 to 452
	value = ((uint32_t)(input[54] & 0x03) << 30) |
			((uint32_t)input[55] << 22) |
			((uint32_t)(input[56] & 0xf8) << 14);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 453 to 467
	value = ((uint32_t)(input[56] & 0x07) << 29) |
			((uint32_t)input[57] << 21) |
			((uint32_t)(input[58] & 0xf0) << 13);
	output[30] = (double)(value >> 17);
}

static void unpack_31_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 465 bits of data.
	// Input must be at least 59 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 364 to 378
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0xe0) << 12);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 379 to 393
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19) |
			((uint32_t)(input[49] & 0xc0) << 11);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 394 to 408
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)input[50] << 18) |
			((uint32_t)(input[51] & 0x80) << 10);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 409 to 423
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)input[52] << 17);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 424 to 438
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xfe) << 16);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 439 to 453
	value = ((uint32_t)(input[54] & 0x01) << 31) |
			((uint32_t)input[55] << 23) |
			((uint32_t)(input[56] & 0xfc) << 15);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 454 to 468
	value = ((uint32_t)(input[56] & 0x03) << 30) |
			((uint32_t)input[57] << 22) |
			((uint32_t)(input[58] & 0xf8) << 14);
	output[30] = (double)(value >> 17);
}

static void unpack_31_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 465 bits of data.
	// Input must be at least 59 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 364 to 378
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0xe0) << 12);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 379 to 393
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19) |
			((uint32_t)(input[49] & 0xc0) << 11);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 394 to 408
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)input[50] << 18) |
			((uint32_t)(input[51] & 0x80) << 10);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 409 to 423
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)input[52] << 17);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 424 to 438
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xfe) << 16);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 439 to 453
	value = ((uint32_t)(input[54] & 0x01) << 31) |
			((uint32_t)input[55] << 23) |
			((uint32_t)(input[56] & 0xfc) << 15);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 454 to 468
	value = ((uint32_t)(input[56] & 0x03) << 30) |
			((uint32_t)input[57] << 22) |
			((uint32_t)(input[58] & 0xf8) << 14);
	output[30] = (double)(value >> 17);
}

static void unpack_31_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 465 bits of data.
	// Input must be at least 59 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 365 to 379
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xf0) << 13);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 380 to 394
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0xe0) << 12);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 395 to 409
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19) |
			((uint32_t)(input[51] & 0xc0) << 11);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 410 to 424
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)input[52] << 18) |
			((uint32_t)(input[53] & 0x80) << 10);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 425 to 439
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)input[54] << 17);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 440 to 454
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xfe) << 16);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 455 to 469
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xfc) << 15);
	output[30] = (double)(value >> 17);
}

static void unpack_31_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 465 bits of data.
	// Input must be at least 59 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 365 to 379
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xf0) << 13);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 380 to 394
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0xe0) << 12);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 395 to 409
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19) |
			((uint32_t)(input[51] & 0xc0) << 11);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 410 to 424
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)input[52] << 18) |
			((uint32_t)(input[53] & 0x80) << 10);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 425 to 439
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)input[54] << 17);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 440 to 454
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xfe) << 16);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 455 to 469
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xfc) << 15);
	output[30] = (double)(value >> 17);
}

static void unpack_31_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 465 bits of data.
	// Input must be at least 59 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 366 to 380
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xf8) << 14);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 381 to 395
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xf0) << 13);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 396 to 410
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0xe0) << 12);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 411 to 425
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)input[52] << 19) |
			((uint32_t)(input[53] & 0xc0) << 11);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 426 to 440
	value = ((uint32_t)(input[53] & 0x3f) << 26) |
			((uint32_t)input[54] << 18) |
			((uint32_t)(input[55] & 0x80) << 10);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 441 to 455
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)input[56] << 17);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 456 to 470
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xfe) << 16);
	output[30] = (double)(value >> 17);
}

static void unpack_31_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 465 bits of data.
	// Input must be at least 59 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 366 to 380
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xf8) << 14);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 381 to 395
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xf0) << 13);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 396 to 410
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0xe0) << 12);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 411 to 425
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)input[52] << 19) |
			((uint32_t)(input[53] & 0xc0) << 11);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 426 to 440
	value = ((uint32_t)(input[53] & 0x3f) << 26) |
			((uint32_t)input[54] << 18) |
			((uint32_t)(input[55] & 0x80) << 10);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 441 to 455
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)input[56] << 17);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 456 to 470
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xfe) << 16);
	output[30] = (double)(value >> 17);
}

static void unpack_31_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 465 bits of data.
	// Input must be at least 59 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 367 to 381
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xfc) << 15);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 382 to 396
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xf8) << 14);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 397 to 411
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xf0) << 13);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 412 to 426
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)input[52] << 20) |
			((uint32_t)(input[53] & 0xe0) << 12);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 427 to 441
	value = ((uint32_t)(input[53] & 0x1f) << 27) |
			((uint32_t)input[54] << 19) |
			((uint32_t)(input[55] & 0xc0) << 11);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 442 to 456
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)input[56] << 18) |
			((uint32_t)(input[57] & 0x80) << 10);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 457 to 471
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)input[58] << 17);
	output[30] = (double)(value >> 17);
}

static void unpack_31_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 465 bits of data.
	// Input must be at least 59 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 367 to 381
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xfc) << 15);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 382 to 396
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xf8) << 14);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 397 to 411
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xf0) << 13);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 412 to 426
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)input[52] << 20) |
			((uint32_t)(input[53] & 0xe0) << 12);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 427 to 441
	value = ((uint32_t)(input[53] & 0x1f) << 27) |
			((uint32_t)input[54] << 19) |
			((uint32_t)(input[55] & 0xc0) << 11);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 442 to 456
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)input[56] << 18) |
			((uint32_t)(input[57] & 0x80) << 10);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 457 to 471
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)input[58] << 17);
	output[30] = (double)(value >> 17);
}

static void unpack_32_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 480 bits of data.
	// Input must be at least 60 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 360 to 374
	value = ((uint32_t)input[45] << 24) |
			((uint32_t)(input[46] & 0xfe) << 16);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 375 to 389
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xfc) << 15);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 390 to 404
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xf8) << 14);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 405 to 419
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xf0) << 13);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 420 to 434
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0xe0) << 12);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 435 to 449
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19) |
			((uint32_t)(input[56] & 0xc0) << 11);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 450 to 464
	value = ((uint32_t)(input[56] & 0x3f) << 26) |
			((uint32_t)input[57] << 18) |
			((uint32_t)(input[58] & 0x80) << 10);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 465 to 479
	value = ((uint32_t)(input[58] & 0x7f) << 25) |
			((uint32_t)input[59] << 17);
	output[31] = (double)(value >> 17);
}

static void unpack_32_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 480 bits of data.
	// Input must be at least 60 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 360 to 374
	value = ((uint32_t)input[45] << 24) |
			((uint32_t)(input[46] & 0xfe) << 16);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 375 to 389
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xfc) << 15);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 390 to 404
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xf8) << 14);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 405 to 419
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xf0) << 13);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 420 to 434
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0xe0) << 12);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 435 to 449
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19) |
			((uint32_t)(input[56] & 0xc0) << 11);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 450 to 464
	value = ((uint32_t)(input[56] & 0x3f) << 26) |
			((uint32_t)input[57] << 18) |
			((uint32_t)(input[58] & 0x80) << 10);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 465 to 479
	value = ((uint32_t)(input[58] & 0x7f) << 25) |
			((uint32_t)input[59] << 17);
	output[31] = (double)(value >> 17);
}

static void unpack_32_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 480 bits of data.
	// Input must be at least 61 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 361 to 375
	value = ((uint32_t)(input[45] & 0x7f) << 25) |
			((uint32_t)input[46] << 17);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 376 to 390
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xfe) << 16);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 391 to 405
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xfc) << 15);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 406 to 420
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xf8) << 14);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 421 to 435
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xf0) << 13);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 436 to 450
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0xe0) << 12);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 451 to 465
	value = ((uint32_t)(input[56] & 0x1f) << 27) |
			((uint32_t)input[57] << 19) |
			((uint32_t)(input[58] & 0xc0) << 11);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 466 to 480
	value = ((uint32_t)(input[58] & 0x3f) << 26) |
			((uint32_t)input[59] << 18) |
			((uint32_t)(input[60] & 0x80) << 10);
	output[31] = (double)(value >> 17);
}

static void unpack_32_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 480 bits of data.
	// Input must be at least 61 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 361 to 375
	value = ((uint32_t)(input[45] & 0x7f) << 25) |
			((uint32_t)input[46] << 17);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 376 to 390
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xfe) << 16);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 391 to 405
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xfc) << 15);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 406 to 420
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xf8) << 14);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 421 to 435
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xf0) << 13);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 436 to 450
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0xe0) << 12);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 451 to 465
	value = ((uint32_t)(input[56] & 0x1f) << 27) |
			((uint32_t)input[57] << 19) |
			((uint32_t)(input[58] & 0xc0) << 11);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 466 to 480
	value = ((uint32_t)(input[58] & 0x3f) << 26) |
			((uint32_t)input[59] << 18) |
			((uint32_t)(input[60] & 0x80) << 10);
	output[31] = (double)(value >> 17);
}

static void unpack_32_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 480 bits of data.
	// Input must be at least 61 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 362 to 376
	value = ((uint32_t)(input[45] & 0x3f) << 26) |
			((uint32_t)input[46] << 18) |
			((uint32_t)(input[47] & 0x80) << 10);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 377 to 391
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)input[48] << 17);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 392 to 406
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xfe) << 16);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 407 to 421
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xfc) << 15);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 422 to 436
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xf8) << 14);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 437 to 451
	value = ((uint32_t)(input[54] & 0x07) << 29) |
			((uint32_t)input[55] << 21) |
			((uint32_t)(input[56] & 0xf0) << 13);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 452 to 466
	value = ((uint32_t)(input[56] & 0x0f) << 28) |
			((uint32_t)input[57] << 20) |
			((uint32_t)(input[58] & 0xe0) << 12);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 467 to 481
	value = ((uint32_t)(input[58] & 0x1f) << 27) |
			((uint32_t)input[59] << 19) |
			((uint32_t)(input[60] & 0xc0) << 11);
	output[31] = (double)(value >> 17);
}

static void unpack_32_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 480 bits of data.
	// Input must be at least 61 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 362 to 376
	value = ((uint32_t)(input[45] & 0x3f) << 26) |
			((uint32_t)input[46] << 18) |
			((uint32_t)(input[47] & 0x80) << 10);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 377 to 391
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)input[48] << 17);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 392 to 406
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xfe) << 16);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 407 to 421
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xfc) << 15);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 422 to 436
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xf8) << 14);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 437 to 451
	value = ((uint32_t)(input[54] & 0x07) << 29) |
			((uint32_t)input[55] << 21) |
			((uint32_t)(input[56] & 0xf0) << 13);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 452 to 466
	value = ((uint32_t)(input[56] & 0x0f) << 28) |
			((uint32_t)input[57] << 20) |
			((uint32_t)(input[58] & 0xe0) << 12);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 467 to 481
	value = ((uint32_t)(input[58] & 0x1f) << 27) |
			((uint32_t)input[59] << 19) |
			((uint32_t)(input[60] & 0xc0) << 11);
	output[31] = (double)(value >> 17);
}

static void unpack_32_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 480 bits of data.
	// Input must be at least 61 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 363 to 377
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)input[46] << 19) |
			((uint32_t)(input[47] & 0xc0) << 11);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 378 to 392
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)input[48] << 18) |
			((uint32_t)(input[49] & 0x80) << 10);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 393 to 407
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)input[50] << 17);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 408 to 422
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xfe) << 16);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 423 to 437
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xfc) << 15);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 438 to 452
	value = ((uint32_t)(input[54] & 0x03) << 30) |
			((uint32_t)input[55] << 22) |
			((uint32_t)(input[56] & 0xf8) << 14);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 453 to 467
	value = ((uint32_t)(input[56] & 0x07) << 29) |
			((uint32_t)input[57] << 21) |
			((uint32_t)(input[58] & 0xf0) << 13);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 468 to 482
	value = ((uint32_t)(input[58] & 0x0f) << 28) |
			((uint32_t)input[59] << 20) |
			((uint32_t)(input[60] & 0xe0) << 12);
	output[31] = (double)(value >> 17);
}

static void unpack_32_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 480 bits of data.
	// Input must be at least 61 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 363 to 377
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)input[46] << 19) |
			((uint32_t)(input[47] & 0xc0) << 11);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 378 to 392
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)input[48] << 18) |
			((uint32_t)(input[49] & 0x80) << 10);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 393 to 407
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)input[50] << 17);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 408 to 422
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xfe) << 16);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 423 to 437
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xfc) << 15);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 438 to 452
	value = ((uint32_t)(input[54] & 0x03) << 30) |
			((uint32_t)input[55] << 22) |
			((uint32_t)(input[56] & 0xf8) << 14);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 453 to 467
	value = ((uint32_t)(input[56] & 0x07) << 29) |
			((uint32_t)input[57] << 21) |
			((uint32_t)(input[58] & 0xf0) << 13);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 468 to 482
	value = ((uint32_t)(input[58] & 0x0f) << 28) |
			((uint32_t)input[59] << 20) |
			((uint32_t)(input[60] & 0xe0) << 12);
	output[31] = (double)(value >> 17);
}

static void unpack_32_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 480 bits of data.
	// Input must be at least 61 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 364 to 378
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0xe0) << 12);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 379 to 393
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19) |
			((uint32_t)(input[49] & 0xc0) << 11);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 394 to 408
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)input[50] << 18) |
			((uint32_t)(input[51] & 0x80) << 10);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 409 to 423
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)input[52] << 17);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 424 to 438
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xfe) << 16);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 439 to 453
	value = ((uint32_t)(input[54] & 0x01) << 31) |
			((uint32_t)input[55] << 23) |
			((uint32_t)(input[56] & 0xfc) << 15);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 454 to 468
	value = ((uint32_t)(input[56] & 0x03) << 30) |
			((uint32_t)input[57] << 22) |
			((uint32_t)(input[58] & 0xf8) << 14);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 469 to 483
	value = ((uint32_t)(input[58] & 0x07) << 29) |
			((uint32_t)input[59] << 21) |
			((uint32_t)(input[60] & 0xf0) << 13);
	output[31] = (double)(value >> 17);
}

static void unpack_32_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 480 bits of data.
	// Input must be at least 61 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 364 to 378
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0xe0) << 12);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 379 to 393
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19) |
			((uint32_t)(input[49] & 0xc0) << 11);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 394 to 408
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)input[50] << 18) |
			((uint32_t)(input[51] & 0x80) << 10);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 409 to 423
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)input[52] << 17);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 424 to 438
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xfe) << 16);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 439 to 453
	value = ((uint32_t)(input[54] & 0x01) << 31) |
			((uint32_t)input[55] << 23) |
			((uint32_t)(input[56] & 0xfc) << 15);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 454 to 468
	value = ((uint32_t)(input[56] & 0x03) << 30) |
			((uint32_t)input[57] << 22) |
			((uint32_t)(input[58] & 0xf8) << 14);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 469 to 483
	value = ((uint32_t)(input[58] & 0x07) << 29) |
			((uint32_t)input[59] << 21) |
			((uint32_t)(input[60] & 0xf0) << 13);
	output[31] = (double)(value >> 17);
}

static void unpack_32_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 480 bits of data.
	// Input must be at least 61 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 365 to 379
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xf0) << 13);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 380 to 394
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0xe0) << 12);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 395 to 409
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19) |
			((uint32_t)(input[51] & 0xc0) << 11);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 410 to 424
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)input[52] << 18) |
			((uint32_t)(input[53] & 0x80) << 10);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 425 to 439
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)input[54] << 17);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 440 to 454
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xfe) << 16);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 455 to 469
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xfc) << 15);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 470 to 484
	value = ((uint32_t)(input[58] & 0x03) << 30) |
			((uint32_t)input[59] << 22) |
			((uint32_t)(input[60] & 0xf8) << 14);
	output[31] = (double)(value >> 17);
}

static void unpack_32_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 480 bits of data.
	// Input must be at least 61 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 365 to 379
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xf0) << 13);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 380 to 394
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0xe0) << 12);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 395 to 409
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19) |
			((uint32_t)(input[51] & 0xc0) << 11);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 410 to 424
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)input[52] << 18) |
			((uint32_t)(input[53] & 0x80) << 10);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 425 to 439
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)input[54] << 17);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 440 to 454
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xfe) << 16);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 455 to 469
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xfc) << 15);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 470 to 484
	value = ((uint32_t)(input[58] & 0x03) << 30) |
			((uint32_t)input[59] << 22) |
			((uint32_t)(input[60] & 0xf8) << 14);
	output[31] = (double)(value >> 17);
}

static void unpack_32_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 480 bits of data.
	// Input must be at least 61 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 366 to 380
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xf8) << 14);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 381 to 395
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xf0) << 13);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 396 to 410
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0xe0) << 12);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 411 to 425
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)input[52] << 19) |
			((uint32_t)(input[53] & 0xc0) << 11);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 426 to 440
	value = ((uint32_t)(input[53] & 0x3f) << 26) |
			((uint32_t)input[54] << 18) |
			((uint32_t)(input[55] & 0x80) << 10);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 441 to 455
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)input[56] << 17);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 456 to 470
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xfe) << 16);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 471 to 485
	value = ((uint32_t)(input[58] & 0x01) << 31) |
			((uint32_t)input[59] << 23) |
			((uint32_t)(input[60] & 0xfc) << 15);
	output[31] = (double)(value >> 17);
}

static void unpack_32_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 480 bits of data.
	// Input must be at least 61 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 366 to 380
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xf8) << 14);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 381 to 395
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xf0) << 13);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 396 to 410
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0xe0) << 12);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 411 to 425
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)input[52] << 19) |
			((uint32_t)(input[53] & 0xc0) << 11);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 426 to 440
	value = ((uint32_t)(input[53] & 0x3f) << 26) |
			((uint32_t)input[54] << 18) |
			((uint32_t)(input[55] & 0x80) << 10);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 441 to 455
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)input[56] << 17);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 456 to 470
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xfe) << 16);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 471 to 485
	value = ((uint32_t)(input[58] & 0x01) << 31) |
			((uint32_t)input[59] << 23) |
			((uint32_t)(input[60] & 0xfc) << 15);
	output[31] = (double)(value >> 17);
}

static void unpack_32_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 480 bits of data.
	// Input must be at least 61 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 367 to 381
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xfc) << 15);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 382 to 396
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xf8) << 14);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 397 to 411
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xf0) << 13);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 412 to 426
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)input[52] << 20) |
			((uint32_t)(input[53] & 0xe0) << 12);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 427 to 441
	value = ((uint32_t)(input[53] & 0x1f) << 27) |
			((uint32_t)input[54] << 19) |
			((uint32_t)(input[55] & 0xc0) << 11);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 442 to 456
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)input[56] << 18) |
			((uint32_t)(input[57] & 0x80) << 10);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 457 to 471
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)input[58] << 17);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 472 to 486
	value = ((uint32_t)input[59] << 24) |
			((uint32_t)(input[60] & 0xfe) << 16);
	output[31] = (double)(value >> 17);
}

static void unpack_32_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 480 bits of data.
	// Input must be at least 61 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 367 to 381
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xfc) << 15);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 382 to 396
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xf8) << 14);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 397 to 411
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xf0) << 13);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 412 to 426
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)input[52] << 20) |
			((uint32_t)(input[53] & 0xe0) << 12);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 427 to 441
	value = ((uint32_t)(input[53] & 0x1f) << 27) |
			((uint32_t)input[54] << 19) |
			((uint32_t)(input[55] & 0xc0) << 11);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 442 to 456
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)input[56] << 18) |
			((uint32_t)(input[57] & 0x80) << 10);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 457 to 471
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)input[58] << 17);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 472 to 486
	value = ((uint32_t)input[59] << 24) |
			((uint32_t)(input[60] & 0xfe) << 16);
	output[31] = (double)(value >> 17);
}

static void unpack_33_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 495 bits of data.
	// Input must be at least 62 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 360 to 374
	value = ((uint32_t)input[45] << 24) |
			((uint32_t)(input[46] & 0xfe) << 16);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 375 to 389
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xfc) << 15);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 390 to 404
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xf8) << 14);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 405 to 419
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xf0) << 13);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 420 to 434
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0xe0) << 12);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 435 to 449
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19) |
			((uint32_t)(input[56] & 0xc0) << 11);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 450 to 464
	value = ((uint32_t)(input[56] & 0x3f) << 26) |
			((uint32_t)input[57] << 18) |
			((uint32_t)(input[58] & 0x80) << 10);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 465 to 479
	value = ((uint32_t)(input[58] & 0x7f) << 25) |
			((uint32_t)input[59] << 17);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 480 to 494
	value = ((uint32_t)input[60] << 24) |
			((uint32_t)(input[61] & 0xfe) << 16);
	output[32] = (double)(value >> 17);
}

static void unpack_33_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 495 bits of data.
	// Input must be at least 62 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 360 to 374
	value = ((uint32_t)input[45] << 24) |
			((uint32_t)(input[46] & 0xfe) << 16);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 375 to 389
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xfc) << 15);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 390 to 404
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xf8) << 14);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 405 to 419
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xf0) << 13);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 420 to 434
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0xe0) << 12);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 435 to 449
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19) |
			((uint32_t)(input[56] & 0xc0) << 11);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 450 to 464
	value = ((uint32_t)(input[56] & 0x3f) << 26) |
			((uint32_t)input[57] << 18) |
			((uint32_t)(input[58] & 0x80) << 10);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 465 to 479
	value = ((uint32_t)(input[58] & 0x7f) << 25) |
			((uint32_t)input[59] << 17);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 480 to 494
	value = ((uint32_t)input[60] << 24) |
			((uint32_t)(input[61] & 0xfe) << 16);
	output[32] = (double)(value >> 17);
}

static void unpack_33_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 495 bits of data.
	// Input must be at least 62 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 361 to 375
	value = ((uint32_t)(input[45] & 0x7f) << 25) |
			((uint32_t)input[46] << 17);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 376 to 390
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xfe) << 16);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 391 to 405
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xfc) << 15);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 406 to 420
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xf8) << 14);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 421 to 435
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xf0) << 13);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 436 to 450
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0xe0) << 12);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 451 to 465
	value = ((uint32_t)(input[56] & 0x1f) << 27) |
			((uint32_t)input[57] << 19) |
			((uint32_t)(input[58] & 0xc0) << 11);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 466 to 480
	value = ((uint32_t)(input[58] & 0x3f) << 26) |
			((uint32_t)input[59] << 18) |
			((uint32_t)(input[60] & 0x80) << 10);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 481 to 495
	value = ((uint32_t)(input[60] & 0x7f) << 25) |
			((uint32_t)input[61] << 17);
	output[32] = (double)(value >> 17);
}

static void unpack_33_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 495 bits of data.
	// Input must be at least 62 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 361 to 375
	value = ((uint32_t)(input[45] & 0x7f) << 25) |
			((uint32_t)input[46] << 17);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 376 to 390
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xfe) << 16);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 391 to 405
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xfc) << 15);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 406 to 420
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xf8) << 14);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 421 to 435
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xf0) << 13);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 436 to 450
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0xe0) << 12);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 451 to 465
	value = ((uint32_t)(input[56] & 0x1f) << 27) |
			((uint32_t)input[57] << 19) |
			((uint32_t)(input[58] & 0xc0) << 11);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 466 to 480
	value = ((uint32_t)(input[58] & 0x3f) << 26) |
			((uint32_t)input[59] << 18) |
			((uint32_t)(input[60] & 0x80) << 10);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 481 to 495
	value = ((uint32_t)(input[60] & 0x7f) << 25) |
			((uint32_t)input[61] << 17);
	output[32] = (double)(value >> 17);
}

static void unpack_33_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 362 to 376
	value = ((uint32_t)(input[45] & 0x3f) << 26) |
			((uint32_t)input[46] << 18) |
			((uint32_t)(input[47] & 0x80) << 10);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 377 to 391
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)input[48] << 17);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 392 to 406
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xfe) << 16);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 407 to 421
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xfc) << 15);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 422 to 436
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xf8) << 14);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 437 to 451
	value = ((uint32_t)(input[54] & 0x07) << 29) |
			((uint32_t)input[55] << 21) |
			((uint32_t)(input[56] & 0xf0) << 13);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 452 to 466
	value = ((uint32_t)(input[56] & 0x0f) << 28) |
			((uint32_t)input[57] << 20) |
			((uint32_t)(input[58] & 0xe0) << 12);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 467 to 481
	value = ((uint32_t)(input[58] & 0x1f) << 27) |
			((uint32_t)input[59] << 19) |
			((uint32_t)(input[60] & 0xc0) << 11);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 482 to 496
	value = ((uint32_t)(input[60] & 0x3f) << 26) |
			((uint32_t)input[61] << 18) |
			((uint32_t)(input[62] & 0x80) << 10);
	output[32] = (double)(value >> 17);
}

static void unpack_33_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 362 to 376
	value = ((uint32_t)(input[45] & 0x3f) << 26) |
			((uint32_t)input[46] << 18) |
			((uint32_t)(input[47] & 0x80) << 10);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 377 to 391
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)input[48] << 17);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 392 to 406
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xfe) << 16);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 407 to 421
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xfc) << 15);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 422 to 436
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xf8) << 14);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 437 to 451
	value = ((uint32_t)(input[54] & 0x07) << 29) |
			((uint32_t)input[55] << 21) |
			((uint32_t)(input[56] & 0xf0) << 13);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 452 to 466
	value = ((uint32_t)(input[56] & 0x0f) << 28) |
			((uint32_t)input[57] << 20) |
			((uint32_t)(input[58] & 0xe0) << 12);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 467 to 481
	value = ((uint32_t)(input[58] & 0x1f) << 27) |
			((uint32_t)input[59] << 19) |
			((uint32_t)(input[60] & 0xc0) << 11);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 482 to 496
	value = ((uint32_t)(input[60] & 0x3f) << 26) |
			((uint32_t)input[61] << 18) |
			((uint32_t)(input[62] & 0x80) << 10);
	output[32] = (double)(value >> 17);
}

static void unpack_33_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 363 to 377
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)input[46] << 19) |
			((uint32_t)(input[47] & 0xc0) << 11);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 378 to 392
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)input[48] << 18) |
			((uint32_t)(input[49] & 0x80) << 10);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 393 to 407
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)input[50] << 17);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 408 to 422
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xfe) << 16);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 423 to 437
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xfc) << 15);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 438 to 452
	value = ((uint32_t)(input[54] & 0x03) << 30) |
			((uint32_t)input[55] << 22) |
			((uint32_t)(input[56] & 0xf8) << 14);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 453 to 467
	value = ((uint32_t)(input[56] & 0x07) << 29) |
			((uint32_t)input[57] << 21) |
			((uint32_t)(input[58] & 0xf0) << 13);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 468 to 482
	value = ((uint32_t)(input[58] & 0x0f) << 28) |
			((uint32_t)input[59] << 20) |
			((uint32_t)(input[60] & 0xe0) << 12);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 483 to 497
	value = ((uint32_t)(input[60] & 0x1f) << 27) |
			((uint32_t)input[61] << 19) |
			((uint32_t)(input[62] & 0xc0) << 11);
	output[32] = (double)(value >> 17);
}

static void unpack_33_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 363 to 377
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)input[46] << 19) |
			((uint32_t)(input[47] & 0xc0) << 11);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 378 to 392
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)input[48] << 18) |
			((uint32_t)(input[49] & 0x80) << 10);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 393 to 407
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)input[50] << 17);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 408 to 422
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xfe) << 16);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 423 to 437
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xfc) << 15);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 438 to 452
	value = ((uint32_t)(input[54] & 0x03) << 30) |
			((uint32_t)input[55] << 22) |
			((uint32_t)(input[56] & 0xf8) << 14);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 453 to 467
	value = ((uint32_t)(input[56] & 0x07) << 29) |
			((uint32_t)input[57] << 21) |
			((uint32_t)(input[58] & 0xf0) << 13);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 468 to 482
	value = ((uint32_t)(input[58] & 0x0f) << 28) |
			((uint32_t)input[59] << 20) |
			((uint32_t)(input[60] & 0xe0) << 12);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 483 to 497
	value = ((uint32_t)(input[60] & 0x1f) << 27) |
			((uint32_t)input[61] << 19) |
			((uint32_t)(input[62] & 0xc0) << 11);
	output[32] = (double)(value >> 17);
}

static void unpack_33_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 364 to 378
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0xe0) << 12);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 379 to 393
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19) |
			((uint32_t)(input[49] & 0xc0) << 11);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 394 to 408
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)input[50] << 18) |
			((uint32_t)(input[51] & 0x80) << 10);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 409 to 423
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)input[52] << 17);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 424 to 438
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xfe) << 16);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 439 to 453
	value = ((uint32_t)(input[54] & 0x01) << 31) |
			((uint32_t)input[55] << 23) |
			((uint32_t)(input[56] & 0xfc) << 15);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 454 to 468
	value = ((uint32_t)(input[56] & 0x03) << 30) |
			((uint32_t)input[57] << 22) |
			((uint32_t)(input[58] & 0xf8) << 14);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 469 to 483
	value = ((uint32_t)(input[58] & 0x07) << 29) |
			((uint32_t)input[59] << 21) |
			((uint32_t)(input[60] & 0xf0) << 13);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 484 to 498
	value = ((uint32_t)(input[60] & 0x0f) << 28) |
			((uint32_t)input[61] << 20) |
			((uint32_t)(input[62] & 0xe0) << 12);
	output[32] = (double)(value >> 17);
}

static void unpack_33_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 364 to 378
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0xe0) << 12);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 379 to 393
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19) |
			((uint32_t)(input[49] & 0xc0) << 11);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 394 to 408
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)input[50] << 18) |
			((uint32_t)(input[51] & 0x80) << 10);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 409 to 423
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)input[52] << 17);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 424 to 438
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xfe) << 16);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 439 to 453
	value = ((uint32_t)(input[54] & 0x01) << 31) |
			((uint32_t)input[55] << 23) |
			((uint32_t)(input[56] & 0xfc) << 15);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 454 to 468
	value = ((uint32_t)(input[56] & 0x03) << 30) |
			((uint32_t)input[57] << 22) |
			((uint32_t)(input[58] & 0xf8) << 14);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 469 to 483
	value = ((uint32_t)(input[58] & 0x07) << 29) |
			((uint32_t)input[59] << 21) |
			((uint32_t)(input[60] & 0xf0) << 13);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 484 to 498
	value = ((uint32_t)(input[60] & 0x0f) << 28) |
			((uint32_t)input[61] << 20) |
			((uint32_t)(input[62] & 0xe0) << 12);
	output[32] = (double)(value >> 17);
}

static void unpack_33_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 365 to 379
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xf0) << 13);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 380 to 394
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0xe0) << 12);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 395 to 409
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19) |
			((uint32_t)(input[51] & 0xc0) << 11);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 410 to 424
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)input[52] << 18) |
			((uint32_t)(input[53] & 0x80) << 10);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 425 to 439
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)input[54] << 17);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 440 to 454
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xfe) << 16);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 455 to 469
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xfc) << 15);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 470 to 484
	value = ((uint32_t)(input[58] & 0x03) << 30) |
			((uint32_t)input[59] << 22) |
			((uint32_t)(input[60] & 0xf8) << 14);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 485 to 499
	value = ((uint32_t)(input[60] & 0x07) << 29) |
			((uint32_t)input[61] << 21) |
			((uint32_t)(input[62] & 0xf0) << 13);
	output[32] = (double)(value >> 17);
}

static void unpack_33_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 365 to 379
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xf0) << 13);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 380 to 394
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0xe0) << 12);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 395 to 409
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19) |
			((uint32_t)(input[51] & 0xc0) << 11);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 410 to 424
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)input[52] << 18) |
			((uint32_t)(input[53] & 0x80) << 10);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 425 to 439
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)input[54] << 17);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 440 to 454
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xfe) << 16);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 455 to 469
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xfc) << 15);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 470 to 484
	value = ((uint32_t)(input[58] & 0x03) << 30) |
			((uint32_t)input[59] << 22) |
			((uint32_t)(input[60] & 0xf8) << 14);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 485 to 499
	value = ((uint32_t)(input[60] & 0x07) << 29) |
			((uint32_t)input[61] << 21) |
			((uint32_t)(input[62] & 0xf0) << 13);
	output[32] = (double)(value >> 17);
}

static void unpack_33_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 366 to 380
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xf8) << 14);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 381 to 395
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xf0) << 13);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 396 to 410
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0xe0) << 12);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 411 to 425
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)input[52] << 19) |
			((uint32_t)(input[53] & 0xc0) << 11);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 426 to 440
	value = ((uint32_t)(input[53] & 0x3f) << 26) |
			((uint32_t)input[54] << 18) |
			((uint32_t)(input[55] & 0x80) << 10);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 441 to 455
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)input[56] << 17);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 456 to 470
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xfe) << 16);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 471 to 485
	value = ((uint32_t)(input[58] & 0x01) << 31) |
			((uint32_t)input[59] << 23) |
			((uint32_t)(input[60] & 0xfc) << 15);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 486 to 500
	value = ((uint32_t)(input[60] & 0x03) << 30) |
			((uint32_t)input[61] << 22) |
			((uint32_t)(input[62] & 0xf8) << 14);
	output[32] = (double)(value >> 17);
}

static void unpack_33_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 366 to 380
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xf8) << 14);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 381 to 395
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xf0) << 13);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 396 to 410
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0xe0) << 12);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 411 to 425
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)input[52] << 19) |
			((uint32_t)(input[53] & 0xc0) << 11);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 426 to 440
	value = ((uint32_t)(input[53] & 0x3f) << 26) |
			((uint32_t)input[54] << 18) |
			((uint32_t)(input[55] & 0x80) << 10);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 441 to 455
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)input[56] << 17);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 456 to 470
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xfe) << 16);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 471 to 485
	value = ((uint32_t)(input[58] & 0x01) << 31) |
			((uint32_t)input[59] << 23) |
			((uint32_t)(input[60] & 0xfc) << 15);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 486 to 500
	value = ((uint32_t)(input[60] & 0x03) << 30) |
			((uint32_t)input[61] << 22) |
			((uint32_t)(input[62] & 0xf8) << 14);
	output[32] = (double)(value >> 17);
}

static void unpack_33_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 367 to 381
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xfc) << 15);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 382 to 396
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xf8) << 14);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 397 to 411
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xf0) << 13);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 412 to 426
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)input[52] << 20) |
			((uint32_t)(input[53] & 0xe0) << 12);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 427 to 441
	value = ((uint32_t)(input[53] & 0x1f) << 27) |
			((uint32_t)input[54] << 19) |
			((uint32_t)(input[55] & 0xc0) << 11);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 442 to 456
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)input[56] << 18) |
			((uint32_t)(input[57] & 0x80) << 10);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 457 to 471
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)input[58] << 17);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 472 to 486
	value = ((uint32_t)input[59] << 24) |
			((uint32_t)(input[60] & 0xfe) << 16);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 487 to 501
	value = ((uint32_t)(input[60] & 0x01) << 31) |
			((uint32_t)input[61] << 23) |
			((uint32_t)(input[62] & 0xfc) << 15);
	output[32] = (double)(value >> 17);
}

static void unpack_33_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 367 to 381
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xfc) << 15);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 382 to 396
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xf8) << 14);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 397 to 411
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xf0) << 13);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 412 to 426
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)input[52] << 20) |
			((uint32_t)(input[53] & 0xe0) << 12);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 427 to 441
	value = ((uint32_t)(input[53] & 0x1f) << 27) |
			((uint32_t)input[54] << 19) |
			((uint32_t)(input[55] & 0xc0) << 11);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 442 to 456
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)input[56] << 18) |
			((uint32_t)(input[57] & 0x80) << 10);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 457 to 471
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)input[58] << 17);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 472 to 486
	value = ((uint32_t)input[59] << 24) |
			((uint32_t)(input[60] & 0xfe) << 16);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 487 to 501
	value = ((uint32_t)(input[60] & 0x01) << 31) |
			((uint32_t)input[61] << 23) |
			((uint32_t)(input[62] & 0xfc) << 15);
	output[32] = (double)(value >> 17);
}

static void unpack_34_15bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 15-bit signed
	// Skip 0 bits, read 510 bits of data.
	// Input must be at least 64 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 360 to 374
	value = ((uint32_t)input[45] << 24) |
			((uint32_t)(input[46] & 0xfe) << 16);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 375 to 389
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xfc) << 15);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 390 to 404
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xf8) << 14);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 405 to 419
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xf0) << 13);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 420 to 434
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0xe0) << 12);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 435 to 449
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19) |
			((uint32_t)(input[56] & 0xc0) << 11);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 450 to 464
	value = ((uint32_t)(input[56] & 0x3f) << 26) |
			((uint32_t)input[57] << 18) |
			((uint32_t)(input[58] & 0x80) << 10);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 465 to 479
	value = ((uint32_t)(input[58] & 0x7f) << 25) |
			((uint32_t)input[59] << 17);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 480 to 494
	value = ((uint32_t)input[60] << 24) |
			((uint32_t)(input[61] & 0xfe) << 16);
	output[32] = (double)(value >> 17);

	// read output[33]: from bit 495 to 509
	value = ((uint32_t)(input[61] & 0x01) << 31) |
			((uint32_t)input[62] << 23) |
			((uint32_t)(input[63] & 0xfc) << 15);
	output[33] = (double)(value >> 17);
}

static void unpack_34_15bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 15-bit unsigned
	// Skip 0 bits, read 510 bits of data.
	// Input must be at least 64 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 14
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xfe) << 16);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 15 to 29
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xfc) << 15);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 30 to 44
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xf8) << 14);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 45 to 59
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xf0) << 13);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 60 to 74
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)input[8] << 20) |
			((uint32_t)(input[9] & 0xe0) << 12);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 75 to 89
	value = ((uint32_t)(input[9] & 0x1f) << 27) |
			((uint32_t)input[10] << 19) |
			((uint32_t)(input[11] & 0xc0) << 11);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 90 to 104
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)input[12] << 18) |
			((uint32_t)(input[13] & 0x80) << 10);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 105 to 119
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)input[14] << 17);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 120 to 134
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xfe) << 16);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 135 to 149
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xfc) << 15);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 150 to 164
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xf8) << 14);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 165 to 179
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21) |
			((uint32_t)(input[22] & 0xf0) << 13);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 180 to 194
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)input[23] << 20) |
			((uint32_t)(input[24] & 0xe0) << 12);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 195 to 209
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19) |
			((uint32_t)(input[26] & 0xc0) << 11);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 210 to 224
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)input[27] << 18) |
			((uint32_t)(input[28] & 0x80) << 10);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 225 to 239
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)input[29] << 17);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 240 to 254
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xfe) << 16);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 255 to 269
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xfc) << 15);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 270 to 284
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0xf8) << 14);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 285 to 299
	value = ((uint32_t)(input[35] & 0x07) << 29) |
			((uint32_t)input[36] << 21) |
			((uint32_t)(input[37] & 0xf0) << 13);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 300 to 314
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0xe0) << 12);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 315 to 329
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19) |
			((uint32_t)(input[41] & 0xc0) << 11);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 330 to 344
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)input[42] << 18) |
			((uint32_t)(input[43] & 0x80) << 10);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 345 to 359
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)input[44] << 17);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 360 to 374
	value = ((uint32_t)input[45] << 24) |
			((uint32_t)(input[46] & 0xfe) << 16);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 375 to 389
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xfc) << 15);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 390 to 404
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xf8) << 14);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 405 to 419
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xf0) << 13);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 420 to 434
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0xe0) << 12);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 435 to 449
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19) |
			((uint32_t)(input[56] & 0xc0) << 11);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 450 to 464
	value = ((uint32_t)(input[56] & 0x3f) << 26) |
			((uint32_t)input[57] << 18) |
			((uint32_t)(input[58] & 0x80) << 10);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 465 to 479
	value = ((uint32_t)(input[58] & 0x7f) << 25) |
			((uint32_t)input[59] << 17);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 480 to 494
	value = ((uint32_t)input[60] << 24) |
			((uint32_t)(input[61] & 0xfe) << 16);
	output[32] = (double)(value >> 17);

	// read output[33]: from bit 495 to 509
	value = ((uint32_t)(input[61] & 0x01) << 31) |
			((uint32_t)input[62] << 23) |
			((uint32_t)(input[63] & 0xfc) << 15);
	output[33] = (double)(value >> 17);
}

static void unpack_34_15bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 15-bit signed
	// Skip 1 bits, read 510 bits of data.
	// Input must be at least 64 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 361 to 375
	value = ((uint32_t)(input[45] & 0x7f) << 25) |
			((uint32_t)input[46] << 17);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 376 to 390
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xfe) << 16);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 391 to 405
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xfc) << 15);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 406 to 420
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xf8) << 14);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 421 to 435
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xf0) << 13);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 436 to 450
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0xe0) << 12);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 451 to 465
	value = ((uint32_t)(input[56] & 0x1f) << 27) |
			((uint32_t)input[57] << 19) |
			((uint32_t)(input[58] & 0xc0) << 11);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 466 to 480
	value = ((uint32_t)(input[58] & 0x3f) << 26) |
			((uint32_t)input[59] << 18) |
			((uint32_t)(input[60] & 0x80) << 10);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 481 to 495
	value = ((uint32_t)(input[60] & 0x7f) << 25) |
			((uint32_t)input[61] << 17);
	output[32] = (double)(value >> 17);

	// read output[33]: from bit 496 to 510
	value = ((uint32_t)input[62] << 24) |
			((uint32_t)(input[63] & 0xfe) << 16);
	output[33] = (double)(value >> 17);
}

static void unpack_34_15bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 15-bit unsigned
	// Skip 1 bits, read 510 bits of data.
	// Input must be at least 64 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 15
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)input[1] << 17);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 16 to 30
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xfe) << 16);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 31 to 45
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xfc) << 15);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 46 to 60
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xf8) << 14);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 61 to 75
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21) |
			((uint32_t)(input[9] & 0xf0) << 13);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 76 to 90
	value = ((uint32_t)(input[9] & 0x0f) << 28) |
			((uint32_t)input[10] << 20) |
			((uint32_t)(input[11] & 0xe0) << 12);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 91 to 105
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19) |
			((uint32_t)(input[13] & 0xc0) << 11);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 106 to 120
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)input[14] << 18) |
			((uint32_t)(input[15] & 0x80) << 10);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 121 to 135
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)input[16] << 17);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 136 to 150
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xfe) << 16);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 151 to 165
	value = ((uint32_t)(input[18] & 0x01) << 31) |
			((uint32_t)input[19] << 23) |
			((uint32_t)(input[20] & 0xfc) << 15);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 166 to 180
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0xf8) << 14);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 181 to 195
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21) |
			((uint32_t)(input[24] & 0xf0) << 13);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 196 to 210
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0xe0) << 12);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 211 to 225
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19) |
			((uint32_t)(input[28] & 0xc0) << 11);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 226 to 240
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)input[29] << 18) |
			((uint32_t)(input[30] & 0x80) << 10);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 241 to 255
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)input[31] << 17);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 256 to 270
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xfe) << 16);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 271 to 285
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xfc) << 15);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 286 to 300
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xf8) << 14);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 301 to 315
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xf0) << 13);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 316 to 330
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0xe0) << 12);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 331 to 345
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19) |
			((uint32_t)(input[43] & 0xc0) << 11);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 346 to 360
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)input[44] << 18) |
			((uint32_t)(input[45] & 0x80) << 10);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 361 to 375
	value = ((uint32_t)(input[45] & 0x7f) << 25) |
			((uint32_t)input[46] << 17);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 376 to 390
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xfe) << 16);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 391 to 405
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xfc) << 15);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 406 to 420
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xf8) << 14);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 421 to 435
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xf0) << 13);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 436 to 450
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0xe0) << 12);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 451 to 465
	value = ((uint32_t)(input[56] & 0x1f) << 27) |
			((uint32_t)input[57] << 19) |
			((uint32_t)(input[58] & 0xc0) << 11);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 466 to 480
	value = ((uint32_t)(input[58] & 0x3f) << 26) |
			((uint32_t)input[59] << 18) |
			((uint32_t)(input[60] & 0x80) << 10);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 481 to 495
	value = ((uint32_t)(input[60] & 0x7f) << 25) |
			((uint32_t)input[61] << 17);
	output[32] = (double)(value >> 17);

	// read output[33]: from bit 496 to 510
	value = ((uint32_t)input[62] << 24) |
			((uint32_t)(input[63] & 0xfe) << 16);
	output[33] = (double)(value >> 17);
}

static void unpack_34_15bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 15-bit signed
	// Skip 2 bits, read 510 bits of data.
	// Input must be at least 64 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 362 to 376
	value = ((uint32_t)(input[45] & 0x3f) << 26) |
			((uint32_t)input[46] << 18) |
			((uint32_t)(input[47] & 0x80) << 10);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 377 to 391
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)input[48] << 17);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 392 to 406
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xfe) << 16);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 407 to 421
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xfc) << 15);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 422 to 436
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xf8) << 14);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 437 to 451
	value = ((uint32_t)(input[54] & 0x07) << 29) |
			((uint32_t)input[55] << 21) |
			((uint32_t)(input[56] & 0xf0) << 13);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 452 to 466
	value = ((uint32_t)(input[56] & 0x0f) << 28) |
			((uint32_t)input[57] << 20) |
			((uint32_t)(input[58] & 0xe0) << 12);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 467 to 481
	value = ((uint32_t)(input[58] & 0x1f) << 27) |
			((uint32_t)input[59] << 19) |
			((uint32_t)(input[60] & 0xc0) << 11);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 482 to 496
	value = ((uint32_t)(input[60] & 0x3f) << 26) |
			((uint32_t)input[61] << 18) |
			((uint32_t)(input[62] & 0x80) << 10);
	output[32] = (double)(value >> 17);

	// read output[33]: from bit 497 to 511
	value = ((uint32_t)(input[62] & 0x7f) << 25) |
			((uint32_t)input[63] << 17);
	output[33] = (double)(value >> 17);
}

static void unpack_34_15bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 15-bit unsigned
	// Skip 2 bits, read 510 bits of data.
	// Input must be at least 64 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 16
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)input[1] << 18) |
			((uint32_t)(input[2] & 0x80) << 10);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 17 to 31
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)input[3] << 17);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 32 to 46
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xfe) << 16);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 47 to 61
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xfc) << 15);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 62 to 76
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0xf8) << 14);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 77 to 91
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21) |
			((uint32_t)(input[11] & 0xf0) << 13);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 92 to 106
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0xe0) << 12);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 107 to 121
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19) |
			((uint32_t)(input[15] & 0xc0) << 11);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 122 to 136
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)input[16] << 18) |
			((uint32_t)(input[17] & 0x80) << 10);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 137 to 151
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)input[18] << 17);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 152 to 166
	value = ((uint32_t)input[19] << 24) |
			((uint32_t)(input[20] & 0xfe) << 16);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 167 to 181
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xfc) << 15);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 182 to 196
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xf8) << 14);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 197 to 211
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xf0) << 13);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 212 to 226
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0xe0) << 12);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 227 to 241
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19) |
			((uint32_t)(input[30] & 0xc0) << 11);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 242 to 256
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)input[31] << 18) |
			((uint32_t)(input[32] & 0x80) << 10);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 257 to 271
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)input[33] << 17);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 272 to 286
	value = ((uint32_t)input[34] << 24) |
			((uint32_t)(input[35] & 0xfe) << 16);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 287 to 301
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xfc) << 15);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 302 to 316
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xf8) << 14);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 317 to 331
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xf0) << 13);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 332 to 346
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0xe0) << 12);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 347 to 361
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)input[44] << 19) |
			((uint32_t)(input[45] & 0xc0) << 11);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 362 to 376
	value = ((uint32_t)(input[45] & 0x3f) << 26) |
			((uint32_t)input[46] << 18) |
			((uint32_t)(input[47] & 0x80) << 10);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 377 to 391
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)input[48] << 17);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 392 to 406
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xfe) << 16);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 407 to 421
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xfc) << 15);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 422 to 436
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xf8) << 14);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 437 to 451
	value = ((uint32_t)(input[54] & 0x07) << 29) |
			((uint32_t)input[55] << 21) |
			((uint32_t)(input[56] & 0xf0) << 13);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 452 to 466
	value = ((uint32_t)(input[56] & 0x0f) << 28) |
			((uint32_t)input[57] << 20) |
			((uint32_t)(input[58] & 0xe0) << 12);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 467 to 481
	value = ((uint32_t)(input[58] & 0x1f) << 27) |
			((uint32_t)input[59] << 19) |
			((uint32_t)(input[60] & 0xc0) << 11);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 482 to 496
	value = ((uint32_t)(input[60] & 0x3f) << 26) |
			((uint32_t)input[61] << 18) |
			((uint32_t)(input[62] & 0x80) << 10);
	output[32] = (double)(value >> 17);

	// read output[33]: from bit 497 to 511
	value = ((uint32_t)(input[62] & 0x7f) << 25) |
			((uint32_t)input[63] << 17);
	output[33] = (double)(value >> 17);
}

static void unpack_34_15bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 15-bit signed
	// Skip 3 bits, read 510 bits of data.
	// Input must be at least 65 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 363 to 377
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)input[46] << 19) |
			((uint32_t)(input[47] & 0xc0) << 11);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 378 to 392
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)input[48] << 18) |
			((uint32_t)(input[49] & 0x80) << 10);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 393 to 407
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)input[50] << 17);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 408 to 422
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xfe) << 16);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 423 to 437
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xfc) << 15);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 438 to 452
	value = ((uint32_t)(input[54] & 0x03) << 30) |
			((uint32_t)input[55] << 22) |
			((uint32_t)(input[56] & 0xf8) << 14);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 453 to 467
	value = ((uint32_t)(input[56] & 0x07) << 29) |
			((uint32_t)input[57] << 21) |
			((uint32_t)(input[58] & 0xf0) << 13);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 468 to 482
	value = ((uint32_t)(input[58] & 0x0f) << 28) |
			((uint32_t)input[59] << 20) |
			((uint32_t)(input[60] & 0xe0) << 12);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 483 to 497
	value = ((uint32_t)(input[60] & 0x1f) << 27) |
			((uint32_t)input[61] << 19) |
			((uint32_t)(input[62] & 0xc0) << 11);
	output[32] = (double)(value >> 17);

	// read output[33]: from bit 498 to 512
	value = ((uint32_t)(input[62] & 0x3f) << 26) |
			((uint32_t)input[63] << 18) |
			((uint32_t)(input[64] & 0x80) << 10);
	output[33] = (double)(value >> 17);
}

static void unpack_34_15bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 15-bit unsigned
	// Skip 3 bits, read 510 bits of data.
	// Input must be at least 65 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 17
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19) |
			((uint32_t)(input[2] & 0xc0) << 11);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 18 to 32
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)input[3] << 18) |
			((uint32_t)(input[4] & 0x80) << 10);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 33 to 47
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)input[5] << 17);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 48 to 62
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xfe) << 16);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 63 to 77
	value = ((uint32_t)(input[7] & 0x01) << 31) |
			((uint32_t)input[8] << 23) |
			((uint32_t)(input[9] & 0xfc) << 15);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 78 to 92
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xf8) << 14);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 93 to 107
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xf0) << 13);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 108 to 122
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0xe0) << 12);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 123 to 137
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19) |
			((uint32_t)(input[17] & 0xc0) << 11);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 138 to 152
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)input[18] << 18) |
			((uint32_t)(input[19] & 0x80) << 10);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 153 to 167
	value = ((uint32_t)(input[19] & 0x7f) << 25) |
			((uint32_t)input[20] << 17);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 168 to 182
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xfe) << 16);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 183 to 197
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xfc) << 15);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 198 to 212
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xf8) << 14);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 213 to 227
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xf0) << 13);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 228 to 242
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0xe0) << 12);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 243 to 257
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)input[31] << 19) |
			((uint32_t)(input[32] & 0xc0) << 11);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 258 to 272
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)input[33] << 18) |
			((uint32_t)(input[34] & 0x80) << 10);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 273 to 287
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)input[35] << 17);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 288 to 302
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xfe) << 16);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 303 to 317
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xfc) << 15);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 318 to 332
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xf8) << 14);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 333 to 347
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21) |
			((uint32_t)(input[43] & 0xf0) << 13);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 348 to 362
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)input[44] << 20) |
			((uint32_t)(input[45] & 0xe0) << 12);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 363 to 377
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)input[46] << 19) |
			((uint32_t)(input[47] & 0xc0) << 11);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 378 to 392
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)input[48] << 18) |
			((uint32_t)(input[49] & 0x80) << 10);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 393 to 407
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)input[50] << 17);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 408 to 422
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xfe) << 16);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 423 to 437
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xfc) << 15);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 438 to 452
	value = ((uint32_t)(input[54] & 0x03) << 30) |
			((uint32_t)input[55] << 22) |
			((uint32_t)(input[56] & 0xf8) << 14);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 453 to 467
	value = ((uint32_t)(input[56] & 0x07) << 29) |
			((uint32_t)input[57] << 21) |
			((uint32_t)(input[58] & 0xf0) << 13);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 468 to 482
	value = ((uint32_t)(input[58] & 0x0f) << 28) |
			((uint32_t)input[59] << 20) |
			((uint32_t)(input[60] & 0xe0) << 12);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 483 to 497
	value = ((uint32_t)(input[60] & 0x1f) << 27) |
			((uint32_t)input[61] << 19) |
			((uint32_t)(input[62] & 0xc0) << 11);
	output[32] = (double)(value >> 17);

	// read output[33]: from bit 498 to 512
	value = ((uint32_t)(input[62] & 0x3f) << 26) |
			((uint32_t)input[63] << 18) |
			((uint32_t)(input[64] & 0x80) << 10);
	output[33] = (double)(value >> 17);
}

static void unpack_34_15bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 15-bit signed
	// Skip 4 bits, read 510 bits of data.
	// Input must be at least 65 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 364 to 378
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0xe0) << 12);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 379 to 393
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19) |
			((uint32_t)(input[49] & 0xc0) << 11);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 394 to 408
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)input[50] << 18) |
			((uint32_t)(input[51] & 0x80) << 10);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 409 to 423
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)input[52] << 17);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 424 to 438
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xfe) << 16);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 439 to 453
	value = ((uint32_t)(input[54] & 0x01) << 31) |
			((uint32_t)input[55] << 23) |
			((uint32_t)(input[56] & 0xfc) << 15);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 454 to 468
	value = ((uint32_t)(input[56] & 0x03) << 30) |
			((uint32_t)input[57] << 22) |
			((uint32_t)(input[58] & 0xf8) << 14);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 469 to 483
	value = ((uint32_t)(input[58] & 0x07) << 29) |
			((uint32_t)input[59] << 21) |
			((uint32_t)(input[60] & 0xf0) << 13);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 484 to 498
	value = ((uint32_t)(input[60] & 0x0f) << 28) |
			((uint32_t)input[61] << 20) |
			((uint32_t)(input[62] & 0xe0) << 12);
	output[32] = (double)(value >> 17);

	// read output[33]: from bit 499 to 513
	value = ((uint32_t)(input[62] & 0x1f) << 27) |
			((uint32_t)input[63] << 19) |
			((uint32_t)(input[64] & 0xc0) << 11);
	output[33] = (double)(value >> 17);
}

static void unpack_34_15bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 15-bit unsigned
	// Skip 4 bits, read 510 bits of data.
	// Input must be at least 65 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 18
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0xe0) << 12);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 19 to 33
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19) |
			((uint32_t)(input[4] & 0xc0) << 11);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 34 to 48
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)input[5] << 18) |
			((uint32_t)(input[6] & 0x80) << 10);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 49 to 63
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)input[7] << 17);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 64 to 78
	value = ((uint32_t)input[8] << 24) |
			((uint32_t)(input[9] & 0xfe) << 16);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 79 to 93
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xfc) << 15);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 94 to 108
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xf8) << 14);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 109 to 123
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xf0) << 13);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 124 to 138
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0xe0) << 12);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 139 to 153
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)input[18] << 19) |
			((uint32_t)(input[19] & 0xc0) << 11);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 154 to 168
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)input[20] << 18) |
			((uint32_t)(input[21] & 0x80) << 10);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 169 to 183
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)input[22] << 17);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 184 to 198
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xfe) << 16);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 199 to 213
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xfc) << 15);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 214 to 228
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xf8) << 14);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 229 to 243
	value = ((uint32_t)(input[28] & 0x07) << 29) |
			((uint32_t)input[29] << 21) |
			((uint32_t)(input[30] & 0xf0) << 13);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 244 to 258
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)input[31] << 20) |
			((uint32_t)(input[32] & 0xe0) << 12);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 259 to 273
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)input[33] << 19) |
			((uint32_t)(input[34] & 0xc0) << 11);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 274 to 288
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)input[35] << 18) |
			((uint32_t)(input[36] & 0x80) << 10);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 289 to 303
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)input[37] << 17);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 304 to 318
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xfe) << 16);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 319 to 333
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xfc) << 15);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 334 to 348
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0xf8) << 14);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 349 to 363
	value = ((uint32_t)(input[43] & 0x07) << 29) |
			((uint32_t)input[44] << 21) |
			((uint32_t)(input[45] & 0xf0) << 13);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 364 to 378
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0xe0) << 12);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 379 to 393
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19) |
			((uint32_t)(input[49] & 0xc0) << 11);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 394 to 408
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)input[50] << 18) |
			((uint32_t)(input[51] & 0x80) << 10);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 409 to 423
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)input[52] << 17);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 424 to 438
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xfe) << 16);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 439 to 453
	value = ((uint32_t)(input[54] & 0x01) << 31) |
			((uint32_t)input[55] << 23) |
			((uint32_t)(input[56] & 0xfc) << 15);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 454 to 468
	value = ((uint32_t)(input[56] & 0x03) << 30) |
			((uint32_t)input[57] << 22) |
			((uint32_t)(input[58] & 0xf8) << 14);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 469 to 483
	value = ((uint32_t)(input[58] & 0x07) << 29) |
			((uint32_t)input[59] << 21) |
			((uint32_t)(input[60] & 0xf0) << 13);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 484 to 498
	value = ((uint32_t)(input[60] & 0x0f) << 28) |
			((uint32_t)input[61] << 20) |
			((uint32_t)(input[62] & 0xe0) << 12);
	output[32] = (double)(value >> 17);

	// read output[33]: from bit 499 to 513
	value = ((uint32_t)(input[62] & 0x1f) << 27) |
			((uint32_t)input[63] << 19) |
			((uint32_t)(input[64] & 0xc0) << 11);
	output[33] = (double)(value >> 17);
}

static void unpack_34_15bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 15-bit signed
	// Skip 5 bits, read 510 bits of data.
	// Input must be at least 65 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 365 to 379
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xf0) << 13);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 380 to 394
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0xe0) << 12);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 395 to 409
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19) |
			((uint32_t)(input[51] & 0xc0) << 11);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 410 to 424
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)input[52] << 18) |
			((uint32_t)(input[53] & 0x80) << 10);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 425 to 439
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)input[54] << 17);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 440 to 454
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xfe) << 16);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 455 to 469
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xfc) << 15);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 470 to 484
	value = ((uint32_t)(input[58] & 0x03) << 30) |
			((uint32_t)input[59] << 22) |
			((uint32_t)(input[60] & 0xf8) << 14);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 485 to 499
	value = ((uint32_t)(input[60] & 0x07) << 29) |
			((uint32_t)input[61] << 21) |
			((uint32_t)(input[62] & 0xf0) << 13);
	output[32] = (double)(value >> 17);

	// read output[33]: from bit 500 to 514
	value = ((uint32_t)(input[62] & 0x0f) << 28) |
			((uint32_t)input[63] << 20) |
			((uint32_t)(input[64] & 0xe0) << 12);
	output[33] = (double)(value >> 17);
}

static void unpack_34_15bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 15-bit unsigned
	// Skip 5 bits, read 510 bits of data.
	// Input must be at least 65 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 19
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xf0) << 13);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 20 to 34
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0xe0) << 12);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 35 to 49
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)input[5] << 19) |
			((uint32_t)(input[6] & 0xc0) << 11);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 50 to 64
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)input[7] << 18) |
			((uint32_t)(input[8] & 0x80) << 10);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 65 to 79
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)input[9] << 17);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 80 to 94
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xfe) << 16);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 95 to 109
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xfc) << 15);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 110 to 124
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xf8) << 14);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 125 to 139
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21) |
			((uint32_t)(input[17] & 0xf0) << 13);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 140 to 154
	value = ((uint32_t)(input[17] & 0x0f) << 28) |
			((uint32_t)input[18] << 20) |
			((uint32_t)(input[19] & 0xe0) << 12);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 155 to 169
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)input[20] << 19) |
			((uint32_t)(input[21] & 0xc0) << 11);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 170 to 184
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)input[22] << 18) |
			((uint32_t)(input[23] & 0x80) << 10);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 185 to 199
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)input[24] << 17);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 200 to 214
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xfe) << 16);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 215 to 229
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xfc) << 15);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 230 to 244
	value = ((uint32_t)(input[28] & 0x03) << 30) |
			((uint32_t)input[29] << 22) |
			((uint32_t)(input[30] & 0xf8) << 14);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 245 to 259
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21) |
			((uint32_t)(input[32] & 0xf0) << 13);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 260 to 274
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0xe0) << 12);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 275 to 289
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19) |
			((uint32_t)(input[36] & 0xc0) << 11);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 290 to 304
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)input[37] << 18) |
			((uint32_t)(input[38] & 0x80) << 10);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 305 to 319
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)input[39] << 17);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 320 to 334
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xfe) << 16);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 335 to 349
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xfc) << 15);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 350 to 364
	value = ((uint32_t)(input[43] & 0x03) << 30) |
			((uint32_t)input[44] << 22) |
			((uint32_t)(input[45] & 0xf8) << 14);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 365 to 379
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xf0) << 13);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 380 to 394
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0xe0) << 12);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 395 to 409
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19) |
			((uint32_t)(input[51] & 0xc0) << 11);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 410 to 424
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)input[52] << 18) |
			((uint32_t)(input[53] & 0x80) << 10);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 425 to 439
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)input[54] << 17);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 440 to 454
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xfe) << 16);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 455 to 469
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xfc) << 15);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 470 to 484
	value = ((uint32_t)(input[58] & 0x03) << 30) |
			((uint32_t)input[59] << 22) |
			((uint32_t)(input[60] & 0xf8) << 14);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 485 to 499
	value = ((uint32_t)(input[60] & 0x07) << 29) |
			((uint32_t)input[61] << 21) |
			((uint32_t)(input[62] & 0xf0) << 13);
	output[32] = (double)(value >> 17);

	// read output[33]: from bit 500 to 514
	value = ((uint32_t)(input[62] & 0x0f) << 28) |
			((uint32_t)input[63] << 20) |
			((uint32_t)(input[64] & 0xe0) << 12);
	output[33] = (double)(value >> 17);
}

static void unpack_34_15bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 15-bit signed
	// Skip 6 bits, read 510 bits of data.
	// Input must be at least 65 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 366 to 380
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xf8) << 14);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 381 to 395
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xf0) << 13);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 396 to 410
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0xe0) << 12);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 411 to 425
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)input[52] << 19) |
			((uint32_t)(input[53] & 0xc0) << 11);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 426 to 440
	value = ((uint32_t)(input[53] & 0x3f) << 26) |
			((uint32_t)input[54] << 18) |
			((uint32_t)(input[55] & 0x80) << 10);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 441 to 455
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)input[56] << 17);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 456 to 470
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xfe) << 16);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 471 to 485
	value = ((uint32_t)(input[58] & 0x01) << 31) |
			((uint32_t)input[59] << 23) |
			((uint32_t)(input[60] & 0xfc) << 15);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 486 to 500
	value = ((uint32_t)(input[60] & 0x03) << 30) |
			((uint32_t)input[61] << 22) |
			((uint32_t)(input[62] & 0xf8) << 14);
	output[32] = (double)(value >> 17);

	// read output[33]: from bit 501 to 515
	value = ((uint32_t)(input[62] & 0x07) << 29) |
			((uint32_t)input[63] << 21) |
			((uint32_t)(input[64] & 0xf0) << 13);
	output[33] = (double)(value >> 17);
}

static void unpack_34_15bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 15-bit unsigned
	// Skip 6 bits, read 510 bits of data.
	// Input must be at least 65 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 20
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xf8) << 14);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 21 to 35
	value = ((uint32_t)(input[2] & 0x07) << 29) |
			((uint32_t)input[3] << 21) |
			((uint32_t)(input[4] & 0xf0) << 13);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 36 to 50
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)input[5] << 20) |
			((uint32_t)(input[6] & 0xe0) << 12);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 51 to 65
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)input[7] << 19) |
			((uint32_t)(input[8] & 0xc0) << 11);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 66 to 80
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)input[9] << 18) |
			((uint32_t)(input[10] & 0x80) << 10);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 81 to 95
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)input[11] << 17);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 96 to 110
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xfe) << 16);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 111 to 125
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xfc) << 15);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 126 to 140
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0xf8) << 14);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 141 to 155
	value = ((uint32_t)(input[17] & 0x07) << 29) |
			((uint32_t)input[18] << 21) |
			((uint32_t)(input[19] & 0xf0) << 13);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 156 to 170
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0xe0) << 12);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 171 to 185
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19) |
			((uint32_t)(input[23] & 0xc0) << 11);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 186 to 200
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)input[24] << 18) |
			((uint32_t)(input[25] & 0x80) << 10);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 201 to 215
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)input[26] << 17);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 216 to 230
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xfe) << 16);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 231 to 245
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xfc) << 15);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 246 to 260
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0xf8) << 14);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 261 to 275
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xf0) << 13);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 276 to 290
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0xe0) << 12);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 291 to 305
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19) |
			((uint32_t)(input[38] & 0xc0) << 11);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 306 to 320
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)input[39] << 18) |
			((uint32_t)(input[40] & 0x80) << 10);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 321 to 335
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)input[41] << 17);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 336 to 350
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xfe) << 16);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 351 to 365
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xfc) << 15);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 366 to 380
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xf8) << 14);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 381 to 395
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xf0) << 13);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 396 to 410
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0xe0) << 12);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 411 to 425
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)input[52] << 19) |
			((uint32_t)(input[53] & 0xc0) << 11);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 426 to 440
	value = ((uint32_t)(input[53] & 0x3f) << 26) |
			((uint32_t)input[54] << 18) |
			((uint32_t)(input[55] & 0x80) << 10);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 441 to 455
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)input[56] << 17);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 456 to 470
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xfe) << 16);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 471 to 485
	value = ((uint32_t)(input[58] & 0x01) << 31) |
			((uint32_t)input[59] << 23) |
			((uint32_t)(input[60] & 0xfc) << 15);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 486 to 500
	value = ((uint32_t)(input[60] & 0x03) << 30) |
			((uint32_t)input[61] << 22) |
			((uint32_t)(input[62] & 0xf8) << 14);
	output[32] = (double)(value >> 17);

	// read output[33]: from bit 501 to 515
	value = ((uint32_t)(input[62] & 0x07) << 29) |
			((uint32_t)input[63] << 21) |
			((uint32_t)(input[64] & 0xf0) << 13);
	output[33] = (double)(value >> 17);
}

static void unpack_34_15bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 15-bit signed
	// Skip 7 bits, read 510 bits of data.
	// Input must be at least 65 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 367 to 381
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xfc) << 15);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 382 to 396
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xf8) << 14);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 397 to 411
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xf0) << 13);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 412 to 426
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)input[52] << 20) |
			((uint32_t)(input[53] & 0xe0) << 12);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 427 to 441
	value = ((uint32_t)(input[53] & 0x1f) << 27) |
			((uint32_t)input[54] << 19) |
			((uint32_t)(input[55] & 0xc0) << 11);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 442 to 456
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)input[56] << 18) |
			((uint32_t)(input[57] & 0x80) << 10);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 457 to 471
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)input[58] << 17);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 472 to 486
	value = ((uint32_t)input[59] << 24) |
			((uint32_t)(input[60] & 0xfe) << 16);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 487 to 501
	value = ((uint32_t)(input[60] & 0x01) << 31) |
			((uint32_t)input[61] << 23) |
			((uint32_t)(input[62] & 0xfc) << 15);
	output[32] = (double)(value >> 17);

	// read output[33]: from bit 502 to 516
	value = ((uint32_t)(input[62] & 0x03) << 30) |
			((uint32_t)input[63] << 22) |
			((uint32_t)(input[64] & 0xf8) << 14);
	output[33] = (double)(value >> 17);
}

static void unpack_34_15bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 15-bit unsigned
	// Skip 7 bits, read 510 bits of data.
	// Input must be at least 65 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 21
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xfc) << 15);
	output[0] = (double)(value >> 17);

	// read output[1]: from bit 22 to 36
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0xf8) << 14);
	output[1] = (double)(value >> 17);

	// read output[2]: from bit 37 to 51
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21) |
			((uint32_t)(input[6] & 0xf0) << 13);
	output[2] = (double)(value >> 17);

	// read output[3]: from bit 52 to 66
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0xe0) << 12);
	output[3] = (double)(value >> 17);

	// read output[4]: from bit 67 to 81
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19) |
			((uint32_t)(input[10] & 0xc0) << 11);
	output[4] = (double)(value >> 17);

	// read output[5]: from bit 82 to 96
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)input[11] << 18) |
			((uint32_t)(input[12] & 0x80) << 10);
	output[5] = (double)(value >> 17);

	// read output[6]: from bit 97 to 111
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)input[13] << 17);
	output[6] = (double)(value >> 17);

	// read output[7]: from bit 112 to 126
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xfe) << 16);
	output[7] = (double)(value >> 17);

	// read output[8]: from bit 127 to 141
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xfc) << 15);
	output[8] = (double)(value >> 17);

	// read output[9]: from bit 142 to 156
	value = ((uint32_t)(input[17] & 0x03) << 30) |
			((uint32_t)input[18] << 22) |
			((uint32_t)(input[19] & 0xf8) << 14);
	output[9] = (double)(value >> 17);

	// read output[10]: from bit 157 to 171
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xf0) << 13);
	output[10] = (double)(value >> 17);

	// read output[11]: from bit 172 to 186
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0xe0) << 12);
	output[11] = (double)(value >> 17);

	// read output[12]: from bit 187 to 201
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19) |
			((uint32_t)(input[25] & 0xc0) << 11);
	output[12] = (double)(value >> 17);

	// read output[13]: from bit 202 to 216
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)input[26] << 18) |
			((uint32_t)(input[27] & 0x80) << 10);
	output[13] = (double)(value >> 17);

	// read output[14]: from bit 217 to 231
	value = ((uint32_t)(input[27] & 0x7f) << 25) |
			((uint32_t)input[28] << 17);
	output[14] = (double)(value >> 17);

	// read output[15]: from bit 232 to 246
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xfe) << 16);
	output[15] = (double)(value >> 17);

	// read output[16]: from bit 247 to 261
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xfc) << 15);
	output[16] = (double)(value >> 17);

	// read output[17]: from bit 262 to 276
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xf8) << 14);
	output[17] = (double)(value >> 17);

	// read output[18]: from bit 277 to 291
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xf0) << 13);
	output[18] = (double)(value >> 17);

	// read output[19]: from bit 292 to 306
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0xe0) << 12);
	output[19] = (double)(value >> 17);

	// read output[20]: from bit 307 to 321
	value = ((uint32_t)(input[38] & 0x1f) << 27) |
			((uint32_t)input[39] << 19) |
			((uint32_t)(input[40] & 0xc0) << 11);
	output[20] = (double)(value >> 17);

	// read output[21]: from bit 322 to 336
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)input[41] << 18) |
			((uint32_t)(input[42] & 0x80) << 10);
	output[21] = (double)(value >> 17);

	// read output[22]: from bit 337 to 351
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)input[43] << 17);
	output[22] = (double)(value >> 17);

	// read output[23]: from bit 352 to 366
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xfe) << 16);
	output[23] = (double)(value >> 17);

	// read output[24]: from bit 367 to 381
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xfc) << 15);
	output[24] = (double)(value >> 17);

	// read output[25]: from bit 382 to 396
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xf8) << 14);
	output[25] = (double)(value >> 17);

	// read output[26]: from bit 397 to 411
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xf0) << 13);
	output[26] = (double)(value >> 17);

	// read output[27]: from bit 412 to 426
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)input[52] << 20) |
			((uint32_t)(input[53] & 0xe0) << 12);
	output[27] = (double)(value >> 17);

	// read output[28]: from bit 427 to 441
	value = ((uint32_t)(input[53] & 0x1f) << 27) |
			((uint32_t)input[54] << 19) |
			((uint32_t)(input[55] & 0xc0) << 11);
	output[28] = (double)(value >> 17);

	// read output[29]: from bit 442 to 456
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)input[56] << 18) |
			((uint32_t)(input[57] & 0x80) << 10);
	output[29] = (double)(value >> 17);

	// read output[30]: from bit 457 to 471
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)input[58] << 17);
	output[30] = (double)(value >> 17);

	// read output[31]: from bit 472 to 486
	value = ((uint32_t)input[59] << 24) |
			((uint32_t)(input[60] & 0xfe) << 16);
	output[31] = (double)(value >> 17);

	// read output[32]: from bit 487 to 501
	value = ((uint32_t)(input[60] & 0x01) << 31) |
			((uint32_t)input[61] << 23) |
			((uint32_t)(input[62] & 0xfc) << 15);
	output[32] = (double)(value >> 17);

	// read output[33]: from bit 502 to 516
	value = ((uint32_t)(input[62] & 0x03) << 30) |
			((uint32_t)input[63] << 22) |
			((uint32_t)(input[64] & 0xf8) << 14);
	output[33] = (double)(value >> 17);
}


unpack_func_t unpack_15bit[UNPACK_15BIT_MAX_COUNT][8][2] = {
	{
		{
			unpack_1_15bit_unsigned_0off,
			unpack_1_15bit_signed_0off,
		},
		{
			unpack_1_15bit_unsigned_1off,
			unpack_1_15bit_signed_1off,
		},
		{
			unpack_1_15bit_unsigned_2off,
			unpack_1_15bit_signed_2off,
		},
		{
			unpack_1_15bit_unsigned_3off,
			unpack_1_15bit_signed_3off,
		},
		{
			unpack_1_15bit_unsigned_4off,
			unpack_1_15bit_signed_4off,
		},
		{
			unpack_1_15bit_unsigned_5off,
			unpack_1_15bit_signed_5off,
		},
		{
			unpack_1_15bit_unsigned_6off,
			unpack_1_15bit_signed_6off,
		},
		{
			unpack_1_15bit_unsigned_7off,
			unpack_1_15bit_signed_7off,
		},
	},
	{
		{
			unpack_2_15bit_unsigned_0off,
			unpack_2_15bit_signed_0off,
		},
		{
			unpack_2_15bit_unsigned_1off,
			unpack_2_15bit_signed_1off,
		},
		{
			unpack_2_15bit_unsigned_2off,
			unpack_2_15bit_signed_2off,
		},
		{
			unpack_2_15bit_unsigned_3off,
			unpack_2_15bit_signed_3off,
		},
		{
			unpack_2_15bit_unsigned_4off,
			unpack_2_15bit_signed_4off,
		},
		{
			unpack_2_15bit_unsigned_5off,
			unpack_2_15bit_signed_5off,
		},
		{
			unpack_2_15bit_unsigned_6off,
			unpack_2_15bit_signed_6off,
		},
		{
			unpack_2_15bit_unsigned_7off,
			unpack_2_15bit_signed_7off,
		},
	},
	{
		{
			unpack_3_15bit_unsigned_0off,
			unpack_3_15bit_signed_0off,
		},
		{
			unpack_3_15bit_unsigned_1off,
			unpack_3_15bit_signed_1off,
		},
		{
			unpack_3_15bit_unsigned_2off,
			unpack_3_15bit_signed_2off,
		},
		{
			unpack_3_15bit_unsigned_3off,
			unpack_3_15bit_signed_3off,
		},
		{
			unpack_3_15bit_unsigned_4off,
			unpack_3_15bit_signed_4off,
		},
		{
			unpack_3_15bit_unsigned_5off,
			unpack_3_15bit_signed_5off,
		},
		{
			unpack_3_15bit_unsigned_6off,
			unpack_3_15bit_signed_6off,
		},
		{
			unpack_3_15bit_unsigned_7off,
			unpack_3_15bit_signed_7off,
		},
	},
	{
		{
			unpack_4_15bit_unsigned_0off,
			unpack_4_15bit_signed_0off,
		},
		{
			unpack_4_15bit_unsigned_1off,
			unpack_4_15bit_signed_1off,
		},
		{
			unpack_4_15bit_unsigned_2off,
			unpack_4_15bit_signed_2off,
		},
		{
			unpack_4_15bit_unsigned_3off,
			unpack_4_15bit_signed_3off,
		},
		{
			unpack_4_15bit_unsigned_4off,
			unpack_4_15bit_signed_4off,
		},
		{
			unpack_4_15bit_unsigned_5off,
			unpack_4_15bit_signed_5off,
		},
		{
			unpack_4_15bit_unsigned_6off,
			unpack_4_15bit_signed_6off,
		},
		{
			unpack_4_15bit_unsigned_7off,
			unpack_4_15bit_signed_7off,
		},
	},
	{
		{
			unpack_5_15bit_unsigned_0off,
			unpack_5_15bit_signed_0off,
		},
		{
			unpack_5_15bit_unsigned_1off,
			unpack_5_15bit_signed_1off,
		},
		{
			unpack_5_15bit_unsigned_2off,
			unpack_5_15bit_signed_2off,
		},
		{
			unpack_5_15bit_unsigned_3off,
			unpack_5_15bit_signed_3off,
		},
		{
			unpack_5_15bit_unsigned_4off,
			unpack_5_15bit_signed_4off,
		},
		{
			unpack_5_15bit_unsigned_5off,
			unpack_5_15bit_signed_5off,
		},
		{
			unpack_5_15bit_unsigned_6off,
			unpack_5_15bit_signed_6off,
		},
		{
			unpack_5_15bit_unsigned_7off,
			unpack_5_15bit_signed_7off,
		},
	},
	{
		{
			unpack_6_15bit_unsigned_0off,
			unpack_6_15bit_signed_0off,
		},
		{
			unpack_6_15bit_unsigned_1off,
			unpack_6_15bit_signed_1off,
		},
		{
			unpack_6_15bit_unsigned_2off,
			unpack_6_15bit_signed_2off,
		},
		{
			unpack_6_15bit_unsigned_3off,
			unpack_6_15bit_signed_3off,
		},
		{
			unpack_6_15bit_unsigned_4off,
			unpack_6_15bit_signed_4off,
		},
		{
			unpack_6_15bit_unsigned_5off,
			unpack_6_15bit_signed_5off,
		},
		{
			unpack_6_15bit_unsigned_6off,
			unpack_6_15bit_signed_6off,
		},
		{
			unpack_6_15bit_unsigned_7off,
			unpack_6_15bit_signed_7off,
		},
	},
	{
		{
			unpack_7_15bit_unsigned_0off,
			unpack_7_15bit_signed_0off,
		},
		{
			unpack_7_15bit_unsigned_1off,
			unpack_7_15bit_signed_1off,
		},
		{
			unpack_7_15bit_unsigned_2off,
			unpack_7_15bit_signed_2off,
		},
		{
			unpack_7_15bit_unsigned_3off,
			unpack_7_15bit_signed_3off,
		},
		{
			unpack_7_15bit_unsigned_4off,
			unpack_7_15bit_signed_4off,
		},
		{
			unpack_7_15bit_unsigned_5off,
			unpack_7_15bit_signed_5off,
		},
		{
			unpack_7_15bit_unsigned_6off,
			unpack_7_15bit_signed_6off,
		},
		{
			unpack_7_15bit_unsigned_7off,
			unpack_7_15bit_signed_7off,
		},
	},
	{
		{
			unpack_8_15bit_unsigned_0off,
			unpack_8_15bit_signed_0off,
		},
		{
			unpack_8_15bit_unsigned_1off,
			unpack_8_15bit_signed_1off,
		},
		{
			unpack_8_15bit_unsigned_2off,
			unpack_8_15bit_signed_2off,
		},
		{
			unpack_8_15bit_unsigned_3off,
			unpack_8_15bit_signed_3off,
		},
		{
			unpack_8_15bit_unsigned_4off,
			unpack_8_15bit_signed_4off,
		},
		{
			unpack_8_15bit_unsigned_5off,
			unpack_8_15bit_signed_5off,
		},
		{
			unpack_8_15bit_unsigned_6off,
			unpack_8_15bit_signed_6off,
		},
		{
			unpack_8_15bit_unsigned_7off,
			unpack_8_15bit_signed_7off,
		},
	},
	{
		{
			unpack_9_15bit_unsigned_0off,
			unpack_9_15bit_signed_0off,
		},
		{
			unpack_9_15bit_unsigned_1off,
			unpack_9_15bit_signed_1off,
		},
		{
			unpack_9_15bit_unsigned_2off,
			unpack_9_15bit_signed_2off,
		},
		{
			unpack_9_15bit_unsigned_3off,
			unpack_9_15bit_signed_3off,
		},
		{
			unpack_9_15bit_unsigned_4off,
			unpack_9_15bit_signed_4off,
		},
		{
			unpack_9_15bit_unsigned_5off,
			unpack_9_15bit_signed_5off,
		},
		{
			unpack_9_15bit_unsigned_6off,
			unpack_9_15bit_signed_6off,
		},
		{
			unpack_9_15bit_unsigned_7off,
			unpack_9_15bit_signed_7off,
		},
	},
	{
		{
			unpack_10_15bit_unsigned_0off,
			unpack_10_15bit_signed_0off,
		},
		{
			unpack_10_15bit_unsigned_1off,
			unpack_10_15bit_signed_1off,
		},
		{
			unpack_10_15bit_unsigned_2off,
			unpack_10_15bit_signed_2off,
		},
		{
			unpack_10_15bit_unsigned_3off,
			unpack_10_15bit_signed_3off,
		},
		{
			unpack_10_15bit_unsigned_4off,
			unpack_10_15bit_signed_4off,
		},
		{
			unpack_10_15bit_unsigned_5off,
			unpack_10_15bit_signed_5off,
		},
		{
			unpack_10_15bit_unsigned_6off,
			unpack_10_15bit_signed_6off,
		},
		{
			unpack_10_15bit_unsigned_7off,
			unpack_10_15bit_signed_7off,
		},
	},
	{
		{
			unpack_11_15bit_unsigned_0off,
			unpack_11_15bit_signed_0off,
		},
		{
			unpack_11_15bit_unsigned_1off,
			unpack_11_15bit_signed_1off,
		},
		{
			unpack_11_15bit_unsigned_2off,
			unpack_11_15bit_signed_2off,
		},
		{
			unpack_11_15bit_unsigned_3off,
			unpack_11_15bit_signed_3off,
		},
		{
			unpack_11_15bit_unsigned_4off,
			unpack_11_15bit_signed_4off,
		},
		{
			unpack_11_15bit_unsigned_5off,
			unpack_11_15bit_signed_5off,
		},
		{
			unpack_11_15bit_unsigned_6off,
			unpack_11_15bit_signed_6off,
		},
		{
			unpack_11_15bit_unsigned_7off,
			unpack_11_15bit_signed_7off,
		},
	},
	{
		{
			unpack_12_15bit_unsigned_0off,
			unpack_12_15bit_signed_0off,
		},
		{
			unpack_12_15bit_unsigned_1off,
			unpack_12_15bit_signed_1off,
		},
		{
			unpack_12_15bit_unsigned_2off,
			unpack_12_15bit_signed_2off,
		},
		{
			unpack_12_15bit_unsigned_3off,
			unpack_12_15bit_signed_3off,
		},
		{
			unpack_12_15bit_unsigned_4off,
			unpack_12_15bit_signed_4off,
		},
		{
			unpack_12_15bit_unsigned_5off,
			unpack_12_15bit_signed_5off,
		},
		{
			unpack_12_15bit_unsigned_6off,
			unpack_12_15bit_signed_6off,
		},
		{
			unpack_12_15bit_unsigned_7off,
			unpack_12_15bit_signed_7off,
		},
	},
	{
		{
			unpack_13_15bit_unsigned_0off,
			unpack_13_15bit_signed_0off,
		},
		{
			unpack_13_15bit_unsigned_1off,
			unpack_13_15bit_signed_1off,
		},
		{
			unpack_13_15bit_unsigned_2off,
			unpack_13_15bit_signed_2off,
		},
		{
			unpack_13_15bit_unsigned_3off,
			unpack_13_15bit_signed_3off,
		},
		{
			unpack_13_15bit_unsigned_4off,
			unpack_13_15bit_signed_4off,
		},
		{
			unpack_13_15bit_unsigned_5off,
			unpack_13_15bit_signed_5off,
		},
		{
			unpack_13_15bit_unsigned_6off,
			unpack_13_15bit_signed_6off,
		},
		{
			unpack_13_15bit_unsigned_7off,
			unpack_13_15bit_signed_7off,
		},
	},
	{
		{
			unpack_14_15bit_unsigned_0off,
			unpack_14_15bit_signed_0off,
		},
		{
			unpack_14_15bit_unsigned_1off,
			unpack_14_15bit_signed_1off,
		},
		{
			unpack_14_15bit_unsigned_2off,
			unpack_14_15bit_signed_2off,
		},
		{
			unpack_14_15bit_unsigned_3off,
			unpack_14_15bit_signed_3off,
		},
		{
			unpack_14_15bit_unsigned_4off,
			unpack_14_15bit_signed_4off,
		},
		{
			unpack_14_15bit_unsigned_5off,
			unpack_14_15bit_signed_5off,
		},
		{
			unpack_14_15bit_unsigned_6off,
			unpack_14_15bit_signed_6off,
		},
		{
			unpack_14_15bit_unsigned_7off,
			unpack_14_15bit_signed_7off,
		},
	},
	{
		{
			unpack_15_15bit_unsigned_0off,
			unpack_15_15bit_signed_0off,
		},
		{
			unpack_15_15bit_unsigned_1off,
			unpack_15_15bit_signed_1off,
		},
		{
			unpack_15_15bit_unsigned_2off,
			unpack_15_15bit_signed_2off,
		},
		{
			unpack_15_15bit_unsigned_3off,
			unpack_15_15bit_signed_3off,
		},
		{
			unpack_15_15bit_unsigned_4off,
			unpack_15_15bit_signed_4off,
		},
		{
			unpack_15_15bit_unsigned_5off,
			unpack_15_15bit_signed_5off,
		},
		{
			unpack_15_15bit_unsigned_6off,
			unpack_15_15bit_signed_6off,
		},
		{
			unpack_15_15bit_unsigned_7off,
			unpack_15_15bit_signed_7off,
		},
	},
	{
		{
			unpack_16_15bit_unsigned_0off,
			unpack_16_15bit_signed_0off,
		},
		{
			unpack_16_15bit_unsigned_1off,
			unpack_16_15bit_signed_1off,
		},
		{
			unpack_16_15bit_unsigned_2off,
			unpack_16_15bit_signed_2off,
		},
		{
			unpack_16_15bit_unsigned_3off,
			unpack_16_15bit_signed_3off,
		},
		{
			unpack_16_15bit_unsigned_4off,
			unpack_16_15bit_signed_4off,
		},
		{
			unpack_16_15bit_unsigned_5off,
			unpack_16_15bit_signed_5off,
		},
		{
			unpack_16_15bit_unsigned_6off,
			unpack_16_15bit_signed_6off,
		},
		{
			unpack_16_15bit_unsigned_7off,
			unpack_16_15bit_signed_7off,
		},
	},
	{
		{
			unpack_17_15bit_unsigned_0off,
			unpack_17_15bit_signed_0off,
		},
		{
			unpack_17_15bit_unsigned_1off,
			unpack_17_15bit_signed_1off,
		},
		{
			unpack_17_15bit_unsigned_2off,
			unpack_17_15bit_signed_2off,
		},
		{
			unpack_17_15bit_unsigned_3off,
			unpack_17_15bit_signed_3off,
		},
		{
			unpack_17_15bit_unsigned_4off,
			unpack_17_15bit_signed_4off,
		},
		{
			unpack_17_15bit_unsigned_5off,
			unpack_17_15bit_signed_5off,
		},
		{
			unpack_17_15bit_unsigned_6off,
			unpack_17_15bit_signed_6off,
		},
		{
			unpack_17_15bit_unsigned_7off,
			unpack_17_15bit_signed_7off,
		},
	},
	{
		{
			unpack_18_15bit_unsigned_0off,
			unpack_18_15bit_signed_0off,
		},
		{
			unpack_18_15bit_unsigned_1off,
			unpack_18_15bit_signed_1off,
		},
		{
			unpack_18_15bit_unsigned_2off,
			unpack_18_15bit_signed_2off,
		},
		{
			unpack_18_15bit_unsigned_3off,
			unpack_18_15bit_signed_3off,
		},
		{
			unpack_18_15bit_unsigned_4off,
			unpack_18_15bit_signed_4off,
		},
		{
			unpack_18_15bit_unsigned_5off,
			unpack_18_15bit_signed_5off,
		},
		{
			unpack_18_15bit_unsigned_6off,
			unpack_18_15bit_signed_6off,
		},
		{
			unpack_18_15bit_unsigned_7off,
			unpack_18_15bit_signed_7off,
		},
	},
	{
		{
			unpack_19_15bit_unsigned_0off,
			unpack_19_15bit_signed_0off,
		},
		{
			unpack_19_15bit_unsigned_1off,
			unpack_19_15bit_signed_1off,
		},
		{
			unpack_19_15bit_unsigned_2off,
			unpack_19_15bit_signed_2off,
		},
		{
			unpack_19_15bit_unsigned_3off,
			unpack_19_15bit_signed_3off,
		},
		{
			unpack_19_15bit_unsigned_4off,
			unpack_19_15bit_signed_4off,
		},
		{
			unpack_19_15bit_unsigned_5off,
			unpack_19_15bit_signed_5off,
		},
		{
			unpack_19_15bit_unsigned_6off,
			unpack_19_15bit_signed_6off,
		},
		{
			unpack_19_15bit_unsigned_7off,
			unpack_19_15bit_signed_7off,
		},
	},
	{
		{
			unpack_20_15bit_unsigned_0off,
			unpack_20_15bit_signed_0off,
		},
		{
			unpack_20_15bit_unsigned_1off,
			unpack_20_15bit_signed_1off,
		},
		{
			unpack_20_15bit_unsigned_2off,
			unpack_20_15bit_signed_2off,
		},
		{
			unpack_20_15bit_unsigned_3off,
			unpack_20_15bit_signed_3off,
		},
		{
			unpack_20_15bit_unsigned_4off,
			unpack_20_15bit_signed_4off,
		},
		{
			unpack_20_15bit_unsigned_5off,
			unpack_20_15bit_signed_5off,
		},
		{
			unpack_20_15bit_unsigned_6off,
			unpack_20_15bit_signed_6off,
		},
		{
			unpack_20_15bit_unsigned_7off,
			unpack_20_15bit_signed_7off,
		},
	},
	{
		{
			unpack_21_15bit_unsigned_0off,
			unpack_21_15bit_signed_0off,
		},
		{
			unpack_21_15bit_unsigned_1off,
			unpack_21_15bit_signed_1off,
		},
		{
			unpack_21_15bit_unsigned_2off,
			unpack_21_15bit_signed_2off,
		},
		{
			unpack_21_15bit_unsigned_3off,
			unpack_21_15bit_signed_3off,
		},
		{
			unpack_21_15bit_unsigned_4off,
			unpack_21_15bit_signed_4off,
		},
		{
			unpack_21_15bit_unsigned_5off,
			unpack_21_15bit_signed_5off,
		},
		{
			unpack_21_15bit_unsigned_6off,
			unpack_21_15bit_signed_6off,
		},
		{
			unpack_21_15bit_unsigned_7off,
			unpack_21_15bit_signed_7off,
		},
	},
	{
		{
			unpack_22_15bit_unsigned_0off,
			unpack_22_15bit_signed_0off,
		},
		{
			unpack_22_15bit_unsigned_1off,
			unpack_22_15bit_signed_1off,
		},
		{
			unpack_22_15bit_unsigned_2off,
			unpack_22_15bit_signed_2off,
		},
		{
			unpack_22_15bit_unsigned_3off,
			unpack_22_15bit_signed_3off,
		},
		{
			unpack_22_15bit_unsigned_4off,
			unpack_22_15bit_signed_4off,
		},
		{
			unpack_22_15bit_unsigned_5off,
			unpack_22_15bit_signed_5off,
		},
		{
			unpack_22_15bit_unsigned_6off,
			unpack_22_15bit_signed_6off,
		},
		{
			unpack_22_15bit_unsigned_7off,
			unpack_22_15bit_signed_7off,
		},
	},
	{
		{
			unpack_23_15bit_unsigned_0off,
			unpack_23_15bit_signed_0off,
		},
		{
			unpack_23_15bit_unsigned_1off,
			unpack_23_15bit_signed_1off,
		},
		{
			unpack_23_15bit_unsigned_2off,
			unpack_23_15bit_signed_2off,
		},
		{
			unpack_23_15bit_unsigned_3off,
			unpack_23_15bit_signed_3off,
		},
		{
			unpack_23_15bit_unsigned_4off,
			unpack_23_15bit_signed_4off,
		},
		{
			unpack_23_15bit_unsigned_5off,
			unpack_23_15bit_signed_5off,
		},
		{
			unpack_23_15bit_unsigned_6off,
			unpack_23_15bit_signed_6off,
		},
		{
			unpack_23_15bit_unsigned_7off,
			unpack_23_15bit_signed_7off,
		},
	},
	{
		{
			unpack_24_15bit_unsigned_0off,
			unpack_24_15bit_signed_0off,
		},
		{
			unpack_24_15bit_unsigned_1off,
			unpack_24_15bit_signed_1off,
		},
		{
			unpack_24_15bit_unsigned_2off,
			unpack_24_15bit_signed_2off,
		},
		{
			unpack_24_15bit_unsigned_3off,
			unpack_24_15bit_signed_3off,
		},
		{
			unpack_24_15bit_unsigned_4off,
			unpack_24_15bit_signed_4off,
		},
		{
			unpack_24_15bit_unsigned_5off,
			unpack_24_15bit_signed_5off,
		},
		{
			unpack_24_15bit_unsigned_6off,
			unpack_24_15bit_signed_6off,
		},
		{
			unpack_24_15bit_unsigned_7off,
			unpack_24_15bit_signed_7off,
		},
	},
	{
		{
			unpack_25_15bit_unsigned_0off,
			unpack_25_15bit_signed_0off,
		},
		{
			unpack_25_15bit_unsigned_1off,
			unpack_25_15bit_signed_1off,
		},
		{
			unpack_25_15bit_unsigned_2off,
			unpack_25_15bit_signed_2off,
		},
		{
			unpack_25_15bit_unsigned_3off,
			unpack_25_15bit_signed_3off,
		},
		{
			unpack_25_15bit_unsigned_4off,
			unpack_25_15bit_signed_4off,
		},
		{
			unpack_25_15bit_unsigned_5off,
			unpack_25_15bit_signed_5off,
		},
		{
			unpack_25_15bit_unsigned_6off,
			unpack_25_15bit_signed_6off,
		},
		{
			unpack_25_15bit_unsigned_7off,
			unpack_25_15bit_signed_7off,
		},
	},
	{
		{
			unpack_26_15bit_unsigned_0off,
			unpack_26_15bit_signed_0off,
		},
		{
			unpack_26_15bit_unsigned_1off,
			unpack_26_15bit_signed_1off,
		},
		{
			unpack_26_15bit_unsigned_2off,
			unpack_26_15bit_signed_2off,
		},
		{
			unpack_26_15bit_unsigned_3off,
			unpack_26_15bit_signed_3off,
		},
		{
			unpack_26_15bit_unsigned_4off,
			unpack_26_15bit_signed_4off,
		},
		{
			unpack_26_15bit_unsigned_5off,
			unpack_26_15bit_signed_5off,
		},
		{
			unpack_26_15bit_unsigned_6off,
			unpack_26_15bit_signed_6off,
		},
		{
			unpack_26_15bit_unsigned_7off,
			unpack_26_15bit_signed_7off,
		},
	},
	{
		{
			unpack_27_15bit_unsigned_0off,
			unpack_27_15bit_signed_0off,
		},
		{
			unpack_27_15bit_unsigned_1off,
			unpack_27_15bit_signed_1off,
		},
		{
			unpack_27_15bit_unsigned_2off,
			unpack_27_15bit_signed_2off,
		},
		{
			unpack_27_15bit_unsigned_3off,
			unpack_27_15bit_signed_3off,
		},
		{
			unpack_27_15bit_unsigned_4off,
			unpack_27_15bit_signed_4off,
		},
		{
			unpack_27_15bit_unsigned_5off,
			unpack_27_15bit_signed_5off,
		},
		{
			unpack_27_15bit_unsigned_6off,
			unpack_27_15bit_signed_6off,
		},
		{
			unpack_27_15bit_unsigned_7off,
			unpack_27_15bit_signed_7off,
		},
	},
	{
		{
			unpack_28_15bit_unsigned_0off,
			unpack_28_15bit_signed_0off,
		},
		{
			unpack_28_15bit_unsigned_1off,
			unpack_28_15bit_signed_1off,
		},
		{
			unpack_28_15bit_unsigned_2off,
			unpack_28_15bit_signed_2off,
		},
		{
			unpack_28_15bit_unsigned_3off,
			unpack_28_15bit_signed_3off,
		},
		{
			unpack_28_15bit_unsigned_4off,
			unpack_28_15bit_signed_4off,
		},
		{
			unpack_28_15bit_unsigned_5off,
			unpack_28_15bit_signed_5off,
		},
		{
			unpack_28_15bit_unsigned_6off,
			unpack_28_15bit_signed_6off,
		},
		{
			unpack_28_15bit_unsigned_7off,
			unpack_28_15bit_signed_7off,
		},
	},
	{
		{
			unpack_29_15bit_unsigned_0off,
			unpack_29_15bit_signed_0off,
		},
		{
			unpack_29_15bit_unsigned_1off,
			unpack_29_15bit_signed_1off,
		},
		{
			unpack_29_15bit_unsigned_2off,
			unpack_29_15bit_signed_2off,
		},
		{
			unpack_29_15bit_unsigned_3off,
			unpack_29_15bit_signed_3off,
		},
		{
			unpack_29_15bit_unsigned_4off,
			unpack_29_15bit_signed_4off,
		},
		{
			unpack_29_15bit_unsigned_5off,
			unpack_29_15bit_signed_5off,
		},
		{
			unpack_29_15bit_unsigned_6off,
			unpack_29_15bit_signed_6off,
		},
		{
			unpack_29_15bit_unsigned_7off,
			unpack_29_15bit_signed_7off,
		},
	},
	{
		{
			unpack_30_15bit_unsigned_0off,
			unpack_30_15bit_signed_0off,
		},
		{
			unpack_30_15bit_unsigned_1off,
			unpack_30_15bit_signed_1off,
		},
		{
			unpack_30_15bit_unsigned_2off,
			unpack_30_15bit_signed_2off,
		},
		{
			unpack_30_15bit_unsigned_3off,
			unpack_30_15bit_signed_3off,
		},
		{
			unpack_30_15bit_unsigned_4off,
			unpack_30_15bit_signed_4off,
		},
		{
			unpack_30_15bit_unsigned_5off,
			unpack_30_15bit_signed_5off,
		},
		{
			unpack_30_15bit_unsigned_6off,
			unpack_30_15bit_signed_6off,
		},
		{
			unpack_30_15bit_unsigned_7off,
			unpack_30_15bit_signed_7off,
		},
	},
	{
		{
			unpack_31_15bit_unsigned_0off,
			unpack_31_15bit_signed_0off,
		},
		{
			unpack_31_15bit_unsigned_1off,
			unpack_31_15bit_signed_1off,
		},
		{
			unpack_31_15bit_unsigned_2off,
			unpack_31_15bit_signed_2off,
		},
		{
			unpack_31_15bit_unsigned_3off,
			unpack_31_15bit_signed_3off,
		},
		{
			unpack_31_15bit_unsigned_4off,
			unpack_31_15bit_signed_4off,
		},
		{
			unpack_31_15bit_unsigned_5off,
			unpack_31_15bit_signed_5off,
		},
		{
			unpack_31_15bit_unsigned_6off,
			unpack_31_15bit_signed_6off,
		},
		{
			unpack_31_15bit_unsigned_7off,
			unpack_31_15bit_signed_7off,
		},
	},
	{
		{
			unpack_32_15bit_unsigned_0off,
			unpack_32_15bit_signed_0off,
		},
		{
			unpack_32_15bit_unsigned_1off,
			unpack_32_15bit_signed_1off,
		},
		{
			unpack_32_15bit_unsigned_2off,
			unpack_32_15bit_signed_2off,
		},
		{
			unpack_32_15bit_unsigned_3off,
			unpack_32_15bit_signed_3off,
		},
		{
			unpack_32_15bit_unsigned_4off,
			unpack_32_15bit_signed_4off,
		},
		{
			unpack_32_15bit_unsigned_5off,
			unpack_32_15bit_signed_5off,
		},
		{
			unpack_32_15bit_unsigned_6off,
			unpack_32_15bit_signed_6off,
		},
		{
			unpack_32_15bit_unsigned_7off,
			unpack_32_15bit_signed_7off,
		},
	},
	{
		{
			unpack_33_15bit_unsigned_0off,
			unpack_33_15bit_signed_0off,
		},
		{
			unpack_33_15bit_unsigned_1off,
			unpack_33_15bit_signed_1off,
		},
		{
			unpack_33_15bit_unsigned_2off,
			unpack_33_15bit_signed_2off,
		},
		{
			unpack_33_15bit_unsigned_3off,
			unpack_33_15bit_signed_3off,
		},
		{
			unpack_33_15bit_unsigned_4off,
			unpack_33_15bit_signed_4off,
		},
		{
			unpack_33_15bit_unsigned_5off,
			unpack_33_15bit_signed_5off,
		},
		{
			unpack_33_15bit_unsigned_6off,
			unpack_33_15bit_signed_6off,
		},
		{
			unpack_33_15bit_unsigned_7off,
			unpack_33_15bit_signed_7off,
		},
	},
	{
		{
			unpack_34_15bit_unsigned_0off,
			unpack_34_15bit_signed_0off,
		},
		{
			unpack_34_15bit_unsigned_1off,
			unpack_34_15bit_signed_1off,
		},
		{
			unpack_34_15bit_unsigned_2off,
			unpack_34_15bit_signed_2off,
		},
		{
			unpack_34_15bit_unsigned_3off,
			unpack_34_15bit_signed_3off,
		},
		{
			unpack_34_15bit_unsigned_4off,
			unpack_34_15bit_signed_4off,
		},
		{
			unpack_34_15bit_unsigned_5off,
			unpack_34_15bit_signed_5off,
		},
		{
			unpack_34_15bit_unsigned_6off,
			unpack_34_15bit_signed_6off,
		},
		{
			unpack_34_15bit_unsigned_7off,
			unpack_34_15bit_signed_7off,
		},
	},
};
