// AUTOGENERATED FILE. DO NOT MODIFY.

#include <stdlib.h>
#include <stdint.h>

#include "unpack11.h"



static void unpack_1_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 11 bits of data.
	// Input must be at least 2 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);
}

static void unpack_1_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 11 bits of data.
	// Input must be at least 2 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);
}

static void unpack_1_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 11 bits of data.
	// Input must be at least 2 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);
}

static void unpack_1_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 11 bits of data.
	// Input must be at least 2 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);
}

static void unpack_1_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 11 bits of data.
	// Input must be at least 2 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);
}

static void unpack_1_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 11 bits of data.
	// Input must be at least 2 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);
}

static void unpack_1_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 11 bits of data.
	// Input must be at least 2 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);
}

static void unpack_1_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 11 bits of data.
	// Input must be at least 2 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);
}

static void unpack_1_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 11 bits of data.
	// Input must be at least 2 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);
}

static void unpack_1_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 11 bits of data.
	// Input must be at least 2 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);
}

static void unpack_1_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 11 bits of data.
	// Input must be at least 2 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);
}

static void unpack_1_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 11 bits of data.
	// Input must be at least 2 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);
}

static void unpack_1_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 11 bits of data.
	// Input must be at least 3 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);
}

static void unpack_1_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 11 bits of data.
	// Input must be at least 3 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);
}

static void unpack_1_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 11 bits of data.
	// Input must be at least 3 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);
}

static void unpack_1_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 11 bits of data.
	// Input must be at least 3 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);
}

static void unpack_2_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 22 bits of data.
	// Input must be at least 3 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);
}

static void unpack_2_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 22 bits of data.
	// Input must be at least 3 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);
}

static void unpack_2_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 22 bits of data.
	// Input must be at least 3 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);
}

static void unpack_2_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 22 bits of data.
	// Input must be at least 3 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);
}

static void unpack_2_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 22 bits of data.
	// Input must be at least 3 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);
}

static void unpack_2_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 22 bits of data.
	// Input must be at least 3 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);
}

static void unpack_2_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 22 bits of data.
	// Input must be at least 4 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);
}

static void unpack_2_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 22 bits of data.
	// Input must be at least 4 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);
}

static void unpack_2_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 22 bits of data.
	// Input must be at least 4 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);
}

static void unpack_2_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 22 bits of data.
	// Input must be at least 4 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);
}

static void unpack_2_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 22 bits of data.
	// Input must be at least 4 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);
}

static void unpack_2_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 22 bits of data.
	// Input must be at least 4 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);
}

static void unpack_2_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 22 bits of data.
	// Input must be at least 4 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);
}

static void unpack_2_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 22 bits of data.
	// Input must be at least 4 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);
}

static void unpack_2_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 22 bits of data.
	// Input must be at least 4 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);
}

static void unpack_2_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 22 bits of data.
	// Input must be at least 4 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);
}

static void unpack_3_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 33 bits of data.
	// Input must be at least 5 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);
}

static void unpack_3_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 33 bits of data.
	// Input must be at least 5 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);
}

static void unpack_3_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 33 bits of data.
	// Input must be at least 5 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);
}

static void unpack_3_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 33 bits of data.
	// Input must be at least 5 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);
}

static void unpack_3_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 33 bits of data.
	// Input must be at least 5 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);
}

static void unpack_3_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 33 bits of data.
	// Input must be at least 5 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);
}

static void unpack_3_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 33 bits of data.
	// Input must be at least 5 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);
}

static void unpack_3_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 33 bits of data.
	// Input must be at least 5 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);
}

static void unpack_3_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 33 bits of data.
	// Input must be at least 5 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);
}

static void unpack_3_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 33 bits of data.
	// Input must be at least 5 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);
}

static void unpack_3_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 33 bits of data.
	// Input must be at least 5 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);
}

static void unpack_3_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 33 bits of data.
	// Input must be at least 5 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);
}

static void unpack_3_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 33 bits of data.
	// Input must be at least 5 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);
}

static void unpack_3_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 33 bits of data.
	// Input must be at least 5 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);
}

static void unpack_3_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 33 bits of data.
	// Input must be at least 5 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);
}

static void unpack_3_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 33 bits of data.
	// Input must be at least 5 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);
}

static void unpack_4_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 44 bits of data.
	// Input must be at least 6 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);
}

static void unpack_4_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 44 bits of data.
	// Input must be at least 6 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);
}

static void unpack_4_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 44 bits of data.
	// Input must be at least 6 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);
}

static void unpack_4_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 44 bits of data.
	// Input must be at least 6 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);
}

static void unpack_4_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 44 bits of data.
	// Input must be at least 6 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);
}

static void unpack_4_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 44 bits of data.
	// Input must be at least 6 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);
}

static void unpack_4_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 44 bits of data.
	// Input must be at least 6 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);
}

static void unpack_4_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 44 bits of data.
	// Input must be at least 6 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);
}

static void unpack_4_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 44 bits of data.
	// Input must be at least 6 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);
}

static void unpack_4_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 44 bits of data.
	// Input must be at least 6 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);
}

static void unpack_4_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 44 bits of data.
	// Input must be at least 7 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);
}

static void unpack_4_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 44 bits of data.
	// Input must be at least 7 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);
}

static void unpack_4_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 44 bits of data.
	// Input must be at least 7 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);
}

static void unpack_4_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 44 bits of data.
	// Input must be at least 7 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);
}

static void unpack_4_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 44 bits of data.
	// Input must be at least 7 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);
}

static void unpack_4_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 44 bits of data.
	// Input must be at least 7 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);
}

static void unpack_5_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 55 bits of data.
	// Input must be at least 7 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);
}

static void unpack_5_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 55 bits of data.
	// Input must be at least 7 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);
}

static void unpack_5_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 55 bits of data.
	// Input must be at least 7 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);
}

static void unpack_5_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 55 bits of data.
	// Input must be at least 7 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);
}

static void unpack_5_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 55 bits of data.
	// Input must be at least 8 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);
}

static void unpack_5_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 55 bits of data.
	// Input must be at least 8 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);
}

static void unpack_5_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 55 bits of data.
	// Input must be at least 8 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);
}

static void unpack_5_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 55 bits of data.
	// Input must be at least 8 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);
}

static void unpack_5_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 55 bits of data.
	// Input must be at least 8 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);
}

static void unpack_5_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 55 bits of data.
	// Input must be at least 8 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);
}

static void unpack_5_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 55 bits of data.
	// Input must be at least 8 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);
}

static void unpack_5_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 55 bits of data.
	// Input must be at least 8 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);
}

static void unpack_5_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 55 bits of data.
	// Input must be at least 8 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);
}

static void unpack_5_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 55 bits of data.
	// Input must be at least 8 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);
}

static void unpack_5_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 55 bits of data.
	// Input must be at least 8 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);
}

static void unpack_5_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 55 bits of data.
	// Input must be at least 8 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);
}

static void unpack_6_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 66 bits of data.
	// Input must be at least 9 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);
}

static void unpack_6_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 66 bits of data.
	// Input must be at least 9 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);
}

static void unpack_6_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 66 bits of data.
	// Input must be at least 9 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);
}

static void unpack_6_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 66 bits of data.
	// Input must be at least 9 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);
}

static void unpack_6_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 66 bits of data.
	// Input must be at least 9 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);
}

static void unpack_6_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 66 bits of data.
	// Input must be at least 9 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);
}

static void unpack_6_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 66 bits of data.
	// Input must be at least 9 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);
}

static void unpack_6_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 66 bits of data.
	// Input must be at least 9 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);
}

static void unpack_6_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 66 bits of data.
	// Input must be at least 9 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);
}

static void unpack_6_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 66 bits of data.
	// Input must be at least 9 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);
}

static void unpack_6_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 66 bits of data.
	// Input must be at least 9 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);
}

static void unpack_6_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 66 bits of data.
	// Input must be at least 9 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);
}

static void unpack_6_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 66 bits of data.
	// Input must be at least 9 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);
}

static void unpack_6_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 66 bits of data.
	// Input must be at least 9 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);
}

static void unpack_6_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 66 bits of data.
	// Input must be at least 10 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);
}

static void unpack_6_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 66 bits of data.
	// Input must be at least 10 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);
}

static void unpack_7_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 77 bits of data.
	// Input must be at least 10 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);
}

static void unpack_7_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 77 bits of data.
	// Input must be at least 10 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);
}

static void unpack_7_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 77 bits of data.
	// Input must be at least 10 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);
}

static void unpack_7_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 77 bits of data.
	// Input must be at least 10 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);
}

static void unpack_7_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 77 bits of data.
	// Input must be at least 10 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);
}

static void unpack_7_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 77 bits of data.
	// Input must be at least 10 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);
}

static void unpack_7_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 77 bits of data.
	// Input must be at least 10 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);
}

static void unpack_7_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 77 bits of data.
	// Input must be at least 10 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);
}

static void unpack_7_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 77 bits of data.
	// Input must be at least 11 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);
}

static void unpack_7_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 77 bits of data.
	// Input must be at least 11 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);
}

static void unpack_7_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 77 bits of data.
	// Input must be at least 11 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);
}

static void unpack_7_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 77 bits of data.
	// Input must be at least 11 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);
}

static void unpack_7_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 77 bits of data.
	// Input must be at least 11 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);
}

static void unpack_7_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 77 bits of data.
	// Input must be at least 11 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);
}

static void unpack_7_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 77 bits of data.
	// Input must be at least 11 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);
}

static void unpack_7_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 77 bits of data.
	// Input must be at least 11 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);
}

static void unpack_8_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 88 bits of data.
	// Input must be at least 11 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);
}

static void unpack_8_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 88 bits of data.
	// Input must be at least 11 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);
}

static void unpack_8_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 88 bits of data.
	// Input must be at least 12 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);
}

static void unpack_8_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 88 bits of data.
	// Input must be at least 12 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);
}

static void unpack_8_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 88 bits of data.
	// Input must be at least 12 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);
}

static void unpack_8_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 88 bits of data.
	// Input must be at least 12 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);
}

static void unpack_8_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 88 bits of data.
	// Input must be at least 12 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);
}

static void unpack_8_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 88 bits of data.
	// Input must be at least 12 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);
}

static void unpack_8_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 88 bits of data.
	// Input must be at least 12 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);
}

static void unpack_8_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 88 bits of data.
	// Input must be at least 12 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);
}

static void unpack_8_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 88 bits of data.
	// Input must be at least 12 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);
}

static void unpack_8_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 88 bits of data.
	// Input must be at least 12 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);
}

static void unpack_8_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 88 bits of data.
	// Input must be at least 12 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);
}

static void unpack_8_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 88 bits of data.
	// Input must be at least 12 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);
}

static void unpack_8_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 88 bits of data.
	// Input must be at least 12 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);
}

static void unpack_8_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 88 bits of data.
	// Input must be at least 12 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);
}

static void unpack_9_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 99 bits of data.
	// Input must be at least 13 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);
}

static void unpack_9_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 99 bits of data.
	// Input must be at least 13 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);
}

static void unpack_9_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 99 bits of data.
	// Input must be at least 13 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);
}

static void unpack_9_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 99 bits of data.
	// Input must be at least 13 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);
}

static void unpack_9_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 99 bits of data.
	// Input must be at least 13 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);
}

static void unpack_9_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 99 bits of data.
	// Input must be at least 13 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);
}

static void unpack_9_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 99 bits of data.
	// Input must be at least 13 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);
}

static void unpack_9_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 99 bits of data.
	// Input must be at least 13 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);
}

static void unpack_9_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 99 bits of data.
	// Input must be at least 13 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);
}

static void unpack_9_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 99 bits of data.
	// Input must be at least 13 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);
}

static void unpack_9_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 99 bits of data.
	// Input must be at least 13 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);
}

static void unpack_9_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 99 bits of data.
	// Input must be at least 13 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);
}

static void unpack_9_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 99 bits of data.
	// Input must be at least 14 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);
}

static void unpack_9_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 99 bits of data.
	// Input must be at least 14 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);
}

static void unpack_9_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 99 bits of data.
	// Input must be at least 14 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);
}

static void unpack_9_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 99 bits of data.
	// Input must be at least 14 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);
}

static void unpack_10_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 110 bits of data.
	// Input must be at least 14 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);
}

static void unpack_10_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 110 bits of data.
	// Input must be at least 14 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);
}

static void unpack_10_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 110 bits of data.
	// Input must be at least 14 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);
}

static void unpack_10_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 110 bits of data.
	// Input must be at least 14 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);
}

static void unpack_10_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 110 bits of data.
	// Input must be at least 14 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);
}

static void unpack_10_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 110 bits of data.
	// Input must be at least 14 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);
}

static void unpack_10_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 110 bits of data.
	// Input must be at least 15 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);
}

static void unpack_10_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 110 bits of data.
	// Input must be at least 15 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);
}

static void unpack_10_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 110 bits of data.
	// Input must be at least 15 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);
}

static void unpack_10_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 110 bits of data.
	// Input must be at least 15 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);
}

static void unpack_10_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 110 bits of data.
	// Input must be at least 15 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);
}

static void unpack_10_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 110 bits of data.
	// Input must be at least 15 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);
}

static void unpack_10_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 110 bits of data.
	// Input must be at least 15 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);
}

static void unpack_10_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 110 bits of data.
	// Input must be at least 15 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);
}

static void unpack_10_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 110 bits of data.
	// Input must be at least 15 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);
}

static void unpack_10_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 110 bits of data.
	// Input must be at least 15 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);
}

static void unpack_11_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 121 bits of data.
	// Input must be at least 16 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);
}

static void unpack_11_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 121 bits of data.
	// Input must be at least 16 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);
}

static void unpack_11_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 121 bits of data.
	// Input must be at least 16 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);
}

static void unpack_11_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 121 bits of data.
	// Input must be at least 16 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);
}

static void unpack_11_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 121 bits of data.
	// Input must be at least 16 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);
}

static void unpack_11_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 121 bits of data.
	// Input must be at least 16 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);
}

static void unpack_11_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 121 bits of data.
	// Input must be at least 16 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);
}

static void unpack_11_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 121 bits of data.
	// Input must be at least 16 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);
}

static void unpack_11_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 121 bits of data.
	// Input must be at least 16 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);
}

static void unpack_11_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 121 bits of data.
	// Input must be at least 16 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);
}

static void unpack_11_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 121 bits of data.
	// Input must be at least 16 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);
}

static void unpack_11_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 121 bits of data.
	// Input must be at least 16 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);
}

static void unpack_11_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 121 bits of data.
	// Input must be at least 16 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);
}

static void unpack_11_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 121 bits of data.
	// Input must be at least 16 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);
}

static void unpack_11_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 121 bits of data.
	// Input must be at least 16 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);
}

static void unpack_11_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 121 bits of data.
	// Input must be at least 16 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);
}

static void unpack_12_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 132 bits of data.
	// Input must be at least 17 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);
}

static void unpack_12_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 132 bits of data.
	// Input must be at least 17 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);
}

static void unpack_12_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 132 bits of data.
	// Input must be at least 17 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);
}

static void unpack_12_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 132 bits of data.
	// Input must be at least 17 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);
}

static void unpack_12_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 132 bits of data.
	// Input must be at least 17 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);
}

static void unpack_12_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 132 bits of data.
	// Input must be at least 17 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);
}

static void unpack_12_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 132 bits of data.
	// Input must be at least 17 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);
}

static void unpack_12_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 132 bits of data.
	// Input must be at least 17 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);
}

static void unpack_12_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 132 bits of data.
	// Input must be at least 17 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);
}

static void unpack_12_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 132 bits of data.
	// Input must be at least 17 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);
}

static void unpack_12_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 132 bits of data.
	// Input must be at least 18 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);
}

static void unpack_12_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 132 bits of data.
	// Input must be at least 18 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);
}

static void unpack_12_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 132 bits of data.
	// Input must be at least 18 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);
}

static void unpack_12_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 132 bits of data.
	// Input must be at least 18 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);
}

static void unpack_12_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 132 bits of data.
	// Input must be at least 18 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);
}

static void unpack_12_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 132 bits of data.
	// Input must be at least 18 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);
}

static void unpack_13_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 143 bits of data.
	// Input must be at least 18 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);
}

static void unpack_13_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 143 bits of data.
	// Input must be at least 18 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);
}

static void unpack_13_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 143 bits of data.
	// Input must be at least 18 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);
}

static void unpack_13_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 143 bits of data.
	// Input must be at least 18 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);
}

static void unpack_13_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);
}

static void unpack_13_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);
}

static void unpack_13_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);
}

static void unpack_13_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);
}

static void unpack_13_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);
}

static void unpack_13_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);
}

static void unpack_13_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);
}

static void unpack_13_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);
}

static void unpack_13_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);
}

static void unpack_13_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);
}

static void unpack_13_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);
}

static void unpack_13_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);
}

static void unpack_14_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 154 bits of data.
	// Input must be at least 20 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);
}

static void unpack_14_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 154 bits of data.
	// Input must be at least 20 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);
}

static void unpack_14_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 154 bits of data.
	// Input must be at least 20 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);
}

static void unpack_14_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 154 bits of data.
	// Input must be at least 20 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);
}

static void unpack_14_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 154 bits of data.
	// Input must be at least 20 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);
}

static void unpack_14_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 154 bits of data.
	// Input must be at least 20 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);
}

static void unpack_14_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 154 bits of data.
	// Input must be at least 20 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);
}

static void unpack_14_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 154 bits of data.
	// Input must be at least 20 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);
}

static void unpack_14_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 154 bits of data.
	// Input must be at least 20 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);
}

static void unpack_14_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 154 bits of data.
	// Input must be at least 20 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);
}

static void unpack_14_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 154 bits of data.
	// Input must be at least 20 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);
}

static void unpack_14_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 154 bits of data.
	// Input must be at least 20 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);
}

static void unpack_14_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 154 bits of data.
	// Input must be at least 20 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);
}

static void unpack_14_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 154 bits of data.
	// Input must be at least 20 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);
}

static void unpack_14_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 154 bits of data.
	// Input must be at least 21 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);
}

static void unpack_14_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 154 bits of data.
	// Input must be at least 21 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);
}

static void unpack_15_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 165 bits of data.
	// Input must be at least 21 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);
}

static void unpack_15_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 165 bits of data.
	// Input must be at least 21 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);
}

static void unpack_15_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 165 bits of data.
	// Input must be at least 21 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);
}

static void unpack_15_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 165 bits of data.
	// Input must be at least 21 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);
}

static void unpack_15_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 165 bits of data.
	// Input must be at least 21 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);
}

static void unpack_15_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 165 bits of data.
	// Input must be at least 21 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);
}

static void unpack_15_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 165 bits of data.
	// Input must be at least 21 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);
}

static void unpack_15_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 165 bits of data.
	// Input must be at least 21 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);
}

static void unpack_15_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 165 bits of data.
	// Input must be at least 22 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);
}

static void unpack_15_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 165 bits of data.
	// Input must be at least 22 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);
}

static void unpack_15_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 165 bits of data.
	// Input must be at least 22 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);
}

static void unpack_15_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 165 bits of data.
	// Input must be at least 22 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);
}

static void unpack_15_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 165 bits of data.
	// Input must be at least 22 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);
}

static void unpack_15_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 165 bits of data.
	// Input must be at least 22 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);
}

static void unpack_15_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 165 bits of data.
	// Input must be at least 22 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);
}

static void unpack_15_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 165 bits of data.
	// Input must be at least 22 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);
}

static void unpack_16_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 176 bits of data.
	// Input must be at least 22 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);
}

static void unpack_16_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 176 bits of data.
	// Input must be at least 22 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);
}

static void unpack_16_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 176 bits of data.
	// Input must be at least 23 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);
}

static void unpack_16_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 176 bits of data.
	// Input must be at least 23 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);
}

static void unpack_16_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 176 bits of data.
	// Input must be at least 23 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);
}

static void unpack_16_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 176 bits of data.
	// Input must be at least 23 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);
}

static void unpack_16_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 176 bits of data.
	// Input must be at least 23 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);
}

static void unpack_16_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 176 bits of data.
	// Input must be at least 23 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);
}

static void unpack_16_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 176 bits of data.
	// Input must be at least 23 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);
}

static void unpack_16_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 176 bits of data.
	// Input must be at least 23 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);
}

static void unpack_16_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 176 bits of data.
	// Input must be at least 23 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);
}

static void unpack_16_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 176 bits of data.
	// Input must be at least 23 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);
}

static void unpack_16_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 176 bits of data.
	// Input must be at least 23 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);
}

static void unpack_16_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 176 bits of data.
	// Input must be at least 23 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);
}

static void unpack_16_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 176 bits of data.
	// Input must be at least 23 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);
}

static void unpack_16_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 176 bits of data.
	// Input must be at least 23 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);
}

static void unpack_17_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 187 bits of data.
	// Input must be at least 24 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);
}

static void unpack_17_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 187 bits of data.
	// Input must be at least 24 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);
}

static void unpack_17_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 187 bits of data.
	// Input must be at least 24 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);
}

static void unpack_17_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 187 bits of data.
	// Input must be at least 24 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);
}

static void unpack_17_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 187 bits of data.
	// Input must be at least 24 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);
}

static void unpack_17_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 187 bits of data.
	// Input must be at least 24 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);
}

static void unpack_17_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 187 bits of data.
	// Input must be at least 24 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);
}

static void unpack_17_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 187 bits of data.
	// Input must be at least 24 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);
}

static void unpack_17_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 187 bits of data.
	// Input must be at least 24 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);
}

static void unpack_17_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 187 bits of data.
	// Input must be at least 24 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);
}

static void unpack_17_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 187 bits of data.
	// Input must be at least 24 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);
}

static void unpack_17_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 187 bits of data.
	// Input must be at least 24 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);
}

static void unpack_17_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 187 bits of data.
	// Input must be at least 25 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);
}

static void unpack_17_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 187 bits of data.
	// Input must be at least 25 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);
}

static void unpack_17_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 187 bits of data.
	// Input must be at least 25 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);
}

static void unpack_17_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 187 bits of data.
	// Input must be at least 25 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);
}

static void unpack_18_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 198 bits of data.
	// Input must be at least 25 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);
}

static void unpack_18_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 198 bits of data.
	// Input must be at least 25 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);
}

static void unpack_18_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 198 bits of data.
	// Input must be at least 25 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);
}

static void unpack_18_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 198 bits of data.
	// Input must be at least 25 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);
}

static void unpack_18_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 198 bits of data.
	// Input must be at least 25 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);
}

static void unpack_18_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 198 bits of data.
	// Input must be at least 25 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);
}

static void unpack_18_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 198 bits of data.
	// Input must be at least 26 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);
}

static void unpack_18_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 198 bits of data.
	// Input must be at least 26 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);
}

static void unpack_18_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 198 bits of data.
	// Input must be at least 26 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);
}

static void unpack_18_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 198 bits of data.
	// Input must be at least 26 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);
}

static void unpack_18_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 198 bits of data.
	// Input must be at least 26 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);
}

static void unpack_18_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 198 bits of data.
	// Input must be at least 26 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);
}

static void unpack_18_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 198 bits of data.
	// Input must be at least 26 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);
}

static void unpack_18_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 198 bits of data.
	// Input must be at least 26 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);
}

static void unpack_18_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 198 bits of data.
	// Input must be at least 26 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);
}

static void unpack_18_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 198 bits of data.
	// Input must be at least 26 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);
}

static void unpack_19_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 209 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);
}

static void unpack_19_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 209 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);
}

static void unpack_19_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 209 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);
}

static void unpack_19_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 209 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);
}

static void unpack_19_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 209 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);
}

static void unpack_19_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 209 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);
}

static void unpack_19_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 209 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);
}

static void unpack_19_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 209 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);
}

static void unpack_19_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 209 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);
}

static void unpack_19_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 209 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);
}

static void unpack_19_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 209 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);
}

static void unpack_19_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 209 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);
}

static void unpack_19_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 209 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);
}

static void unpack_19_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 209 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);
}

static void unpack_19_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 209 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);
}

static void unpack_19_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 209 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);
}

static void unpack_20_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 220 bits of data.
	// Input must be at least 28 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);
}

static void unpack_20_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 220 bits of data.
	// Input must be at least 28 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);
}

static void unpack_20_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 220 bits of data.
	// Input must be at least 28 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);
}

static void unpack_20_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 220 bits of data.
	// Input must be at least 28 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);
}

static void unpack_20_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 220 bits of data.
	// Input must be at least 28 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);
}

static void unpack_20_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 220 bits of data.
	// Input must be at least 28 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);
}

static void unpack_20_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 220 bits of data.
	// Input must be at least 28 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);
}

static void unpack_20_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 220 bits of data.
	// Input must be at least 28 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);
}

static void unpack_20_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 220 bits of data.
	// Input must be at least 28 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);
}

static void unpack_20_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 220 bits of data.
	// Input must be at least 28 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);
}

static void unpack_20_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 220 bits of data.
	// Input must be at least 29 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);
}

static void unpack_20_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 220 bits of data.
	// Input must be at least 29 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);
}

static void unpack_20_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 220 bits of data.
	// Input must be at least 29 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);
}

static void unpack_20_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 220 bits of data.
	// Input must be at least 29 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);
}

static void unpack_20_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 220 bits of data.
	// Input must be at least 29 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);
}

static void unpack_20_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 220 bits of data.
	// Input must be at least 29 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);
}

static void unpack_21_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 231 bits of data.
	// Input must be at least 29 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);
}

static void unpack_21_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 231 bits of data.
	// Input must be at least 29 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);
}

static void unpack_21_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 231 bits of data.
	// Input must be at least 29 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);
}

static void unpack_21_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 231 bits of data.
	// Input must be at least 29 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);
}

static void unpack_21_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 231 bits of data.
	// Input must be at least 30 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);
}

static void unpack_21_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 231 bits of data.
	// Input must be at least 30 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);
}

static void unpack_21_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 231 bits of data.
	// Input must be at least 30 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);
}

static void unpack_21_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 231 bits of data.
	// Input must be at least 30 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);
}

static void unpack_21_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 231 bits of data.
	// Input must be at least 30 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);
}

static void unpack_21_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 231 bits of data.
	// Input must be at least 30 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);
}

static void unpack_21_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 231 bits of data.
	// Input must be at least 30 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);
}

static void unpack_21_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 231 bits of data.
	// Input must be at least 30 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);
}

static void unpack_21_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 231 bits of data.
	// Input must be at least 30 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);
}

static void unpack_21_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 231 bits of data.
	// Input must be at least 30 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);
}

static void unpack_21_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 231 bits of data.
	// Input must be at least 30 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);
}

static void unpack_21_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 231 bits of data.
	// Input must be at least 30 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);
}

static void unpack_22_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 242 bits of data.
	// Input must be at least 31 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);
}

static void unpack_22_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 242 bits of data.
	// Input must be at least 31 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);
}

static void unpack_22_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 242 bits of data.
	// Input must be at least 31 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);
}

static void unpack_22_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 242 bits of data.
	// Input must be at least 31 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);
}

static void unpack_22_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 242 bits of data.
	// Input must be at least 31 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);
}

static void unpack_22_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 242 bits of data.
	// Input must be at least 31 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);
}

static void unpack_22_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 242 bits of data.
	// Input must be at least 31 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);
}

static void unpack_22_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 242 bits of data.
	// Input must be at least 31 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);
}

static void unpack_22_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 242 bits of data.
	// Input must be at least 31 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);
}

static void unpack_22_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 242 bits of data.
	// Input must be at least 31 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);
}

static void unpack_22_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 242 bits of data.
	// Input must be at least 31 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);
}

static void unpack_22_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 242 bits of data.
	// Input must be at least 31 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);
}

static void unpack_22_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 242 bits of data.
	// Input must be at least 31 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);
}

static void unpack_22_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 242 bits of data.
	// Input must be at least 31 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);
}

static void unpack_22_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 242 bits of data.
	// Input must be at least 32 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);
}

static void unpack_22_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 242 bits of data.
	// Input must be at least 32 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);
}

static void unpack_23_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 253 bits of data.
	// Input must be at least 32 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);
}

static void unpack_23_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 253 bits of data.
	// Input must be at least 32 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);
}

static void unpack_23_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 253 bits of data.
	// Input must be at least 32 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);
}

static void unpack_23_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 253 bits of data.
	// Input must be at least 32 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);
}

static void unpack_23_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 253 bits of data.
	// Input must be at least 32 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);
}

static void unpack_23_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 253 bits of data.
	// Input must be at least 32 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);
}

static void unpack_23_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 253 bits of data.
	// Input must be at least 32 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);
}

static void unpack_23_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 253 bits of data.
	// Input must be at least 32 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);
}

static void unpack_23_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 253 bits of data.
	// Input must be at least 33 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);
}

static void unpack_23_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 253 bits of data.
	// Input must be at least 33 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);
}

static void unpack_23_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 253 bits of data.
	// Input must be at least 33 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);
}

static void unpack_23_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 253 bits of data.
	// Input must be at least 33 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);
}

static void unpack_23_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 253 bits of data.
	// Input must be at least 33 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);
}

static void unpack_23_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 253 bits of data.
	// Input must be at least 33 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);
}

static void unpack_23_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 253 bits of data.
	// Input must be at least 33 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);
}

static void unpack_23_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 253 bits of data.
	// Input must be at least 33 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);
}

static void unpack_24_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 264 bits of data.
	// Input must be at least 33 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);
}

static void unpack_24_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 264 bits of data.
	// Input must be at least 33 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);
}

static void unpack_24_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 264 bits of data.
	// Input must be at least 34 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);
}

static void unpack_24_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 264 bits of data.
	// Input must be at least 34 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);
}

static void unpack_24_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 264 bits of data.
	// Input must be at least 34 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);
}

static void unpack_24_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 264 bits of data.
	// Input must be at least 34 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);
}

static void unpack_24_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 264 bits of data.
	// Input must be at least 34 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);
}

static void unpack_24_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 264 bits of data.
	// Input must be at least 34 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);
}

static void unpack_24_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 264 bits of data.
	// Input must be at least 34 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);
}

static void unpack_24_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 264 bits of data.
	// Input must be at least 34 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);
}

static void unpack_24_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 264 bits of data.
	// Input must be at least 34 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);
}

static void unpack_24_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 264 bits of data.
	// Input must be at least 34 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);
}

static void unpack_24_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 264 bits of data.
	// Input must be at least 34 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);
}

static void unpack_24_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 264 bits of data.
	// Input must be at least 34 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);
}

static void unpack_24_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 264 bits of data.
	// Input must be at least 34 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);
}

static void unpack_24_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 264 bits of data.
	// Input must be at least 34 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);
}

static void unpack_25_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 275 bits of data.
	// Input must be at least 35 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);
}

static void unpack_25_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 275 bits of data.
	// Input must be at least 35 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);
}

static void unpack_25_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 275 bits of data.
	// Input must be at least 35 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);
}

static void unpack_25_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 275 bits of data.
	// Input must be at least 35 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);
}

static void unpack_25_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 275 bits of data.
	// Input must be at least 35 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);
}

static void unpack_25_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 275 bits of data.
	// Input must be at least 35 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);
}

static void unpack_25_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 275 bits of data.
	// Input must be at least 35 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);
}

static void unpack_25_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 275 bits of data.
	// Input must be at least 35 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);
}

static void unpack_25_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 275 bits of data.
	// Input must be at least 35 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);
}

static void unpack_25_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 275 bits of data.
	// Input must be at least 35 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);
}

static void unpack_25_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 275 bits of data.
	// Input must be at least 35 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);
}

static void unpack_25_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 275 bits of data.
	// Input must be at least 35 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);
}

static void unpack_25_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 275 bits of data.
	// Input must be at least 36 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);
}

static void unpack_25_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 275 bits of data.
	// Input must be at least 36 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);
}

static void unpack_25_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 275 bits of data.
	// Input must be at least 36 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);
}

static void unpack_25_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 275 bits of data.
	// Input must be at least 36 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);
}

static void unpack_26_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 286 bits of data.
	// Input must be at least 36 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);
}

static void unpack_26_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 286 bits of data.
	// Input must be at least 36 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);
}

static void unpack_26_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 286 bits of data.
	// Input must be at least 36 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);
}

static void unpack_26_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 286 bits of data.
	// Input must be at least 36 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);
}

static void unpack_26_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 286 bits of data.
	// Input must be at least 36 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);
}

static void unpack_26_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 286 bits of data.
	// Input must be at least 36 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);
}

static void unpack_26_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 286 bits of data.
	// Input must be at least 37 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);
}

static void unpack_26_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 286 bits of data.
	// Input must be at least 37 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);
}

static void unpack_26_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 286 bits of data.
	// Input must be at least 37 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);
}

static void unpack_26_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 286 bits of data.
	// Input must be at least 37 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);
}

static void unpack_26_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 286 bits of data.
	// Input must be at least 37 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);
}

static void unpack_26_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 286 bits of data.
	// Input must be at least 37 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);
}

static void unpack_26_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 286 bits of data.
	// Input must be at least 37 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);
}

static void unpack_26_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 286 bits of data.
	// Input must be at least 37 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);
}

static void unpack_26_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 286 bits of data.
	// Input must be at least 37 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);
}

static void unpack_26_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 286 bits of data.
	// Input must be at least 37 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);
}

static void unpack_27_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 297 bits of data.
	// Input must be at least 38 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);
}

static void unpack_27_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 297 bits of data.
	// Input must be at least 38 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);
}

static void unpack_27_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 297 bits of data.
	// Input must be at least 38 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);
}

static void unpack_27_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 297 bits of data.
	// Input must be at least 38 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);
}

static void unpack_27_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 297 bits of data.
	// Input must be at least 38 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);
}

static void unpack_27_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 297 bits of data.
	// Input must be at least 38 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);
}

static void unpack_27_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 297 bits of data.
	// Input must be at least 38 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);
}

static void unpack_27_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 297 bits of data.
	// Input must be at least 38 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);
}

static void unpack_27_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 297 bits of data.
	// Input must be at least 38 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);
}

static void unpack_27_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 297 bits of data.
	// Input must be at least 38 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);
}

static void unpack_27_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 297 bits of data.
	// Input must be at least 38 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);
}

static void unpack_27_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 297 bits of data.
	// Input must be at least 38 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);
}

static void unpack_27_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 297 bits of data.
	// Input must be at least 38 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);
}

static void unpack_27_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 297 bits of data.
	// Input must be at least 38 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);
}

static void unpack_27_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 297 bits of data.
	// Input must be at least 38 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);
}

static void unpack_27_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 297 bits of data.
	// Input must be at least 38 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);
}

static void unpack_28_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 308 bits of data.
	// Input must be at least 39 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);
}

static void unpack_28_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 308 bits of data.
	// Input must be at least 39 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);
}

static void unpack_28_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 308 bits of data.
	// Input must be at least 39 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);
}

static void unpack_28_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 308 bits of data.
	// Input must be at least 39 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);
}

static void unpack_28_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 308 bits of data.
	// Input must be at least 39 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);
}

static void unpack_28_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 308 bits of data.
	// Input must be at least 39 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);
}

static void unpack_28_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 308 bits of data.
	// Input must be at least 39 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);
}

static void unpack_28_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 308 bits of data.
	// Input must be at least 39 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);
}

static void unpack_28_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 308 bits of data.
	// Input must be at least 39 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);
}

static void unpack_28_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 308 bits of data.
	// Input must be at least 39 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);
}

static void unpack_28_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 308 bits of data.
	// Input must be at least 40 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);
}

static void unpack_28_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 308 bits of data.
	// Input must be at least 40 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);
}

static void unpack_28_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 308 bits of data.
	// Input must be at least 40 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);
}

static void unpack_28_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 308 bits of data.
	// Input must be at least 40 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);
}

static void unpack_28_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 308 bits of data.
	// Input must be at least 40 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);
}

static void unpack_28_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 308 bits of data.
	// Input must be at least 40 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);
}

static void unpack_29_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 319 bits of data.
	// Input must be at least 40 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);
}

static void unpack_29_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 319 bits of data.
	// Input must be at least 40 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);
}

static void unpack_29_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 319 bits of data.
	// Input must be at least 40 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);
}

static void unpack_29_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 319 bits of data.
	// Input must be at least 40 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);
}

static void unpack_29_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 319 bits of data.
	// Input must be at least 41 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);
}

static void unpack_29_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 319 bits of data.
	// Input must be at least 41 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);
}

static void unpack_29_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 319 bits of data.
	// Input must be at least 41 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);
}

static void unpack_29_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 319 bits of data.
	// Input must be at least 41 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);
}

static void unpack_29_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 319 bits of data.
	// Input must be at least 41 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);
}

static void unpack_29_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 319 bits of data.
	// Input must be at least 41 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);
}

static void unpack_29_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 319 bits of data.
	// Input must be at least 41 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);
}

static void unpack_29_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 319 bits of data.
	// Input must be at least 41 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);
}

static void unpack_29_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 319 bits of data.
	// Input must be at least 41 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);
}

static void unpack_29_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 319 bits of data.
	// Input must be at least 41 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);
}

static void unpack_29_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 319 bits of data.
	// Input must be at least 41 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);
}

static void unpack_29_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 319 bits of data.
	// Input must be at least 41 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);
}

static void unpack_30_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);
}

static void unpack_30_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);
}

static void unpack_30_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);
}

static void unpack_30_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);
}

static void unpack_30_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);
}

static void unpack_30_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);
}

static void unpack_30_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);
}

static void unpack_30_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);
}

static void unpack_30_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);
}

static void unpack_30_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);
}

static void unpack_30_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);
}

static void unpack_30_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);
}

static void unpack_30_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);
}

static void unpack_30_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 330 bits of data.
	// Input must be at least 42 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);
}

static void unpack_30_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 330 bits of data.
	// Input must be at least 43 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);
}

static void unpack_30_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 330 bits of data.
	// Input must be at least 43 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);
}

static void unpack_31_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 341 bits of data.
	// Input must be at least 43 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);
}

static void unpack_31_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 341 bits of data.
	// Input must be at least 43 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);
}

static void unpack_31_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 341 bits of data.
	// Input must be at least 43 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);
}

static void unpack_31_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 341 bits of data.
	// Input must be at least 43 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);
}

static void unpack_31_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 341 bits of data.
	// Input must be at least 43 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);
}

static void unpack_31_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 341 bits of data.
	// Input must be at least 43 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);
}

static void unpack_31_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 341 bits of data.
	// Input must be at least 43 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);
}

static void unpack_31_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 341 bits of data.
	// Input must be at least 43 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);
}

static void unpack_31_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 341 bits of data.
	// Input must be at least 44 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);
}

static void unpack_31_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 341 bits of data.
	// Input must be at least 44 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);
}

static void unpack_31_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 341 bits of data.
	// Input must be at least 44 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);
}

static void unpack_31_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 341 bits of data.
	// Input must be at least 44 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);
}

static void unpack_31_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 341 bits of data.
	// Input must be at least 44 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);
}

static void unpack_31_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 341 bits of data.
	// Input must be at least 44 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);
}

static void unpack_31_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 341 bits of data.
	// Input must be at least 44 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);
}

static void unpack_31_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 341 bits of data.
	// Input must be at least 44 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);
}

static void unpack_32_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 352 bits of data.
	// Input must be at least 44 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);
}

static void unpack_32_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 352 bits of data.
	// Input must be at least 44 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);
}

static void unpack_32_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 352 bits of data.
	// Input must be at least 45 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);
}

static void unpack_32_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 352 bits of data.
	// Input must be at least 45 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);
}

static void unpack_32_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 352 bits of data.
	// Input must be at least 45 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);
}

static void unpack_32_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 352 bits of data.
	// Input must be at least 45 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);
}

static void unpack_32_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 352 bits of data.
	// Input must be at least 45 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);
}

static void unpack_32_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 352 bits of data.
	// Input must be at least 45 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);
}

static void unpack_32_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 352 bits of data.
	// Input must be at least 45 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);
}

static void unpack_32_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 352 bits of data.
	// Input must be at least 45 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);
}

static void unpack_32_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 352 bits of data.
	// Input must be at least 45 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);
}

static void unpack_32_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 352 bits of data.
	// Input must be at least 45 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);
}

static void unpack_32_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 352 bits of data.
	// Input must be at least 45 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);
}

static void unpack_32_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 352 bits of data.
	// Input must be at least 45 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);
}

static void unpack_32_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 352 bits of data.
	// Input must be at least 45 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);
}

static void unpack_32_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 352 bits of data.
	// Input must be at least 45 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);
}

static void unpack_33_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 363 bits of data.
	// Input must be at least 46 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);
}

static void unpack_33_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 363 bits of data.
	// Input must be at least 46 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);
}

static void unpack_33_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 363 bits of data.
	// Input must be at least 46 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);
}

static void unpack_33_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 363 bits of data.
	// Input must be at least 46 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);
}

static void unpack_33_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 363 bits of data.
	// Input must be at least 46 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);
}

static void unpack_33_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 363 bits of data.
	// Input must be at least 46 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);
}

static void unpack_33_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 363 bits of data.
	// Input must be at least 46 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);
}

static void unpack_33_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 363 bits of data.
	// Input must be at least 46 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);
}

static void unpack_33_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 363 bits of data.
	// Input must be at least 46 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);
}

static void unpack_33_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 363 bits of data.
	// Input must be at least 46 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);
}

static void unpack_33_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 363 bits of data.
	// Input must be at least 46 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);
}

static void unpack_33_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 363 bits of data.
	// Input must be at least 46 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);
}

static void unpack_33_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 363 bits of data.
	// Input must be at least 47 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);
}

static void unpack_33_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 363 bits of data.
	// Input must be at least 47 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);
}

static void unpack_33_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 363 bits of data.
	// Input must be at least 47 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);
}

static void unpack_33_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 363 bits of data.
	// Input must be at least 47 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);
}

static void unpack_34_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 374 bits of data.
	// Input must be at least 47 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);
}

static void unpack_34_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 374 bits of data.
	// Input must be at least 47 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);
}

static void unpack_34_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 374 bits of data.
	// Input must be at least 47 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);
}

static void unpack_34_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 374 bits of data.
	// Input must be at least 47 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);
}

static void unpack_34_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 374 bits of data.
	// Input must be at least 47 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);
}

static void unpack_34_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 374 bits of data.
	// Input must be at least 47 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);
}

static void unpack_34_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 374 bits of data.
	// Input must be at least 48 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);
}

static void unpack_34_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 374 bits of data.
	// Input must be at least 48 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);
}

static void unpack_34_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 374 bits of data.
	// Input must be at least 48 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);
}

static void unpack_34_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 374 bits of data.
	// Input must be at least 48 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);
}

static void unpack_34_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 374 bits of data.
	// Input must be at least 48 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);
}

static void unpack_34_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 374 bits of data.
	// Input must be at least 48 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);
}

static void unpack_34_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 374 bits of data.
	// Input must be at least 48 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);
}

static void unpack_34_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 374 bits of data.
	// Input must be at least 48 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);
}

static void unpack_34_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 374 bits of data.
	// Input must be at least 48 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);
}

static void unpack_34_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 374 bits of data.
	// Input must be at least 48 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);
}

static void unpack_35_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 385 bits of data.
	// Input must be at least 49 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);
}

static void unpack_35_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 385 bits of data.
	// Input must be at least 49 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);
}

static void unpack_35_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 385 bits of data.
	// Input must be at least 49 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);
}

static void unpack_35_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 385 bits of data.
	// Input must be at least 49 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);
}

static void unpack_35_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 385 bits of data.
	// Input must be at least 49 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);
}

static void unpack_35_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 385 bits of data.
	// Input must be at least 49 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);
}

static void unpack_35_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 385 bits of data.
	// Input must be at least 49 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);
}

static void unpack_35_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 385 bits of data.
	// Input must be at least 49 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);
}

static void unpack_35_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 385 bits of data.
	// Input must be at least 49 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);
}

static void unpack_35_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 385 bits of data.
	// Input must be at least 49 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);
}

static void unpack_35_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 385 bits of data.
	// Input must be at least 49 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);
}

static void unpack_35_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 385 bits of data.
	// Input must be at least 49 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);
}

static void unpack_35_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 385 bits of data.
	// Input must be at least 49 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);
}

static void unpack_35_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 385 bits of data.
	// Input must be at least 49 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);
}

static void unpack_35_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 385 bits of data.
	// Input must be at least 49 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);
}

static void unpack_35_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 385 bits of data.
	// Input must be at least 49 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);
}

static void unpack_36_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 396 bits of data.
	// Input must be at least 50 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);
}

static void unpack_36_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 396 bits of data.
	// Input must be at least 50 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);
}

static void unpack_36_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 396 bits of data.
	// Input must be at least 50 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);
}

static void unpack_36_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 396 bits of data.
	// Input must be at least 50 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);
}

static void unpack_36_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 396 bits of data.
	// Input must be at least 50 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);
}

static void unpack_36_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 396 bits of data.
	// Input must be at least 50 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);
}

static void unpack_36_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 396 bits of data.
	// Input must be at least 50 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);
}

static void unpack_36_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 396 bits of data.
	// Input must be at least 50 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);
}

static void unpack_36_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 396 bits of data.
	// Input must be at least 50 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);
}

static void unpack_36_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 396 bits of data.
	// Input must be at least 50 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);
}

static void unpack_36_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 396 bits of data.
	// Input must be at least 51 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);
}

static void unpack_36_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 396 bits of data.
	// Input must be at least 51 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);
}

static void unpack_36_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 396 bits of data.
	// Input must be at least 51 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);
}

static void unpack_36_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 396 bits of data.
	// Input must be at least 51 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);
}

static void unpack_36_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 396 bits of data.
	// Input must be at least 51 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);
}

static void unpack_36_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 396 bits of data.
	// Input must be at least 51 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);
}

static void unpack_37_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 407 bits of data.
	// Input must be at least 51 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 396 to 406
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)(input[50] & 0xfe) << 20);
	output[36] = (double)(value >> 21);
}

static void unpack_37_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 407 bits of data.
	// Input must be at least 51 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 396 to 406
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)(input[50] & 0xfe) << 20);
	output[36] = (double)(value >> 21);
}

static void unpack_37_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 407 bits of data.
	// Input must be at least 51 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 397 to 407
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21);
	output[36] = (double)(value >> 21);
}

static void unpack_37_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 407 bits of data.
	// Input must be at least 51 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 397 to 407
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21);
	output[36] = (double)(value >> 21);
}

static void unpack_37_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 407 bits of data.
	// Input must be at least 52 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 398 to 408
	value = ((uint32_t)(input[49] & 0x03) << 30) |
			((uint32_t)input[50] << 22) |
			((uint32_t)(input[51] & 0x80) << 14);
	output[36] = (double)(value >> 21);
}

static void unpack_37_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 407 bits of data.
	// Input must be at least 52 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 398 to 408
	value = ((uint32_t)(input[49] & 0x03) << 30) |
			((uint32_t)input[50] << 22) |
			((uint32_t)(input[51] & 0x80) << 14);
	output[36] = (double)(value >> 21);
}

static void unpack_37_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 407 bits of data.
	// Input must be at least 52 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 399 to 409
	value = ((uint32_t)(input[49] & 0x01) << 31) |
			((uint32_t)input[50] << 23) |
			((uint32_t)(input[51] & 0xc0) << 15);
	output[36] = (double)(value >> 21);
}

static void unpack_37_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 407 bits of data.
	// Input must be at least 52 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 399 to 409
	value = ((uint32_t)(input[49] & 0x01) << 31) |
			((uint32_t)input[50] << 23) |
			((uint32_t)(input[51] & 0xc0) << 15);
	output[36] = (double)(value >> 21);
}

static void unpack_37_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 407 bits of data.
	// Input must be at least 52 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 400 to 410
	value = ((uint32_t)input[50] << 24) |
			((uint32_t)(input[51] & 0xe0) << 16);
	output[36] = (double)(value >> 21);
}

static void unpack_37_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 407 bits of data.
	// Input must be at least 52 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 400 to 410
	value = ((uint32_t)input[50] << 24) |
			((uint32_t)(input[51] & 0xe0) << 16);
	output[36] = (double)(value >> 21);
}

static void unpack_37_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 407 bits of data.
	// Input must be at least 52 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 401 to 411
	value = ((uint32_t)(input[50] & 0x7f) << 25) |
			((uint32_t)(input[51] & 0xf0) << 17);
	output[36] = (double)(value >> 21);
}

static void unpack_37_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 407 bits of data.
	// Input must be at least 52 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 401 to 411
	value = ((uint32_t)(input[50] & 0x7f) << 25) |
			((uint32_t)(input[51] & 0xf0) << 17);
	output[36] = (double)(value >> 21);
}

static void unpack_37_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 407 bits of data.
	// Input must be at least 52 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 402 to 412
	value = ((uint32_t)(input[50] & 0x3f) << 26) |
			((uint32_t)(input[51] & 0xf8) << 18);
	output[36] = (double)(value >> 21);
}

static void unpack_37_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 407 bits of data.
	// Input must be at least 52 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 402 to 412
	value = ((uint32_t)(input[50] & 0x3f) << 26) |
			((uint32_t)(input[51] & 0xf8) << 18);
	output[36] = (double)(value >> 21);
}

static void unpack_37_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 407 bits of data.
	// Input must be at least 52 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 403 to 413
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)(input[51] & 0xfc) << 19);
	output[36] = (double)(value >> 21);
}

static void unpack_37_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 407 bits of data.
	// Input must be at least 52 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 403 to 413
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)(input[51] & 0xfc) << 19);
	output[36] = (double)(value >> 21);
}

static void unpack_38_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 418 bits of data.
	// Input must be at least 53 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 396 to 406
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)(input[50] & 0xfe) << 20);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 407 to 417
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xc0) << 15);
	output[37] = (double)(value >> 21);
}

static void unpack_38_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 418 bits of data.
	// Input must be at least 53 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 396 to 406
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)(input[50] & 0xfe) << 20);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 407 to 417
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xc0) << 15);
	output[37] = (double)(value >> 21);
}

static void unpack_38_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 418 bits of data.
	// Input must be at least 53 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 397 to 407
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 408 to 418
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xe0) << 16);
	output[37] = (double)(value >> 21);
}

static void unpack_38_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 418 bits of data.
	// Input must be at least 53 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 397 to 407
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 408 to 418
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xe0) << 16);
	output[37] = (double)(value >> 21);
}

static void unpack_38_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 418 bits of data.
	// Input must be at least 53 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 398 to 408
	value = ((uint32_t)(input[49] & 0x03) << 30) |
			((uint32_t)input[50] << 22) |
			((uint32_t)(input[51] & 0x80) << 14);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 409 to 419
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xf0) << 17);
	output[37] = (double)(value >> 21);
}

static void unpack_38_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 418 bits of data.
	// Input must be at least 53 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 398 to 408
	value = ((uint32_t)(input[49] & 0x03) << 30) |
			((uint32_t)input[50] << 22) |
			((uint32_t)(input[51] & 0x80) << 14);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 409 to 419
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xf0) << 17);
	output[37] = (double)(value >> 21);
}

static void unpack_38_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 418 bits of data.
	// Input must be at least 53 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 399 to 409
	value = ((uint32_t)(input[49] & 0x01) << 31) |
			((uint32_t)input[50] << 23) |
			((uint32_t)(input[51] & 0xc0) << 15);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 410 to 420
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xf8) << 18);
	output[37] = (double)(value >> 21);
}

static void unpack_38_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 418 bits of data.
	// Input must be at least 53 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 399 to 409
	value = ((uint32_t)(input[49] & 0x01) << 31) |
			((uint32_t)input[50] << 23) |
			((uint32_t)(input[51] & 0xc0) << 15);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 410 to 420
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xf8) << 18);
	output[37] = (double)(value >> 21);
}

static void unpack_38_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 418 bits of data.
	// Input must be at least 53 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 400 to 410
	value = ((uint32_t)input[50] << 24) |
			((uint32_t)(input[51] & 0xe0) << 16);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 411 to 421
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)(input[52] & 0xfc) << 19);
	output[37] = (double)(value >> 21);
}

static void unpack_38_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 418 bits of data.
	// Input must be at least 53 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 400 to 410
	value = ((uint32_t)input[50] << 24) |
			((uint32_t)(input[51] & 0xe0) << 16);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 411 to 421
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)(input[52] & 0xfc) << 19);
	output[37] = (double)(value >> 21);
}

static void unpack_38_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 418 bits of data.
	// Input must be at least 53 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 401 to 411
	value = ((uint32_t)(input[50] & 0x7f) << 25) |
			((uint32_t)(input[51] & 0xf0) << 17);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 412 to 422
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)(input[52] & 0xfe) << 20);
	output[37] = (double)(value >> 21);
}

static void unpack_38_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 418 bits of data.
	// Input must be at least 53 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 401 to 411
	value = ((uint32_t)(input[50] & 0x7f) << 25) |
			((uint32_t)(input[51] & 0xf0) << 17);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 412 to 422
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)(input[52] & 0xfe) << 20);
	output[37] = (double)(value >> 21);
}

static void unpack_38_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 418 bits of data.
	// Input must be at least 53 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 402 to 412
	value = ((uint32_t)(input[50] & 0x3f) << 26) |
			((uint32_t)(input[51] & 0xf8) << 18);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 413 to 423
	value = ((uint32_t)(input[51] & 0x07) << 29) |
			((uint32_t)input[52] << 21);
	output[37] = (double)(value >> 21);
}

static void unpack_38_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 418 bits of data.
	// Input must be at least 53 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 402 to 412
	value = ((uint32_t)(input[50] & 0x3f) << 26) |
			((uint32_t)(input[51] & 0xf8) << 18);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 413 to 423
	value = ((uint32_t)(input[51] & 0x07) << 29) |
			((uint32_t)input[52] << 21);
	output[37] = (double)(value >> 21);
}

static void unpack_38_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 418 bits of data.
	// Input must be at least 54 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 403 to 413
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)(input[51] & 0xfc) << 19);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 414 to 424
	value = ((uint32_t)(input[51] & 0x03) << 30) |
			((uint32_t)input[52] << 22) |
			((uint32_t)(input[53] & 0x80) << 14);
	output[37] = (double)(value >> 21);
}

static void unpack_38_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 418 bits of data.
	// Input must be at least 54 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 403 to 413
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)(input[51] & 0xfc) << 19);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 414 to 424
	value = ((uint32_t)(input[51] & 0x03) << 30) |
			((uint32_t)input[52] << 22) |
			((uint32_t)(input[53] & 0x80) << 14);
	output[37] = (double)(value >> 21);
}

static void unpack_39_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 429 bits of data.
	// Input must be at least 54 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 396 to 406
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)(input[50] & 0xfe) << 20);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 407 to 417
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xc0) << 15);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 418 to 428
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xf8) << 18);
	output[38] = (double)(value >> 21);
}

static void unpack_39_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 429 bits of data.
	// Input must be at least 54 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 396 to 406
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)(input[50] & 0xfe) << 20);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 407 to 417
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xc0) << 15);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 418 to 428
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xf8) << 18);
	output[38] = (double)(value >> 21);
}

static void unpack_39_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 429 bits of data.
	// Input must be at least 54 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 397 to 407
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 408 to 418
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xe0) << 16);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 419 to 429
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)(input[53] & 0xfc) << 19);
	output[38] = (double)(value >> 21);
}

static void unpack_39_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 429 bits of data.
	// Input must be at least 54 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 397 to 407
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 408 to 418
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xe0) << 16);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 419 to 429
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)(input[53] & 0xfc) << 19);
	output[38] = (double)(value >> 21);
}

static void unpack_39_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 429 bits of data.
	// Input must be at least 54 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 398 to 408
	value = ((uint32_t)(input[49] & 0x03) << 30) |
			((uint32_t)input[50] << 22) |
			((uint32_t)(input[51] & 0x80) << 14);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 409 to 419
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xf0) << 17);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 420 to 430
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)(input[53] & 0xfe) << 20);
	output[38] = (double)(value >> 21);
}

static void unpack_39_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 429 bits of data.
	// Input must be at least 54 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 398 to 408
	value = ((uint32_t)(input[49] & 0x03) << 30) |
			((uint32_t)input[50] << 22) |
			((uint32_t)(input[51] & 0x80) << 14);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 409 to 419
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xf0) << 17);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 420 to 430
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)(input[53] & 0xfe) << 20);
	output[38] = (double)(value >> 21);
}

static void unpack_39_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 429 bits of data.
	// Input must be at least 54 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 399 to 409
	value = ((uint32_t)(input[49] & 0x01) << 31) |
			((uint32_t)input[50] << 23) |
			((uint32_t)(input[51] & 0xc0) << 15);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 410 to 420
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xf8) << 18);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 421 to 431
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21);
	output[38] = (double)(value >> 21);
}

static void unpack_39_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 429 bits of data.
	// Input must be at least 54 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 399 to 409
	value = ((uint32_t)(input[49] & 0x01) << 31) |
			((uint32_t)input[50] << 23) |
			((uint32_t)(input[51] & 0xc0) << 15);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 410 to 420
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xf8) << 18);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 421 to 431
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21);
	output[38] = (double)(value >> 21);
}

static void unpack_39_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 429 bits of data.
	// Input must be at least 55 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 400 to 410
	value = ((uint32_t)input[50] << 24) |
			((uint32_t)(input[51] & 0xe0) << 16);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 411 to 421
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)(input[52] & 0xfc) << 19);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 422 to 432
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0x80) << 14);
	output[38] = (double)(value >> 21);
}

static void unpack_39_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 429 bits of data.
	// Input must be at least 55 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 400 to 410
	value = ((uint32_t)input[50] << 24) |
			((uint32_t)(input[51] & 0xe0) << 16);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 411 to 421
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)(input[52] & 0xfc) << 19);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 422 to 432
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0x80) << 14);
	output[38] = (double)(value >> 21);
}

static void unpack_39_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 429 bits of data.
	// Input must be at least 55 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 401 to 411
	value = ((uint32_t)(input[50] & 0x7f) << 25) |
			((uint32_t)(input[51] & 0xf0) << 17);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 412 to 422
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)(input[52] & 0xfe) << 20);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 423 to 433
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xc0) << 15);
	output[38] = (double)(value >> 21);
}

static void unpack_39_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 429 bits of data.
	// Input must be at least 55 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 401 to 411
	value = ((uint32_t)(input[50] & 0x7f) << 25) |
			((uint32_t)(input[51] & 0xf0) << 17);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 412 to 422
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)(input[52] & 0xfe) << 20);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 423 to 433
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xc0) << 15);
	output[38] = (double)(value >> 21);
}

static void unpack_39_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 429 bits of data.
	// Input must be at least 55 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 402 to 412
	value = ((uint32_t)(input[50] & 0x3f) << 26) |
			((uint32_t)(input[51] & 0xf8) << 18);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 413 to 423
	value = ((uint32_t)(input[51] & 0x07) << 29) |
			((uint32_t)input[52] << 21);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 424 to 434
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xe0) << 16);
	output[38] = (double)(value >> 21);
}

static void unpack_39_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 429 bits of data.
	// Input must be at least 55 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 402 to 412
	value = ((uint32_t)(input[50] & 0x3f) << 26) |
			((uint32_t)(input[51] & 0xf8) << 18);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 413 to 423
	value = ((uint32_t)(input[51] & 0x07) << 29) |
			((uint32_t)input[52] << 21);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 424 to 434
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xe0) << 16);
	output[38] = (double)(value >> 21);
}

static void unpack_39_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 429 bits of data.
	// Input must be at least 55 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 403 to 413
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)(input[51] & 0xfc) << 19);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 414 to 424
	value = ((uint32_t)(input[51] & 0x03) << 30) |
			((uint32_t)input[52] << 22) |
			((uint32_t)(input[53] & 0x80) << 14);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 425 to 435
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)(input[54] & 0xf0) << 17);
	output[38] = (double)(value >> 21);
}

static void unpack_39_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 429 bits of data.
	// Input must be at least 55 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 403 to 413
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)(input[51] & 0xfc) << 19);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 414 to 424
	value = ((uint32_t)(input[51] & 0x03) << 30) |
			((uint32_t)input[52] << 22) |
			((uint32_t)(input[53] & 0x80) << 14);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 425 to 435
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)(input[54] & 0xf0) << 17);
	output[38] = (double)(value >> 21);
}

static void unpack_40_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 40 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 440 bits of data.
	// Input must be at least 55 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 396 to 406
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)(input[50] & 0xfe) << 20);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 407 to 417
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xc0) << 15);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 418 to 428
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xf8) << 18);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 429 to 439
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21);
	output[39] = (double)(value >> 21);
}

static void unpack_40_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 40 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 440 bits of data.
	// Input must be at least 55 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 396 to 406
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)(input[50] & 0xfe) << 20);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 407 to 417
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xc0) << 15);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 418 to 428
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xf8) << 18);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 429 to 439
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21);
	output[39] = (double)(value >> 21);
}

static void unpack_40_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 40 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 440 bits of data.
	// Input must be at least 56 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 397 to 407
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 408 to 418
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xe0) << 16);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 419 to 429
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)(input[53] & 0xfc) << 19);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 430 to 440
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0x80) << 14);
	output[39] = (double)(value >> 21);
}

static void unpack_40_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 40 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 440 bits of data.
	// Input must be at least 56 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 397 to 407
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 408 to 418
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xe0) << 16);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 419 to 429
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)(input[53] & 0xfc) << 19);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 430 to 440
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0x80) << 14);
	output[39] = (double)(value >> 21);
}

static void unpack_40_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 40 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 440 bits of data.
	// Input must be at least 56 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 398 to 408
	value = ((uint32_t)(input[49] & 0x03) << 30) |
			((uint32_t)input[50] << 22) |
			((uint32_t)(input[51] & 0x80) << 14);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 409 to 419
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xf0) << 17);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 420 to 430
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)(input[53] & 0xfe) << 20);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 431 to 441
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xc0) << 15);
	output[39] = (double)(value >> 21);
}

static void unpack_40_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 40 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 440 bits of data.
	// Input must be at least 56 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 398 to 408
	value = ((uint32_t)(input[49] & 0x03) << 30) |
			((uint32_t)input[50] << 22) |
			((uint32_t)(input[51] & 0x80) << 14);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 409 to 419
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xf0) << 17);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 420 to 430
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)(input[53] & 0xfe) << 20);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 431 to 441
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xc0) << 15);
	output[39] = (double)(value >> 21);
}

static void unpack_40_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 40 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 440 bits of data.
	// Input must be at least 56 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 399 to 409
	value = ((uint32_t)(input[49] & 0x01) << 31) |
			((uint32_t)input[50] << 23) |
			((uint32_t)(input[51] & 0xc0) << 15);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 410 to 420
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xf8) << 18);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 421 to 431
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 432 to 442
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xe0) << 16);
	output[39] = (double)(value >> 21);
}

static void unpack_40_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 40 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 440 bits of data.
	// Input must be at least 56 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 399 to 409
	value = ((uint32_t)(input[49] & 0x01) << 31) |
			((uint32_t)input[50] << 23) |
			((uint32_t)(input[51] & 0xc0) << 15);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 410 to 420
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xf8) << 18);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 421 to 431
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 432 to 442
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xe0) << 16);
	output[39] = (double)(value >> 21);
}

static void unpack_40_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 40 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 440 bits of data.
	// Input must be at least 56 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 400 to 410
	value = ((uint32_t)input[50] << 24) |
			((uint32_t)(input[51] & 0xe0) << 16);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 411 to 421
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)(input[52] & 0xfc) << 19);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 422 to 432
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0x80) << 14);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 433 to 443
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xf0) << 17);
	output[39] = (double)(value >> 21);
}

static void unpack_40_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 40 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 440 bits of data.
	// Input must be at least 56 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 400 to 410
	value = ((uint32_t)input[50] << 24) |
			((uint32_t)(input[51] & 0xe0) << 16);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 411 to 421
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)(input[52] & 0xfc) << 19);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 422 to 432
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0x80) << 14);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 433 to 443
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xf0) << 17);
	output[39] = (double)(value >> 21);
}

static void unpack_40_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 40 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 440 bits of data.
	// Input must be at least 56 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 401 to 411
	value = ((uint32_t)(input[50] & 0x7f) << 25) |
			((uint32_t)(input[51] & 0xf0) << 17);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 412 to 422
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)(input[52] & 0xfe) << 20);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 423 to 433
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xc0) << 15);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 434 to 444
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xf8) << 18);
	output[39] = (double)(value >> 21);
}

static void unpack_40_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 40 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 440 bits of data.
	// Input must be at least 56 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 401 to 411
	value = ((uint32_t)(input[50] & 0x7f) << 25) |
			((uint32_t)(input[51] & 0xf0) << 17);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 412 to 422
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)(input[52] & 0xfe) << 20);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 423 to 433
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xc0) << 15);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 434 to 444
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xf8) << 18);
	output[39] = (double)(value >> 21);
}

static void unpack_40_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 40 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 440 bits of data.
	// Input must be at least 56 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 402 to 412
	value = ((uint32_t)(input[50] & 0x3f) << 26) |
			((uint32_t)(input[51] & 0xf8) << 18);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 413 to 423
	value = ((uint32_t)(input[51] & 0x07) << 29) |
			((uint32_t)input[52] << 21);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 424 to 434
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xe0) << 16);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 435 to 445
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)(input[55] & 0xfc) << 19);
	output[39] = (double)(value >> 21);
}

static void unpack_40_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 40 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 440 bits of data.
	// Input must be at least 56 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 402 to 412
	value = ((uint32_t)(input[50] & 0x3f) << 26) |
			((uint32_t)(input[51] & 0xf8) << 18);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 413 to 423
	value = ((uint32_t)(input[51] & 0x07) << 29) |
			((uint32_t)input[52] << 21);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 424 to 434
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xe0) << 16);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 435 to 445
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)(input[55] & 0xfc) << 19);
	output[39] = (double)(value >> 21);
}

static void unpack_40_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 40 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 440 bits of data.
	// Input must be at least 56 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 403 to 413
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)(input[51] & 0xfc) << 19);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 414 to 424
	value = ((uint32_t)(input[51] & 0x03) << 30) |
			((uint32_t)input[52] << 22) |
			((uint32_t)(input[53] & 0x80) << 14);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 425 to 435
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)(input[54] & 0xf0) << 17);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 436 to 446
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)(input[55] & 0xfe) << 20);
	output[39] = (double)(value >> 21);
}

static void unpack_40_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 40 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 440 bits of data.
	// Input must be at least 56 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 403 to 413
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)(input[51] & 0xfc) << 19);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 414 to 424
	value = ((uint32_t)(input[51] & 0x03) << 30) |
			((uint32_t)input[52] << 22) |
			((uint32_t)(input[53] & 0x80) << 14);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 425 to 435
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)(input[54] & 0xf0) << 17);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 436 to 446
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)(input[55] & 0xfe) << 20);
	output[39] = (double)(value >> 21);
}

static void unpack_41_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 41 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 451 bits of data.
	// Input must be at least 57 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 396 to 406
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)(input[50] & 0xfe) << 20);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 407 to 417
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xc0) << 15);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 418 to 428
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xf8) << 18);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 429 to 439
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 440 to 450
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xe0) << 16);
	output[40] = (double)(value >> 21);
}

static void unpack_41_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 41 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 451 bits of data.
	// Input must be at least 57 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 396 to 406
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)(input[50] & 0xfe) << 20);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 407 to 417
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xc0) << 15);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 418 to 428
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xf8) << 18);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 429 to 439
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 440 to 450
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xe0) << 16);
	output[40] = (double)(value >> 21);
}

static void unpack_41_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 41 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 451 bits of data.
	// Input must be at least 57 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 397 to 407
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 408 to 418
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xe0) << 16);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 419 to 429
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)(input[53] & 0xfc) << 19);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 430 to 440
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0x80) << 14);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 441 to 451
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)(input[56] & 0xf0) << 17);
	output[40] = (double)(value >> 21);
}

static void unpack_41_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 41 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 451 bits of data.
	// Input must be at least 57 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 397 to 407
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 408 to 418
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xe0) << 16);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 419 to 429
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)(input[53] & 0xfc) << 19);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 430 to 440
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0x80) << 14);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 441 to 451
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)(input[56] & 0xf0) << 17);
	output[40] = (double)(value >> 21);
}

static void unpack_41_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 41 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 451 bits of data.
	// Input must be at least 57 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 398 to 408
	value = ((uint32_t)(input[49] & 0x03) << 30) |
			((uint32_t)input[50] << 22) |
			((uint32_t)(input[51] & 0x80) << 14);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 409 to 419
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xf0) << 17);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 420 to 430
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)(input[53] & 0xfe) << 20);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 431 to 441
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xc0) << 15);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 442 to 452
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xf8) << 18);
	output[40] = (double)(value >> 21);
}

static void unpack_41_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 41 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 451 bits of data.
	// Input must be at least 57 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 398 to 408
	value = ((uint32_t)(input[49] & 0x03) << 30) |
			((uint32_t)input[50] << 22) |
			((uint32_t)(input[51] & 0x80) << 14);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 409 to 419
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xf0) << 17);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 420 to 430
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)(input[53] & 0xfe) << 20);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 431 to 441
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xc0) << 15);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 442 to 452
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xf8) << 18);
	output[40] = (double)(value >> 21);
}

static void unpack_41_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 41 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 451 bits of data.
	// Input must be at least 57 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 399 to 409
	value = ((uint32_t)(input[49] & 0x01) << 31) |
			((uint32_t)input[50] << 23) |
			((uint32_t)(input[51] & 0xc0) << 15);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 410 to 420
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xf8) << 18);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 421 to 431
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 432 to 442
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xe0) << 16);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 443 to 453
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)(input[56] & 0xfc) << 19);
	output[40] = (double)(value >> 21);
}

static void unpack_41_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 41 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 451 bits of data.
	// Input must be at least 57 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 399 to 409
	value = ((uint32_t)(input[49] & 0x01) << 31) |
			((uint32_t)input[50] << 23) |
			((uint32_t)(input[51] & 0xc0) << 15);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 410 to 420
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xf8) << 18);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 421 to 431
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 432 to 442
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xe0) << 16);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 443 to 453
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)(input[56] & 0xfc) << 19);
	output[40] = (double)(value >> 21);
}

static void unpack_41_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 41 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 451 bits of data.
	// Input must be at least 57 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 400 to 410
	value = ((uint32_t)input[50] << 24) |
			((uint32_t)(input[51] & 0xe0) << 16);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 411 to 421
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)(input[52] & 0xfc) << 19);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 422 to 432
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0x80) << 14);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 433 to 443
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xf0) << 17);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 444 to 454
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)(input[56] & 0xfe) << 20);
	output[40] = (double)(value >> 21);
}

static void unpack_41_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 41 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 451 bits of data.
	// Input must be at least 57 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 400 to 410
	value = ((uint32_t)input[50] << 24) |
			((uint32_t)(input[51] & 0xe0) << 16);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 411 to 421
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)(input[52] & 0xfc) << 19);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 422 to 432
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0x80) << 14);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 433 to 443
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xf0) << 17);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 444 to 454
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)(input[56] & 0xfe) << 20);
	output[40] = (double)(value >> 21);
}

static void unpack_41_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 41 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 451 bits of data.
	// Input must be at least 57 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 401 to 411
	value = ((uint32_t)(input[50] & 0x7f) << 25) |
			((uint32_t)(input[51] & 0xf0) << 17);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 412 to 422
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)(input[52] & 0xfe) << 20);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 423 to 433
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xc0) << 15);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 434 to 444
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xf8) << 18);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 445 to 455
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21);
	output[40] = (double)(value >> 21);
}

static void unpack_41_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 41 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 451 bits of data.
	// Input must be at least 57 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 401 to 411
	value = ((uint32_t)(input[50] & 0x7f) << 25) |
			((uint32_t)(input[51] & 0xf0) << 17);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 412 to 422
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)(input[52] & 0xfe) << 20);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 423 to 433
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xc0) << 15);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 434 to 444
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xf8) << 18);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 445 to 455
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21);
	output[40] = (double)(value >> 21);
}

static void unpack_41_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 41 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 451 bits of data.
	// Input must be at least 58 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 402 to 412
	value = ((uint32_t)(input[50] & 0x3f) << 26) |
			((uint32_t)(input[51] & 0xf8) << 18);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 413 to 423
	value = ((uint32_t)(input[51] & 0x07) << 29) |
			((uint32_t)input[52] << 21);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 424 to 434
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xe0) << 16);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 435 to 445
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)(input[55] & 0xfc) << 19);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 446 to 456
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0x80) << 14);
	output[40] = (double)(value >> 21);
}

static void unpack_41_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 41 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 451 bits of data.
	// Input must be at least 58 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 402 to 412
	value = ((uint32_t)(input[50] & 0x3f) << 26) |
			((uint32_t)(input[51] & 0xf8) << 18);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 413 to 423
	value = ((uint32_t)(input[51] & 0x07) << 29) |
			((uint32_t)input[52] << 21);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 424 to 434
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xe0) << 16);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 435 to 445
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)(input[55] & 0xfc) << 19);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 446 to 456
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0x80) << 14);
	output[40] = (double)(value >> 21);
}

static void unpack_41_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 41 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 451 bits of data.
	// Input must be at least 58 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 403 to 413
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)(input[51] & 0xfc) << 19);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 414 to 424
	value = ((uint32_t)(input[51] & 0x03) << 30) |
			((uint32_t)input[52] << 22) |
			((uint32_t)(input[53] & 0x80) << 14);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 425 to 435
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)(input[54] & 0xf0) << 17);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 436 to 446
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)(input[55] & 0xfe) << 20);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 447 to 457
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xc0) << 15);
	output[40] = (double)(value >> 21);
}

static void unpack_41_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 41 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 451 bits of data.
	// Input must be at least 58 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 403 to 413
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)(input[51] & 0xfc) << 19);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 414 to 424
	value = ((uint32_t)(input[51] & 0x03) << 30) |
			((uint32_t)input[52] << 22) |
			((uint32_t)(input[53] & 0x80) << 14);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 425 to 435
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)(input[54] & 0xf0) << 17);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 436 to 446
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)(input[55] & 0xfe) << 20);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 447 to 457
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xc0) << 15);
	output[40] = (double)(value >> 21);
}

static void unpack_42_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 42 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 462 bits of data.
	// Input must be at least 58 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 396 to 406
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)(input[50] & 0xfe) << 20);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 407 to 417
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xc0) << 15);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 418 to 428
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xf8) << 18);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 429 to 439
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 440 to 450
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xe0) << 16);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 451 to 461
	value = ((uint32_t)(input[56] & 0x1f) << 27) |
			((uint32_t)(input[57] & 0xfc) << 19);
	output[41] = (double)(value >> 21);
}

static void unpack_42_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 42 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 462 bits of data.
	// Input must be at least 58 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 396 to 406
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)(input[50] & 0xfe) << 20);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 407 to 417
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xc0) << 15);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 418 to 428
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xf8) << 18);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 429 to 439
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 440 to 450
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xe0) << 16);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 451 to 461
	value = ((uint32_t)(input[56] & 0x1f) << 27) |
			((uint32_t)(input[57] & 0xfc) << 19);
	output[41] = (double)(value >> 21);
}

static void unpack_42_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 42 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 462 bits of data.
	// Input must be at least 58 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 397 to 407
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 408 to 418
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xe0) << 16);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 419 to 429
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)(input[53] & 0xfc) << 19);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 430 to 440
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0x80) << 14);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 441 to 451
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)(input[56] & 0xf0) << 17);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 452 to 462
	value = ((uint32_t)(input[56] & 0x0f) << 28) |
			((uint32_t)(input[57] & 0xfe) << 20);
	output[41] = (double)(value >> 21);
}

static void unpack_42_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 42 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 462 bits of data.
	// Input must be at least 58 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 397 to 407
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 408 to 418
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xe0) << 16);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 419 to 429
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)(input[53] & 0xfc) << 19);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 430 to 440
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0x80) << 14);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 441 to 451
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)(input[56] & 0xf0) << 17);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 452 to 462
	value = ((uint32_t)(input[56] & 0x0f) << 28) |
			((uint32_t)(input[57] & 0xfe) << 20);
	output[41] = (double)(value >> 21);
}

static void unpack_42_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 42 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 462 bits of data.
	// Input must be at least 58 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 398 to 408
	value = ((uint32_t)(input[49] & 0x03) << 30) |
			((uint32_t)input[50] << 22) |
			((uint32_t)(input[51] & 0x80) << 14);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 409 to 419
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xf0) << 17);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 420 to 430
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)(input[53] & 0xfe) << 20);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 431 to 441
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xc0) << 15);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 442 to 452
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xf8) << 18);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 453 to 463
	value = ((uint32_t)(input[56] & 0x07) << 29) |
			((uint32_t)input[57] << 21);
	output[41] = (double)(value >> 21);
}

static void unpack_42_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 42 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 462 bits of data.
	// Input must be at least 58 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 398 to 408
	value = ((uint32_t)(input[49] & 0x03) << 30) |
			((uint32_t)input[50] << 22) |
			((uint32_t)(input[51] & 0x80) << 14);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 409 to 419
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xf0) << 17);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 420 to 430
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)(input[53] & 0xfe) << 20);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 431 to 441
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xc0) << 15);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 442 to 452
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xf8) << 18);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 453 to 463
	value = ((uint32_t)(input[56] & 0x07) << 29) |
			((uint32_t)input[57] << 21);
	output[41] = (double)(value >> 21);
}

static void unpack_42_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 42 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 462 bits of data.
	// Input must be at least 59 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 399 to 409
	value = ((uint32_t)(input[49] & 0x01) << 31) |
			((uint32_t)input[50] << 23) |
			((uint32_t)(input[51] & 0xc0) << 15);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 410 to 420
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xf8) << 18);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 421 to 431
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 432 to 442
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xe0) << 16);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 443 to 453
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)(input[56] & 0xfc) << 19);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 454 to 464
	value = ((uint32_t)(input[56] & 0x03) << 30) |
			((uint32_t)input[57] << 22) |
			((uint32_t)(input[58] & 0x80) << 14);
	output[41] = (double)(value >> 21);
}

static void unpack_42_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 42 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 462 bits of data.
	// Input must be at least 59 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 399 to 409
	value = ((uint32_t)(input[49] & 0x01) << 31) |
			((uint32_t)input[50] << 23) |
			((uint32_t)(input[51] & 0xc0) << 15);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 410 to 420
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xf8) << 18);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 421 to 431
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 432 to 442
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xe0) << 16);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 443 to 453
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)(input[56] & 0xfc) << 19);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 454 to 464
	value = ((uint32_t)(input[56] & 0x03) << 30) |
			((uint32_t)input[57] << 22) |
			((uint32_t)(input[58] & 0x80) << 14);
	output[41] = (double)(value >> 21);
}

static void unpack_42_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 42 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 462 bits of data.
	// Input must be at least 59 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 400 to 410
	value = ((uint32_t)input[50] << 24) |
			((uint32_t)(input[51] & 0xe0) << 16);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 411 to 421
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)(input[52] & 0xfc) << 19);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 422 to 432
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0x80) << 14);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 433 to 443
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xf0) << 17);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 444 to 454
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)(input[56] & 0xfe) << 20);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 455 to 465
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xc0) << 15);
	output[41] = (double)(value >> 21);
}

static void unpack_42_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 42 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 462 bits of data.
	// Input must be at least 59 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 400 to 410
	value = ((uint32_t)input[50] << 24) |
			((uint32_t)(input[51] & 0xe0) << 16);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 411 to 421
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)(input[52] & 0xfc) << 19);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 422 to 432
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0x80) << 14);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 433 to 443
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xf0) << 17);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 444 to 454
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)(input[56] & 0xfe) << 20);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 455 to 465
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xc0) << 15);
	output[41] = (double)(value >> 21);
}

static void unpack_42_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 42 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 462 bits of data.
	// Input must be at least 59 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 401 to 411
	value = ((uint32_t)(input[50] & 0x7f) << 25) |
			((uint32_t)(input[51] & 0xf0) << 17);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 412 to 422
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)(input[52] & 0xfe) << 20);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 423 to 433
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xc0) << 15);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 434 to 444
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xf8) << 18);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 445 to 455
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 456 to 466
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xe0) << 16);
	output[41] = (double)(value >> 21);
}

static void unpack_42_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 42 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 462 bits of data.
	// Input must be at least 59 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 401 to 411
	value = ((uint32_t)(input[50] & 0x7f) << 25) |
			((uint32_t)(input[51] & 0xf0) << 17);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 412 to 422
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)(input[52] & 0xfe) << 20);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 423 to 433
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xc0) << 15);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 434 to 444
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xf8) << 18);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 445 to 455
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 456 to 466
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xe0) << 16);
	output[41] = (double)(value >> 21);
}

static void unpack_42_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 42 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 462 bits of data.
	// Input must be at least 59 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 402 to 412
	value = ((uint32_t)(input[50] & 0x3f) << 26) |
			((uint32_t)(input[51] & 0xf8) << 18);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 413 to 423
	value = ((uint32_t)(input[51] & 0x07) << 29) |
			((uint32_t)input[52] << 21);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 424 to 434
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xe0) << 16);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 435 to 445
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)(input[55] & 0xfc) << 19);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 446 to 456
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0x80) << 14);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 457 to 467
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)(input[58] & 0xf0) << 17);
	output[41] = (double)(value >> 21);
}

static void unpack_42_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 42 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 462 bits of data.
	// Input must be at least 59 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 402 to 412
	value = ((uint32_t)(input[50] & 0x3f) << 26) |
			((uint32_t)(input[51] & 0xf8) << 18);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 413 to 423
	value = ((uint32_t)(input[51] & 0x07) << 29) |
			((uint32_t)input[52] << 21);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 424 to 434
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xe0) << 16);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 435 to 445
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)(input[55] & 0xfc) << 19);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 446 to 456
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0x80) << 14);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 457 to 467
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)(input[58] & 0xf0) << 17);
	output[41] = (double)(value >> 21);
}

static void unpack_42_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 42 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 462 bits of data.
	// Input must be at least 59 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 403 to 413
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)(input[51] & 0xfc) << 19);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 414 to 424
	value = ((uint32_t)(input[51] & 0x03) << 30) |
			((uint32_t)input[52] << 22) |
			((uint32_t)(input[53] & 0x80) << 14);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 425 to 435
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)(input[54] & 0xf0) << 17);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 436 to 446
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)(input[55] & 0xfe) << 20);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 447 to 457
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xc0) << 15);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 458 to 468
	value = ((uint32_t)(input[57] & 0x3f) << 26) |
			((uint32_t)(input[58] & 0xf8) << 18);
	output[41] = (double)(value >> 21);
}

static void unpack_42_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 42 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 462 bits of data.
	// Input must be at least 59 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 403 to 413
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)(input[51] & 0xfc) << 19);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 414 to 424
	value = ((uint32_t)(input[51] & 0x03) << 30) |
			((uint32_t)input[52] << 22) |
			((uint32_t)(input[53] & 0x80) << 14);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 425 to 435
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)(input[54] & 0xf0) << 17);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 436 to 446
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)(input[55] & 0xfe) << 20);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 447 to 457
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xc0) << 15);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 458 to 468
	value = ((uint32_t)(input[57] & 0x3f) << 26) |
			((uint32_t)(input[58] & 0xf8) << 18);
	output[41] = (double)(value >> 21);
}

static void unpack_43_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 43 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 473 bits of data.
	// Input must be at least 60 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 396 to 406
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)(input[50] & 0xfe) << 20);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 407 to 417
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xc0) << 15);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 418 to 428
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xf8) << 18);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 429 to 439
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 440 to 450
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xe0) << 16);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 451 to 461
	value = ((uint32_t)(input[56] & 0x1f) << 27) |
			((uint32_t)(input[57] & 0xfc) << 19);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 462 to 472
	value = ((uint32_t)(input[57] & 0x03) << 30) |
			((uint32_t)input[58] << 22) |
			((uint32_t)(input[59] & 0x80) << 14);
	output[42] = (double)(value >> 21);
}

static void unpack_43_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 43 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 473 bits of data.
	// Input must be at least 60 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 396 to 406
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)(input[50] & 0xfe) << 20);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 407 to 417
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xc0) << 15);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 418 to 428
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xf8) << 18);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 429 to 439
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 440 to 450
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xe0) << 16);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 451 to 461
	value = ((uint32_t)(input[56] & 0x1f) << 27) |
			((uint32_t)(input[57] & 0xfc) << 19);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 462 to 472
	value = ((uint32_t)(input[57] & 0x03) << 30) |
			((uint32_t)input[58] << 22) |
			((uint32_t)(input[59] & 0x80) << 14);
	output[42] = (double)(value >> 21);
}

static void unpack_43_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 43 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 473 bits of data.
	// Input must be at least 60 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 397 to 407
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 408 to 418
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xe0) << 16);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 419 to 429
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)(input[53] & 0xfc) << 19);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 430 to 440
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0x80) << 14);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 441 to 451
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)(input[56] & 0xf0) << 17);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 452 to 462
	value = ((uint32_t)(input[56] & 0x0f) << 28) |
			((uint32_t)(input[57] & 0xfe) << 20);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 463 to 473
	value = ((uint32_t)(input[57] & 0x01) << 31) |
			((uint32_t)input[58] << 23) |
			((uint32_t)(input[59] & 0xc0) << 15);
	output[42] = (double)(value >> 21);
}

static void unpack_43_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 43 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 473 bits of data.
	// Input must be at least 60 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 397 to 407
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 408 to 418
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xe0) << 16);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 419 to 429
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)(input[53] & 0xfc) << 19);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 430 to 440
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0x80) << 14);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 441 to 451
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)(input[56] & 0xf0) << 17);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 452 to 462
	value = ((uint32_t)(input[56] & 0x0f) << 28) |
			((uint32_t)(input[57] & 0xfe) << 20);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 463 to 473
	value = ((uint32_t)(input[57] & 0x01) << 31) |
			((uint32_t)input[58] << 23) |
			((uint32_t)(input[59] & 0xc0) << 15);
	output[42] = (double)(value >> 21);
}

static void unpack_43_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 43 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 473 bits of data.
	// Input must be at least 60 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 398 to 408
	value = ((uint32_t)(input[49] & 0x03) << 30) |
			((uint32_t)input[50] << 22) |
			((uint32_t)(input[51] & 0x80) << 14);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 409 to 419
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xf0) << 17);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 420 to 430
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)(input[53] & 0xfe) << 20);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 431 to 441
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xc0) << 15);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 442 to 452
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xf8) << 18);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 453 to 463
	value = ((uint32_t)(input[56] & 0x07) << 29) |
			((uint32_t)input[57] << 21);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 464 to 474
	value = ((uint32_t)input[58] << 24) |
			((uint32_t)(input[59] & 0xe0) << 16);
	output[42] = (double)(value >> 21);
}

static void unpack_43_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 43 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 473 bits of data.
	// Input must be at least 60 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 398 to 408
	value = ((uint32_t)(input[49] & 0x03) << 30) |
			((uint32_t)input[50] << 22) |
			((uint32_t)(input[51] & 0x80) << 14);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 409 to 419
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xf0) << 17);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 420 to 430
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)(input[53] & 0xfe) << 20);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 431 to 441
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xc0) << 15);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 442 to 452
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xf8) << 18);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 453 to 463
	value = ((uint32_t)(input[56] & 0x07) << 29) |
			((uint32_t)input[57] << 21);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 464 to 474
	value = ((uint32_t)input[58] << 24) |
			((uint32_t)(input[59] & 0xe0) << 16);
	output[42] = (double)(value >> 21);
}

static void unpack_43_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 43 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 473 bits of data.
	// Input must be at least 60 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 399 to 409
	value = ((uint32_t)(input[49] & 0x01) << 31) |
			((uint32_t)input[50] << 23) |
			((uint32_t)(input[51] & 0xc0) << 15);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 410 to 420
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xf8) << 18);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 421 to 431
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 432 to 442
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xe0) << 16);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 443 to 453
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)(input[56] & 0xfc) << 19);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 454 to 464
	value = ((uint32_t)(input[56] & 0x03) << 30) |
			((uint32_t)input[57] << 22) |
			((uint32_t)(input[58] & 0x80) << 14);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 465 to 475
	value = ((uint32_t)(input[58] & 0x7f) << 25) |
			((uint32_t)(input[59] & 0xf0) << 17);
	output[42] = (double)(value >> 21);
}

static void unpack_43_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 43 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 473 bits of data.
	// Input must be at least 60 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 399 to 409
	value = ((uint32_t)(input[49] & 0x01) << 31) |
			((uint32_t)input[50] << 23) |
			((uint32_t)(input[51] & 0xc0) << 15);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 410 to 420
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xf8) << 18);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 421 to 431
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 432 to 442
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xe0) << 16);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 443 to 453
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)(input[56] & 0xfc) << 19);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 454 to 464
	value = ((uint32_t)(input[56] & 0x03) << 30) |
			((uint32_t)input[57] << 22) |
			((uint32_t)(input[58] & 0x80) << 14);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 465 to 475
	value = ((uint32_t)(input[58] & 0x7f) << 25) |
			((uint32_t)(input[59] & 0xf0) << 17);
	output[42] = (double)(value >> 21);
}

static void unpack_43_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 43 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 473 bits of data.
	// Input must be at least 60 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 400 to 410
	value = ((uint32_t)input[50] << 24) |
			((uint32_t)(input[51] & 0xe0) << 16);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 411 to 421
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)(input[52] & 0xfc) << 19);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 422 to 432
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0x80) << 14);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 433 to 443
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xf0) << 17);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 444 to 454
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)(input[56] & 0xfe) << 20);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 455 to 465
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xc0) << 15);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 466 to 476
	value = ((uint32_t)(input[58] & 0x3f) << 26) |
			((uint32_t)(input[59] & 0xf8) << 18);
	output[42] = (double)(value >> 21);
}

static void unpack_43_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 43 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 473 bits of data.
	// Input must be at least 60 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 400 to 410
	value = ((uint32_t)input[50] << 24) |
			((uint32_t)(input[51] & 0xe0) << 16);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 411 to 421
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)(input[52] & 0xfc) << 19);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 422 to 432
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0x80) << 14);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 433 to 443
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xf0) << 17);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 444 to 454
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)(input[56] & 0xfe) << 20);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 455 to 465
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xc0) << 15);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 466 to 476
	value = ((uint32_t)(input[58] & 0x3f) << 26) |
			((uint32_t)(input[59] & 0xf8) << 18);
	output[42] = (double)(value >> 21);
}

static void unpack_43_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 43 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 473 bits of data.
	// Input must be at least 60 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 401 to 411
	value = ((uint32_t)(input[50] & 0x7f) << 25) |
			((uint32_t)(input[51] & 0xf0) << 17);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 412 to 422
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)(input[52] & 0xfe) << 20);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 423 to 433
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xc0) << 15);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 434 to 444
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xf8) << 18);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 445 to 455
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 456 to 466
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xe0) << 16);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 467 to 477
	value = ((uint32_t)(input[58] & 0x1f) << 27) |
			((uint32_t)(input[59] & 0xfc) << 19);
	output[42] = (double)(value >> 21);
}

static void unpack_43_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 43 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 473 bits of data.
	// Input must be at least 60 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 401 to 411
	value = ((uint32_t)(input[50] & 0x7f) << 25) |
			((uint32_t)(input[51] & 0xf0) << 17);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 412 to 422
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)(input[52] & 0xfe) << 20);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 423 to 433
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xc0) << 15);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 434 to 444
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xf8) << 18);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 445 to 455
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 456 to 466
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xe0) << 16);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 467 to 477
	value = ((uint32_t)(input[58] & 0x1f) << 27) |
			((uint32_t)(input[59] & 0xfc) << 19);
	output[42] = (double)(value >> 21);
}

static void unpack_43_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 43 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 473 bits of data.
	// Input must be at least 60 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 402 to 412
	value = ((uint32_t)(input[50] & 0x3f) << 26) |
			((uint32_t)(input[51] & 0xf8) << 18);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 413 to 423
	value = ((uint32_t)(input[51] & 0x07) << 29) |
			((uint32_t)input[52] << 21);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 424 to 434
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xe0) << 16);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 435 to 445
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)(input[55] & 0xfc) << 19);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 446 to 456
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0x80) << 14);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 457 to 467
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)(input[58] & 0xf0) << 17);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 468 to 478
	value = ((uint32_t)(input[58] & 0x0f) << 28) |
			((uint32_t)(input[59] & 0xfe) << 20);
	output[42] = (double)(value >> 21);
}

static void unpack_43_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 43 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 473 bits of data.
	// Input must be at least 60 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 402 to 412
	value = ((uint32_t)(input[50] & 0x3f) << 26) |
			((uint32_t)(input[51] & 0xf8) << 18);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 413 to 423
	value = ((uint32_t)(input[51] & 0x07) << 29) |
			((uint32_t)input[52] << 21);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 424 to 434
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xe0) << 16);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 435 to 445
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)(input[55] & 0xfc) << 19);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 446 to 456
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0x80) << 14);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 457 to 467
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)(input[58] & 0xf0) << 17);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 468 to 478
	value = ((uint32_t)(input[58] & 0x0f) << 28) |
			((uint32_t)(input[59] & 0xfe) << 20);
	output[42] = (double)(value >> 21);
}

static void unpack_43_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 43 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 473 bits of data.
	// Input must be at least 60 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 403 to 413
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)(input[51] & 0xfc) << 19);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 414 to 424
	value = ((uint32_t)(input[51] & 0x03) << 30) |
			((uint32_t)input[52] << 22) |
			((uint32_t)(input[53] & 0x80) << 14);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 425 to 435
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)(input[54] & 0xf0) << 17);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 436 to 446
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)(input[55] & 0xfe) << 20);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 447 to 457
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xc0) << 15);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 458 to 468
	value = ((uint32_t)(input[57] & 0x3f) << 26) |
			((uint32_t)(input[58] & 0xf8) << 18);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 469 to 479
	value = ((uint32_t)(input[58] & 0x07) << 29) |
			((uint32_t)input[59] << 21);
	output[42] = (double)(value >> 21);
}

static void unpack_43_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 43 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 473 bits of data.
	// Input must be at least 60 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 403 to 413
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)(input[51] & 0xfc) << 19);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 414 to 424
	value = ((uint32_t)(input[51] & 0x03) << 30) |
			((uint32_t)input[52] << 22) |
			((uint32_t)(input[53] & 0x80) << 14);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 425 to 435
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)(input[54] & 0xf0) << 17);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 436 to 446
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)(input[55] & 0xfe) << 20);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 447 to 457
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xc0) << 15);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 458 to 468
	value = ((uint32_t)(input[57] & 0x3f) << 26) |
			((uint32_t)(input[58] & 0xf8) << 18);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 469 to 479
	value = ((uint32_t)(input[58] & 0x07) << 29) |
			((uint32_t)input[59] << 21);
	output[42] = (double)(value >> 21);
}

static void unpack_44_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 44 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 484 bits of data.
	// Input must be at least 61 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 396 to 406
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)(input[50] & 0xfe) << 20);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 407 to 417
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xc0) << 15);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 418 to 428
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xf8) << 18);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 429 to 439
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 440 to 450
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xe0) << 16);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 451 to 461
	value = ((uint32_t)(input[56] & 0x1f) << 27) |
			((uint32_t)(input[57] & 0xfc) << 19);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 462 to 472
	value = ((uint32_t)(input[57] & 0x03) << 30) |
			((uint32_t)input[58] << 22) |
			((uint32_t)(input[59] & 0x80) << 14);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 473 to 483
	value = ((uint32_t)(input[59] & 0x7f) << 25) |
			((uint32_t)(input[60] & 0xf0) << 17);
	output[43] = (double)(value >> 21);
}

static void unpack_44_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 44 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 484 bits of data.
	// Input must be at least 61 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 396 to 406
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)(input[50] & 0xfe) << 20);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 407 to 417
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xc0) << 15);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 418 to 428
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xf8) << 18);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 429 to 439
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 440 to 450
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xe0) << 16);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 451 to 461
	value = ((uint32_t)(input[56] & 0x1f) << 27) |
			((uint32_t)(input[57] & 0xfc) << 19);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 462 to 472
	value = ((uint32_t)(input[57] & 0x03) << 30) |
			((uint32_t)input[58] << 22) |
			((uint32_t)(input[59] & 0x80) << 14);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 473 to 483
	value = ((uint32_t)(input[59] & 0x7f) << 25) |
			((uint32_t)(input[60] & 0xf0) << 17);
	output[43] = (double)(value >> 21);
}

static void unpack_44_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 44 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 484 bits of data.
	// Input must be at least 61 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 397 to 407
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 408 to 418
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xe0) << 16);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 419 to 429
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)(input[53] & 0xfc) << 19);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 430 to 440
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0x80) << 14);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 441 to 451
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)(input[56] & 0xf0) << 17);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 452 to 462
	value = ((uint32_t)(input[56] & 0x0f) << 28) |
			((uint32_t)(input[57] & 0xfe) << 20);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 463 to 473
	value = ((uint32_t)(input[57] & 0x01) << 31) |
			((uint32_t)input[58] << 23) |
			((uint32_t)(input[59] & 0xc0) << 15);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 474 to 484
	value = ((uint32_t)(input[59] & 0x3f) << 26) |
			((uint32_t)(input[60] & 0xf8) << 18);
	output[43] = (double)(value >> 21);
}

static void unpack_44_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 44 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 484 bits of data.
	// Input must be at least 61 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 397 to 407
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 408 to 418
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xe0) << 16);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 419 to 429
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)(input[53] & 0xfc) << 19);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 430 to 440
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0x80) << 14);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 441 to 451
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)(input[56] & 0xf0) << 17);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 452 to 462
	value = ((uint32_t)(input[56] & 0x0f) << 28) |
			((uint32_t)(input[57] & 0xfe) << 20);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 463 to 473
	value = ((uint32_t)(input[57] & 0x01) << 31) |
			((uint32_t)input[58] << 23) |
			((uint32_t)(input[59] & 0xc0) << 15);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 474 to 484
	value = ((uint32_t)(input[59] & 0x3f) << 26) |
			((uint32_t)(input[60] & 0xf8) << 18);
	output[43] = (double)(value >> 21);
}

static void unpack_44_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 44 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 484 bits of data.
	// Input must be at least 61 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 398 to 408
	value = ((uint32_t)(input[49] & 0x03) << 30) |
			((uint32_t)input[50] << 22) |
			((uint32_t)(input[51] & 0x80) << 14);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 409 to 419
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xf0) << 17);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 420 to 430
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)(input[53] & 0xfe) << 20);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 431 to 441
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xc0) << 15);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 442 to 452
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xf8) << 18);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 453 to 463
	value = ((uint32_t)(input[56] & 0x07) << 29) |
			((uint32_t)input[57] << 21);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 464 to 474
	value = ((uint32_t)input[58] << 24) |
			((uint32_t)(input[59] & 0xe0) << 16);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 475 to 485
	value = ((uint32_t)(input[59] & 0x1f) << 27) |
			((uint32_t)(input[60] & 0xfc) << 19);
	output[43] = (double)(value >> 21);
}

static void unpack_44_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 44 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 484 bits of data.
	// Input must be at least 61 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 398 to 408
	value = ((uint32_t)(input[49] & 0x03) << 30) |
			((uint32_t)input[50] << 22) |
			((uint32_t)(input[51] & 0x80) << 14);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 409 to 419
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xf0) << 17);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 420 to 430
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)(input[53] & 0xfe) << 20);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 431 to 441
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xc0) << 15);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 442 to 452
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xf8) << 18);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 453 to 463
	value = ((uint32_t)(input[56] & 0x07) << 29) |
			((uint32_t)input[57] << 21);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 464 to 474
	value = ((uint32_t)input[58] << 24) |
			((uint32_t)(input[59] & 0xe0) << 16);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 475 to 485
	value = ((uint32_t)(input[59] & 0x1f) << 27) |
			((uint32_t)(input[60] & 0xfc) << 19);
	output[43] = (double)(value >> 21);
}

static void unpack_44_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 44 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 484 bits of data.
	// Input must be at least 61 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 399 to 409
	value = ((uint32_t)(input[49] & 0x01) << 31) |
			((uint32_t)input[50] << 23) |
			((uint32_t)(input[51] & 0xc0) << 15);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 410 to 420
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xf8) << 18);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 421 to 431
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 432 to 442
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xe0) << 16);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 443 to 453
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)(input[56] & 0xfc) << 19);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 454 to 464
	value = ((uint32_t)(input[56] & 0x03) << 30) |
			((uint32_t)input[57] << 22) |
			((uint32_t)(input[58] & 0x80) << 14);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 465 to 475
	value = ((uint32_t)(input[58] & 0x7f) << 25) |
			((uint32_t)(input[59] & 0xf0) << 17);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 476 to 486
	value = ((uint32_t)(input[59] & 0x0f) << 28) |
			((uint32_t)(input[60] & 0xfe) << 20);
	output[43] = (double)(value >> 21);
}

static void unpack_44_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 44 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 484 bits of data.
	// Input must be at least 61 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 399 to 409
	value = ((uint32_t)(input[49] & 0x01) << 31) |
			((uint32_t)input[50] << 23) |
			((uint32_t)(input[51] & 0xc0) << 15);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 410 to 420
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xf8) << 18);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 421 to 431
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 432 to 442
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xe0) << 16);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 443 to 453
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)(input[56] & 0xfc) << 19);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 454 to 464
	value = ((uint32_t)(input[56] & 0x03) << 30) |
			((uint32_t)input[57] << 22) |
			((uint32_t)(input[58] & 0x80) << 14);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 465 to 475
	value = ((uint32_t)(input[58] & 0x7f) << 25) |
			((uint32_t)(input[59] & 0xf0) << 17);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 476 to 486
	value = ((uint32_t)(input[59] & 0x0f) << 28) |
			((uint32_t)(input[60] & 0xfe) << 20);
	output[43] = (double)(value >> 21);
}

static void unpack_44_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 44 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 484 bits of data.
	// Input must be at least 61 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 400 to 410
	value = ((uint32_t)input[50] << 24) |
			((uint32_t)(input[51] & 0xe0) << 16);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 411 to 421
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)(input[52] & 0xfc) << 19);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 422 to 432
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0x80) << 14);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 433 to 443
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xf0) << 17);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 444 to 454
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)(input[56] & 0xfe) << 20);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 455 to 465
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xc0) << 15);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 466 to 476
	value = ((uint32_t)(input[58] & 0x3f) << 26) |
			((uint32_t)(input[59] & 0xf8) << 18);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 477 to 487
	value = ((uint32_t)(input[59] & 0x07) << 29) |
			((uint32_t)input[60] << 21);
	output[43] = (double)(value >> 21);
}

static void unpack_44_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 44 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 484 bits of data.
	// Input must be at least 61 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 400 to 410
	value = ((uint32_t)input[50] << 24) |
			((uint32_t)(input[51] & 0xe0) << 16);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 411 to 421
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)(input[52] & 0xfc) << 19);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 422 to 432
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0x80) << 14);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 433 to 443
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xf0) << 17);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 444 to 454
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)(input[56] & 0xfe) << 20);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 455 to 465
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xc0) << 15);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 466 to 476
	value = ((uint32_t)(input[58] & 0x3f) << 26) |
			((uint32_t)(input[59] & 0xf8) << 18);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 477 to 487
	value = ((uint32_t)(input[59] & 0x07) << 29) |
			((uint32_t)input[60] << 21);
	output[43] = (double)(value >> 21);
}

static void unpack_44_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 44 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 484 bits of data.
	// Input must be at least 62 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 401 to 411
	value = ((uint32_t)(input[50] & 0x7f) << 25) |
			((uint32_t)(input[51] & 0xf0) << 17);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 412 to 422
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)(input[52] & 0xfe) << 20);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 423 to 433
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xc0) << 15);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 434 to 444
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xf8) << 18);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 445 to 455
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 456 to 466
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xe0) << 16);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 467 to 477
	value = ((uint32_t)(input[58] & 0x1f) << 27) |
			((uint32_t)(input[59] & 0xfc) << 19);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 478 to 488
	value = ((uint32_t)(input[59] & 0x03) << 30) |
			((uint32_t)input[60] << 22) |
			((uint32_t)(input[61] & 0x80) << 14);
	output[43] = (double)(value >> 21);
}

static void unpack_44_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 44 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 484 bits of data.
	// Input must be at least 62 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 401 to 411
	value = ((uint32_t)(input[50] & 0x7f) << 25) |
			((uint32_t)(input[51] & 0xf0) << 17);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 412 to 422
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)(input[52] & 0xfe) << 20);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 423 to 433
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xc0) << 15);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 434 to 444
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xf8) << 18);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 445 to 455
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 456 to 466
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xe0) << 16);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 467 to 477
	value = ((uint32_t)(input[58] & 0x1f) << 27) |
			((uint32_t)(input[59] & 0xfc) << 19);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 478 to 488
	value = ((uint32_t)(input[59] & 0x03) << 30) |
			((uint32_t)input[60] << 22) |
			((uint32_t)(input[61] & 0x80) << 14);
	output[43] = (double)(value >> 21);
}

static void unpack_44_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 44 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 484 bits of data.
	// Input must be at least 62 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 402 to 412
	value = ((uint32_t)(input[50] & 0x3f) << 26) |
			((uint32_t)(input[51] & 0xf8) << 18);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 413 to 423
	value = ((uint32_t)(input[51] & 0x07) << 29) |
			((uint32_t)input[52] << 21);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 424 to 434
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xe0) << 16);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 435 to 445
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)(input[55] & 0xfc) << 19);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 446 to 456
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0x80) << 14);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 457 to 467
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)(input[58] & 0xf0) << 17);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 468 to 478
	value = ((uint32_t)(input[58] & 0x0f) << 28) |
			((uint32_t)(input[59] & 0xfe) << 20);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 479 to 489
	value = ((uint32_t)(input[59] & 0x01) << 31) |
			((uint32_t)input[60] << 23) |
			((uint32_t)(input[61] & 0xc0) << 15);
	output[43] = (double)(value >> 21);
}

static void unpack_44_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 44 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 484 bits of data.
	// Input must be at least 62 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 402 to 412
	value = ((uint32_t)(input[50] & 0x3f) << 26) |
			((uint32_t)(input[51] & 0xf8) << 18);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 413 to 423
	value = ((uint32_t)(input[51] & 0x07) << 29) |
			((uint32_t)input[52] << 21);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 424 to 434
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xe0) << 16);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 435 to 445
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)(input[55] & 0xfc) << 19);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 446 to 456
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0x80) << 14);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 457 to 467
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)(input[58] & 0xf0) << 17);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 468 to 478
	value = ((uint32_t)(input[58] & 0x0f) << 28) |
			((uint32_t)(input[59] & 0xfe) << 20);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 479 to 489
	value = ((uint32_t)(input[59] & 0x01) << 31) |
			((uint32_t)input[60] << 23) |
			((uint32_t)(input[61] & 0xc0) << 15);
	output[43] = (double)(value >> 21);
}

static void unpack_44_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 44 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 484 bits of data.
	// Input must be at least 62 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 403 to 413
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)(input[51] & 0xfc) << 19);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 414 to 424
	value = ((uint32_t)(input[51] & 0x03) << 30) |
			((uint32_t)input[52] << 22) |
			((uint32_t)(input[53] & 0x80) << 14);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 425 to 435
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)(input[54] & 0xf0) << 17);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 436 to 446
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)(input[55] & 0xfe) << 20);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 447 to 457
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xc0) << 15);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 458 to 468
	value = ((uint32_t)(input[57] & 0x3f) << 26) |
			((uint32_t)(input[58] & 0xf8) << 18);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 469 to 479
	value = ((uint32_t)(input[58] & 0x07) << 29) |
			((uint32_t)input[59] << 21);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 480 to 490
	value = ((uint32_t)input[60] << 24) |
			((uint32_t)(input[61] & 0xe0) << 16);
	output[43] = (double)(value >> 21);
}

static void unpack_44_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 44 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 484 bits of data.
	// Input must be at least 62 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 403 to 413
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)(input[51] & 0xfc) << 19);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 414 to 424
	value = ((uint32_t)(input[51] & 0x03) << 30) |
			((uint32_t)input[52] << 22) |
			((uint32_t)(input[53] & 0x80) << 14);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 425 to 435
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)(input[54] & 0xf0) << 17);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 436 to 446
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)(input[55] & 0xfe) << 20);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 447 to 457
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xc0) << 15);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 458 to 468
	value = ((uint32_t)(input[57] & 0x3f) << 26) |
			((uint32_t)(input[58] & 0xf8) << 18);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 469 to 479
	value = ((uint32_t)(input[58] & 0x07) << 29) |
			((uint32_t)input[59] << 21);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 480 to 490
	value = ((uint32_t)input[60] << 24) |
			((uint32_t)(input[61] & 0xe0) << 16);
	output[43] = (double)(value >> 21);
}

static void unpack_45_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 45 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 495 bits of data.
	// Input must be at least 62 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 396 to 406
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)(input[50] & 0xfe) << 20);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 407 to 417
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xc0) << 15);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 418 to 428
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xf8) << 18);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 429 to 439
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 440 to 450
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xe0) << 16);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 451 to 461
	value = ((uint32_t)(input[56] & 0x1f) << 27) |
			((uint32_t)(input[57] & 0xfc) << 19);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 462 to 472
	value = ((uint32_t)(input[57] & 0x03) << 30) |
			((uint32_t)input[58] << 22) |
			((uint32_t)(input[59] & 0x80) << 14);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 473 to 483
	value = ((uint32_t)(input[59] & 0x7f) << 25) |
			((uint32_t)(input[60] & 0xf0) << 17);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 484 to 494
	value = ((uint32_t)(input[60] & 0x0f) << 28) |
			((uint32_t)(input[61] & 0xfe) << 20);
	output[44] = (double)(value >> 21);
}

static void unpack_45_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 45 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 495 bits of data.
	// Input must be at least 62 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 396 to 406
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)(input[50] & 0xfe) << 20);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 407 to 417
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xc0) << 15);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 418 to 428
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xf8) << 18);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 429 to 439
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 440 to 450
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xe0) << 16);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 451 to 461
	value = ((uint32_t)(input[56] & 0x1f) << 27) |
			((uint32_t)(input[57] & 0xfc) << 19);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 462 to 472
	value = ((uint32_t)(input[57] & 0x03) << 30) |
			((uint32_t)input[58] << 22) |
			((uint32_t)(input[59] & 0x80) << 14);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 473 to 483
	value = ((uint32_t)(input[59] & 0x7f) << 25) |
			((uint32_t)(input[60] & 0xf0) << 17);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 484 to 494
	value = ((uint32_t)(input[60] & 0x0f) << 28) |
			((uint32_t)(input[61] & 0xfe) << 20);
	output[44] = (double)(value >> 21);
}

static void unpack_45_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 45 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 495 bits of data.
	// Input must be at least 62 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 397 to 407
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 408 to 418
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xe0) << 16);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 419 to 429
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)(input[53] & 0xfc) << 19);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 430 to 440
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0x80) << 14);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 441 to 451
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)(input[56] & 0xf0) << 17);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 452 to 462
	value = ((uint32_t)(input[56] & 0x0f) << 28) |
			((uint32_t)(input[57] & 0xfe) << 20);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 463 to 473
	value = ((uint32_t)(input[57] & 0x01) << 31) |
			((uint32_t)input[58] << 23) |
			((uint32_t)(input[59] & 0xc0) << 15);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 474 to 484
	value = ((uint32_t)(input[59] & 0x3f) << 26) |
			((uint32_t)(input[60] & 0xf8) << 18);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 485 to 495
	value = ((uint32_t)(input[60] & 0x07) << 29) |
			((uint32_t)input[61] << 21);
	output[44] = (double)(value >> 21);
}

static void unpack_45_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 45 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 495 bits of data.
	// Input must be at least 62 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 397 to 407
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 408 to 418
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xe0) << 16);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 419 to 429
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)(input[53] & 0xfc) << 19);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 430 to 440
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0x80) << 14);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 441 to 451
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)(input[56] & 0xf0) << 17);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 452 to 462
	value = ((uint32_t)(input[56] & 0x0f) << 28) |
			((uint32_t)(input[57] & 0xfe) << 20);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 463 to 473
	value = ((uint32_t)(input[57] & 0x01) << 31) |
			((uint32_t)input[58] << 23) |
			((uint32_t)(input[59] & 0xc0) << 15);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 474 to 484
	value = ((uint32_t)(input[59] & 0x3f) << 26) |
			((uint32_t)(input[60] & 0xf8) << 18);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 485 to 495
	value = ((uint32_t)(input[60] & 0x07) << 29) |
			((uint32_t)input[61] << 21);
	output[44] = (double)(value >> 21);
}

static void unpack_45_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 45 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 398 to 408
	value = ((uint32_t)(input[49] & 0x03) << 30) |
			((uint32_t)input[50] << 22) |
			((uint32_t)(input[51] & 0x80) << 14);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 409 to 419
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xf0) << 17);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 420 to 430
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)(input[53] & 0xfe) << 20);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 431 to 441
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xc0) << 15);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 442 to 452
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xf8) << 18);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 453 to 463
	value = ((uint32_t)(input[56] & 0x07) << 29) |
			((uint32_t)input[57] << 21);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 464 to 474
	value = ((uint32_t)input[58] << 24) |
			((uint32_t)(input[59] & 0xe0) << 16);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 475 to 485
	value = ((uint32_t)(input[59] & 0x1f) << 27) |
			((uint32_t)(input[60] & 0xfc) << 19);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 486 to 496
	value = ((uint32_t)(input[60] & 0x03) << 30) |
			((uint32_t)input[61] << 22) |
			((uint32_t)(input[62] & 0x80) << 14);
	output[44] = (double)(value >> 21);
}

static void unpack_45_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 45 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 398 to 408
	value = ((uint32_t)(input[49] & 0x03) << 30) |
			((uint32_t)input[50] << 22) |
			((uint32_t)(input[51] & 0x80) << 14);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 409 to 419
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xf0) << 17);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 420 to 430
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)(input[53] & 0xfe) << 20);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 431 to 441
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xc0) << 15);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 442 to 452
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xf8) << 18);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 453 to 463
	value = ((uint32_t)(input[56] & 0x07) << 29) |
			((uint32_t)input[57] << 21);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 464 to 474
	value = ((uint32_t)input[58] << 24) |
			((uint32_t)(input[59] & 0xe0) << 16);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 475 to 485
	value = ((uint32_t)(input[59] & 0x1f) << 27) |
			((uint32_t)(input[60] & 0xfc) << 19);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 486 to 496
	value = ((uint32_t)(input[60] & 0x03) << 30) |
			((uint32_t)input[61] << 22) |
			((uint32_t)(input[62] & 0x80) << 14);
	output[44] = (double)(value >> 21);
}

static void unpack_45_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 45 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 399 to 409
	value = ((uint32_t)(input[49] & 0x01) << 31) |
			((uint32_t)input[50] << 23) |
			((uint32_t)(input[51] & 0xc0) << 15);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 410 to 420
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xf8) << 18);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 421 to 431
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 432 to 442
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xe0) << 16);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 443 to 453
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)(input[56] & 0xfc) << 19);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 454 to 464
	value = ((uint32_t)(input[56] & 0x03) << 30) |
			((uint32_t)input[57] << 22) |
			((uint32_t)(input[58] & 0x80) << 14);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 465 to 475
	value = ((uint32_t)(input[58] & 0x7f) << 25) |
			((uint32_t)(input[59] & 0xf0) << 17);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 476 to 486
	value = ((uint32_t)(input[59] & 0x0f) << 28) |
			((uint32_t)(input[60] & 0xfe) << 20);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 487 to 497
	value = ((uint32_t)(input[60] & 0x01) << 31) |
			((uint32_t)input[61] << 23) |
			((uint32_t)(input[62] & 0xc0) << 15);
	output[44] = (double)(value >> 21);
}

static void unpack_45_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 45 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 399 to 409
	value = ((uint32_t)(input[49] & 0x01) << 31) |
			((uint32_t)input[50] << 23) |
			((uint32_t)(input[51] & 0xc0) << 15);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 410 to 420
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xf8) << 18);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 421 to 431
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 432 to 442
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xe0) << 16);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 443 to 453
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)(input[56] & 0xfc) << 19);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 454 to 464
	value = ((uint32_t)(input[56] & 0x03) << 30) |
			((uint32_t)input[57] << 22) |
			((uint32_t)(input[58] & 0x80) << 14);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 465 to 475
	value = ((uint32_t)(input[58] & 0x7f) << 25) |
			((uint32_t)(input[59] & 0xf0) << 17);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 476 to 486
	value = ((uint32_t)(input[59] & 0x0f) << 28) |
			((uint32_t)(input[60] & 0xfe) << 20);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 487 to 497
	value = ((uint32_t)(input[60] & 0x01) << 31) |
			((uint32_t)input[61] << 23) |
			((uint32_t)(input[62] & 0xc0) << 15);
	output[44] = (double)(value >> 21);
}

static void unpack_45_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 45 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 400 to 410
	value = ((uint32_t)input[50] << 24) |
			((uint32_t)(input[51] & 0xe0) << 16);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 411 to 421
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)(input[52] & 0xfc) << 19);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 422 to 432
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0x80) << 14);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 433 to 443
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xf0) << 17);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 444 to 454
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)(input[56] & 0xfe) << 20);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 455 to 465
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xc0) << 15);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 466 to 476
	value = ((uint32_t)(input[58] & 0x3f) << 26) |
			((uint32_t)(input[59] & 0xf8) << 18);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 477 to 487
	value = ((uint32_t)(input[59] & 0x07) << 29) |
			((uint32_t)input[60] << 21);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 488 to 498
	value = ((uint32_t)input[61] << 24) |
			((uint32_t)(input[62] & 0xe0) << 16);
	output[44] = (double)(value >> 21);
}

static void unpack_45_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 45 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 400 to 410
	value = ((uint32_t)input[50] << 24) |
			((uint32_t)(input[51] & 0xe0) << 16);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 411 to 421
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)(input[52] & 0xfc) << 19);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 422 to 432
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0x80) << 14);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 433 to 443
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xf0) << 17);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 444 to 454
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)(input[56] & 0xfe) << 20);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 455 to 465
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xc0) << 15);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 466 to 476
	value = ((uint32_t)(input[58] & 0x3f) << 26) |
			((uint32_t)(input[59] & 0xf8) << 18);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 477 to 487
	value = ((uint32_t)(input[59] & 0x07) << 29) |
			((uint32_t)input[60] << 21);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 488 to 498
	value = ((uint32_t)input[61] << 24) |
			((uint32_t)(input[62] & 0xe0) << 16);
	output[44] = (double)(value >> 21);
}

static void unpack_45_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 45 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 401 to 411
	value = ((uint32_t)(input[50] & 0x7f) << 25) |
			((uint32_t)(input[51] & 0xf0) << 17);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 412 to 422
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)(input[52] & 0xfe) << 20);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 423 to 433
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xc0) << 15);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 434 to 444
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xf8) << 18);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 445 to 455
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 456 to 466
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xe0) << 16);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 467 to 477
	value = ((uint32_t)(input[58] & 0x1f) << 27) |
			((uint32_t)(input[59] & 0xfc) << 19);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 478 to 488
	value = ((uint32_t)(input[59] & 0x03) << 30) |
			((uint32_t)input[60] << 22) |
			((uint32_t)(input[61] & 0x80) << 14);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 489 to 499
	value = ((uint32_t)(input[61] & 0x7f) << 25) |
			((uint32_t)(input[62] & 0xf0) << 17);
	output[44] = (double)(value >> 21);
}

static void unpack_45_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 45 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 401 to 411
	value = ((uint32_t)(input[50] & 0x7f) << 25) |
			((uint32_t)(input[51] & 0xf0) << 17);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 412 to 422
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)(input[52] & 0xfe) << 20);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 423 to 433
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xc0) << 15);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 434 to 444
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xf8) << 18);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 445 to 455
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 456 to 466
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xe0) << 16);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 467 to 477
	value = ((uint32_t)(input[58] & 0x1f) << 27) |
			((uint32_t)(input[59] & 0xfc) << 19);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 478 to 488
	value = ((uint32_t)(input[59] & 0x03) << 30) |
			((uint32_t)input[60] << 22) |
			((uint32_t)(input[61] & 0x80) << 14);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 489 to 499
	value = ((uint32_t)(input[61] & 0x7f) << 25) |
			((uint32_t)(input[62] & 0xf0) << 17);
	output[44] = (double)(value >> 21);
}

static void unpack_45_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 45 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 402 to 412
	value = ((uint32_t)(input[50] & 0x3f) << 26) |
			((uint32_t)(input[51] & 0xf8) << 18);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 413 to 423
	value = ((uint32_t)(input[51] & 0x07) << 29) |
			((uint32_t)input[52] << 21);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 424 to 434
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xe0) << 16);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 435 to 445
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)(input[55] & 0xfc) << 19);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 446 to 456
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0x80) << 14);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 457 to 467
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)(input[58] & 0xf0) << 17);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 468 to 478
	value = ((uint32_t)(input[58] & 0x0f) << 28) |
			((uint32_t)(input[59] & 0xfe) << 20);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 479 to 489
	value = ((uint32_t)(input[59] & 0x01) << 31) |
			((uint32_t)input[60] << 23) |
			((uint32_t)(input[61] & 0xc0) << 15);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 490 to 500
	value = ((uint32_t)(input[61] & 0x3f) << 26) |
			((uint32_t)(input[62] & 0xf8) << 18);
	output[44] = (double)(value >> 21);
}

static void unpack_45_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 45 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 402 to 412
	value = ((uint32_t)(input[50] & 0x3f) << 26) |
			((uint32_t)(input[51] & 0xf8) << 18);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 413 to 423
	value = ((uint32_t)(input[51] & 0x07) << 29) |
			((uint32_t)input[52] << 21);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 424 to 434
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xe0) << 16);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 435 to 445
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)(input[55] & 0xfc) << 19);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 446 to 456
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0x80) << 14);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 457 to 467
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)(input[58] & 0xf0) << 17);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 468 to 478
	value = ((uint32_t)(input[58] & 0x0f) << 28) |
			((uint32_t)(input[59] & 0xfe) << 20);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 479 to 489
	value = ((uint32_t)(input[59] & 0x01) << 31) |
			((uint32_t)input[60] << 23) |
			((uint32_t)(input[61] & 0xc0) << 15);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 490 to 500
	value = ((uint32_t)(input[61] & 0x3f) << 26) |
			((uint32_t)(input[62] & 0xf8) << 18);
	output[44] = (double)(value >> 21);
}

static void unpack_45_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 45 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 403 to 413
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)(input[51] & 0xfc) << 19);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 414 to 424
	value = ((uint32_t)(input[51] & 0x03) << 30) |
			((uint32_t)input[52] << 22) |
			((uint32_t)(input[53] & 0x80) << 14);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 425 to 435
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)(input[54] & 0xf0) << 17);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 436 to 446
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)(input[55] & 0xfe) << 20);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 447 to 457
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xc0) << 15);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 458 to 468
	value = ((uint32_t)(input[57] & 0x3f) << 26) |
			((uint32_t)(input[58] & 0xf8) << 18);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 469 to 479
	value = ((uint32_t)(input[58] & 0x07) << 29) |
			((uint32_t)input[59] << 21);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 480 to 490
	value = ((uint32_t)input[60] << 24) |
			((uint32_t)(input[61] & 0xe0) << 16);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 491 to 501
	value = ((uint32_t)(input[61] & 0x1f) << 27) |
			((uint32_t)(input[62] & 0xfc) << 19);
	output[44] = (double)(value >> 21);
}

static void unpack_45_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 45 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 495 bits of data.
	// Input must be at least 63 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 403 to 413
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)(input[51] & 0xfc) << 19);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 414 to 424
	value = ((uint32_t)(input[51] & 0x03) << 30) |
			((uint32_t)input[52] << 22) |
			((uint32_t)(input[53] & 0x80) << 14);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 425 to 435
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)(input[54] & 0xf0) << 17);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 436 to 446
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)(input[55] & 0xfe) << 20);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 447 to 457
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xc0) << 15);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 458 to 468
	value = ((uint32_t)(input[57] & 0x3f) << 26) |
			((uint32_t)(input[58] & 0xf8) << 18);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 469 to 479
	value = ((uint32_t)(input[58] & 0x07) << 29) |
			((uint32_t)input[59] << 21);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 480 to 490
	value = ((uint32_t)input[60] << 24) |
			((uint32_t)(input[61] & 0xe0) << 16);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 491 to 501
	value = ((uint32_t)(input[61] & 0x1f) << 27) |
			((uint32_t)(input[62] & 0xfc) << 19);
	output[44] = (double)(value >> 21);
}

static void unpack_46_11bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 46 values into output
	// Values are 11-bit signed
	// Skip 0 bits, read 506 bits of data.
	// Input must be at least 64 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 396 to 406
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)(input[50] & 0xfe) << 20);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 407 to 417
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xc0) << 15);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 418 to 428
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xf8) << 18);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 429 to 439
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 440 to 450
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xe0) << 16);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 451 to 461
	value = ((uint32_t)(input[56] & 0x1f) << 27) |
			((uint32_t)(input[57] & 0xfc) << 19);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 462 to 472
	value = ((uint32_t)(input[57] & 0x03) << 30) |
			((uint32_t)input[58] << 22) |
			((uint32_t)(input[59] & 0x80) << 14);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 473 to 483
	value = ((uint32_t)(input[59] & 0x7f) << 25) |
			((uint32_t)(input[60] & 0xf0) << 17);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 484 to 494
	value = ((uint32_t)(input[60] & 0x0f) << 28) |
			((uint32_t)(input[61] & 0xfe) << 20);
	output[44] = (double)(value >> 21);

	// read output[45]: from bit 495 to 505
	value = ((uint32_t)(input[61] & 0x01) << 31) |
			((uint32_t)input[62] << 23) |
			((uint32_t)(input[63] & 0xc0) << 15);
	output[45] = (double)(value >> 21);
}

static void unpack_46_11bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 46 values into output
	// Values are 11-bit unsigned
	// Skip 0 bits, read 506 bits of data.
	// Input must be at least 64 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 10
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xe0) << 16);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 11 to 21
	value = ((uint32_t)(input[1] & 0x1f) << 27) |
			((uint32_t)(input[2] & 0xfc) << 19);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 22 to 32
	value = ((uint32_t)(input[2] & 0x03) << 30) |
			((uint32_t)input[3] << 22) |
			((uint32_t)(input[4] & 0x80) << 14);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 33 to 43
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xf0) << 17);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 44 to 54
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)(input[6] & 0xfe) << 20);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 55 to 65
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xc0) << 15);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 66 to 76
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xf8) << 18);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 77 to 87
	value = ((uint32_t)(input[9] & 0x07) << 29) |
			((uint32_t)input[10] << 21);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 88 to 98
	value = ((uint32_t)input[11] << 24) |
			((uint32_t)(input[12] & 0xe0) << 16);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 99 to 109
	value = ((uint32_t)(input[12] & 0x1f) << 27) |
			((uint32_t)(input[13] & 0xfc) << 19);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 110 to 120
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0x80) << 14);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 121 to 131
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xf0) << 17);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 132 to 142
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)(input[17] & 0xfe) << 20);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 143 to 153
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xc0) << 15);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 154 to 164
	value = ((uint32_t)(input[19] & 0x3f) << 26) |
			((uint32_t)(input[20] & 0xf8) << 18);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 165 to 175
	value = ((uint32_t)(input[20] & 0x07) << 29) |
			((uint32_t)input[21] << 21);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 176 to 186
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xe0) << 16);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 187 to 197
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)(input[24] & 0xfc) << 19);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 198 to 208
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0x80) << 14);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 209 to 219
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xf0) << 17);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 220 to 230
	value = ((uint32_t)(input[27] & 0x0f) << 28) |
			((uint32_t)(input[28] & 0xfe) << 20);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 231 to 241
	value = ((uint32_t)(input[28] & 0x01) << 31) |
			((uint32_t)input[29] << 23) |
			((uint32_t)(input[30] & 0xc0) << 15);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 242 to 252
	value = ((uint32_t)(input[30] & 0x3f) << 26) |
			((uint32_t)(input[31] & 0xf8) << 18);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 253 to 263
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 264 to 274
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xe0) << 16);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 275 to 285
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)(input[35] & 0xfc) << 19);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 286 to 296
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0x80) << 14);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 297 to 307
	value = ((uint32_t)(input[37] & 0x7f) << 25) |
			((uint32_t)(input[38] & 0xf0) << 17);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 308 to 318
	value = ((uint32_t)(input[38] & 0x0f) << 28) |
			((uint32_t)(input[39] & 0xfe) << 20);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 319 to 329
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xc0) << 15);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 330 to 340
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xf8) << 18);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 341 to 351
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 352 to 362
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xe0) << 16);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 363 to 373
	value = ((uint32_t)(input[45] & 0x1f) << 27) |
			((uint32_t)(input[46] & 0xfc) << 19);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 374 to 384
	value = ((uint32_t)(input[46] & 0x03) << 30) |
			((uint32_t)input[47] << 22) |
			((uint32_t)(input[48] & 0x80) << 14);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 385 to 395
	value = ((uint32_t)(input[48] & 0x7f) << 25) |
			((uint32_t)(input[49] & 0xf0) << 17);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 396 to 406
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)(input[50] & 0xfe) << 20);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 407 to 417
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xc0) << 15);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 418 to 428
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xf8) << 18);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 429 to 439
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 440 to 450
	value = ((uint32_t)input[55] << 24) |
			((uint32_t)(input[56] & 0xe0) << 16);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 451 to 461
	value = ((uint32_t)(input[56] & 0x1f) << 27) |
			((uint32_t)(input[57] & 0xfc) << 19);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 462 to 472
	value = ((uint32_t)(input[57] & 0x03) << 30) |
			((uint32_t)input[58] << 22) |
			((uint32_t)(input[59] & 0x80) << 14);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 473 to 483
	value = ((uint32_t)(input[59] & 0x7f) << 25) |
			((uint32_t)(input[60] & 0xf0) << 17);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 484 to 494
	value = ((uint32_t)(input[60] & 0x0f) << 28) |
			((uint32_t)(input[61] & 0xfe) << 20);
	output[44] = (double)(value >> 21);

	// read output[45]: from bit 495 to 505
	value = ((uint32_t)(input[61] & 0x01) << 31) |
			((uint32_t)input[62] << 23) |
			((uint32_t)(input[63] & 0xc0) << 15);
	output[45] = (double)(value >> 21);
}

static void unpack_46_11bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 46 values into output
	// Values are 11-bit signed
	// Skip 1 bits, read 506 bits of data.
	// Input must be at least 64 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 397 to 407
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 408 to 418
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xe0) << 16);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 419 to 429
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)(input[53] & 0xfc) << 19);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 430 to 440
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0x80) << 14);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 441 to 451
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)(input[56] & 0xf0) << 17);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 452 to 462
	value = ((uint32_t)(input[56] & 0x0f) << 28) |
			((uint32_t)(input[57] & 0xfe) << 20);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 463 to 473
	value = ((uint32_t)(input[57] & 0x01) << 31) |
			((uint32_t)input[58] << 23) |
			((uint32_t)(input[59] & 0xc0) << 15);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 474 to 484
	value = ((uint32_t)(input[59] & 0x3f) << 26) |
			((uint32_t)(input[60] & 0xf8) << 18);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 485 to 495
	value = ((uint32_t)(input[60] & 0x07) << 29) |
			((uint32_t)input[61] << 21);
	output[44] = (double)(value >> 21);

	// read output[45]: from bit 496 to 506
	value = ((uint32_t)input[62] << 24) |
			((uint32_t)(input[63] & 0xe0) << 16);
	output[45] = (double)(value >> 21);
}

static void unpack_46_11bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 46 values into output
	// Values are 11-bit unsigned
	// Skip 1 bits, read 506 bits of data.
	// Input must be at least 64 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 11
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xf0) << 17);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 12 to 22
	value = ((uint32_t)(input[1] & 0x0f) << 28) |
			((uint32_t)(input[2] & 0xfe) << 20);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 23 to 33
	value = ((uint32_t)(input[2] & 0x01) << 31) |
			((uint32_t)input[3] << 23) |
			((uint32_t)(input[4] & 0xc0) << 15);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 34 to 44
	value = ((uint32_t)(input[4] & 0x3f) << 26) |
			((uint32_t)(input[5] & 0xf8) << 18);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 45 to 55
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 56 to 66
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xe0) << 16);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 67 to 77
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)(input[9] & 0xfc) << 19);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 78 to 88
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0x80) << 14);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 89 to 99
	value = ((uint32_t)(input[11] & 0x7f) << 25) |
			((uint32_t)(input[12] & 0xf0) << 17);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 100 to 110
	value = ((uint32_t)(input[12] & 0x0f) << 28) |
			((uint32_t)(input[13] & 0xfe) << 20);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 111 to 121
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xc0) << 15);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 122 to 132
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xf8) << 18);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 133 to 143
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 144 to 154
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xe0) << 16);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 155 to 165
	value = ((uint32_t)(input[19] & 0x1f) << 27) |
			((uint32_t)(input[20] & 0xfc) << 19);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 166 to 176
	value = ((uint32_t)(input[20] & 0x03) << 30) |
			((uint32_t)input[21] << 22) |
			((uint32_t)(input[22] & 0x80) << 14);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 177 to 187
	value = ((uint32_t)(input[22] & 0x7f) << 25) |
			((uint32_t)(input[23] & 0xf0) << 17);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 188 to 198
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)(input[24] & 0xfe) << 20);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 199 to 209
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xc0) << 15);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 210 to 220
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xf8) << 18);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 221 to 231
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 232 to 242
	value = ((uint32_t)input[29] << 24) |
			((uint32_t)(input[30] & 0xe0) << 16);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 243 to 253
	value = ((uint32_t)(input[30] & 0x1f) << 27) |
			((uint32_t)(input[31] & 0xfc) << 19);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 254 to 264
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0x80) << 14);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 265 to 275
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xf0) << 17);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 276 to 286
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)(input[35] & 0xfe) << 20);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 287 to 297
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xc0) << 15);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 298 to 308
	value = ((uint32_t)(input[37] & 0x3f) << 26) |
			((uint32_t)(input[38] & 0xf8) << 18);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 309 to 319
	value = ((uint32_t)(input[38] & 0x07) << 29) |
			((uint32_t)input[39] << 21);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 320 to 330
	value = ((uint32_t)input[40] << 24) |
			((uint32_t)(input[41] & 0xe0) << 16);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 331 to 341
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)(input[42] & 0xfc) << 19);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 342 to 352
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0x80) << 14);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 353 to 363
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xf0) << 17);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 364 to 374
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)(input[46] & 0xfe) << 20);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 375 to 385
	value = ((uint32_t)(input[46] & 0x01) << 31) |
			((uint32_t)input[47] << 23) |
			((uint32_t)(input[48] & 0xc0) << 15);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 386 to 396
	value = ((uint32_t)(input[48] & 0x3f) << 26) |
			((uint32_t)(input[49] & 0xf8) << 18);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 397 to 407
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 408 to 418
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xe0) << 16);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 419 to 429
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)(input[53] & 0xfc) << 19);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 430 to 440
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0x80) << 14);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 441 to 451
	value = ((uint32_t)(input[55] & 0x7f) << 25) |
			((uint32_t)(input[56] & 0xf0) << 17);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 452 to 462
	value = ((uint32_t)(input[56] & 0x0f) << 28) |
			((uint32_t)(input[57] & 0xfe) << 20);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 463 to 473
	value = ((uint32_t)(input[57] & 0x01) << 31) |
			((uint32_t)input[58] << 23) |
			((uint32_t)(input[59] & 0xc0) << 15);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 474 to 484
	value = ((uint32_t)(input[59] & 0x3f) << 26) |
			((uint32_t)(input[60] & 0xf8) << 18);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 485 to 495
	value = ((uint32_t)(input[60] & 0x07) << 29) |
			((uint32_t)input[61] << 21);
	output[44] = (double)(value >> 21);

	// read output[45]: from bit 496 to 506
	value = ((uint32_t)input[62] << 24) |
			((uint32_t)(input[63] & 0xe0) << 16);
	output[45] = (double)(value >> 21);
}

static void unpack_46_11bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 46 values into output
	// Values are 11-bit signed
	// Skip 2 bits, read 506 bits of data.
	// Input must be at least 64 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 398 to 408
	value = ((uint32_t)(input[49] & 0x03) << 30) |
			((uint32_t)input[50] << 22) |
			((uint32_t)(input[51] & 0x80) << 14);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 409 to 419
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xf0) << 17);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 420 to 430
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)(input[53] & 0xfe) << 20);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 431 to 441
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xc0) << 15);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 442 to 452
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xf8) << 18);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 453 to 463
	value = ((uint32_t)(input[56] & 0x07) << 29) |
			((uint32_t)input[57] << 21);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 464 to 474
	value = ((uint32_t)input[58] << 24) |
			((uint32_t)(input[59] & 0xe0) << 16);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 475 to 485
	value = ((uint32_t)(input[59] & 0x1f) << 27) |
			((uint32_t)(input[60] & 0xfc) << 19);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 486 to 496
	value = ((uint32_t)(input[60] & 0x03) << 30) |
			((uint32_t)input[61] << 22) |
			((uint32_t)(input[62] & 0x80) << 14);
	output[44] = (double)(value >> 21);

	// read output[45]: from bit 497 to 507
	value = ((uint32_t)(input[62] & 0x7f) << 25) |
			((uint32_t)(input[63] & 0xf0) << 17);
	output[45] = (double)(value >> 21);
}

static void unpack_46_11bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 46 values into output
	// Values are 11-bit unsigned
	// Skip 2 bits, read 506 bits of data.
	// Input must be at least 64 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 12
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xf8) << 18);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 13 to 23
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 24 to 34
	value = ((uint32_t)input[3] << 24) |
			((uint32_t)(input[4] & 0xe0) << 16);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 35 to 45
	value = ((uint32_t)(input[4] & 0x1f) << 27) |
			((uint32_t)(input[5] & 0xfc) << 19);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 46 to 56
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0x80) << 14);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 57 to 67
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xf0) << 17);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 68 to 78
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)(input[9] & 0xfe) << 20);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 79 to 89
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xc0) << 15);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 90 to 100
	value = ((uint32_t)(input[11] & 0x3f) << 26) |
			((uint32_t)(input[12] & 0xf8) << 18);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 101 to 111
	value = ((uint32_t)(input[12] & 0x07) << 29) |
			((uint32_t)input[13] << 21);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 112 to 122
	value = ((uint32_t)input[14] << 24) |
			((uint32_t)(input[15] & 0xe0) << 16);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 123 to 133
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)(input[16] & 0xfc) << 19);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 134 to 144
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0x80) << 14);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 145 to 155
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xf0) << 17);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 156 to 166
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)(input[20] & 0xfe) << 20);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 167 to 177
	value = ((uint32_t)(input[20] & 0x01) << 31) |
			((uint32_t)input[21] << 23) |
			((uint32_t)(input[22] & 0xc0) << 15);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 178 to 188
	value = ((uint32_t)(input[22] & 0x3f) << 26) |
			((uint32_t)(input[23] & 0xf8) << 18);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 189 to 199
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 200 to 210
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xe0) << 16);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 211 to 221
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)(input[27] & 0xfc) << 19);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 222 to 232
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0x80) << 14);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 233 to 243
	value = ((uint32_t)(input[29] & 0x7f) << 25) |
			((uint32_t)(input[30] & 0xf0) << 17);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 244 to 254
	value = ((uint32_t)(input[30] & 0x0f) << 28) |
			((uint32_t)(input[31] & 0xfe) << 20);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 255 to 265
	value = ((uint32_t)(input[31] & 0x01) << 31) |
			((uint32_t)input[32] << 23) |
			((uint32_t)(input[33] & 0xc0) << 15);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 266 to 276
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xf8) << 18);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 277 to 287
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 288 to 298
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xe0) << 16);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 299 to 309
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)(input[38] & 0xfc) << 19);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 310 to 320
	value = ((uint32_t)(input[38] & 0x03) << 30) |
			((uint32_t)input[39] << 22) |
			((uint32_t)(input[40] & 0x80) << 14);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 321 to 331
	value = ((uint32_t)(input[40] & 0x7f) << 25) |
			((uint32_t)(input[41] & 0xf0) << 17);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 332 to 342
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)(input[42] & 0xfe) << 20);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 343 to 353
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xc0) << 15);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 354 to 364
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xf8) << 18);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 365 to 375
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 376 to 386
	value = ((uint32_t)input[47] << 24) |
			((uint32_t)(input[48] & 0xe0) << 16);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 387 to 397
	value = ((uint32_t)(input[48] & 0x1f) << 27) |
			((uint32_t)(input[49] & 0xfc) << 19);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 398 to 408
	value = ((uint32_t)(input[49] & 0x03) << 30) |
			((uint32_t)input[50] << 22) |
			((uint32_t)(input[51] & 0x80) << 14);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 409 to 419
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xf0) << 17);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 420 to 430
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)(input[53] & 0xfe) << 20);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 431 to 441
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xc0) << 15);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 442 to 452
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xf8) << 18);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 453 to 463
	value = ((uint32_t)(input[56] & 0x07) << 29) |
			((uint32_t)input[57] << 21);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 464 to 474
	value = ((uint32_t)input[58] << 24) |
			((uint32_t)(input[59] & 0xe0) << 16);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 475 to 485
	value = ((uint32_t)(input[59] & 0x1f) << 27) |
			((uint32_t)(input[60] & 0xfc) << 19);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 486 to 496
	value = ((uint32_t)(input[60] & 0x03) << 30) |
			((uint32_t)input[61] << 22) |
			((uint32_t)(input[62] & 0x80) << 14);
	output[44] = (double)(value >> 21);

	// read output[45]: from bit 497 to 507
	value = ((uint32_t)(input[62] & 0x7f) << 25) |
			((uint32_t)(input[63] & 0xf0) << 17);
	output[45] = (double)(value >> 21);
}

static void unpack_46_11bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 46 values into output
	// Values are 11-bit signed
	// Skip 3 bits, read 506 bits of data.
	// Input must be at least 64 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 399 to 409
	value = ((uint32_t)(input[49] & 0x01) << 31) |
			((uint32_t)input[50] << 23) |
			((uint32_t)(input[51] & 0xc0) << 15);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 410 to 420
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xf8) << 18);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 421 to 431
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 432 to 442
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xe0) << 16);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 443 to 453
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)(input[56] & 0xfc) << 19);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 454 to 464
	value = ((uint32_t)(input[56] & 0x03) << 30) |
			((uint32_t)input[57] << 22) |
			((uint32_t)(input[58] & 0x80) << 14);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 465 to 475
	value = ((uint32_t)(input[58] & 0x7f) << 25) |
			((uint32_t)(input[59] & 0xf0) << 17);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 476 to 486
	value = ((uint32_t)(input[59] & 0x0f) << 28) |
			((uint32_t)(input[60] & 0xfe) << 20);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 487 to 497
	value = ((uint32_t)(input[60] & 0x01) << 31) |
			((uint32_t)input[61] << 23) |
			((uint32_t)(input[62] & 0xc0) << 15);
	output[44] = (double)(value >> 21);

	// read output[45]: from bit 498 to 508
	value = ((uint32_t)(input[62] & 0x3f) << 26) |
			((uint32_t)(input[63] & 0xf8) << 18);
	output[45] = (double)(value >> 21);
}

static void unpack_46_11bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 46 values into output
	// Values are 11-bit unsigned
	// Skip 3 bits, read 506 bits of data.
	// Input must be at least 64 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 13
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)(input[1] & 0xfc) << 19);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 14 to 24
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0x80) << 14);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 25 to 35
	value = ((uint32_t)(input[3] & 0x7f) << 25) |
			((uint32_t)(input[4] & 0xf0) << 17);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 36 to 46
	value = ((uint32_t)(input[4] & 0x0f) << 28) |
			((uint32_t)(input[5] & 0xfe) << 20);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 47 to 57
	value = ((uint32_t)(input[5] & 0x01) << 31) |
			((uint32_t)input[6] << 23) |
			((uint32_t)(input[7] & 0xc0) << 15);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 58 to 68
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xf8) << 18);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 69 to 79
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 80 to 90
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xe0) << 16);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 91 to 101
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)(input[12] & 0xfc) << 19);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 102 to 112
	value = ((uint32_t)(input[12] & 0x03) << 30) |
			((uint32_t)input[13] << 22) |
			((uint32_t)(input[14] & 0x80) << 14);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 113 to 123
	value = ((uint32_t)(input[14] & 0x7f) << 25) |
			((uint32_t)(input[15] & 0xf0) << 17);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 124 to 134
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)(input[16] & 0xfe) << 20);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 135 to 145
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xc0) << 15);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 146 to 156
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xf8) << 18);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 157 to 167
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 168 to 178
	value = ((uint32_t)input[21] << 24) |
			((uint32_t)(input[22] & 0xe0) << 16);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 179 to 189
	value = ((uint32_t)(input[22] & 0x1f) << 27) |
			((uint32_t)(input[23] & 0xfc) << 19);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 190 to 200
	value = ((uint32_t)(input[23] & 0x03) << 30) |
			((uint32_t)input[24] << 22) |
			((uint32_t)(input[25] & 0x80) << 14);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 201 to 211
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xf0) << 17);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 212 to 222
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)(input[27] & 0xfe) << 20);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 223 to 233
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xc0) << 15);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 234 to 244
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xf8) << 18);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 245 to 255
	value = ((uint32_t)(input[30] & 0x07) << 29) |
			((uint32_t)input[31] << 21);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 256 to 266
	value = ((uint32_t)input[32] << 24) |
			((uint32_t)(input[33] & 0xe0) << 16);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 267 to 277
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)(input[34] & 0xfc) << 19);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 278 to 288
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0x80) << 14);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 289 to 299
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xf0) << 17);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 300 to 310
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)(input[38] & 0xfe) << 20);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 311 to 321
	value = ((uint32_t)(input[38] & 0x01) << 31) |
			((uint32_t)input[39] << 23) |
			((uint32_t)(input[40] & 0xc0) << 15);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 322 to 332
	value = ((uint32_t)(input[40] & 0x3f) << 26) |
			((uint32_t)(input[41] & 0xf8) << 18);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 333 to 343
	value = ((uint32_t)(input[41] & 0x07) << 29) |
			((uint32_t)input[42] << 21);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 344 to 354
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xe0) << 16);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 355 to 365
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)(input[45] & 0xfc) << 19);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 366 to 376
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0x80) << 14);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 377 to 387
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xf0) << 17);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 388 to 398
	value = ((uint32_t)(input[48] & 0x0f) << 28) |
			((uint32_t)(input[49] & 0xfe) << 20);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 399 to 409
	value = ((uint32_t)(input[49] & 0x01) << 31) |
			((uint32_t)input[50] << 23) |
			((uint32_t)(input[51] & 0xc0) << 15);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 410 to 420
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xf8) << 18);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 421 to 431
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 432 to 442
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xe0) << 16);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 443 to 453
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)(input[56] & 0xfc) << 19);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 454 to 464
	value = ((uint32_t)(input[56] & 0x03) << 30) |
			((uint32_t)input[57] << 22) |
			((uint32_t)(input[58] & 0x80) << 14);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 465 to 475
	value = ((uint32_t)(input[58] & 0x7f) << 25) |
			((uint32_t)(input[59] & 0xf0) << 17);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 476 to 486
	value = ((uint32_t)(input[59] & 0x0f) << 28) |
			((uint32_t)(input[60] & 0xfe) << 20);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 487 to 497
	value = ((uint32_t)(input[60] & 0x01) << 31) |
			((uint32_t)input[61] << 23) |
			((uint32_t)(input[62] & 0xc0) << 15);
	output[44] = (double)(value >> 21);

	// read output[45]: from bit 498 to 508
	value = ((uint32_t)(input[62] & 0x3f) << 26) |
			((uint32_t)(input[63] & 0xf8) << 18);
	output[45] = (double)(value >> 21);
}

static void unpack_46_11bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 46 values into output
	// Values are 11-bit signed
	// Skip 4 bits, read 506 bits of data.
	// Input must be at least 64 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 400 to 410
	value = ((uint32_t)input[50] << 24) |
			((uint32_t)(input[51] & 0xe0) << 16);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 411 to 421
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)(input[52] & 0xfc) << 19);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 422 to 432
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0x80) << 14);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 433 to 443
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xf0) << 17);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 444 to 454
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)(input[56] & 0xfe) << 20);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 455 to 465
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xc0) << 15);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 466 to 476
	value = ((uint32_t)(input[58] & 0x3f) << 26) |
			((uint32_t)(input[59] & 0xf8) << 18);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 477 to 487
	value = ((uint32_t)(input[59] & 0x07) << 29) |
			((uint32_t)input[60] << 21);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 488 to 498
	value = ((uint32_t)input[61] << 24) |
			((uint32_t)(input[62] & 0xe0) << 16);
	output[44] = (double)(value >> 21);

	// read output[45]: from bit 499 to 509
	value = ((uint32_t)(input[62] & 0x1f) << 27) |
			((uint32_t)(input[63] & 0xfc) << 19);
	output[45] = (double)(value >> 21);
}

static void unpack_46_11bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 46 values into output
	// Values are 11-bit unsigned
	// Skip 4 bits, read 506 bits of data.
	// Input must be at least 64 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 14
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)(input[1] & 0xfe) << 20);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 15 to 25
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xc0) << 15);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 26 to 36
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xf8) << 18);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 37 to 47
	value = ((uint32_t)(input[4] & 0x07) << 29) |
			((uint32_t)input[5] << 21);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 48 to 58
	value = ((uint32_t)input[6] << 24) |
			((uint32_t)(input[7] & 0xe0) << 16);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 59 to 69
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)(input[8] & 0xfc) << 19);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 70 to 80
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0x80) << 14);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 81 to 91
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xf0) << 17);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 92 to 102
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)(input[12] & 0xfe) << 20);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 103 to 113
	value = ((uint32_t)(input[12] & 0x01) << 31) |
			((uint32_t)input[13] << 23) |
			((uint32_t)(input[14] & 0xc0) << 15);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 114 to 124
	value = ((uint32_t)(input[14] & 0x3f) << 26) |
			((uint32_t)(input[15] & 0xf8) << 18);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 125 to 135
	value = ((uint32_t)(input[15] & 0x07) << 29) |
			((uint32_t)input[16] << 21);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 136 to 146
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xe0) << 16);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 147 to 157
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)(input[19] & 0xfc) << 19);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 158 to 168
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0x80) << 14);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 169 to 179
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xf0) << 17);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 180 to 190
	value = ((uint32_t)(input[22] & 0x0f) << 28) |
			((uint32_t)(input[23] & 0xfe) << 20);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 191 to 201
	value = ((uint32_t)(input[23] & 0x01) << 31) |
			((uint32_t)input[24] << 23) |
			((uint32_t)(input[25] & 0xc0) << 15);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 202 to 212
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xf8) << 18);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 213 to 223
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 224 to 234
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xe0) << 16);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 235 to 245
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)(input[30] & 0xfc) << 19);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 246 to 256
	value = ((uint32_t)(input[30] & 0x03) << 30) |
			((uint32_t)input[31] << 22) |
			((uint32_t)(input[32] & 0x80) << 14);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 257 to 267
	value = ((uint32_t)(input[32] & 0x7f) << 25) |
			((uint32_t)(input[33] & 0xf0) << 17);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 268 to 278
	value = ((uint32_t)(input[33] & 0x0f) << 28) |
			((uint32_t)(input[34] & 0xfe) << 20);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 279 to 289
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xc0) << 15);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 290 to 300
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xf8) << 18);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 301 to 311
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 312 to 322
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xe0) << 16);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 323 to 333
	value = ((uint32_t)(input[40] & 0x1f) << 27) |
			((uint32_t)(input[41] & 0xfc) << 19);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 334 to 344
	value = ((uint32_t)(input[41] & 0x03) << 30) |
			((uint32_t)input[42] << 22) |
			((uint32_t)(input[43] & 0x80) << 14);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 345 to 355
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xf0) << 17);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 356 to 366
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)(input[45] & 0xfe) << 20);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 367 to 377
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xc0) << 15);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 378 to 388
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xf8) << 18);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 389 to 399
	value = ((uint32_t)(input[48] & 0x07) << 29) |
			((uint32_t)input[49] << 21);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 400 to 410
	value = ((uint32_t)input[50] << 24) |
			((uint32_t)(input[51] & 0xe0) << 16);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 411 to 421
	value = ((uint32_t)(input[51] & 0x1f) << 27) |
			((uint32_t)(input[52] & 0xfc) << 19);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 422 to 432
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0x80) << 14);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 433 to 443
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xf0) << 17);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 444 to 454
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)(input[56] & 0xfe) << 20);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 455 to 465
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xc0) << 15);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 466 to 476
	value = ((uint32_t)(input[58] & 0x3f) << 26) |
			((uint32_t)(input[59] & 0xf8) << 18);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 477 to 487
	value = ((uint32_t)(input[59] & 0x07) << 29) |
			((uint32_t)input[60] << 21);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 488 to 498
	value = ((uint32_t)input[61] << 24) |
			((uint32_t)(input[62] & 0xe0) << 16);
	output[44] = (double)(value >> 21);

	// read output[45]: from bit 499 to 509
	value = ((uint32_t)(input[62] & 0x1f) << 27) |
			((uint32_t)(input[63] & 0xfc) << 19);
	output[45] = (double)(value >> 21);
}

static void unpack_46_11bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 46 values into output
	// Values are 11-bit signed
	// Skip 5 bits, read 506 bits of data.
	// Input must be at least 64 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 401 to 411
	value = ((uint32_t)(input[50] & 0x7f) << 25) |
			((uint32_t)(input[51] & 0xf0) << 17);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 412 to 422
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)(input[52] & 0xfe) << 20);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 423 to 433
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xc0) << 15);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 434 to 444
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xf8) << 18);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 445 to 455
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 456 to 466
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xe0) << 16);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 467 to 477
	value = ((uint32_t)(input[58] & 0x1f) << 27) |
			((uint32_t)(input[59] & 0xfc) << 19);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 478 to 488
	value = ((uint32_t)(input[59] & 0x03) << 30) |
			((uint32_t)input[60] << 22) |
			((uint32_t)(input[61] & 0x80) << 14);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 489 to 499
	value = ((uint32_t)(input[61] & 0x7f) << 25) |
			((uint32_t)(input[62] & 0xf0) << 17);
	output[44] = (double)(value >> 21);

	// read output[45]: from bit 500 to 510
	value = ((uint32_t)(input[62] & 0x0f) << 28) |
			((uint32_t)(input[63] & 0xfe) << 20);
	output[45] = (double)(value >> 21);
}

static void unpack_46_11bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 46 values into output
	// Values are 11-bit unsigned
	// Skip 5 bits, read 506 bits of data.
	// Input must be at least 64 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 15
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 16 to 26
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xe0) << 16);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 27 to 37
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)(input[4] & 0xfc) << 19);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 38 to 48
	value = ((uint32_t)(input[4] & 0x03) << 30) |
			((uint32_t)input[5] << 22) |
			((uint32_t)(input[6] & 0x80) << 14);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 49 to 59
	value = ((uint32_t)(input[6] & 0x7f) << 25) |
			((uint32_t)(input[7] & 0xf0) << 17);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 60 to 70
	value = ((uint32_t)(input[7] & 0x0f) << 28) |
			((uint32_t)(input[8] & 0xfe) << 20);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 71 to 81
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xc0) << 15);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 82 to 92
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xf8) << 18);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 93 to 103
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 104 to 114
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xe0) << 16);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 115 to 125
	value = ((uint32_t)(input[14] & 0x1f) << 27) |
			((uint32_t)(input[15] & 0xfc) << 19);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 126 to 136
	value = ((uint32_t)(input[15] & 0x03) << 30) |
			((uint32_t)input[16] << 22) |
			((uint32_t)(input[17] & 0x80) << 14);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 137 to 147
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xf0) << 17);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 148 to 158
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)(input[19] & 0xfe) << 20);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 159 to 169
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xc0) << 15);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 170 to 180
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xf8) << 18);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 181 to 191
	value = ((uint32_t)(input[22] & 0x07) << 29) |
			((uint32_t)input[23] << 21);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 192 to 202
	value = ((uint32_t)input[24] << 24) |
			((uint32_t)(input[25] & 0xe0) << 16);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 203 to 213
	value = ((uint32_t)(input[25] & 0x1f) << 27) |
			((uint32_t)(input[26] & 0xfc) << 19);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 214 to 224
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0x80) << 14);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 225 to 235
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xf0) << 17);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 236 to 246
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)(input[30] & 0xfe) << 20);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 247 to 257
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xc0) << 15);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 258 to 268
	value = ((uint32_t)(input[32] & 0x3f) << 26) |
			((uint32_t)(input[33] & 0xf8) << 18);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 269 to 279
	value = ((uint32_t)(input[33] & 0x07) << 29) |
			((uint32_t)input[34] << 21);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 280 to 290
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xe0) << 16);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 291 to 301
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)(input[37] & 0xfc) << 19);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 302 to 312
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0x80) << 14);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 313 to 323
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xf0) << 17);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 324 to 334
	value = ((uint32_t)(input[40] & 0x0f) << 28) |
			((uint32_t)(input[41] & 0xfe) << 20);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 335 to 345
	value = ((uint32_t)(input[41] & 0x01) << 31) |
			((uint32_t)input[42] << 23) |
			((uint32_t)(input[43] & 0xc0) << 15);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 346 to 356
	value = ((uint32_t)(input[43] & 0x3f) << 26) |
			((uint32_t)(input[44] & 0xf8) << 18);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 357 to 367
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 368 to 378
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xe0) << 16);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 379 to 389
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)(input[48] & 0xfc) << 19);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 390 to 400
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0x80) << 14);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 401 to 411
	value = ((uint32_t)(input[50] & 0x7f) << 25) |
			((uint32_t)(input[51] & 0xf0) << 17);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 412 to 422
	value = ((uint32_t)(input[51] & 0x0f) << 28) |
			((uint32_t)(input[52] & 0xfe) << 20);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 423 to 433
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xc0) << 15);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 434 to 444
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xf8) << 18);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 445 to 455
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 456 to 466
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xe0) << 16);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 467 to 477
	value = ((uint32_t)(input[58] & 0x1f) << 27) |
			((uint32_t)(input[59] & 0xfc) << 19);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 478 to 488
	value = ((uint32_t)(input[59] & 0x03) << 30) |
			((uint32_t)input[60] << 22) |
			((uint32_t)(input[61] & 0x80) << 14);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 489 to 499
	value = ((uint32_t)(input[61] & 0x7f) << 25) |
			((uint32_t)(input[62] & 0xf0) << 17);
	output[44] = (double)(value >> 21);

	// read output[45]: from bit 500 to 510
	value = ((uint32_t)(input[62] & 0x0f) << 28) |
			((uint32_t)(input[63] & 0xfe) << 20);
	output[45] = (double)(value >> 21);
}

static void unpack_46_11bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 46 values into output
	// Values are 11-bit signed
	// Skip 6 bits, read 506 bits of data.
	// Input must be at least 64 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 402 to 412
	value = ((uint32_t)(input[50] & 0x3f) << 26) |
			((uint32_t)(input[51] & 0xf8) << 18);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 413 to 423
	value = ((uint32_t)(input[51] & 0x07) << 29) |
			((uint32_t)input[52] << 21);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 424 to 434
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xe0) << 16);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 435 to 445
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)(input[55] & 0xfc) << 19);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 446 to 456
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0x80) << 14);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 457 to 467
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)(input[58] & 0xf0) << 17);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 468 to 478
	value = ((uint32_t)(input[58] & 0x0f) << 28) |
			((uint32_t)(input[59] & 0xfe) << 20);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 479 to 489
	value = ((uint32_t)(input[59] & 0x01) << 31) |
			((uint32_t)input[60] << 23) |
			((uint32_t)(input[61] & 0xc0) << 15);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 490 to 500
	value = ((uint32_t)(input[61] & 0x3f) << 26) |
			((uint32_t)(input[62] & 0xf8) << 18);
	output[44] = (double)(value >> 21);

	// read output[45]: from bit 501 to 511
	value = ((uint32_t)(input[62] & 0x07) << 29) |
			((uint32_t)input[63] << 21);
	output[45] = (double)(value >> 21);
}

static void unpack_46_11bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 46 values into output
	// Values are 11-bit unsigned
	// Skip 6 bits, read 506 bits of data.
	// Input must be at least 64 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 16
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0x80) << 14);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 17 to 27
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xf0) << 17);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 28 to 38
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)(input[4] & 0xfe) << 20);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 39 to 49
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xc0) << 15);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 50 to 60
	value = ((uint32_t)(input[6] & 0x3f) << 26) |
			((uint32_t)(input[7] & 0xf8) << 18);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 61 to 71
	value = ((uint32_t)(input[7] & 0x07) << 29) |
			((uint32_t)input[8] << 21);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 72 to 82
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xe0) << 16);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 83 to 93
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)(input[11] & 0xfc) << 19);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 94 to 104
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0x80) << 14);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 105 to 115
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xf0) << 17);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 116 to 126
	value = ((uint32_t)(input[14] & 0x0f) << 28) |
			((uint32_t)(input[15] & 0xfe) << 20);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 127 to 137
	value = ((uint32_t)(input[15] & 0x01) << 31) |
			((uint32_t)input[16] << 23) |
			((uint32_t)(input[17] & 0xc0) << 15);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 138 to 148
	value = ((uint32_t)(input[17] & 0x3f) << 26) |
			((uint32_t)(input[18] & 0xf8) << 18);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 149 to 159
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 160 to 170
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xe0) << 16);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 171 to 181
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)(input[22] & 0xfc) << 19);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 182 to 192
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0x80) << 14);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 193 to 203
	value = ((uint32_t)(input[24] & 0x7f) << 25) |
			((uint32_t)(input[25] & 0xf0) << 17);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 204 to 214
	value = ((uint32_t)(input[25] & 0x0f) << 28) |
			((uint32_t)(input[26] & 0xfe) << 20);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 215 to 225
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xc0) << 15);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 226 to 236
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xf8) << 18);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 237 to 247
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 248 to 258
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xe0) << 16);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 259 to 269
	value = ((uint32_t)(input[32] & 0x1f) << 27) |
			((uint32_t)(input[33] & 0xfc) << 19);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 270 to 280
	value = ((uint32_t)(input[33] & 0x03) << 30) |
			((uint32_t)input[34] << 22) |
			((uint32_t)(input[35] & 0x80) << 14);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 281 to 291
	value = ((uint32_t)(input[35] & 0x7f) << 25) |
			((uint32_t)(input[36] & 0xf0) << 17);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 292 to 302
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)(input[37] & 0xfe) << 20);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 303 to 313
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xc0) << 15);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 314 to 324
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xf8) << 18);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 325 to 335
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 336 to 346
	value = ((uint32_t)input[42] << 24) |
			((uint32_t)(input[43] & 0xe0) << 16);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 347 to 357
	value = ((uint32_t)(input[43] & 0x1f) << 27) |
			((uint32_t)(input[44] & 0xfc) << 19);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 358 to 368
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0x80) << 14);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 369 to 379
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xf0) << 17);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 380 to 390
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)(input[48] & 0xfe) << 20);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 391 to 401
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xc0) << 15);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 402 to 412
	value = ((uint32_t)(input[50] & 0x3f) << 26) |
			((uint32_t)(input[51] & 0xf8) << 18);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 413 to 423
	value = ((uint32_t)(input[51] & 0x07) << 29) |
			((uint32_t)input[52] << 21);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 424 to 434
	value = ((uint32_t)input[53] << 24) |
			((uint32_t)(input[54] & 0xe0) << 16);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 435 to 445
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)(input[55] & 0xfc) << 19);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 446 to 456
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0x80) << 14);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 457 to 467
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)(input[58] & 0xf0) << 17);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 468 to 478
	value = ((uint32_t)(input[58] & 0x0f) << 28) |
			((uint32_t)(input[59] & 0xfe) << 20);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 479 to 489
	value = ((uint32_t)(input[59] & 0x01) << 31) |
			((uint32_t)input[60] << 23) |
			((uint32_t)(input[61] & 0xc0) << 15);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 490 to 500
	value = ((uint32_t)(input[61] & 0x3f) << 26) |
			((uint32_t)(input[62] & 0xf8) << 18);
	output[44] = (double)(value >> 21);

	// read output[45]: from bit 501 to 511
	value = ((uint32_t)(input[62] & 0x07) << 29) |
			((uint32_t)input[63] << 21);
	output[45] = (double)(value >> 21);
}

static void unpack_46_11bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 46 values into output
	// Values are 11-bit signed
	// Skip 7 bits, read 506 bits of data.
	// Input must be at least 65 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 403 to 413
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)(input[51] & 0xfc) << 19);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 414 to 424
	value = ((uint32_t)(input[51] & 0x03) << 30) |
			((uint32_t)input[52] << 22) |
			((uint32_t)(input[53] & 0x80) << 14);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 425 to 435
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)(input[54] & 0xf0) << 17);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 436 to 446
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)(input[55] & 0xfe) << 20);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 447 to 457
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xc0) << 15);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 458 to 468
	value = ((uint32_t)(input[57] & 0x3f) << 26) |
			((uint32_t)(input[58] & 0xf8) << 18);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 469 to 479
	value = ((uint32_t)(input[58] & 0x07) << 29) |
			((uint32_t)input[59] << 21);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 480 to 490
	value = ((uint32_t)input[60] << 24) |
			((uint32_t)(input[61] & 0xe0) << 16);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 491 to 501
	value = ((uint32_t)(input[61] & 0x1f) << 27) |
			((uint32_t)(input[62] & 0xfc) << 19);
	output[44] = (double)(value >> 21);

	// read output[45]: from bit 502 to 512
	value = ((uint32_t)(input[62] & 0x03) << 30) |
			((uint32_t)input[63] << 22) |
			((uint32_t)(input[64] & 0x80) << 14);
	output[45] = (double)(value >> 21);
}

static void unpack_46_11bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 46 values into output
	// Values are 11-bit unsigned
	// Skip 7 bits, read 506 bits of data.
	// Input must be at least 65 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 17
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xc0) << 15);
	output[0] = (double)(value >> 21);

	// read output[1]: from bit 18 to 28
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xf8) << 18);
	output[1] = (double)(value >> 21);

	// read output[2]: from bit 29 to 39
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21);
	output[2] = (double)(value >> 21);

	// read output[3]: from bit 40 to 50
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xe0) << 16);
	output[3] = (double)(value >> 21);

	// read output[4]: from bit 51 to 61
	value = ((uint32_t)(input[6] & 0x1f) << 27) |
			((uint32_t)(input[7] & 0xfc) << 19);
	output[4] = (double)(value >> 21);

	// read output[5]: from bit 62 to 72
	value = ((uint32_t)(input[7] & 0x03) << 30) |
			((uint32_t)input[8] << 22) |
			((uint32_t)(input[9] & 0x80) << 14);
	output[5] = (double)(value >> 21);

	// read output[6]: from bit 73 to 83
	value = ((uint32_t)(input[9] & 0x7f) << 25) |
			((uint32_t)(input[10] & 0xf0) << 17);
	output[6] = (double)(value >> 21);

	// read output[7]: from bit 84 to 94
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)(input[11] & 0xfe) << 20);
	output[7] = (double)(value >> 21);

	// read output[8]: from bit 95 to 105
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xc0) << 15);
	output[8] = (double)(value >> 21);

	// read output[9]: from bit 106 to 116
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xf8) << 18);
	output[9] = (double)(value >> 21);

	// read output[10]: from bit 117 to 127
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21);
	output[10] = (double)(value >> 21);

	// read output[11]: from bit 128 to 138
	value = ((uint32_t)input[16] << 24) |
			((uint32_t)(input[17] & 0xe0) << 16);
	output[11] = (double)(value >> 21);

	// read output[12]: from bit 139 to 149
	value = ((uint32_t)(input[17] & 0x1f) << 27) |
			((uint32_t)(input[18] & 0xfc) << 19);
	output[12] = (double)(value >> 21);

	// read output[13]: from bit 150 to 160
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0x80) << 14);
	output[13] = (double)(value >> 21);

	// read output[14]: from bit 161 to 171
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xf0) << 17);
	output[14] = (double)(value >> 21);

	// read output[15]: from bit 172 to 182
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)(input[22] & 0xfe) << 20);
	output[15] = (double)(value >> 21);

	// read output[16]: from bit 183 to 193
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xc0) << 15);
	output[16] = (double)(value >> 21);

	// read output[17]: from bit 194 to 204
	value = ((uint32_t)(input[24] & 0x3f) << 26) |
			((uint32_t)(input[25] & 0xf8) << 18);
	output[17] = (double)(value >> 21);

	// read output[18]: from bit 205 to 215
	value = ((uint32_t)(input[25] & 0x07) << 29) |
			((uint32_t)input[26] << 21);
	output[18] = (double)(value >> 21);

	// read output[19]: from bit 216 to 226
	value = ((uint32_t)input[27] << 24) |
			((uint32_t)(input[28] & 0xe0) << 16);
	output[19] = (double)(value >> 21);

	// read output[20]: from bit 227 to 237
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)(input[29] & 0xfc) << 19);
	output[20] = (double)(value >> 21);

	// read output[21]: from bit 238 to 248
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0x80) << 14);
	output[21] = (double)(value >> 21);

	// read output[22]: from bit 249 to 259
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xf0) << 17);
	output[22] = (double)(value >> 21);

	// read output[23]: from bit 260 to 270
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)(input[33] & 0xfe) << 20);
	output[23] = (double)(value >> 21);

	// read output[24]: from bit 271 to 281
	value = ((uint32_t)(input[33] & 0x01) << 31) |
			((uint32_t)input[34] << 23) |
			((uint32_t)(input[35] & 0xc0) << 15);
	output[24] = (double)(value >> 21);

	// read output[25]: from bit 282 to 292
	value = ((uint32_t)(input[35] & 0x3f) << 26) |
			((uint32_t)(input[36] & 0xf8) << 18);
	output[25] = (double)(value >> 21);

	// read output[26]: from bit 293 to 303
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21);
	output[26] = (double)(value >> 21);

	// read output[27]: from bit 304 to 314
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xe0) << 16);
	output[27] = (double)(value >> 21);

	// read output[28]: from bit 315 to 325
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)(input[40] & 0xfc) << 19);
	output[28] = (double)(value >> 21);

	// read output[29]: from bit 326 to 336
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0x80) << 14);
	output[29] = (double)(value >> 21);

	// read output[30]: from bit 337 to 347
	value = ((uint32_t)(input[42] & 0x7f) << 25) |
			((uint32_t)(input[43] & 0xf0) << 17);
	output[30] = (double)(value >> 21);

	// read output[31]: from bit 348 to 358
	value = ((uint32_t)(input[43] & 0x0f) << 28) |
			((uint32_t)(input[44] & 0xfe) << 20);
	output[31] = (double)(value >> 21);

	// read output[32]: from bit 359 to 369
	value = ((uint32_t)(input[44] & 0x01) << 31) |
			((uint32_t)input[45] << 23) |
			((uint32_t)(input[46] & 0xc0) << 15);
	output[32] = (double)(value >> 21);

	// read output[33]: from bit 370 to 380
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xf8) << 18);
	output[33] = (double)(value >> 21);

	// read output[34]: from bit 381 to 391
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21);
	output[34] = (double)(value >> 21);

	// read output[35]: from bit 392 to 402
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xe0) << 16);
	output[35] = (double)(value >> 21);

	// read output[36]: from bit 403 to 413
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)(input[51] & 0xfc) << 19);
	output[36] = (double)(value >> 21);

	// read output[37]: from bit 414 to 424
	value = ((uint32_t)(input[51] & 0x03) << 30) |
			((uint32_t)input[52] << 22) |
			((uint32_t)(input[53] & 0x80) << 14);
	output[37] = (double)(value >> 21);

	// read output[38]: from bit 425 to 435
	value = ((uint32_t)(input[53] & 0x7f) << 25) |
			((uint32_t)(input[54] & 0xf0) << 17);
	output[38] = (double)(value >> 21);

	// read output[39]: from bit 436 to 446
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)(input[55] & 0xfe) << 20);
	output[39] = (double)(value >> 21);

	// read output[40]: from bit 447 to 457
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xc0) << 15);
	output[40] = (double)(value >> 21);

	// read output[41]: from bit 458 to 468
	value = ((uint32_t)(input[57] & 0x3f) << 26) |
			((uint32_t)(input[58] & 0xf8) << 18);
	output[41] = (double)(value >> 21);

	// read output[42]: from bit 469 to 479
	value = ((uint32_t)(input[58] & 0x07) << 29) |
			((uint32_t)input[59] << 21);
	output[42] = (double)(value >> 21);

	// read output[43]: from bit 480 to 490
	value = ((uint32_t)input[60] << 24) |
			((uint32_t)(input[61] & 0xe0) << 16);
	output[43] = (double)(value >> 21);

	// read output[44]: from bit 491 to 501
	value = ((uint32_t)(input[61] & 0x1f) << 27) |
			((uint32_t)(input[62] & 0xfc) << 19);
	output[44] = (double)(value >> 21);

	// read output[45]: from bit 502 to 512
	value = ((uint32_t)(input[62] & 0x03) << 30) |
			((uint32_t)input[63] << 22) |
			((uint32_t)(input[64] & 0x80) << 14);
	output[45] = (double)(value >> 21);
}


unpack_func_t unpack_11bit[UNPACK_11BIT_MAX_COUNT][8][2] = {
	{
		{
			unpack_1_11bit_unsigned_0off,
			unpack_1_11bit_signed_0off,
		},
		{
			unpack_1_11bit_unsigned_1off,
			unpack_1_11bit_signed_1off,
		},
		{
			unpack_1_11bit_unsigned_2off,
			unpack_1_11bit_signed_2off,
		},
		{
			unpack_1_11bit_unsigned_3off,
			unpack_1_11bit_signed_3off,
		},
		{
			unpack_1_11bit_unsigned_4off,
			unpack_1_11bit_signed_4off,
		},
		{
			unpack_1_11bit_unsigned_5off,
			unpack_1_11bit_signed_5off,
		},
		{
			unpack_1_11bit_unsigned_6off,
			unpack_1_11bit_signed_6off,
		},
		{
			unpack_1_11bit_unsigned_7off,
			unpack_1_11bit_signed_7off,
		},
	},
	{
		{
			unpack_2_11bit_unsigned_0off,
			unpack_2_11bit_signed_0off,
		},
		{
			unpack_2_11bit_unsigned_1off,
			unpack_2_11bit_signed_1off,
		},
		{
			unpack_2_11bit_unsigned_2off,
			unpack_2_11bit_signed_2off,
		},
		{
			unpack_2_11bit_unsigned_3off,
			unpack_2_11bit_signed_3off,
		},
		{
			unpack_2_11bit_unsigned_4off,
			unpack_2_11bit_signed_4off,
		},
		{
			unpack_2_11bit_unsigned_5off,
			unpack_2_11bit_signed_5off,
		},
		{
			unpack_2_11bit_unsigned_6off,
			unpack_2_11bit_signed_6off,
		},
		{
			unpack_2_11bit_unsigned_7off,
			unpack_2_11bit_signed_7off,
		},
	},
	{
		{
			unpack_3_11bit_unsigned_0off,
			unpack_3_11bit_signed_0off,
		},
		{
			unpack_3_11bit_unsigned_1off,
			unpack_3_11bit_signed_1off,
		},
		{
			unpack_3_11bit_unsigned_2off,
			unpack_3_11bit_signed_2off,
		},
		{
			unpack_3_11bit_unsigned_3off,
			unpack_3_11bit_signed_3off,
		},
		{
			unpack_3_11bit_unsigned_4off,
			unpack_3_11bit_signed_4off,
		},
		{
			unpack_3_11bit_unsigned_5off,
			unpack_3_11bit_signed_5off,
		},
		{
			unpack_3_11bit_unsigned_6off,
			unpack_3_11bit_signed_6off,
		},
		{
			unpack_3_11bit_unsigned_7off,
			unpack_3_11bit_signed_7off,
		},
	},
	{
		{
			unpack_4_11bit_unsigned_0off,
			unpack_4_11bit_signed_0off,
		},
		{
			unpack_4_11bit_unsigned_1off,
			unpack_4_11bit_signed_1off,
		},
		{
			unpack_4_11bit_unsigned_2off,
			unpack_4_11bit_signed_2off,
		},
		{
			unpack_4_11bit_unsigned_3off,
			unpack_4_11bit_signed_3off,
		},
		{
			unpack_4_11bit_unsigned_4off,
			unpack_4_11bit_signed_4off,
		},
		{
			unpack_4_11bit_unsigned_5off,
			unpack_4_11bit_signed_5off,
		},
		{
			unpack_4_11bit_unsigned_6off,
			unpack_4_11bit_signed_6off,
		},
		{
			unpack_4_11bit_unsigned_7off,
			unpack_4_11bit_signed_7off,
		},
	},
	{
		{
			unpack_5_11bit_unsigned_0off,
			unpack_5_11bit_signed_0off,
		},
		{
			unpack_5_11bit_unsigned_1off,
			unpack_5_11bit_signed_1off,
		},
		{
			unpack_5_11bit_unsigned_2off,
			unpack_5_11bit_signed_2off,
		},
		{
			unpack_5_11bit_unsigned_3off,
			unpack_5_11bit_signed_3off,
		},
		{
			unpack_5_11bit_unsigned_4off,
			unpack_5_11bit_signed_4off,
		},
		{
			unpack_5_11bit_unsigned_5off,
			unpack_5_11bit_signed_5off,
		},
		{
			unpack_5_11bit_unsigned_6off,
			unpack_5_11bit_signed_6off,
		},
		{
			unpack_5_11bit_unsigned_7off,
			unpack_5_11bit_signed_7off,
		},
	},
	{
		{
			unpack_6_11bit_unsigned_0off,
			unpack_6_11bit_signed_0off,
		},
		{
			unpack_6_11bit_unsigned_1off,
			unpack_6_11bit_signed_1off,
		},
		{
			unpack_6_11bit_unsigned_2off,
			unpack_6_11bit_signed_2off,
		},
		{
			unpack_6_11bit_unsigned_3off,
			unpack_6_11bit_signed_3off,
		},
		{
			unpack_6_11bit_unsigned_4off,
			unpack_6_11bit_signed_4off,
		},
		{
			unpack_6_11bit_unsigned_5off,
			unpack_6_11bit_signed_5off,
		},
		{
			unpack_6_11bit_unsigned_6off,
			unpack_6_11bit_signed_6off,
		},
		{
			unpack_6_11bit_unsigned_7off,
			unpack_6_11bit_signed_7off,
		},
	},
	{
		{
			unpack_7_11bit_unsigned_0off,
			unpack_7_11bit_signed_0off,
		},
		{
			unpack_7_11bit_unsigned_1off,
			unpack_7_11bit_signed_1off,
		},
		{
			unpack_7_11bit_unsigned_2off,
			unpack_7_11bit_signed_2off,
		},
		{
			unpack_7_11bit_unsigned_3off,
			unpack_7_11bit_signed_3off,
		},
		{
			unpack_7_11bit_unsigned_4off,
			unpack_7_11bit_signed_4off,
		},
		{
			unpack_7_11bit_unsigned_5off,
			unpack_7_11bit_signed_5off,
		},
		{
			unpack_7_11bit_unsigned_6off,
			unpack_7_11bit_signed_6off,
		},
		{
			unpack_7_11bit_unsigned_7off,
			unpack_7_11bit_signed_7off,
		},
	},
	{
		{
			unpack_8_11bit_unsigned_0off,
			unpack_8_11bit_signed_0off,
		},
		{
			unpack_8_11bit_unsigned_1off,
			unpack_8_11bit_signed_1off,
		},
		{
			unpack_8_11bit_unsigned_2off,
			unpack_8_11bit_signed_2off,
		},
		{
			unpack_8_11bit_unsigned_3off,
			unpack_8_11bit_signed_3off,
		},
		{
			unpack_8_11bit_unsigned_4off,
			unpack_8_11bit_signed_4off,
		},
		{
			unpack_8_11bit_unsigned_5off,
			unpack_8_11bit_signed_5off,
		},
		{
			unpack_8_11bit_unsigned_6off,
			unpack_8_11bit_signed_6off,
		},
		{
			unpack_8_11bit_unsigned_7off,
			unpack_8_11bit_signed_7off,
		},
	},
	{
		{
			unpack_9_11bit_unsigned_0off,
			unpack_9_11bit_signed_0off,
		},
		{
			unpack_9_11bit_unsigned_1off,
			unpack_9_11bit_signed_1off,
		},
		{
			unpack_9_11bit_unsigned_2off,
			unpack_9_11bit_signed_2off,
		},
		{
			unpack_9_11bit_unsigned_3off,
			unpack_9_11bit_signed_3off,
		},
		{
			unpack_9_11bit_unsigned_4off,
			unpack_9_11bit_signed_4off,
		},
		{
			unpack_9_11bit_unsigned_5off,
			unpack_9_11bit_signed_5off,
		},
		{
			unpack_9_11bit_unsigned_6off,
			unpack_9_11bit_signed_6off,
		},
		{
			unpack_9_11bit_unsigned_7off,
			unpack_9_11bit_signed_7off,
		},
	},
	{
		{
			unpack_10_11bit_unsigned_0off,
			unpack_10_11bit_signed_0off,
		},
		{
			unpack_10_11bit_unsigned_1off,
			unpack_10_11bit_signed_1off,
		},
		{
			unpack_10_11bit_unsigned_2off,
			unpack_10_11bit_signed_2off,
		},
		{
			unpack_10_11bit_unsigned_3off,
			unpack_10_11bit_signed_3off,
		},
		{
			unpack_10_11bit_unsigned_4off,
			unpack_10_11bit_signed_4off,
		},
		{
			unpack_10_11bit_unsigned_5off,
			unpack_10_11bit_signed_5off,
		},
		{
			unpack_10_11bit_unsigned_6off,
			unpack_10_11bit_signed_6off,
		},
		{
			unpack_10_11bit_unsigned_7off,
			unpack_10_11bit_signed_7off,
		},
	},
	{
		{
			unpack_11_11bit_unsigned_0off,
			unpack_11_11bit_signed_0off,
		},
		{
			unpack_11_11bit_unsigned_1off,
			unpack_11_11bit_signed_1off,
		},
		{
			unpack_11_11bit_unsigned_2off,
			unpack_11_11bit_signed_2off,
		},
		{
			unpack_11_11bit_unsigned_3off,
			unpack_11_11bit_signed_3off,
		},
		{
			unpack_11_11bit_unsigned_4off,
			unpack_11_11bit_signed_4off,
		},
		{
			unpack_11_11bit_unsigned_5off,
			unpack_11_11bit_signed_5off,
		},
		{
			unpack_11_11bit_unsigned_6off,
			unpack_11_11bit_signed_6off,
		},
		{
			unpack_11_11bit_unsigned_7off,
			unpack_11_11bit_signed_7off,
		},
	},
	{
		{
			unpack_12_11bit_unsigned_0off,
			unpack_12_11bit_signed_0off,
		},
		{
			unpack_12_11bit_unsigned_1off,
			unpack_12_11bit_signed_1off,
		},
		{
			unpack_12_11bit_unsigned_2off,
			unpack_12_11bit_signed_2off,
		},
		{
			unpack_12_11bit_unsigned_3off,
			unpack_12_11bit_signed_3off,
		},
		{
			unpack_12_11bit_unsigned_4off,
			unpack_12_11bit_signed_4off,
		},
		{
			unpack_12_11bit_unsigned_5off,
			unpack_12_11bit_signed_5off,
		},
		{
			unpack_12_11bit_unsigned_6off,
			unpack_12_11bit_signed_6off,
		},
		{
			unpack_12_11bit_unsigned_7off,
			unpack_12_11bit_signed_7off,
		},
	},
	{
		{
			unpack_13_11bit_unsigned_0off,
			unpack_13_11bit_signed_0off,
		},
		{
			unpack_13_11bit_unsigned_1off,
			unpack_13_11bit_signed_1off,
		},
		{
			unpack_13_11bit_unsigned_2off,
			unpack_13_11bit_signed_2off,
		},
		{
			unpack_13_11bit_unsigned_3off,
			unpack_13_11bit_signed_3off,
		},
		{
			unpack_13_11bit_unsigned_4off,
			unpack_13_11bit_signed_4off,
		},
		{
			unpack_13_11bit_unsigned_5off,
			unpack_13_11bit_signed_5off,
		},
		{
			unpack_13_11bit_unsigned_6off,
			unpack_13_11bit_signed_6off,
		},
		{
			unpack_13_11bit_unsigned_7off,
			unpack_13_11bit_signed_7off,
		},
	},
	{
		{
			unpack_14_11bit_unsigned_0off,
			unpack_14_11bit_signed_0off,
		},
		{
			unpack_14_11bit_unsigned_1off,
			unpack_14_11bit_signed_1off,
		},
		{
			unpack_14_11bit_unsigned_2off,
			unpack_14_11bit_signed_2off,
		},
		{
			unpack_14_11bit_unsigned_3off,
			unpack_14_11bit_signed_3off,
		},
		{
			unpack_14_11bit_unsigned_4off,
			unpack_14_11bit_signed_4off,
		},
		{
			unpack_14_11bit_unsigned_5off,
			unpack_14_11bit_signed_5off,
		},
		{
			unpack_14_11bit_unsigned_6off,
			unpack_14_11bit_signed_6off,
		},
		{
			unpack_14_11bit_unsigned_7off,
			unpack_14_11bit_signed_7off,
		},
	},
	{
		{
			unpack_15_11bit_unsigned_0off,
			unpack_15_11bit_signed_0off,
		},
		{
			unpack_15_11bit_unsigned_1off,
			unpack_15_11bit_signed_1off,
		},
		{
			unpack_15_11bit_unsigned_2off,
			unpack_15_11bit_signed_2off,
		},
		{
			unpack_15_11bit_unsigned_3off,
			unpack_15_11bit_signed_3off,
		},
		{
			unpack_15_11bit_unsigned_4off,
			unpack_15_11bit_signed_4off,
		},
		{
			unpack_15_11bit_unsigned_5off,
			unpack_15_11bit_signed_5off,
		},
		{
			unpack_15_11bit_unsigned_6off,
			unpack_15_11bit_signed_6off,
		},
		{
			unpack_15_11bit_unsigned_7off,
			unpack_15_11bit_signed_7off,
		},
	},
	{
		{
			unpack_16_11bit_unsigned_0off,
			unpack_16_11bit_signed_0off,
		},
		{
			unpack_16_11bit_unsigned_1off,
			unpack_16_11bit_signed_1off,
		},
		{
			unpack_16_11bit_unsigned_2off,
			unpack_16_11bit_signed_2off,
		},
		{
			unpack_16_11bit_unsigned_3off,
			unpack_16_11bit_signed_3off,
		},
		{
			unpack_16_11bit_unsigned_4off,
			unpack_16_11bit_signed_4off,
		},
		{
			unpack_16_11bit_unsigned_5off,
			unpack_16_11bit_signed_5off,
		},
		{
			unpack_16_11bit_unsigned_6off,
			unpack_16_11bit_signed_6off,
		},
		{
			unpack_16_11bit_unsigned_7off,
			unpack_16_11bit_signed_7off,
		},
	},
	{
		{
			unpack_17_11bit_unsigned_0off,
			unpack_17_11bit_signed_0off,
		},
		{
			unpack_17_11bit_unsigned_1off,
			unpack_17_11bit_signed_1off,
		},
		{
			unpack_17_11bit_unsigned_2off,
			unpack_17_11bit_signed_2off,
		},
		{
			unpack_17_11bit_unsigned_3off,
			unpack_17_11bit_signed_3off,
		},
		{
			unpack_17_11bit_unsigned_4off,
			unpack_17_11bit_signed_4off,
		},
		{
			unpack_17_11bit_unsigned_5off,
			unpack_17_11bit_signed_5off,
		},
		{
			unpack_17_11bit_unsigned_6off,
			unpack_17_11bit_signed_6off,
		},
		{
			unpack_17_11bit_unsigned_7off,
			unpack_17_11bit_signed_7off,
		},
	},
	{
		{
			unpack_18_11bit_unsigned_0off,
			unpack_18_11bit_signed_0off,
		},
		{
			unpack_18_11bit_unsigned_1off,
			unpack_18_11bit_signed_1off,
		},
		{
			unpack_18_11bit_unsigned_2off,
			unpack_18_11bit_signed_2off,
		},
		{
			unpack_18_11bit_unsigned_3off,
			unpack_18_11bit_signed_3off,
		},
		{
			unpack_18_11bit_unsigned_4off,
			unpack_18_11bit_signed_4off,
		},
		{
			unpack_18_11bit_unsigned_5off,
			unpack_18_11bit_signed_5off,
		},
		{
			unpack_18_11bit_unsigned_6off,
			unpack_18_11bit_signed_6off,
		},
		{
			unpack_18_11bit_unsigned_7off,
			unpack_18_11bit_signed_7off,
		},
	},
	{
		{
			unpack_19_11bit_unsigned_0off,
			unpack_19_11bit_signed_0off,
		},
		{
			unpack_19_11bit_unsigned_1off,
			unpack_19_11bit_signed_1off,
		},
		{
			unpack_19_11bit_unsigned_2off,
			unpack_19_11bit_signed_2off,
		},
		{
			unpack_19_11bit_unsigned_3off,
			unpack_19_11bit_signed_3off,
		},
		{
			unpack_19_11bit_unsigned_4off,
			unpack_19_11bit_signed_4off,
		},
		{
			unpack_19_11bit_unsigned_5off,
			unpack_19_11bit_signed_5off,
		},
		{
			unpack_19_11bit_unsigned_6off,
			unpack_19_11bit_signed_6off,
		},
		{
			unpack_19_11bit_unsigned_7off,
			unpack_19_11bit_signed_7off,
		},
	},
	{
		{
			unpack_20_11bit_unsigned_0off,
			unpack_20_11bit_signed_0off,
		},
		{
			unpack_20_11bit_unsigned_1off,
			unpack_20_11bit_signed_1off,
		},
		{
			unpack_20_11bit_unsigned_2off,
			unpack_20_11bit_signed_2off,
		},
		{
			unpack_20_11bit_unsigned_3off,
			unpack_20_11bit_signed_3off,
		},
		{
			unpack_20_11bit_unsigned_4off,
			unpack_20_11bit_signed_4off,
		},
		{
			unpack_20_11bit_unsigned_5off,
			unpack_20_11bit_signed_5off,
		},
		{
			unpack_20_11bit_unsigned_6off,
			unpack_20_11bit_signed_6off,
		},
		{
			unpack_20_11bit_unsigned_7off,
			unpack_20_11bit_signed_7off,
		},
	},
	{
		{
			unpack_21_11bit_unsigned_0off,
			unpack_21_11bit_signed_0off,
		},
		{
			unpack_21_11bit_unsigned_1off,
			unpack_21_11bit_signed_1off,
		},
		{
			unpack_21_11bit_unsigned_2off,
			unpack_21_11bit_signed_2off,
		},
		{
			unpack_21_11bit_unsigned_3off,
			unpack_21_11bit_signed_3off,
		},
		{
			unpack_21_11bit_unsigned_4off,
			unpack_21_11bit_signed_4off,
		},
		{
			unpack_21_11bit_unsigned_5off,
			unpack_21_11bit_signed_5off,
		},
		{
			unpack_21_11bit_unsigned_6off,
			unpack_21_11bit_signed_6off,
		},
		{
			unpack_21_11bit_unsigned_7off,
			unpack_21_11bit_signed_7off,
		},
	},
	{
		{
			unpack_22_11bit_unsigned_0off,
			unpack_22_11bit_signed_0off,
		},
		{
			unpack_22_11bit_unsigned_1off,
			unpack_22_11bit_signed_1off,
		},
		{
			unpack_22_11bit_unsigned_2off,
			unpack_22_11bit_signed_2off,
		},
		{
			unpack_22_11bit_unsigned_3off,
			unpack_22_11bit_signed_3off,
		},
		{
			unpack_22_11bit_unsigned_4off,
			unpack_22_11bit_signed_4off,
		},
		{
			unpack_22_11bit_unsigned_5off,
			unpack_22_11bit_signed_5off,
		},
		{
			unpack_22_11bit_unsigned_6off,
			unpack_22_11bit_signed_6off,
		},
		{
			unpack_22_11bit_unsigned_7off,
			unpack_22_11bit_signed_7off,
		},
	},
	{
		{
			unpack_23_11bit_unsigned_0off,
			unpack_23_11bit_signed_0off,
		},
		{
			unpack_23_11bit_unsigned_1off,
			unpack_23_11bit_signed_1off,
		},
		{
			unpack_23_11bit_unsigned_2off,
			unpack_23_11bit_signed_2off,
		},
		{
			unpack_23_11bit_unsigned_3off,
			unpack_23_11bit_signed_3off,
		},
		{
			unpack_23_11bit_unsigned_4off,
			unpack_23_11bit_signed_4off,
		},
		{
			unpack_23_11bit_unsigned_5off,
			unpack_23_11bit_signed_5off,
		},
		{
			unpack_23_11bit_unsigned_6off,
			unpack_23_11bit_signed_6off,
		},
		{
			unpack_23_11bit_unsigned_7off,
			unpack_23_11bit_signed_7off,
		},
	},
	{
		{
			unpack_24_11bit_unsigned_0off,
			unpack_24_11bit_signed_0off,
		},
		{
			unpack_24_11bit_unsigned_1off,
			unpack_24_11bit_signed_1off,
		},
		{
			unpack_24_11bit_unsigned_2off,
			unpack_24_11bit_signed_2off,
		},
		{
			unpack_24_11bit_unsigned_3off,
			unpack_24_11bit_signed_3off,
		},
		{
			unpack_24_11bit_unsigned_4off,
			unpack_24_11bit_signed_4off,
		},
		{
			unpack_24_11bit_unsigned_5off,
			unpack_24_11bit_signed_5off,
		},
		{
			unpack_24_11bit_unsigned_6off,
			unpack_24_11bit_signed_6off,
		},
		{
			unpack_24_11bit_unsigned_7off,
			unpack_24_11bit_signed_7off,
		},
	},
	{
		{
			unpack_25_11bit_unsigned_0off,
			unpack_25_11bit_signed_0off,
		},
		{
			unpack_25_11bit_unsigned_1off,
			unpack_25_11bit_signed_1off,
		},
		{
			unpack_25_11bit_unsigned_2off,
			unpack_25_11bit_signed_2off,
		},
		{
			unpack_25_11bit_unsigned_3off,
			unpack_25_11bit_signed_3off,
		},
		{
			unpack_25_11bit_unsigned_4off,
			unpack_25_11bit_signed_4off,
		},
		{
			unpack_25_11bit_unsigned_5off,
			unpack_25_11bit_signed_5off,
		},
		{
			unpack_25_11bit_unsigned_6off,
			unpack_25_11bit_signed_6off,
		},
		{
			unpack_25_11bit_unsigned_7off,
			unpack_25_11bit_signed_7off,
		},
	},
	{
		{
			unpack_26_11bit_unsigned_0off,
			unpack_26_11bit_signed_0off,
		},
		{
			unpack_26_11bit_unsigned_1off,
			unpack_26_11bit_signed_1off,
		},
		{
			unpack_26_11bit_unsigned_2off,
			unpack_26_11bit_signed_2off,
		},
		{
			unpack_26_11bit_unsigned_3off,
			unpack_26_11bit_signed_3off,
		},
		{
			unpack_26_11bit_unsigned_4off,
			unpack_26_11bit_signed_4off,
		},
		{
			unpack_26_11bit_unsigned_5off,
			unpack_26_11bit_signed_5off,
		},
		{
			unpack_26_11bit_unsigned_6off,
			unpack_26_11bit_signed_6off,
		},
		{
			unpack_26_11bit_unsigned_7off,
			unpack_26_11bit_signed_7off,
		},
	},
	{
		{
			unpack_27_11bit_unsigned_0off,
			unpack_27_11bit_signed_0off,
		},
		{
			unpack_27_11bit_unsigned_1off,
			unpack_27_11bit_signed_1off,
		},
		{
			unpack_27_11bit_unsigned_2off,
			unpack_27_11bit_signed_2off,
		},
		{
			unpack_27_11bit_unsigned_3off,
			unpack_27_11bit_signed_3off,
		},
		{
			unpack_27_11bit_unsigned_4off,
			unpack_27_11bit_signed_4off,
		},
		{
			unpack_27_11bit_unsigned_5off,
			unpack_27_11bit_signed_5off,
		},
		{
			unpack_27_11bit_unsigned_6off,
			unpack_27_11bit_signed_6off,
		},
		{
			unpack_27_11bit_unsigned_7off,
			unpack_27_11bit_signed_7off,
		},
	},
	{
		{
			unpack_28_11bit_unsigned_0off,
			unpack_28_11bit_signed_0off,
		},
		{
			unpack_28_11bit_unsigned_1off,
			unpack_28_11bit_signed_1off,
		},
		{
			unpack_28_11bit_unsigned_2off,
			unpack_28_11bit_signed_2off,
		},
		{
			unpack_28_11bit_unsigned_3off,
			unpack_28_11bit_signed_3off,
		},
		{
			unpack_28_11bit_unsigned_4off,
			unpack_28_11bit_signed_4off,
		},
		{
			unpack_28_11bit_unsigned_5off,
			unpack_28_11bit_signed_5off,
		},
		{
			unpack_28_11bit_unsigned_6off,
			unpack_28_11bit_signed_6off,
		},
		{
			unpack_28_11bit_unsigned_7off,
			unpack_28_11bit_signed_7off,
		},
	},
	{
		{
			unpack_29_11bit_unsigned_0off,
			unpack_29_11bit_signed_0off,
		},
		{
			unpack_29_11bit_unsigned_1off,
			unpack_29_11bit_signed_1off,
		},
		{
			unpack_29_11bit_unsigned_2off,
			unpack_29_11bit_signed_2off,
		},
		{
			unpack_29_11bit_unsigned_3off,
			unpack_29_11bit_signed_3off,
		},
		{
			unpack_29_11bit_unsigned_4off,
			unpack_29_11bit_signed_4off,
		},
		{
			unpack_29_11bit_unsigned_5off,
			unpack_29_11bit_signed_5off,
		},
		{
			unpack_29_11bit_unsigned_6off,
			unpack_29_11bit_signed_6off,
		},
		{
			unpack_29_11bit_unsigned_7off,
			unpack_29_11bit_signed_7off,
		},
	},
	{
		{
			unpack_30_11bit_unsigned_0off,
			unpack_30_11bit_signed_0off,
		},
		{
			unpack_30_11bit_unsigned_1off,
			unpack_30_11bit_signed_1off,
		},
		{
			unpack_30_11bit_unsigned_2off,
			unpack_30_11bit_signed_2off,
		},
		{
			unpack_30_11bit_unsigned_3off,
			unpack_30_11bit_signed_3off,
		},
		{
			unpack_30_11bit_unsigned_4off,
			unpack_30_11bit_signed_4off,
		},
		{
			unpack_30_11bit_unsigned_5off,
			unpack_30_11bit_signed_5off,
		},
		{
			unpack_30_11bit_unsigned_6off,
			unpack_30_11bit_signed_6off,
		},
		{
			unpack_30_11bit_unsigned_7off,
			unpack_30_11bit_signed_7off,
		},
	},
	{
		{
			unpack_31_11bit_unsigned_0off,
			unpack_31_11bit_signed_0off,
		},
		{
			unpack_31_11bit_unsigned_1off,
			unpack_31_11bit_signed_1off,
		},
		{
			unpack_31_11bit_unsigned_2off,
			unpack_31_11bit_signed_2off,
		},
		{
			unpack_31_11bit_unsigned_3off,
			unpack_31_11bit_signed_3off,
		},
		{
			unpack_31_11bit_unsigned_4off,
			unpack_31_11bit_signed_4off,
		},
		{
			unpack_31_11bit_unsigned_5off,
			unpack_31_11bit_signed_5off,
		},
		{
			unpack_31_11bit_unsigned_6off,
			unpack_31_11bit_signed_6off,
		},
		{
			unpack_31_11bit_unsigned_7off,
			unpack_31_11bit_signed_7off,
		},
	},
	{
		{
			unpack_32_11bit_unsigned_0off,
			unpack_32_11bit_signed_0off,
		},
		{
			unpack_32_11bit_unsigned_1off,
			unpack_32_11bit_signed_1off,
		},
		{
			unpack_32_11bit_unsigned_2off,
			unpack_32_11bit_signed_2off,
		},
		{
			unpack_32_11bit_unsigned_3off,
			unpack_32_11bit_signed_3off,
		},
		{
			unpack_32_11bit_unsigned_4off,
			unpack_32_11bit_signed_4off,
		},
		{
			unpack_32_11bit_unsigned_5off,
			unpack_32_11bit_signed_5off,
		},
		{
			unpack_32_11bit_unsigned_6off,
			unpack_32_11bit_signed_6off,
		},
		{
			unpack_32_11bit_unsigned_7off,
			unpack_32_11bit_signed_7off,
		},
	},
	{
		{
			unpack_33_11bit_unsigned_0off,
			unpack_33_11bit_signed_0off,
		},
		{
			unpack_33_11bit_unsigned_1off,
			unpack_33_11bit_signed_1off,
		},
		{
			unpack_33_11bit_unsigned_2off,
			unpack_33_11bit_signed_2off,
		},
		{
			unpack_33_11bit_unsigned_3off,
			unpack_33_11bit_signed_3off,
		},
		{
			unpack_33_11bit_unsigned_4off,
			unpack_33_11bit_signed_4off,
		},
		{
			unpack_33_11bit_unsigned_5off,
			unpack_33_11bit_signed_5off,
		},
		{
			unpack_33_11bit_unsigned_6off,
			unpack_33_11bit_signed_6off,
		},
		{
			unpack_33_11bit_unsigned_7off,
			unpack_33_11bit_signed_7off,
		},
	},
	{
		{
			unpack_34_11bit_unsigned_0off,
			unpack_34_11bit_signed_0off,
		},
		{
			unpack_34_11bit_unsigned_1off,
			unpack_34_11bit_signed_1off,
		},
		{
			unpack_34_11bit_unsigned_2off,
			unpack_34_11bit_signed_2off,
		},
		{
			unpack_34_11bit_unsigned_3off,
			unpack_34_11bit_signed_3off,
		},
		{
			unpack_34_11bit_unsigned_4off,
			unpack_34_11bit_signed_4off,
		},
		{
			unpack_34_11bit_unsigned_5off,
			unpack_34_11bit_signed_5off,
		},
		{
			unpack_34_11bit_unsigned_6off,
			unpack_34_11bit_signed_6off,
		},
		{
			unpack_34_11bit_unsigned_7off,
			unpack_34_11bit_signed_7off,
		},
	},
	{
		{
			unpack_35_11bit_unsigned_0off,
			unpack_35_11bit_signed_0off,
		},
		{
			unpack_35_11bit_unsigned_1off,
			unpack_35_11bit_signed_1off,
		},
		{
			unpack_35_11bit_unsigned_2off,
			unpack_35_11bit_signed_2off,
		},
		{
			unpack_35_11bit_unsigned_3off,
			unpack_35_11bit_signed_3off,
		},
		{
			unpack_35_11bit_unsigned_4off,
			unpack_35_11bit_signed_4off,
		},
		{
			unpack_35_11bit_unsigned_5off,
			unpack_35_11bit_signed_5off,
		},
		{
			unpack_35_11bit_unsigned_6off,
			unpack_35_11bit_signed_6off,
		},
		{
			unpack_35_11bit_unsigned_7off,
			unpack_35_11bit_signed_7off,
		},
	},
	{
		{
			unpack_36_11bit_unsigned_0off,
			unpack_36_11bit_signed_0off,
		},
		{
			unpack_36_11bit_unsigned_1off,
			unpack_36_11bit_signed_1off,
		},
		{
			unpack_36_11bit_unsigned_2off,
			unpack_36_11bit_signed_2off,
		},
		{
			unpack_36_11bit_unsigned_3off,
			unpack_36_11bit_signed_3off,
		},
		{
			unpack_36_11bit_unsigned_4off,
			unpack_36_11bit_signed_4off,
		},
		{
			unpack_36_11bit_unsigned_5off,
			unpack_36_11bit_signed_5off,
		},
		{
			unpack_36_11bit_unsigned_6off,
			unpack_36_11bit_signed_6off,
		},
		{
			unpack_36_11bit_unsigned_7off,
			unpack_36_11bit_signed_7off,
		},
	},
	{
		{
			unpack_37_11bit_unsigned_0off,
			unpack_37_11bit_signed_0off,
		},
		{
			unpack_37_11bit_unsigned_1off,
			unpack_37_11bit_signed_1off,
		},
		{
			unpack_37_11bit_unsigned_2off,
			unpack_37_11bit_signed_2off,
		},
		{
			unpack_37_11bit_unsigned_3off,
			unpack_37_11bit_signed_3off,
		},
		{
			unpack_37_11bit_unsigned_4off,
			unpack_37_11bit_signed_4off,
		},
		{
			unpack_37_11bit_unsigned_5off,
			unpack_37_11bit_signed_5off,
		},
		{
			unpack_37_11bit_unsigned_6off,
			unpack_37_11bit_signed_6off,
		},
		{
			unpack_37_11bit_unsigned_7off,
			unpack_37_11bit_signed_7off,
		},
	},
	{
		{
			unpack_38_11bit_unsigned_0off,
			unpack_38_11bit_signed_0off,
		},
		{
			unpack_38_11bit_unsigned_1off,
			unpack_38_11bit_signed_1off,
		},
		{
			unpack_38_11bit_unsigned_2off,
			unpack_38_11bit_signed_2off,
		},
		{
			unpack_38_11bit_unsigned_3off,
			unpack_38_11bit_signed_3off,
		},
		{
			unpack_38_11bit_unsigned_4off,
			unpack_38_11bit_signed_4off,
		},
		{
			unpack_38_11bit_unsigned_5off,
			unpack_38_11bit_signed_5off,
		},
		{
			unpack_38_11bit_unsigned_6off,
			unpack_38_11bit_signed_6off,
		},
		{
			unpack_38_11bit_unsigned_7off,
			unpack_38_11bit_signed_7off,
		},
	},
	{
		{
			unpack_39_11bit_unsigned_0off,
			unpack_39_11bit_signed_0off,
		},
		{
			unpack_39_11bit_unsigned_1off,
			unpack_39_11bit_signed_1off,
		},
		{
			unpack_39_11bit_unsigned_2off,
			unpack_39_11bit_signed_2off,
		},
		{
			unpack_39_11bit_unsigned_3off,
			unpack_39_11bit_signed_3off,
		},
		{
			unpack_39_11bit_unsigned_4off,
			unpack_39_11bit_signed_4off,
		},
		{
			unpack_39_11bit_unsigned_5off,
			unpack_39_11bit_signed_5off,
		},
		{
			unpack_39_11bit_unsigned_6off,
			unpack_39_11bit_signed_6off,
		},
		{
			unpack_39_11bit_unsigned_7off,
			unpack_39_11bit_signed_7off,
		},
	},
	{
		{
			unpack_40_11bit_unsigned_0off,
			unpack_40_11bit_signed_0off,
		},
		{
			unpack_40_11bit_unsigned_1off,
			unpack_40_11bit_signed_1off,
		},
		{
			unpack_40_11bit_unsigned_2off,
			unpack_40_11bit_signed_2off,
		},
		{
			unpack_40_11bit_unsigned_3off,
			unpack_40_11bit_signed_3off,
		},
		{
			unpack_40_11bit_unsigned_4off,
			unpack_40_11bit_signed_4off,
		},
		{
			unpack_40_11bit_unsigned_5off,
			unpack_40_11bit_signed_5off,
		},
		{
			unpack_40_11bit_unsigned_6off,
			unpack_40_11bit_signed_6off,
		},
		{
			unpack_40_11bit_unsigned_7off,
			unpack_40_11bit_signed_7off,
		},
	},
	{
		{
			unpack_41_11bit_unsigned_0off,
			unpack_41_11bit_signed_0off,
		},
		{
			unpack_41_11bit_unsigned_1off,
			unpack_41_11bit_signed_1off,
		},
		{
			unpack_41_11bit_unsigned_2off,
			unpack_41_11bit_signed_2off,
		},
		{
			unpack_41_11bit_unsigned_3off,
			unpack_41_11bit_signed_3off,
		},
		{
			unpack_41_11bit_unsigned_4off,
			unpack_41_11bit_signed_4off,
		},
		{
			unpack_41_11bit_unsigned_5off,
			unpack_41_11bit_signed_5off,
		},
		{
			unpack_41_11bit_unsigned_6off,
			unpack_41_11bit_signed_6off,
		},
		{
			unpack_41_11bit_unsigned_7off,
			unpack_41_11bit_signed_7off,
		},
	},
	{
		{
			unpack_42_11bit_unsigned_0off,
			unpack_42_11bit_signed_0off,
		},
		{
			unpack_42_11bit_unsigned_1off,
			unpack_42_11bit_signed_1off,
		},
		{
			unpack_42_11bit_unsigned_2off,
			unpack_42_11bit_signed_2off,
		},
		{
			unpack_42_11bit_unsigned_3off,
			unpack_42_11bit_signed_3off,
		},
		{
			unpack_42_11bit_unsigned_4off,
			unpack_42_11bit_signed_4off,
		},
		{
			unpack_42_11bit_unsigned_5off,
			unpack_42_11bit_signed_5off,
		},
		{
			unpack_42_11bit_unsigned_6off,
			unpack_42_11bit_signed_6off,
		},
		{
			unpack_42_11bit_unsigned_7off,
			unpack_42_11bit_signed_7off,
		},
	},
	{
		{
			unpack_43_11bit_unsigned_0off,
			unpack_43_11bit_signed_0off,
		},
		{
			unpack_43_11bit_unsigned_1off,
			unpack_43_11bit_signed_1off,
		},
		{
			unpack_43_11bit_unsigned_2off,
			unpack_43_11bit_signed_2off,
		},
		{
			unpack_43_11bit_unsigned_3off,
			unpack_43_11bit_signed_3off,
		},
		{
			unpack_43_11bit_unsigned_4off,
			unpack_43_11bit_signed_4off,
		},
		{
			unpack_43_11bit_unsigned_5off,
			unpack_43_11bit_signed_5off,
		},
		{
			unpack_43_11bit_unsigned_6off,
			unpack_43_11bit_signed_6off,
		},
		{
			unpack_43_11bit_unsigned_7off,
			unpack_43_11bit_signed_7off,
		},
	},
	{
		{
			unpack_44_11bit_unsigned_0off,
			unpack_44_11bit_signed_0off,
		},
		{
			unpack_44_11bit_unsigned_1off,
			unpack_44_11bit_signed_1off,
		},
		{
			unpack_44_11bit_unsigned_2off,
			unpack_44_11bit_signed_2off,
		},
		{
			unpack_44_11bit_unsigned_3off,
			unpack_44_11bit_signed_3off,
		},
		{
			unpack_44_11bit_unsigned_4off,
			unpack_44_11bit_signed_4off,
		},
		{
			unpack_44_11bit_unsigned_5off,
			unpack_44_11bit_signed_5off,
		},
		{
			unpack_44_11bit_unsigned_6off,
			unpack_44_11bit_signed_6off,
		},
		{
			unpack_44_11bit_unsigned_7off,
			unpack_44_11bit_signed_7off,
		},
	},
	{
		{
			unpack_45_11bit_unsigned_0off,
			unpack_45_11bit_signed_0off,
		},
		{
			unpack_45_11bit_unsigned_1off,
			unpack_45_11bit_signed_1off,
		},
		{
			unpack_45_11bit_unsigned_2off,
			unpack_45_11bit_signed_2off,
		},
		{
			unpack_45_11bit_unsigned_3off,
			unpack_45_11bit_signed_3off,
		},
		{
			unpack_45_11bit_unsigned_4off,
			unpack_45_11bit_signed_4off,
		},
		{
			unpack_45_11bit_unsigned_5off,
			unpack_45_11bit_signed_5off,
		},
		{
			unpack_45_11bit_unsigned_6off,
			unpack_45_11bit_signed_6off,
		},
		{
			unpack_45_11bit_unsigned_7off,
			unpack_45_11bit_signed_7off,
		},
	},
	{
		{
			unpack_46_11bit_unsigned_0off,
			unpack_46_11bit_signed_0off,
		},
		{
			unpack_46_11bit_unsigned_1off,
			unpack_46_11bit_signed_1off,
		},
		{
			unpack_46_11bit_unsigned_2off,
			unpack_46_11bit_signed_2off,
		},
		{
			unpack_46_11bit_unsigned_3off,
			unpack_46_11bit_signed_3off,
		},
		{
			unpack_46_11bit_unsigned_4off,
			unpack_46_11bit_signed_4off,
		},
		{
			unpack_46_11bit_unsigned_5off,
			unpack_46_11bit_signed_5off,
		},
		{
			unpack_46_11bit_unsigned_6off,
			unpack_46_11bit_signed_6off,
		},
		{
			unpack_46_11bit_unsigned_7off,
			unpack_46_11bit_signed_7off,
		},
	},
};
