// AUTOGENERATED FILE. DO NOT MODIFY.

#include <stdlib.h>
#include <stdint.h>

#include "unpack13.h"



static void unpack_1_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 13 bits of data.
	// Input must be at least 2 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);
}

static void unpack_1_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 13 bits of data.
	// Input must be at least 2 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);
}

static void unpack_1_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 13 bits of data.
	// Input must be at least 2 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);
}

static void unpack_1_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 13 bits of data.
	// Input must be at least 2 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);
}

static void unpack_1_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 13 bits of data.
	// Input must be at least 2 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);
}

static void unpack_1_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 13 bits of data.
	// Input must be at least 2 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);
}

static void unpack_1_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 13 bits of data.
	// Input must be at least 2 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);
}

static void unpack_1_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 13 bits of data.
	// Input must be at least 2 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);
}

static void unpack_1_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 13 bits of data.
	// Input must be at least 3 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);
}

static void unpack_1_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 13 bits of data.
	// Input must be at least 3 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);
}

static void unpack_1_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 13 bits of data.
	// Input must be at least 3 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);
}

static void unpack_1_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 13 bits of data.
	// Input must be at least 3 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);
}

static void unpack_1_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 13 bits of data.
	// Input must be at least 3 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);
}

static void unpack_1_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 13 bits of data.
	// Input must be at least 3 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);
}

static void unpack_1_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 13 bits of data.
	// Input must be at least 3 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);
}

static void unpack_1_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 1 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 13 bits of data.
	// Input must be at least 3 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);
}

static void unpack_2_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 26 bits of data.
	// Input must be at least 4 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);
}

static void unpack_2_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 26 bits of data.
	// Input must be at least 4 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);
}

static void unpack_2_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 26 bits of data.
	// Input must be at least 4 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);
}

static void unpack_2_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 26 bits of data.
	// Input must be at least 4 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);
}

static void unpack_2_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 26 bits of data.
	// Input must be at least 4 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);
}

static void unpack_2_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 26 bits of data.
	// Input must be at least 4 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);
}

static void unpack_2_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 26 bits of data.
	// Input must be at least 4 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);
}

static void unpack_2_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 26 bits of data.
	// Input must be at least 4 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);
}

static void unpack_2_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 26 bits of data.
	// Input must be at least 4 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);
}

static void unpack_2_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 26 bits of data.
	// Input must be at least 4 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);
}

static void unpack_2_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 26 bits of data.
	// Input must be at least 4 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);
}

static void unpack_2_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 26 bits of data.
	// Input must be at least 4 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);
}

static void unpack_2_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 26 bits of data.
	// Input must be at least 4 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);
}

static void unpack_2_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 26 bits of data.
	// Input must be at least 4 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);
}

static void unpack_2_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 26 bits of data.
	// Input must be at least 5 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);
}

static void unpack_2_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 2 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 26 bits of data.
	// Input must be at least 5 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);
}

static void unpack_3_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 39 bits of data.
	// Input must be at least 5 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);
}

static void unpack_3_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 39 bits of data.
	// Input must be at least 5 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);
}

static void unpack_3_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 39 bits of data.
	// Input must be at least 5 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);
}

static void unpack_3_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 39 bits of data.
	// Input must be at least 5 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);
}

static void unpack_3_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 39 bits of data.
	// Input must be at least 6 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);
}

static void unpack_3_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 39 bits of data.
	// Input must be at least 6 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);
}

static void unpack_3_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 39 bits of data.
	// Input must be at least 6 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);
}

static void unpack_3_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 39 bits of data.
	// Input must be at least 6 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);
}

static void unpack_3_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 39 bits of data.
	// Input must be at least 6 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);
}

static void unpack_3_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 39 bits of data.
	// Input must be at least 6 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);
}

static void unpack_3_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 39 bits of data.
	// Input must be at least 6 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);
}

static void unpack_3_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 39 bits of data.
	// Input must be at least 6 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);
}

static void unpack_3_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 39 bits of data.
	// Input must be at least 6 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);
}

static void unpack_3_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 39 bits of data.
	// Input must be at least 6 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);
}

static void unpack_3_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 39 bits of data.
	// Input must be at least 6 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);
}

static void unpack_3_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 3 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 39 bits of data.
	// Input must be at least 6 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);
}

static void unpack_4_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 52 bits of data.
	// Input must be at least 7 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);
}

static void unpack_4_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 52 bits of data.
	// Input must be at least 7 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);
}

static void unpack_4_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 52 bits of data.
	// Input must be at least 7 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);
}

static void unpack_4_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 52 bits of data.
	// Input must be at least 7 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);
}

static void unpack_4_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 52 bits of data.
	// Input must be at least 7 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);
}

static void unpack_4_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 52 bits of data.
	// Input must be at least 7 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);
}

static void unpack_4_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 52 bits of data.
	// Input must be at least 7 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);
}

static void unpack_4_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 52 bits of data.
	// Input must be at least 7 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);
}

static void unpack_4_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 52 bits of data.
	// Input must be at least 7 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);
}

static void unpack_4_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 52 bits of data.
	// Input must be at least 7 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);
}

static void unpack_4_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 52 bits of data.
	// Input must be at least 8 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);
}

static void unpack_4_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 52 bits of data.
	// Input must be at least 8 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);
}

static void unpack_4_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 52 bits of data.
	// Input must be at least 8 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);
}

static void unpack_4_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 52 bits of data.
	// Input must be at least 8 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);
}

static void unpack_4_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 52 bits of data.
	// Input must be at least 8 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);
}

static void unpack_4_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 4 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 52 bits of data.
	// Input must be at least 8 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);
}

static void unpack_5_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 65 bits of data.
	// Input must be at least 9 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);
}

static void unpack_5_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 65 bits of data.
	// Input must be at least 9 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);
}

static void unpack_5_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 65 bits of data.
	// Input must be at least 9 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);
}

static void unpack_5_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 65 bits of data.
	// Input must be at least 9 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);
}

static void unpack_5_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 65 bits of data.
	// Input must be at least 9 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);
}

static void unpack_5_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 65 bits of data.
	// Input must be at least 9 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);
}

static void unpack_5_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 65 bits of data.
	// Input must be at least 9 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);
}

static void unpack_5_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 65 bits of data.
	// Input must be at least 9 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);
}

static void unpack_5_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 65 bits of data.
	// Input must be at least 9 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);
}

static void unpack_5_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 65 bits of data.
	// Input must be at least 9 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);
}

static void unpack_5_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 65 bits of data.
	// Input must be at least 9 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);
}

static void unpack_5_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 65 bits of data.
	// Input must be at least 9 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);
}

static void unpack_5_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 65 bits of data.
	// Input must be at least 9 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);
}

static void unpack_5_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 65 bits of data.
	// Input must be at least 9 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);
}

static void unpack_5_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 65 bits of data.
	// Input must be at least 9 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);
}

static void unpack_5_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 5 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 65 bits of data.
	// Input must be at least 9 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);
}

static void unpack_6_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 78 bits of data.
	// Input must be at least 10 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);
}

static void unpack_6_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 78 bits of data.
	// Input must be at least 10 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);
}

static void unpack_6_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 78 bits of data.
	// Input must be at least 10 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);
}

static void unpack_6_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 78 bits of data.
	// Input must be at least 10 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);
}

static void unpack_6_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 78 bits of data.
	// Input must be at least 10 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);
}

static void unpack_6_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 78 bits of data.
	// Input must be at least 10 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);
}

static void unpack_6_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 78 bits of data.
	// Input must be at least 11 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);
}

static void unpack_6_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 78 bits of data.
	// Input must be at least 11 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);
}

static void unpack_6_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 78 bits of data.
	// Input must be at least 11 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);
}

static void unpack_6_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 78 bits of data.
	// Input must be at least 11 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);
}

static void unpack_6_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 78 bits of data.
	// Input must be at least 11 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);
}

static void unpack_6_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 78 bits of data.
	// Input must be at least 11 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);
}

static void unpack_6_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 78 bits of data.
	// Input must be at least 11 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);
}

static void unpack_6_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 78 bits of data.
	// Input must be at least 11 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);
}

static void unpack_6_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 78 bits of data.
	// Input must be at least 11 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);
}

static void unpack_6_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 6 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 78 bits of data.
	// Input must be at least 11 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);
}

static void unpack_7_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 91 bits of data.
	// Input must be at least 12 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);
}

static void unpack_7_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 91 bits of data.
	// Input must be at least 12 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);
}

static void unpack_7_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 91 bits of data.
	// Input must be at least 12 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);
}

static void unpack_7_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 91 bits of data.
	// Input must be at least 12 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);
}

static void unpack_7_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 91 bits of data.
	// Input must be at least 12 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);
}

static void unpack_7_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 91 bits of data.
	// Input must be at least 12 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);
}

static void unpack_7_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 91 bits of data.
	// Input must be at least 12 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);
}

static void unpack_7_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 91 bits of data.
	// Input must be at least 12 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);
}

static void unpack_7_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 91 bits of data.
	// Input must be at least 12 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);
}

static void unpack_7_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 91 bits of data.
	// Input must be at least 12 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);
}

static void unpack_7_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 91 bits of data.
	// Input must be at least 12 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);
}

static void unpack_7_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 91 bits of data.
	// Input must be at least 12 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);
}

static void unpack_7_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 91 bits of data.
	// Input must be at least 13 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);
}

static void unpack_7_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 91 bits of data.
	// Input must be at least 13 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);
}

static void unpack_7_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 91 bits of data.
	// Input must be at least 13 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);
}

static void unpack_7_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 7 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 91 bits of data.
	// Input must be at least 13 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);
}

static void unpack_8_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 104 bits of data.
	// Input must be at least 13 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);
}

static void unpack_8_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 104 bits of data.
	// Input must be at least 13 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);
}

static void unpack_8_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 104 bits of data.
	// Input must be at least 14 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);
}

static void unpack_8_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 104 bits of data.
	// Input must be at least 14 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);
}

static void unpack_8_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 104 bits of data.
	// Input must be at least 14 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);
}

static void unpack_8_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 104 bits of data.
	// Input must be at least 14 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);
}

static void unpack_8_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 104 bits of data.
	// Input must be at least 14 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);
}

static void unpack_8_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 104 bits of data.
	// Input must be at least 14 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);
}

static void unpack_8_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 104 bits of data.
	// Input must be at least 14 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);
}

static void unpack_8_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 104 bits of data.
	// Input must be at least 14 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);
}

static void unpack_8_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 104 bits of data.
	// Input must be at least 14 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);
}

static void unpack_8_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 104 bits of data.
	// Input must be at least 14 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);
}

static void unpack_8_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 104 bits of data.
	// Input must be at least 14 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);
}

static void unpack_8_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 104 bits of data.
	// Input must be at least 14 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);
}

static void unpack_8_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 104 bits of data.
	// Input must be at least 14 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);
}

static void unpack_8_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 8 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 104 bits of data.
	// Input must be at least 14 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);
}

static void unpack_9_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 117 bits of data.
	// Input must be at least 15 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);
}

static void unpack_9_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 117 bits of data.
	// Input must be at least 15 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);
}

static void unpack_9_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 117 bits of data.
	// Input must be at least 15 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);
}

static void unpack_9_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 117 bits of data.
	// Input must be at least 15 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);
}

static void unpack_9_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 117 bits of data.
	// Input must be at least 15 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);
}

static void unpack_9_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 117 bits of data.
	// Input must be at least 15 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);
}

static void unpack_9_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 117 bits of data.
	// Input must be at least 15 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);
}

static void unpack_9_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 117 bits of data.
	// Input must be at least 15 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);
}

static void unpack_9_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 117 bits of data.
	// Input must be at least 16 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);
}

static void unpack_9_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 117 bits of data.
	// Input must be at least 16 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);
}

static void unpack_9_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 117 bits of data.
	// Input must be at least 16 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);
}

static void unpack_9_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 117 bits of data.
	// Input must be at least 16 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);
}

static void unpack_9_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 117 bits of data.
	// Input must be at least 16 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);
}

static void unpack_9_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 117 bits of data.
	// Input must be at least 16 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);
}

static void unpack_9_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 117 bits of data.
	// Input must be at least 16 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);
}

static void unpack_9_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 9 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 117 bits of data.
	// Input must be at least 16 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);
}

static void unpack_10_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 130 bits of data.
	// Input must be at least 17 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);
}

static void unpack_10_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 130 bits of data.
	// Input must be at least 17 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);
}

static void unpack_10_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 130 bits of data.
	// Input must be at least 17 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);
}

static void unpack_10_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 130 bits of data.
	// Input must be at least 17 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);
}

static void unpack_10_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 130 bits of data.
	// Input must be at least 17 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);
}

static void unpack_10_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 130 bits of data.
	// Input must be at least 17 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);
}

static void unpack_10_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 130 bits of data.
	// Input must be at least 17 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);
}

static void unpack_10_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 130 bits of data.
	// Input must be at least 17 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);
}

static void unpack_10_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 130 bits of data.
	// Input must be at least 17 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);
}

static void unpack_10_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 130 bits of data.
	// Input must be at least 17 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);
}

static void unpack_10_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 130 bits of data.
	// Input must be at least 17 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);
}

static void unpack_10_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 130 bits of data.
	// Input must be at least 17 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);
}

static void unpack_10_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 130 bits of data.
	// Input must be at least 17 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);
}

static void unpack_10_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 130 bits of data.
	// Input must be at least 17 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);
}

static void unpack_10_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 130 bits of data.
	// Input must be at least 18 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);
}

static void unpack_10_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 10 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 130 bits of data.
	// Input must be at least 18 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);
}

static void unpack_11_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 143 bits of data.
	// Input must be at least 18 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);
}

static void unpack_11_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 143 bits of data.
	// Input must be at least 18 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);
}

static void unpack_11_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 143 bits of data.
	// Input must be at least 18 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);
}

static void unpack_11_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 143 bits of data.
	// Input must be at least 18 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);
}

static void unpack_11_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);
}

static void unpack_11_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);
}

static void unpack_11_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);
}

static void unpack_11_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);
}

static void unpack_11_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);
}

static void unpack_11_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);
}

static void unpack_11_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);
}

static void unpack_11_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);
}

static void unpack_11_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);
}

static void unpack_11_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);
}

static void unpack_11_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);
}

static void unpack_11_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 11 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 143 bits of data.
	// Input must be at least 19 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);
}

static void unpack_12_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 156 bits of data.
	// Input must be at least 20 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);
}

static void unpack_12_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 156 bits of data.
	// Input must be at least 20 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);
}

static void unpack_12_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 156 bits of data.
	// Input must be at least 20 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);
}

static void unpack_12_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 156 bits of data.
	// Input must be at least 20 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);
}

static void unpack_12_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 156 bits of data.
	// Input must be at least 20 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);
}

static void unpack_12_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 156 bits of data.
	// Input must be at least 20 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);
}

static void unpack_12_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 156 bits of data.
	// Input must be at least 20 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);
}

static void unpack_12_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 156 bits of data.
	// Input must be at least 20 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);
}

static void unpack_12_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 156 bits of data.
	// Input must be at least 20 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);
}

static void unpack_12_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 156 bits of data.
	// Input must be at least 20 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);
}

static void unpack_12_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 156 bits of data.
	// Input must be at least 21 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);
}

static void unpack_12_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 156 bits of data.
	// Input must be at least 21 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);
}

static void unpack_12_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 156 bits of data.
	// Input must be at least 21 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);
}

static void unpack_12_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 156 bits of data.
	// Input must be at least 21 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);
}

static void unpack_12_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 156 bits of data.
	// Input must be at least 21 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);
}

static void unpack_12_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 12 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 156 bits of data.
	// Input must be at least 21 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);
}

static void unpack_13_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 169 bits of data.
	// Input must be at least 22 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);
}

static void unpack_13_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 169 bits of data.
	// Input must be at least 22 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);
}

static void unpack_13_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 169 bits of data.
	// Input must be at least 22 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);
}

static void unpack_13_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 169 bits of data.
	// Input must be at least 22 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);
}

static void unpack_13_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 169 bits of data.
	// Input must be at least 22 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);
}

static void unpack_13_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 169 bits of data.
	// Input must be at least 22 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);
}

static void unpack_13_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 169 bits of data.
	// Input must be at least 22 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);
}

static void unpack_13_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 169 bits of data.
	// Input must be at least 22 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);
}

static void unpack_13_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 169 bits of data.
	// Input must be at least 22 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);
}

static void unpack_13_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 169 bits of data.
	// Input must be at least 22 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);
}

static void unpack_13_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 169 bits of data.
	// Input must be at least 22 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);
}

static void unpack_13_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 169 bits of data.
	// Input must be at least 22 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);
}

static void unpack_13_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 169 bits of data.
	// Input must be at least 22 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);
}

static void unpack_13_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 169 bits of data.
	// Input must be at least 22 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);
}

static void unpack_13_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 169 bits of data.
	// Input must be at least 22 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);
}

static void unpack_13_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 13 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 169 bits of data.
	// Input must be at least 22 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);
}

static void unpack_14_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 182 bits of data.
	// Input must be at least 23 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);
}

static void unpack_14_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 182 bits of data.
	// Input must be at least 23 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);
}

static void unpack_14_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 182 bits of data.
	// Input must be at least 23 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);
}

static void unpack_14_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 182 bits of data.
	// Input must be at least 23 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);
}

static void unpack_14_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 182 bits of data.
	// Input must be at least 23 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);
}

static void unpack_14_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 182 bits of data.
	// Input must be at least 23 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);
}

static void unpack_14_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 182 bits of data.
	// Input must be at least 24 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);
}

static void unpack_14_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 182 bits of data.
	// Input must be at least 24 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);
}

static void unpack_14_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 182 bits of data.
	// Input must be at least 24 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);
}

static void unpack_14_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 182 bits of data.
	// Input must be at least 24 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);
}

static void unpack_14_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 182 bits of data.
	// Input must be at least 24 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);
}

static void unpack_14_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 182 bits of data.
	// Input must be at least 24 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);
}

static void unpack_14_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 182 bits of data.
	// Input must be at least 24 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);
}

static void unpack_14_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 182 bits of data.
	// Input must be at least 24 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);
}

static void unpack_14_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 182 bits of data.
	// Input must be at least 24 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);
}

static void unpack_14_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 14 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 182 bits of data.
	// Input must be at least 24 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);
}

static void unpack_15_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);
}

static void unpack_15_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);
}

static void unpack_15_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);
}

static void unpack_15_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);
}

static void unpack_15_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);
}

static void unpack_15_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);
}

static void unpack_15_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);
}

static void unpack_15_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);
}

static void unpack_15_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);
}

static void unpack_15_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);
}

static void unpack_15_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);
}

static void unpack_15_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 195 bits of data.
	// Input must be at least 25 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);
}

static void unpack_15_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 195 bits of data.
	// Input must be at least 26 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);
}

static void unpack_15_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 195 bits of data.
	// Input must be at least 26 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);
}

static void unpack_15_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 195 bits of data.
	// Input must be at least 26 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);
}

static void unpack_15_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 15 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 195 bits of data.
	// Input must be at least 26 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);
}

static void unpack_16_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 208 bits of data.
	// Input must be at least 26 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);
}

static void unpack_16_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 208 bits of data.
	// Input must be at least 26 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);
}

static void unpack_16_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 208 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);
}

static void unpack_16_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 208 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);
}

static void unpack_16_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 208 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);
}

static void unpack_16_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 208 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);
}

static void unpack_16_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 208 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);
}

static void unpack_16_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 208 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);
}

static void unpack_16_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 208 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);
}

static void unpack_16_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 208 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);
}

static void unpack_16_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 208 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);
}

static void unpack_16_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 208 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);
}

static void unpack_16_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 208 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);
}

static void unpack_16_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 208 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);
}

static void unpack_16_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 208 bits of data.
	// Input must be at least 27 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);
}

static void unpack_16_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 16 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 208 bits of data.
	// Input must be at least 27 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);
}

static void unpack_17_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 221 bits of data.
	// Input must be at least 28 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);
}

static void unpack_17_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 221 bits of data.
	// Input must be at least 28 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);
}

static void unpack_17_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 221 bits of data.
	// Input must be at least 28 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);
}

static void unpack_17_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 221 bits of data.
	// Input must be at least 28 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);
}

static void unpack_17_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 221 bits of data.
	// Input must be at least 28 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);
}

static void unpack_17_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 221 bits of data.
	// Input must be at least 28 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);
}

static void unpack_17_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 221 bits of data.
	// Input must be at least 28 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);
}

static void unpack_17_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 221 bits of data.
	// Input must be at least 28 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);
}

static void unpack_17_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 221 bits of data.
	// Input must be at least 29 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);
}

static void unpack_17_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 221 bits of data.
	// Input must be at least 29 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);
}

static void unpack_17_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 221 bits of data.
	// Input must be at least 29 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);
}

static void unpack_17_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 221 bits of data.
	// Input must be at least 29 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);
}

static void unpack_17_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 221 bits of data.
	// Input must be at least 29 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);
}

static void unpack_17_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 221 bits of data.
	// Input must be at least 29 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);
}

static void unpack_17_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 221 bits of data.
	// Input must be at least 29 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);
}

static void unpack_17_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 17 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 221 bits of data.
	// Input must be at least 29 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);
}

static void unpack_18_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 234 bits of data.
	// Input must be at least 30 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);
}

static void unpack_18_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 234 bits of data.
	// Input must be at least 30 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);
}

static void unpack_18_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 234 bits of data.
	// Input must be at least 30 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);
}

static void unpack_18_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 234 bits of data.
	// Input must be at least 30 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);
}

static void unpack_18_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 234 bits of data.
	// Input must be at least 30 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);
}

static void unpack_18_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 234 bits of data.
	// Input must be at least 30 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);
}

static void unpack_18_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 234 bits of data.
	// Input must be at least 30 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);
}

static void unpack_18_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 234 bits of data.
	// Input must be at least 30 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);
}

static void unpack_18_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 234 bits of data.
	// Input must be at least 30 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);
}

static void unpack_18_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 234 bits of data.
	// Input must be at least 30 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);
}

static void unpack_18_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 234 bits of data.
	// Input must be at least 30 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);
}

static void unpack_18_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 234 bits of data.
	// Input must be at least 30 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);
}

static void unpack_18_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 234 bits of data.
	// Input must be at least 30 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);
}

static void unpack_18_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 234 bits of data.
	// Input must be at least 30 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);
}

static void unpack_18_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 234 bits of data.
	// Input must be at least 31 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);
}

static void unpack_18_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 18 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 234 bits of data.
	// Input must be at least 31 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);
}

static void unpack_19_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 247 bits of data.
	// Input must be at least 31 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);
}

static void unpack_19_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 247 bits of data.
	// Input must be at least 31 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);
}

static void unpack_19_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 247 bits of data.
	// Input must be at least 31 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);
}

static void unpack_19_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 247 bits of data.
	// Input must be at least 31 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);
}

static void unpack_19_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 247 bits of data.
	// Input must be at least 32 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);
}

static void unpack_19_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 247 bits of data.
	// Input must be at least 32 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);
}

static void unpack_19_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 247 bits of data.
	// Input must be at least 32 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);
}

static void unpack_19_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 247 bits of data.
	// Input must be at least 32 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);
}

static void unpack_19_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 247 bits of data.
	// Input must be at least 32 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);
}

static void unpack_19_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 247 bits of data.
	// Input must be at least 32 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);
}

static void unpack_19_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 247 bits of data.
	// Input must be at least 32 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);
}

static void unpack_19_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 247 bits of data.
	// Input must be at least 32 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);
}

static void unpack_19_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 247 bits of data.
	// Input must be at least 32 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);
}

static void unpack_19_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 247 bits of data.
	// Input must be at least 32 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);
}

static void unpack_19_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 247 bits of data.
	// Input must be at least 32 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);
}

static void unpack_19_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 19 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 247 bits of data.
	// Input must be at least 32 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);
}

static void unpack_20_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 260 bits of data.
	// Input must be at least 33 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);
}

static void unpack_20_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 260 bits of data.
	// Input must be at least 33 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);
}

static void unpack_20_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 260 bits of data.
	// Input must be at least 33 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);
}

static void unpack_20_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 260 bits of data.
	// Input must be at least 33 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);
}

static void unpack_20_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 260 bits of data.
	// Input must be at least 33 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);
}

static void unpack_20_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 260 bits of data.
	// Input must be at least 33 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);
}

static void unpack_20_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 260 bits of data.
	// Input must be at least 33 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);
}

static void unpack_20_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 260 bits of data.
	// Input must be at least 33 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);
}

static void unpack_20_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 260 bits of data.
	// Input must be at least 33 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);
}

static void unpack_20_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 260 bits of data.
	// Input must be at least 33 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);
}

static void unpack_20_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 260 bits of data.
	// Input must be at least 34 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);
}

static void unpack_20_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 260 bits of data.
	// Input must be at least 34 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);
}

static void unpack_20_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 260 bits of data.
	// Input must be at least 34 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);
}

static void unpack_20_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 260 bits of data.
	// Input must be at least 34 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);
}

static void unpack_20_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 260 bits of data.
	// Input must be at least 34 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);
}

static void unpack_20_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 20 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 260 bits of data.
	// Input must be at least 34 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);
}

static void unpack_21_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 273 bits of data.
	// Input must be at least 35 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);
}

static void unpack_21_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 273 bits of data.
	// Input must be at least 35 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);
}

static void unpack_21_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 273 bits of data.
	// Input must be at least 35 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);
}

static void unpack_21_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 273 bits of data.
	// Input must be at least 35 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);
}

static void unpack_21_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 273 bits of data.
	// Input must be at least 35 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);
}

static void unpack_21_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 273 bits of data.
	// Input must be at least 35 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);
}

static void unpack_21_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 273 bits of data.
	// Input must be at least 35 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);
}

static void unpack_21_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 273 bits of data.
	// Input must be at least 35 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);
}

static void unpack_21_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 273 bits of data.
	// Input must be at least 35 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);
}

static void unpack_21_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 273 bits of data.
	// Input must be at least 35 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);
}

static void unpack_21_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 273 bits of data.
	// Input must be at least 35 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);
}

static void unpack_21_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 273 bits of data.
	// Input must be at least 35 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);
}

static void unpack_21_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 273 bits of data.
	// Input must be at least 35 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);
}

static void unpack_21_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 273 bits of data.
	// Input must be at least 35 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);
}

static void unpack_21_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 273 bits of data.
	// Input must be at least 35 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);
}

static void unpack_21_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 21 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 273 bits of data.
	// Input must be at least 35 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);
}

static void unpack_22_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 286 bits of data.
	// Input must be at least 36 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);
}

static void unpack_22_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 286 bits of data.
	// Input must be at least 36 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);
}

static void unpack_22_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 286 bits of data.
	// Input must be at least 36 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);
}

static void unpack_22_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 286 bits of data.
	// Input must be at least 36 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);
}

static void unpack_22_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 286 bits of data.
	// Input must be at least 36 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);
}

static void unpack_22_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 286 bits of data.
	// Input must be at least 36 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);
}

static void unpack_22_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 286 bits of data.
	// Input must be at least 37 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);
}

static void unpack_22_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 286 bits of data.
	// Input must be at least 37 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);
}

static void unpack_22_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 286 bits of data.
	// Input must be at least 37 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);
}

static void unpack_22_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 286 bits of data.
	// Input must be at least 37 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);
}

static void unpack_22_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 286 bits of data.
	// Input must be at least 37 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);
}

static void unpack_22_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 286 bits of data.
	// Input must be at least 37 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);
}

static void unpack_22_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 286 bits of data.
	// Input must be at least 37 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);
}

static void unpack_22_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 286 bits of data.
	// Input must be at least 37 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);
}

static void unpack_22_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 286 bits of data.
	// Input must be at least 37 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);
}

static void unpack_22_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 22 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 286 bits of data.
	// Input must be at least 37 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);
}

static void unpack_23_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 299 bits of data.
	// Input must be at least 38 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);
}

static void unpack_23_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 299 bits of data.
	// Input must be at least 38 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);
}

static void unpack_23_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 299 bits of data.
	// Input must be at least 38 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);
}

static void unpack_23_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 299 bits of data.
	// Input must be at least 38 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);
}

static void unpack_23_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 299 bits of data.
	// Input must be at least 38 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);
}

static void unpack_23_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 299 bits of data.
	// Input must be at least 38 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);
}

static void unpack_23_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 299 bits of data.
	// Input must be at least 38 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);
}

static void unpack_23_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 299 bits of data.
	// Input must be at least 38 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);
}

static void unpack_23_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 299 bits of data.
	// Input must be at least 38 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);
}

static void unpack_23_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 299 bits of data.
	// Input must be at least 38 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);
}

static void unpack_23_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 299 bits of data.
	// Input must be at least 38 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);
}

static void unpack_23_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 299 bits of data.
	// Input must be at least 38 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);
}

static void unpack_23_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 299 bits of data.
	// Input must be at least 39 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);
}

static void unpack_23_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 299 bits of data.
	// Input must be at least 39 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);
}

static void unpack_23_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 299 bits of data.
	// Input must be at least 39 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);
}

static void unpack_23_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 23 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 299 bits of data.
	// Input must be at least 39 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);
}

static void unpack_24_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 312 bits of data.
	// Input must be at least 39 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);
}

static void unpack_24_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 312 bits of data.
	// Input must be at least 39 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);
}

static void unpack_24_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 312 bits of data.
	// Input must be at least 40 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);
}

static void unpack_24_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 312 bits of data.
	// Input must be at least 40 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);
}

static void unpack_24_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 312 bits of data.
	// Input must be at least 40 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);
}

static void unpack_24_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 312 bits of data.
	// Input must be at least 40 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);
}

static void unpack_24_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 312 bits of data.
	// Input must be at least 40 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);
}

static void unpack_24_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 312 bits of data.
	// Input must be at least 40 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);
}

static void unpack_24_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 312 bits of data.
	// Input must be at least 40 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);
}

static void unpack_24_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 312 bits of data.
	// Input must be at least 40 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);
}

static void unpack_24_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 312 bits of data.
	// Input must be at least 40 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);
}

static void unpack_24_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 312 bits of data.
	// Input must be at least 40 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);
}

static void unpack_24_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 312 bits of data.
	// Input must be at least 40 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);
}

static void unpack_24_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 312 bits of data.
	// Input must be at least 40 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);
}

static void unpack_24_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 312 bits of data.
	// Input must be at least 40 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);
}

static void unpack_24_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 24 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 312 bits of data.
	// Input must be at least 40 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);
}

static void unpack_25_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 325 bits of data.
	// Input must be at least 41 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);
}

static void unpack_25_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 325 bits of data.
	// Input must be at least 41 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);
}

static void unpack_25_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 325 bits of data.
	// Input must be at least 41 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);
}

static void unpack_25_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 325 bits of data.
	// Input must be at least 41 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);
}

static void unpack_25_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 325 bits of data.
	// Input must be at least 41 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);
}

static void unpack_25_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 325 bits of data.
	// Input must be at least 41 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);
}

static void unpack_25_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 325 bits of data.
	// Input must be at least 41 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);
}

static void unpack_25_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 325 bits of data.
	// Input must be at least 41 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);
}

static void unpack_25_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 325 bits of data.
	// Input must be at least 42 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);
}

static void unpack_25_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 325 bits of data.
	// Input must be at least 42 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);
}

static void unpack_25_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 325 bits of data.
	// Input must be at least 42 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);
}

static void unpack_25_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 325 bits of data.
	// Input must be at least 42 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);
}

static void unpack_25_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 325 bits of data.
	// Input must be at least 42 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);
}

static void unpack_25_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 325 bits of data.
	// Input must be at least 42 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);
}

static void unpack_25_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 325 bits of data.
	// Input must be at least 42 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);
}

static void unpack_25_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 25 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 325 bits of data.
	// Input must be at least 42 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);
}

static void unpack_26_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 338 bits of data.
	// Input must be at least 43 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);
}

static void unpack_26_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 338 bits of data.
	// Input must be at least 43 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);
}

static void unpack_26_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 338 bits of data.
	// Input must be at least 43 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);
}

static void unpack_26_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 338 bits of data.
	// Input must be at least 43 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);
}

static void unpack_26_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 338 bits of data.
	// Input must be at least 43 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);
}

static void unpack_26_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 338 bits of data.
	// Input must be at least 43 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);
}

static void unpack_26_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 338 bits of data.
	// Input must be at least 43 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);
}

static void unpack_26_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 338 bits of data.
	// Input must be at least 43 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);
}

static void unpack_26_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 338 bits of data.
	// Input must be at least 43 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);
}

static void unpack_26_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 338 bits of data.
	// Input must be at least 43 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);
}

static void unpack_26_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 338 bits of data.
	// Input must be at least 43 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);
}

static void unpack_26_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 338 bits of data.
	// Input must be at least 43 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);
}

static void unpack_26_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 338 bits of data.
	// Input must be at least 43 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);
}

static void unpack_26_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 338 bits of data.
	// Input must be at least 43 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);
}

static void unpack_26_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 338 bits of data.
	// Input must be at least 44 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);
}

static void unpack_26_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 26 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 338 bits of data.
	// Input must be at least 44 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);
}

static void unpack_27_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 351 bits of data.
	// Input must be at least 44 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);
}

static void unpack_27_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 351 bits of data.
	// Input must be at least 44 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);
}

static void unpack_27_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 351 bits of data.
	// Input must be at least 44 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);
}

static void unpack_27_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 351 bits of data.
	// Input must be at least 44 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);
}

static void unpack_27_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 351 bits of data.
	// Input must be at least 45 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);
}

static void unpack_27_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 351 bits of data.
	// Input must be at least 45 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);
}

static void unpack_27_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 351 bits of data.
	// Input must be at least 45 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);
}

static void unpack_27_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 351 bits of data.
	// Input must be at least 45 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);
}

static void unpack_27_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 351 bits of data.
	// Input must be at least 45 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);
}

static void unpack_27_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 351 bits of data.
	// Input must be at least 45 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);
}

static void unpack_27_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 351 bits of data.
	// Input must be at least 45 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);
}

static void unpack_27_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 351 bits of data.
	// Input must be at least 45 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);
}

static void unpack_27_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 351 bits of data.
	// Input must be at least 45 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);
}

static void unpack_27_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 351 bits of data.
	// Input must be at least 45 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);
}

static void unpack_27_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 351 bits of data.
	// Input must be at least 45 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);
}

static void unpack_27_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 27 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 351 bits of data.
	// Input must be at least 45 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);
}

static void unpack_28_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 364 bits of data.
	// Input must be at least 46 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);
}

static void unpack_28_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 364 bits of data.
	// Input must be at least 46 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);
}

static void unpack_28_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 364 bits of data.
	// Input must be at least 46 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);
}

static void unpack_28_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 364 bits of data.
	// Input must be at least 46 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);
}

static void unpack_28_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 364 bits of data.
	// Input must be at least 46 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);
}

static void unpack_28_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 364 bits of data.
	// Input must be at least 46 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);
}

static void unpack_28_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 364 bits of data.
	// Input must be at least 46 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);
}

static void unpack_28_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 364 bits of data.
	// Input must be at least 46 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);
}

static void unpack_28_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 364 bits of data.
	// Input must be at least 46 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);
}

static void unpack_28_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 364 bits of data.
	// Input must be at least 46 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);
}

static void unpack_28_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 364 bits of data.
	// Input must be at least 47 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);
}

static void unpack_28_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 364 bits of data.
	// Input must be at least 47 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);
}

static void unpack_28_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 364 bits of data.
	// Input must be at least 47 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);
}

static void unpack_28_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 364 bits of data.
	// Input must be at least 47 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);
}

static void unpack_28_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 364 bits of data.
	// Input must be at least 47 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);
}

static void unpack_28_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 28 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 364 bits of data.
	// Input must be at least 47 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);
}

static void unpack_29_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 377 bits of data.
	// Input must be at least 48 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);
}

static void unpack_29_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 377 bits of data.
	// Input must be at least 48 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);
}

static void unpack_29_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 377 bits of data.
	// Input must be at least 48 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);
}

static void unpack_29_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 377 bits of data.
	// Input must be at least 48 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);
}

static void unpack_29_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 377 bits of data.
	// Input must be at least 48 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);
}

static void unpack_29_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 377 bits of data.
	// Input must be at least 48 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);
}

static void unpack_29_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 377 bits of data.
	// Input must be at least 48 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);
}

static void unpack_29_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 377 bits of data.
	// Input must be at least 48 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);
}

static void unpack_29_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 377 bits of data.
	// Input must be at least 48 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);
}

static void unpack_29_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 377 bits of data.
	// Input must be at least 48 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);
}

static void unpack_29_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 377 bits of data.
	// Input must be at least 48 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);
}

static void unpack_29_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 377 bits of data.
	// Input must be at least 48 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);
}

static void unpack_29_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 377 bits of data.
	// Input must be at least 48 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);
}

static void unpack_29_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 377 bits of data.
	// Input must be at least 48 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);
}

static void unpack_29_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 377 bits of data.
	// Input must be at least 48 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);
}

static void unpack_29_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 29 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 377 bits of data.
	// Input must be at least 48 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);
}

static void unpack_30_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 390 bits of data.
	// Input must be at least 49 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 377 to 389
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xfc) << 17);
	output[29] = (double)(value >> 19);
}

static void unpack_30_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 390 bits of data.
	// Input must be at least 49 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 377 to 389
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xfc) << 17);
	output[29] = (double)(value >> 19);
}

static void unpack_30_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 390 bits of data.
	// Input must be at least 49 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 378 to 390
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xfe) << 18);
	output[29] = (double)(value >> 19);
}

static void unpack_30_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 390 bits of data.
	// Input must be at least 49 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 378 to 390
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xfe) << 18);
	output[29] = (double)(value >> 19);
}

static void unpack_30_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 390 bits of data.
	// Input must be at least 49 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 379 to 391
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19);
	output[29] = (double)(value >> 19);
}

static void unpack_30_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 390 bits of data.
	// Input must be at least 49 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 379 to 391
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19);
	output[29] = (double)(value >> 19);
}

static void unpack_30_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 390 bits of data.
	// Input must be at least 50 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 380 to 392
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0x80) << 12);
	output[29] = (double)(value >> 19);
}

static void unpack_30_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 390 bits of data.
	// Input must be at least 50 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 380 to 392
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0x80) << 12);
	output[29] = (double)(value >> 19);
}

static void unpack_30_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 390 bits of data.
	// Input must be at least 50 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 381 to 393
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xc0) << 13);
	output[29] = (double)(value >> 19);
}

static void unpack_30_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 390 bits of data.
	// Input must be at least 50 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 381 to 393
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xc0) << 13);
	output[29] = (double)(value >> 19);
}

static void unpack_30_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 390 bits of data.
	// Input must be at least 50 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 382 to 394
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xe0) << 14);
	output[29] = (double)(value >> 19);
}

static void unpack_30_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 390 bits of data.
	// Input must be at least 50 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 382 to 394
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xe0) << 14);
	output[29] = (double)(value >> 19);
}

static void unpack_30_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 390 bits of data.
	// Input must be at least 50 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 383 to 395
	value = ((uint32_t)(input[47] & 0x01) << 31) |
			((uint32_t)input[48] << 23) |
			((uint32_t)(input[49] & 0xf0) << 15);
	output[29] = (double)(value >> 19);
}

static void unpack_30_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 390 bits of data.
	// Input must be at least 50 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 383 to 395
	value = ((uint32_t)(input[47] & 0x01) << 31) |
			((uint32_t)input[48] << 23) |
			((uint32_t)(input[49] & 0xf0) << 15);
	output[29] = (double)(value >> 19);
}

static void unpack_30_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 390 bits of data.
	// Input must be at least 50 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 384 to 396
	value = ((uint32_t)input[48] << 24) |
			((uint32_t)(input[49] & 0xf8) << 16);
	output[29] = (double)(value >> 19);
}

static void unpack_30_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 30 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 390 bits of data.
	// Input must be at least 50 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 384 to 396
	value = ((uint32_t)input[48] << 24) |
			((uint32_t)(input[49] & 0xf8) << 16);
	output[29] = (double)(value >> 19);
}

static void unpack_31_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 403 bits of data.
	// Input must be at least 51 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 377 to 389
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xfc) << 17);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 390 to 402
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xe0) << 14);
	output[30] = (double)(value >> 19);
}

static void unpack_31_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 403 bits of data.
	// Input must be at least 51 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 377 to 389
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xfc) << 17);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 390 to 402
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xe0) << 14);
	output[30] = (double)(value >> 19);
}

static void unpack_31_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 403 bits of data.
	// Input must be at least 51 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 378 to 390
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xfe) << 18);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 391 to 403
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xf0) << 15);
	output[30] = (double)(value >> 19);
}

static void unpack_31_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 403 bits of data.
	// Input must be at least 51 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 378 to 390
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xfe) << 18);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 391 to 403
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xf0) << 15);
	output[30] = (double)(value >> 19);
}

static void unpack_31_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 403 bits of data.
	// Input must be at least 51 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 379 to 391
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 392 to 404
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xf8) << 16);
	output[30] = (double)(value >> 19);
}

static void unpack_31_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 403 bits of data.
	// Input must be at least 51 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 379 to 391
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 392 to 404
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xf8) << 16);
	output[30] = (double)(value >> 19);
}

static void unpack_31_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 403 bits of data.
	// Input must be at least 51 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 380 to 392
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0x80) << 12);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 393 to 405
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)(input[50] & 0xfc) << 17);
	output[30] = (double)(value >> 19);
}

static void unpack_31_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 403 bits of data.
	// Input must be at least 51 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 380 to 392
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0x80) << 12);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 393 to 405
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)(input[50] & 0xfc) << 17);
	output[30] = (double)(value >> 19);
}

static void unpack_31_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 403 bits of data.
	// Input must be at least 51 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 381 to 393
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xc0) << 13);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 394 to 406
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)(input[50] & 0xfe) << 18);
	output[30] = (double)(value >> 19);
}

static void unpack_31_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 403 bits of data.
	// Input must be at least 51 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 381 to 393
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xc0) << 13);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 394 to 406
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)(input[50] & 0xfe) << 18);
	output[30] = (double)(value >> 19);
}

static void unpack_31_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 403 bits of data.
	// Input must be at least 51 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 382 to 394
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xe0) << 14);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 395 to 407
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19);
	output[30] = (double)(value >> 19);
}

static void unpack_31_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 403 bits of data.
	// Input must be at least 51 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 382 to 394
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xe0) << 14);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 395 to 407
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19);
	output[30] = (double)(value >> 19);
}

static void unpack_31_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 403 bits of data.
	// Input must be at least 52 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 383 to 395
	value = ((uint32_t)(input[47] & 0x01) << 31) |
			((uint32_t)input[48] << 23) |
			((uint32_t)(input[49] & 0xf0) << 15);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 396 to 408
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0x80) << 12);
	output[30] = (double)(value >> 19);
}

static void unpack_31_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 403 bits of data.
	// Input must be at least 52 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 383 to 395
	value = ((uint32_t)(input[47] & 0x01) << 31) |
			((uint32_t)input[48] << 23) |
			((uint32_t)(input[49] & 0xf0) << 15);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 396 to 408
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0x80) << 12);
	output[30] = (double)(value >> 19);
}

static void unpack_31_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 403 bits of data.
	// Input must be at least 52 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 384 to 396
	value = ((uint32_t)input[48] << 24) |
			((uint32_t)(input[49] & 0xf8) << 16);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 397 to 409
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xc0) << 13);
	output[30] = (double)(value >> 19);
}

static void unpack_31_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 31 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 403 bits of data.
	// Input must be at least 52 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 384 to 396
	value = ((uint32_t)input[48] << 24) |
			((uint32_t)(input[49] & 0xf8) << 16);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 397 to 409
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xc0) << 13);
	output[30] = (double)(value >> 19);
}

static void unpack_32_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 416 bits of data.
	// Input must be at least 52 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 377 to 389
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xfc) << 17);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 390 to 402
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xe0) << 14);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 403 to 415
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)input[51] << 19);
	output[31] = (double)(value >> 19);
}

static void unpack_32_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 416 bits of data.
	// Input must be at least 52 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 377 to 389
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xfc) << 17);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 390 to 402
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xe0) << 14);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 403 to 415
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)input[51] << 19);
	output[31] = (double)(value >> 19);
}

static void unpack_32_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 416 bits of data.
	// Input must be at least 53 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 378 to 390
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xfe) << 18);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 391 to 403
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xf0) << 15);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 404 to 416
	value = ((uint32_t)(input[50] & 0x0f) << 28) |
			((uint32_t)input[51] << 20) |
			((uint32_t)(input[52] & 0x80) << 12);
	output[31] = (double)(value >> 19);
}

static void unpack_32_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 416 bits of data.
	// Input must be at least 53 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 378 to 390
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xfe) << 18);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 391 to 403
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xf0) << 15);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 404 to 416
	value = ((uint32_t)(input[50] & 0x0f) << 28) |
			((uint32_t)input[51] << 20) |
			((uint32_t)(input[52] & 0x80) << 12);
	output[31] = (double)(value >> 19);
}

static void unpack_32_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 416 bits of data.
	// Input must be at least 53 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 379 to 391
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 392 to 404
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xf8) << 16);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 405 to 417
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xc0) << 13);
	output[31] = (double)(value >> 19);
}

static void unpack_32_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 416 bits of data.
	// Input must be at least 53 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 379 to 391
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 392 to 404
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xf8) << 16);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 405 to 417
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xc0) << 13);
	output[31] = (double)(value >> 19);
}

static void unpack_32_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 416 bits of data.
	// Input must be at least 53 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 380 to 392
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0x80) << 12);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 393 to 405
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)(input[50] & 0xfc) << 17);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 406 to 418
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xe0) << 14);
	output[31] = (double)(value >> 19);
}

static void unpack_32_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 416 bits of data.
	// Input must be at least 53 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 380 to 392
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0x80) << 12);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 393 to 405
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)(input[50] & 0xfc) << 17);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 406 to 418
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xe0) << 14);
	output[31] = (double)(value >> 19);
}

static void unpack_32_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 416 bits of data.
	// Input must be at least 53 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 381 to 393
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xc0) << 13);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 394 to 406
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)(input[50] & 0xfe) << 18);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 407 to 419
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xf0) << 15);
	output[31] = (double)(value >> 19);
}

static void unpack_32_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 416 bits of data.
	// Input must be at least 53 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 381 to 393
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xc0) << 13);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 394 to 406
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)(input[50] & 0xfe) << 18);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 407 to 419
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xf0) << 15);
	output[31] = (double)(value >> 19);
}

static void unpack_32_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 416 bits of data.
	// Input must be at least 53 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 382 to 394
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xe0) << 14);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 395 to 407
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 408 to 420
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xf8) << 16);
	output[31] = (double)(value >> 19);
}

static void unpack_32_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 416 bits of data.
	// Input must be at least 53 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 382 to 394
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xe0) << 14);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 395 to 407
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 408 to 420
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xf8) << 16);
	output[31] = (double)(value >> 19);
}

static void unpack_32_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 416 bits of data.
	// Input must be at least 53 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 383 to 395
	value = ((uint32_t)(input[47] & 0x01) << 31) |
			((uint32_t)input[48] << 23) |
			((uint32_t)(input[49] & 0xf0) << 15);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 396 to 408
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0x80) << 12);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 409 to 421
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xfc) << 17);
	output[31] = (double)(value >> 19);
}

static void unpack_32_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 416 bits of data.
	// Input must be at least 53 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 383 to 395
	value = ((uint32_t)(input[47] & 0x01) << 31) |
			((uint32_t)input[48] << 23) |
			((uint32_t)(input[49] & 0xf0) << 15);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 396 to 408
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0x80) << 12);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 409 to 421
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xfc) << 17);
	output[31] = (double)(value >> 19);
}

static void unpack_32_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 416 bits of data.
	// Input must be at least 53 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 384 to 396
	value = ((uint32_t)input[48] << 24) |
			((uint32_t)(input[49] & 0xf8) << 16);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 397 to 409
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xc0) << 13);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 410 to 422
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xfe) << 18);
	output[31] = (double)(value >> 19);
}

static void unpack_32_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 32 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 416 bits of data.
	// Input must be at least 53 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 384 to 396
	value = ((uint32_t)input[48] << 24) |
			((uint32_t)(input[49] & 0xf8) << 16);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 397 to 409
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xc0) << 13);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 410 to 422
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xfe) << 18);
	output[31] = (double)(value >> 19);
}

static void unpack_33_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 429 bits of data.
	// Input must be at least 54 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 377 to 389
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xfc) << 17);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 390 to 402
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xe0) << 14);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 403 to 415
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)input[51] << 19);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 416 to 428
	value = ((uint32_t)input[52] << 24) |
			((uint32_t)(input[53] & 0xf8) << 16);
	output[32] = (double)(value >> 19);
}

static void unpack_33_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 429 bits of data.
	// Input must be at least 54 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 377 to 389
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xfc) << 17);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 390 to 402
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xe0) << 14);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 403 to 415
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)input[51] << 19);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 416 to 428
	value = ((uint32_t)input[52] << 24) |
			((uint32_t)(input[53] & 0xf8) << 16);
	output[32] = (double)(value >> 19);
}

static void unpack_33_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 429 bits of data.
	// Input must be at least 54 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 378 to 390
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xfe) << 18);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 391 to 403
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xf0) << 15);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 404 to 416
	value = ((uint32_t)(input[50] & 0x0f) << 28) |
			((uint32_t)input[51] << 20) |
			((uint32_t)(input[52] & 0x80) << 12);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 417 to 429
	value = ((uint32_t)(input[52] & 0x7f) << 25) |
			((uint32_t)(input[53] & 0xfc) << 17);
	output[32] = (double)(value >> 19);
}

static void unpack_33_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 429 bits of data.
	// Input must be at least 54 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 378 to 390
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xfe) << 18);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 391 to 403
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xf0) << 15);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 404 to 416
	value = ((uint32_t)(input[50] & 0x0f) << 28) |
			((uint32_t)input[51] << 20) |
			((uint32_t)(input[52] & 0x80) << 12);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 417 to 429
	value = ((uint32_t)(input[52] & 0x7f) << 25) |
			((uint32_t)(input[53] & 0xfc) << 17);
	output[32] = (double)(value >> 19);
}

static void unpack_33_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 429 bits of data.
	// Input must be at least 54 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 379 to 391
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 392 to 404
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xf8) << 16);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 405 to 417
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xc0) << 13);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 418 to 430
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xfe) << 18);
	output[32] = (double)(value >> 19);
}

static void unpack_33_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 429 bits of data.
	// Input must be at least 54 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 379 to 391
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 392 to 404
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xf8) << 16);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 405 to 417
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xc0) << 13);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 418 to 430
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xfe) << 18);
	output[32] = (double)(value >> 19);
}

static void unpack_33_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 429 bits of data.
	// Input must be at least 54 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 380 to 392
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0x80) << 12);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 393 to 405
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)(input[50] & 0xfc) << 17);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 406 to 418
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xe0) << 14);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 419 to 431
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)input[53] << 19);
	output[32] = (double)(value >> 19);
}

static void unpack_33_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 429 bits of data.
	// Input must be at least 54 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 380 to 392
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0x80) << 12);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 393 to 405
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)(input[50] & 0xfc) << 17);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 406 to 418
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xe0) << 14);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 419 to 431
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)input[53] << 19);
	output[32] = (double)(value >> 19);
}

static void unpack_33_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 429 bits of data.
	// Input must be at least 55 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 381 to 393
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xc0) << 13);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 394 to 406
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)(input[50] & 0xfe) << 18);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 407 to 419
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xf0) << 15);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 420 to 432
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0x80) << 12);
	output[32] = (double)(value >> 19);
}

static void unpack_33_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 429 bits of data.
	// Input must be at least 55 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 381 to 393
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xc0) << 13);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 394 to 406
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)(input[50] & 0xfe) << 18);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 407 to 419
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xf0) << 15);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 420 to 432
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0x80) << 12);
	output[32] = (double)(value >> 19);
}

static void unpack_33_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 429 bits of data.
	// Input must be at least 55 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 382 to 394
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xe0) << 14);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 395 to 407
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 408 to 420
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xf8) << 16);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 421 to 433
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xc0) << 13);
	output[32] = (double)(value >> 19);
}

static void unpack_33_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 429 bits of data.
	// Input must be at least 55 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 382 to 394
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xe0) << 14);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 395 to 407
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 408 to 420
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xf8) << 16);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 421 to 433
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xc0) << 13);
	output[32] = (double)(value >> 19);
}

static void unpack_33_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 429 bits of data.
	// Input must be at least 55 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 383 to 395
	value = ((uint32_t)(input[47] & 0x01) << 31) |
			((uint32_t)input[48] << 23) |
			((uint32_t)(input[49] & 0xf0) << 15);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 396 to 408
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0x80) << 12);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 409 to 421
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xfc) << 17);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 422 to 434
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xe0) << 14);
	output[32] = (double)(value >> 19);
}

static void unpack_33_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 429 bits of data.
	// Input must be at least 55 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 383 to 395
	value = ((uint32_t)(input[47] & 0x01) << 31) |
			((uint32_t)input[48] << 23) |
			((uint32_t)(input[49] & 0xf0) << 15);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 396 to 408
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0x80) << 12);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 409 to 421
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xfc) << 17);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 422 to 434
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xe0) << 14);
	output[32] = (double)(value >> 19);
}

static void unpack_33_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 429 bits of data.
	// Input must be at least 55 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 384 to 396
	value = ((uint32_t)input[48] << 24) |
			((uint32_t)(input[49] & 0xf8) << 16);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 397 to 409
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xc0) << 13);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 410 to 422
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xfe) << 18);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 423 to 435
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xf0) << 15);
	output[32] = (double)(value >> 19);
}

static void unpack_33_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 33 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 429 bits of data.
	// Input must be at least 55 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 384 to 396
	value = ((uint32_t)input[48] << 24) |
			((uint32_t)(input[49] & 0xf8) << 16);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 397 to 409
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xc0) << 13);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 410 to 422
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xfe) << 18);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 423 to 435
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xf0) << 15);
	output[32] = (double)(value >> 19);
}

static void unpack_34_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 442 bits of data.
	// Input must be at least 56 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 377 to 389
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xfc) << 17);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 390 to 402
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xe0) << 14);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 403 to 415
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)input[51] << 19);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 416 to 428
	value = ((uint32_t)input[52] << 24) |
			((uint32_t)(input[53] & 0xf8) << 16);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 429 to 441
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21) |
			((uint32_t)(input[55] & 0xc0) << 13);
	output[33] = (double)(value >> 19);
}

static void unpack_34_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 442 bits of data.
	// Input must be at least 56 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 377 to 389
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xfc) << 17);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 390 to 402
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xe0) << 14);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 403 to 415
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)input[51] << 19);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 416 to 428
	value = ((uint32_t)input[52] << 24) |
			((uint32_t)(input[53] & 0xf8) << 16);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 429 to 441
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21) |
			((uint32_t)(input[55] & 0xc0) << 13);
	output[33] = (double)(value >> 19);
}

static void unpack_34_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 442 bits of data.
	// Input must be at least 56 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 378 to 390
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xfe) << 18);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 391 to 403
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xf0) << 15);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 404 to 416
	value = ((uint32_t)(input[50] & 0x0f) << 28) |
			((uint32_t)input[51] << 20) |
			((uint32_t)(input[52] & 0x80) << 12);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 417 to 429
	value = ((uint32_t)(input[52] & 0x7f) << 25) |
			((uint32_t)(input[53] & 0xfc) << 17);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 430 to 442
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0xe0) << 14);
	output[33] = (double)(value >> 19);
}

static void unpack_34_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 442 bits of data.
	// Input must be at least 56 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 378 to 390
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xfe) << 18);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 391 to 403
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xf0) << 15);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 404 to 416
	value = ((uint32_t)(input[50] & 0x0f) << 28) |
			((uint32_t)input[51] << 20) |
			((uint32_t)(input[52] & 0x80) << 12);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 417 to 429
	value = ((uint32_t)(input[52] & 0x7f) << 25) |
			((uint32_t)(input[53] & 0xfc) << 17);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 430 to 442
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0xe0) << 14);
	output[33] = (double)(value >> 19);
}

static void unpack_34_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 442 bits of data.
	// Input must be at least 56 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 379 to 391
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 392 to 404
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xf8) << 16);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 405 to 417
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xc0) << 13);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 418 to 430
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xfe) << 18);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 431 to 443
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xf0) << 15);
	output[33] = (double)(value >> 19);
}

static void unpack_34_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 442 bits of data.
	// Input must be at least 56 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 379 to 391
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 392 to 404
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xf8) << 16);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 405 to 417
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xc0) << 13);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 418 to 430
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xfe) << 18);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 431 to 443
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xf0) << 15);
	output[33] = (double)(value >> 19);
}

static void unpack_34_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 442 bits of data.
	// Input must be at least 56 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 380 to 392
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0x80) << 12);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 393 to 405
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)(input[50] & 0xfc) << 17);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 406 to 418
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xe0) << 14);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 419 to 431
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)input[53] << 19);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 432 to 444
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xf8) << 16);
	output[33] = (double)(value >> 19);
}

static void unpack_34_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 442 bits of data.
	// Input must be at least 56 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 380 to 392
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0x80) << 12);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 393 to 405
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)(input[50] & 0xfc) << 17);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 406 to 418
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xe0) << 14);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 419 to 431
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)input[53] << 19);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 432 to 444
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xf8) << 16);
	output[33] = (double)(value >> 19);
}

static void unpack_34_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 442 bits of data.
	// Input must be at least 56 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 381 to 393
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xc0) << 13);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 394 to 406
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)(input[50] & 0xfe) << 18);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 407 to 419
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xf0) << 15);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 420 to 432
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0x80) << 12);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 433 to 445
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xfc) << 17);
	output[33] = (double)(value >> 19);
}

static void unpack_34_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 442 bits of data.
	// Input must be at least 56 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 381 to 393
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xc0) << 13);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 394 to 406
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)(input[50] & 0xfe) << 18);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 407 to 419
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xf0) << 15);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 420 to 432
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0x80) << 12);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 433 to 445
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xfc) << 17);
	output[33] = (double)(value >> 19);
}

static void unpack_34_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 442 bits of data.
	// Input must be at least 56 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 382 to 394
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xe0) << 14);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 395 to 407
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 408 to 420
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xf8) << 16);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 421 to 433
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xc0) << 13);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 434 to 446
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xfe) << 18);
	output[33] = (double)(value >> 19);
}

static void unpack_34_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 442 bits of data.
	// Input must be at least 56 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 382 to 394
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xe0) << 14);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 395 to 407
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 408 to 420
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xf8) << 16);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 421 to 433
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xc0) << 13);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 434 to 446
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xfe) << 18);
	output[33] = (double)(value >> 19);
}

static void unpack_34_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 442 bits of data.
	// Input must be at least 56 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 383 to 395
	value = ((uint32_t)(input[47] & 0x01) << 31) |
			((uint32_t)input[48] << 23) |
			((uint32_t)(input[49] & 0xf0) << 15);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 396 to 408
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0x80) << 12);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 409 to 421
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xfc) << 17);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 422 to 434
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xe0) << 14);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 435 to 447
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19);
	output[33] = (double)(value >> 19);
}

static void unpack_34_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 442 bits of data.
	// Input must be at least 56 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 383 to 395
	value = ((uint32_t)(input[47] & 0x01) << 31) |
			((uint32_t)input[48] << 23) |
			((uint32_t)(input[49] & 0xf0) << 15);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 396 to 408
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0x80) << 12);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 409 to 421
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xfc) << 17);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 422 to 434
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xe0) << 14);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 435 to 447
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19);
	output[33] = (double)(value >> 19);
}

static void unpack_34_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 442 bits of data.
	// Input must be at least 57 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 384 to 396
	value = ((uint32_t)input[48] << 24) |
			((uint32_t)(input[49] & 0xf8) << 16);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 397 to 409
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xc0) << 13);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 410 to 422
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xfe) << 18);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 423 to 435
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xf0) << 15);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 436 to 448
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0x80) << 12);
	output[33] = (double)(value >> 19);
}

static void unpack_34_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 34 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 442 bits of data.
	// Input must be at least 57 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 384 to 396
	value = ((uint32_t)input[48] << 24) |
			((uint32_t)(input[49] & 0xf8) << 16);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 397 to 409
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xc0) << 13);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 410 to 422
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xfe) << 18);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 423 to 435
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xf0) << 15);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 436 to 448
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0x80) << 12);
	output[33] = (double)(value >> 19);
}

static void unpack_35_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 455 bits of data.
	// Input must be at least 57 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 377 to 389
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xfc) << 17);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 390 to 402
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xe0) << 14);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 403 to 415
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)input[51] << 19);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 416 to 428
	value = ((uint32_t)input[52] << 24) |
			((uint32_t)(input[53] & 0xf8) << 16);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 429 to 441
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21) |
			((uint32_t)(input[55] & 0xc0) << 13);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 442 to 454
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xfe) << 18);
	output[34] = (double)(value >> 19);
}

static void unpack_35_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 455 bits of data.
	// Input must be at least 57 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 377 to 389
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xfc) << 17);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 390 to 402
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xe0) << 14);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 403 to 415
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)input[51] << 19);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 416 to 428
	value = ((uint32_t)input[52] << 24) |
			((uint32_t)(input[53] & 0xf8) << 16);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 429 to 441
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21) |
			((uint32_t)(input[55] & 0xc0) << 13);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 442 to 454
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xfe) << 18);
	output[34] = (double)(value >> 19);
}

static void unpack_35_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 455 bits of data.
	// Input must be at least 57 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 378 to 390
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xfe) << 18);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 391 to 403
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xf0) << 15);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 404 to 416
	value = ((uint32_t)(input[50] & 0x0f) << 28) |
			((uint32_t)input[51] << 20) |
			((uint32_t)(input[52] & 0x80) << 12);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 417 to 429
	value = ((uint32_t)(input[52] & 0x7f) << 25) |
			((uint32_t)(input[53] & 0xfc) << 17);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 430 to 442
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0xe0) << 14);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 443 to 455
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)input[56] << 19);
	output[34] = (double)(value >> 19);
}

static void unpack_35_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 455 bits of data.
	// Input must be at least 57 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 378 to 390
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xfe) << 18);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 391 to 403
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xf0) << 15);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 404 to 416
	value = ((uint32_t)(input[50] & 0x0f) << 28) |
			((uint32_t)input[51] << 20) |
			((uint32_t)(input[52] & 0x80) << 12);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 417 to 429
	value = ((uint32_t)(input[52] & 0x7f) << 25) |
			((uint32_t)(input[53] & 0xfc) << 17);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 430 to 442
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0xe0) << 14);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 443 to 455
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)input[56] << 19);
	output[34] = (double)(value >> 19);
}

static void unpack_35_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 455 bits of data.
	// Input must be at least 58 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 379 to 391
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 392 to 404
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xf8) << 16);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 405 to 417
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xc0) << 13);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 418 to 430
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xfe) << 18);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 431 to 443
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xf0) << 15);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 444 to 456
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)input[56] << 20) |
			((uint32_t)(input[57] & 0x80) << 12);
	output[34] = (double)(value >> 19);
}

static void unpack_35_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 455 bits of data.
	// Input must be at least 58 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 379 to 391
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 392 to 404
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xf8) << 16);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 405 to 417
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xc0) << 13);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 418 to 430
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xfe) << 18);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 431 to 443
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xf0) << 15);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 444 to 456
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)input[56] << 20) |
			((uint32_t)(input[57] & 0x80) << 12);
	output[34] = (double)(value >> 19);
}

static void unpack_35_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 455 bits of data.
	// Input must be at least 58 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 380 to 392
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0x80) << 12);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 393 to 405
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)(input[50] & 0xfc) << 17);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 406 to 418
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xe0) << 14);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 419 to 431
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)input[53] << 19);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 432 to 444
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xf8) << 16);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 445 to 457
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21) |
			((uint32_t)(input[57] & 0xc0) << 13);
	output[34] = (double)(value >> 19);
}

static void unpack_35_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 455 bits of data.
	// Input must be at least 58 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 380 to 392
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0x80) << 12);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 393 to 405
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)(input[50] & 0xfc) << 17);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 406 to 418
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xe0) << 14);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 419 to 431
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)input[53] << 19);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 432 to 444
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xf8) << 16);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 445 to 457
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21) |
			((uint32_t)(input[57] & 0xc0) << 13);
	output[34] = (double)(value >> 19);
}

static void unpack_35_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 455 bits of data.
	// Input must be at least 58 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 381 to 393
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xc0) << 13);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 394 to 406
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)(input[50] & 0xfe) << 18);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 407 to 419
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xf0) << 15);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 420 to 432
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0x80) << 12);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 433 to 445
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xfc) << 17);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 446 to 458
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0xe0) << 14);
	output[34] = (double)(value >> 19);
}

static void unpack_35_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 455 bits of data.
	// Input must be at least 58 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 381 to 393
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xc0) << 13);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 394 to 406
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)(input[50] & 0xfe) << 18);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 407 to 419
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xf0) << 15);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 420 to 432
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0x80) << 12);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 433 to 445
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xfc) << 17);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 446 to 458
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0xe0) << 14);
	output[34] = (double)(value >> 19);
}

static void unpack_35_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 455 bits of data.
	// Input must be at least 58 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 382 to 394
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xe0) << 14);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 395 to 407
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 408 to 420
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xf8) << 16);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 421 to 433
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xc0) << 13);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 434 to 446
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xfe) << 18);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 447 to 459
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xf0) << 15);
	output[34] = (double)(value >> 19);
}

static void unpack_35_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 455 bits of data.
	// Input must be at least 58 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 382 to 394
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xe0) << 14);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 395 to 407
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 408 to 420
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xf8) << 16);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 421 to 433
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xc0) << 13);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 434 to 446
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xfe) << 18);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 447 to 459
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xf0) << 15);
	output[34] = (double)(value >> 19);
}

static void unpack_35_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 455 bits of data.
	// Input must be at least 58 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 383 to 395
	value = ((uint32_t)(input[47] & 0x01) << 31) |
			((uint32_t)input[48] << 23) |
			((uint32_t)(input[49] & 0xf0) << 15);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 396 to 408
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0x80) << 12);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 409 to 421
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xfc) << 17);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 422 to 434
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xe0) << 14);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 435 to 447
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 448 to 460
	value = ((uint32_t)input[56] << 24) |
			((uint32_t)(input[57] & 0xf8) << 16);
	output[34] = (double)(value >> 19);
}

static void unpack_35_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 455 bits of data.
	// Input must be at least 58 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 383 to 395
	value = ((uint32_t)(input[47] & 0x01) << 31) |
			((uint32_t)input[48] << 23) |
			((uint32_t)(input[49] & 0xf0) << 15);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 396 to 408
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0x80) << 12);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 409 to 421
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xfc) << 17);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 422 to 434
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xe0) << 14);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 435 to 447
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 448 to 460
	value = ((uint32_t)input[56] << 24) |
			((uint32_t)(input[57] & 0xf8) << 16);
	output[34] = (double)(value >> 19);
}

static void unpack_35_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 455 bits of data.
	// Input must be at least 58 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 384 to 396
	value = ((uint32_t)input[48] << 24) |
			((uint32_t)(input[49] & 0xf8) << 16);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 397 to 409
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xc0) << 13);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 410 to 422
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xfe) << 18);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 423 to 435
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xf0) << 15);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 436 to 448
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0x80) << 12);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 449 to 461
	value = ((uint32_t)(input[56] & 0x7f) << 25) |
			((uint32_t)(input[57] & 0xfc) << 17);
	output[34] = (double)(value >> 19);
}

static void unpack_35_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 35 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 455 bits of data.
	// Input must be at least 58 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 384 to 396
	value = ((uint32_t)input[48] << 24) |
			((uint32_t)(input[49] & 0xf8) << 16);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 397 to 409
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xc0) << 13);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 410 to 422
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xfe) << 18);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 423 to 435
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xf0) << 15);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 436 to 448
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0x80) << 12);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 449 to 461
	value = ((uint32_t)(input[56] & 0x7f) << 25) |
			((uint32_t)(input[57] & 0xfc) << 17);
	output[34] = (double)(value >> 19);
}

static void unpack_36_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 468 bits of data.
	// Input must be at least 59 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 377 to 389
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xfc) << 17);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 390 to 402
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xe0) << 14);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 403 to 415
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)input[51] << 19);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 416 to 428
	value = ((uint32_t)input[52] << 24) |
			((uint32_t)(input[53] & 0xf8) << 16);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 429 to 441
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21) |
			((uint32_t)(input[55] & 0xc0) << 13);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 442 to 454
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xfe) << 18);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 455 to 467
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xf0) << 15);
	output[35] = (double)(value >> 19);
}

static void unpack_36_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 468 bits of data.
	// Input must be at least 59 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 377 to 389
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xfc) << 17);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 390 to 402
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xe0) << 14);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 403 to 415
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)input[51] << 19);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 416 to 428
	value = ((uint32_t)input[52] << 24) |
			((uint32_t)(input[53] & 0xf8) << 16);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 429 to 441
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21) |
			((uint32_t)(input[55] & 0xc0) << 13);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 442 to 454
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xfe) << 18);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 455 to 467
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xf0) << 15);
	output[35] = (double)(value >> 19);
}

static void unpack_36_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 468 bits of data.
	// Input must be at least 59 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 378 to 390
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xfe) << 18);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 391 to 403
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xf0) << 15);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 404 to 416
	value = ((uint32_t)(input[50] & 0x0f) << 28) |
			((uint32_t)input[51] << 20) |
			((uint32_t)(input[52] & 0x80) << 12);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 417 to 429
	value = ((uint32_t)(input[52] & 0x7f) << 25) |
			((uint32_t)(input[53] & 0xfc) << 17);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 430 to 442
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0xe0) << 14);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 443 to 455
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)input[56] << 19);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 456 to 468
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xf8) << 16);
	output[35] = (double)(value >> 19);
}

static void unpack_36_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 468 bits of data.
	// Input must be at least 59 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 378 to 390
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xfe) << 18);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 391 to 403
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xf0) << 15);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 404 to 416
	value = ((uint32_t)(input[50] & 0x0f) << 28) |
			((uint32_t)input[51] << 20) |
			((uint32_t)(input[52] & 0x80) << 12);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 417 to 429
	value = ((uint32_t)(input[52] & 0x7f) << 25) |
			((uint32_t)(input[53] & 0xfc) << 17);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 430 to 442
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0xe0) << 14);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 443 to 455
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)input[56] << 19);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 456 to 468
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xf8) << 16);
	output[35] = (double)(value >> 19);
}

static void unpack_36_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 468 bits of data.
	// Input must be at least 59 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 379 to 391
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 392 to 404
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xf8) << 16);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 405 to 417
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xc0) << 13);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 418 to 430
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xfe) << 18);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 431 to 443
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xf0) << 15);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 444 to 456
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)input[56] << 20) |
			((uint32_t)(input[57] & 0x80) << 12);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 457 to 469
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)(input[58] & 0xfc) << 17);
	output[35] = (double)(value >> 19);
}

static void unpack_36_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 468 bits of data.
	// Input must be at least 59 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 379 to 391
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 392 to 404
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xf8) << 16);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 405 to 417
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xc0) << 13);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 418 to 430
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xfe) << 18);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 431 to 443
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xf0) << 15);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 444 to 456
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)input[56] << 20) |
			((uint32_t)(input[57] & 0x80) << 12);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 457 to 469
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)(input[58] & 0xfc) << 17);
	output[35] = (double)(value >> 19);
}

static void unpack_36_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 468 bits of data.
	// Input must be at least 59 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 380 to 392
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0x80) << 12);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 393 to 405
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)(input[50] & 0xfc) << 17);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 406 to 418
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xe0) << 14);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 419 to 431
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)input[53] << 19);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 432 to 444
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xf8) << 16);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 445 to 457
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21) |
			((uint32_t)(input[57] & 0xc0) << 13);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 458 to 470
	value = ((uint32_t)(input[57] & 0x3f) << 26) |
			((uint32_t)(input[58] & 0xfe) << 18);
	output[35] = (double)(value >> 19);
}

static void unpack_36_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 468 bits of data.
	// Input must be at least 59 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 380 to 392
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0x80) << 12);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 393 to 405
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)(input[50] & 0xfc) << 17);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 406 to 418
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xe0) << 14);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 419 to 431
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)input[53] << 19);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 432 to 444
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xf8) << 16);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 445 to 457
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21) |
			((uint32_t)(input[57] & 0xc0) << 13);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 458 to 470
	value = ((uint32_t)(input[57] & 0x3f) << 26) |
			((uint32_t)(input[58] & 0xfe) << 18);
	output[35] = (double)(value >> 19);
}

static void unpack_36_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 468 bits of data.
	// Input must be at least 59 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 381 to 393
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xc0) << 13);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 394 to 406
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)(input[50] & 0xfe) << 18);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 407 to 419
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xf0) << 15);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 420 to 432
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0x80) << 12);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 433 to 445
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xfc) << 17);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 446 to 458
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0xe0) << 14);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 459 to 471
	value = ((uint32_t)(input[57] & 0x1f) << 27) |
			((uint32_t)input[58] << 19);
	output[35] = (double)(value >> 19);
}

static void unpack_36_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 468 bits of data.
	// Input must be at least 59 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 381 to 393
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xc0) << 13);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 394 to 406
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)(input[50] & 0xfe) << 18);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 407 to 419
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xf0) << 15);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 420 to 432
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0x80) << 12);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 433 to 445
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xfc) << 17);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 446 to 458
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0xe0) << 14);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 459 to 471
	value = ((uint32_t)(input[57] & 0x1f) << 27) |
			((uint32_t)input[58] << 19);
	output[35] = (double)(value >> 19);
}

static void unpack_36_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 468 bits of data.
	// Input must be at least 60 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 382 to 394
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xe0) << 14);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 395 to 407
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 408 to 420
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xf8) << 16);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 421 to 433
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xc0) << 13);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 434 to 446
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xfe) << 18);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 447 to 459
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xf0) << 15);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 460 to 472
	value = ((uint32_t)(input[57] & 0x0f) << 28) |
			((uint32_t)input[58] << 20) |
			((uint32_t)(input[59] & 0x80) << 12);
	output[35] = (double)(value >> 19);
}

static void unpack_36_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 468 bits of data.
	// Input must be at least 60 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 382 to 394
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xe0) << 14);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 395 to 407
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 408 to 420
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xf8) << 16);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 421 to 433
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xc0) << 13);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 434 to 446
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xfe) << 18);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 447 to 459
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xf0) << 15);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 460 to 472
	value = ((uint32_t)(input[57] & 0x0f) << 28) |
			((uint32_t)input[58] << 20) |
			((uint32_t)(input[59] & 0x80) << 12);
	output[35] = (double)(value >> 19);
}

static void unpack_36_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 468 bits of data.
	// Input must be at least 60 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 383 to 395
	value = ((uint32_t)(input[47] & 0x01) << 31) |
			((uint32_t)input[48] << 23) |
			((uint32_t)(input[49] & 0xf0) << 15);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 396 to 408
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0x80) << 12);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 409 to 421
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xfc) << 17);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 422 to 434
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xe0) << 14);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 435 to 447
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 448 to 460
	value = ((uint32_t)input[56] << 24) |
			((uint32_t)(input[57] & 0xf8) << 16);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 461 to 473
	value = ((uint32_t)(input[57] & 0x07) << 29) |
			((uint32_t)input[58] << 21) |
			((uint32_t)(input[59] & 0xc0) << 13);
	output[35] = (double)(value >> 19);
}

static void unpack_36_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 468 bits of data.
	// Input must be at least 60 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 383 to 395
	value = ((uint32_t)(input[47] & 0x01) << 31) |
			((uint32_t)input[48] << 23) |
			((uint32_t)(input[49] & 0xf0) << 15);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 396 to 408
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0x80) << 12);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 409 to 421
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xfc) << 17);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 422 to 434
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xe0) << 14);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 435 to 447
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 448 to 460
	value = ((uint32_t)input[56] << 24) |
			((uint32_t)(input[57] & 0xf8) << 16);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 461 to 473
	value = ((uint32_t)(input[57] & 0x07) << 29) |
			((uint32_t)input[58] << 21) |
			((uint32_t)(input[59] & 0xc0) << 13);
	output[35] = (double)(value >> 19);
}

static void unpack_36_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 468 bits of data.
	// Input must be at least 60 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 384 to 396
	value = ((uint32_t)input[48] << 24) |
			((uint32_t)(input[49] & 0xf8) << 16);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 397 to 409
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xc0) << 13);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 410 to 422
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xfe) << 18);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 423 to 435
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xf0) << 15);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 436 to 448
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0x80) << 12);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 449 to 461
	value = ((uint32_t)(input[56] & 0x7f) << 25) |
			((uint32_t)(input[57] & 0xfc) << 17);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 462 to 474
	value = ((uint32_t)(input[57] & 0x03) << 30) |
			((uint32_t)input[58] << 22) |
			((uint32_t)(input[59] & 0xe0) << 14);
	output[35] = (double)(value >> 19);
}

static void unpack_36_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 36 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 468 bits of data.
	// Input must be at least 60 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 384 to 396
	value = ((uint32_t)input[48] << 24) |
			((uint32_t)(input[49] & 0xf8) << 16);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 397 to 409
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xc0) << 13);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 410 to 422
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xfe) << 18);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 423 to 435
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xf0) << 15);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 436 to 448
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0x80) << 12);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 449 to 461
	value = ((uint32_t)(input[56] & 0x7f) << 25) |
			((uint32_t)(input[57] & 0xfc) << 17);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 462 to 474
	value = ((uint32_t)(input[57] & 0x03) << 30) |
			((uint32_t)input[58] << 22) |
			((uint32_t)(input[59] & 0xe0) << 14);
	output[35] = (double)(value >> 19);
}

static void unpack_37_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 481 bits of data.
	// Input must be at least 61 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 377 to 389
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xfc) << 17);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 390 to 402
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xe0) << 14);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 403 to 415
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)input[51] << 19);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 416 to 428
	value = ((uint32_t)input[52] << 24) |
			((uint32_t)(input[53] & 0xf8) << 16);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 429 to 441
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21) |
			((uint32_t)(input[55] & 0xc0) << 13);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 442 to 454
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xfe) << 18);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 455 to 467
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xf0) << 15);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 468 to 480
	value = ((uint32_t)(input[58] & 0x0f) << 28) |
			((uint32_t)input[59] << 20) |
			((uint32_t)(input[60] & 0x80) << 12);
	output[36] = (double)(value >> 19);
}

static void unpack_37_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 481 bits of data.
	// Input must be at least 61 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 377 to 389
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xfc) << 17);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 390 to 402
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xe0) << 14);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 403 to 415
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)input[51] << 19);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 416 to 428
	value = ((uint32_t)input[52] << 24) |
			((uint32_t)(input[53] & 0xf8) << 16);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 429 to 441
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21) |
			((uint32_t)(input[55] & 0xc0) << 13);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 442 to 454
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xfe) << 18);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 455 to 467
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xf0) << 15);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 468 to 480
	value = ((uint32_t)(input[58] & 0x0f) << 28) |
			((uint32_t)input[59] << 20) |
			((uint32_t)(input[60] & 0x80) << 12);
	output[36] = (double)(value >> 19);
}

static void unpack_37_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 481 bits of data.
	// Input must be at least 61 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 378 to 390
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xfe) << 18);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 391 to 403
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xf0) << 15);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 404 to 416
	value = ((uint32_t)(input[50] & 0x0f) << 28) |
			((uint32_t)input[51] << 20) |
			((uint32_t)(input[52] & 0x80) << 12);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 417 to 429
	value = ((uint32_t)(input[52] & 0x7f) << 25) |
			((uint32_t)(input[53] & 0xfc) << 17);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 430 to 442
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0xe0) << 14);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 443 to 455
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)input[56] << 19);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 456 to 468
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xf8) << 16);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 469 to 481
	value = ((uint32_t)(input[58] & 0x07) << 29) |
			((uint32_t)input[59] << 21) |
			((uint32_t)(input[60] & 0xc0) << 13);
	output[36] = (double)(value >> 19);
}

static void unpack_37_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 481 bits of data.
	// Input must be at least 61 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 378 to 390
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xfe) << 18);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 391 to 403
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xf0) << 15);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 404 to 416
	value = ((uint32_t)(input[50] & 0x0f) << 28) |
			((uint32_t)input[51] << 20) |
			((uint32_t)(input[52] & 0x80) << 12);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 417 to 429
	value = ((uint32_t)(input[52] & 0x7f) << 25) |
			((uint32_t)(input[53] & 0xfc) << 17);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 430 to 442
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0xe0) << 14);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 443 to 455
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)input[56] << 19);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 456 to 468
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xf8) << 16);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 469 to 481
	value = ((uint32_t)(input[58] & 0x07) << 29) |
			((uint32_t)input[59] << 21) |
			((uint32_t)(input[60] & 0xc0) << 13);
	output[36] = (double)(value >> 19);
}

static void unpack_37_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 481 bits of data.
	// Input must be at least 61 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 379 to 391
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 392 to 404
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xf8) << 16);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 405 to 417
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xc0) << 13);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 418 to 430
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xfe) << 18);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 431 to 443
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xf0) << 15);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 444 to 456
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)input[56] << 20) |
			((uint32_t)(input[57] & 0x80) << 12);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 457 to 469
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)(input[58] & 0xfc) << 17);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 470 to 482
	value = ((uint32_t)(input[58] & 0x03) << 30) |
			((uint32_t)input[59] << 22) |
			((uint32_t)(input[60] & 0xe0) << 14);
	output[36] = (double)(value >> 19);
}

static void unpack_37_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 481 bits of data.
	// Input must be at least 61 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 379 to 391
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 392 to 404
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xf8) << 16);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 405 to 417
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xc0) << 13);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 418 to 430
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xfe) << 18);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 431 to 443
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xf0) << 15);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 444 to 456
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)input[56] << 20) |
			((uint32_t)(input[57] & 0x80) << 12);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 457 to 469
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)(input[58] & 0xfc) << 17);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 470 to 482
	value = ((uint32_t)(input[58] & 0x03) << 30) |
			((uint32_t)input[59] << 22) |
			((uint32_t)(input[60] & 0xe0) << 14);
	output[36] = (double)(value >> 19);
}

static void unpack_37_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 481 bits of data.
	// Input must be at least 61 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 380 to 392
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0x80) << 12);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 393 to 405
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)(input[50] & 0xfc) << 17);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 406 to 418
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xe0) << 14);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 419 to 431
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)input[53] << 19);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 432 to 444
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xf8) << 16);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 445 to 457
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21) |
			((uint32_t)(input[57] & 0xc0) << 13);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 458 to 470
	value = ((uint32_t)(input[57] & 0x3f) << 26) |
			((uint32_t)(input[58] & 0xfe) << 18);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 471 to 483
	value = ((uint32_t)(input[58] & 0x01) << 31) |
			((uint32_t)input[59] << 23) |
			((uint32_t)(input[60] & 0xf0) << 15);
	output[36] = (double)(value >> 19);
}

static void unpack_37_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 481 bits of data.
	// Input must be at least 61 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 380 to 392
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0x80) << 12);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 393 to 405
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)(input[50] & 0xfc) << 17);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 406 to 418
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xe0) << 14);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 419 to 431
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)input[53] << 19);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 432 to 444
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xf8) << 16);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 445 to 457
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21) |
			((uint32_t)(input[57] & 0xc0) << 13);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 458 to 470
	value = ((uint32_t)(input[57] & 0x3f) << 26) |
			((uint32_t)(input[58] & 0xfe) << 18);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 471 to 483
	value = ((uint32_t)(input[58] & 0x01) << 31) |
			((uint32_t)input[59] << 23) |
			((uint32_t)(input[60] & 0xf0) << 15);
	output[36] = (double)(value >> 19);
}

static void unpack_37_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 481 bits of data.
	// Input must be at least 61 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 381 to 393
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xc0) << 13);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 394 to 406
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)(input[50] & 0xfe) << 18);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 407 to 419
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xf0) << 15);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 420 to 432
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0x80) << 12);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 433 to 445
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xfc) << 17);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 446 to 458
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0xe0) << 14);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 459 to 471
	value = ((uint32_t)(input[57] & 0x1f) << 27) |
			((uint32_t)input[58] << 19);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 472 to 484
	value = ((uint32_t)input[59] << 24) |
			((uint32_t)(input[60] & 0xf8) << 16);
	output[36] = (double)(value >> 19);
}

static void unpack_37_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 481 bits of data.
	// Input must be at least 61 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 381 to 393
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xc0) << 13);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 394 to 406
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)(input[50] & 0xfe) << 18);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 407 to 419
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xf0) << 15);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 420 to 432
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0x80) << 12);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 433 to 445
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xfc) << 17);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 446 to 458
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0xe0) << 14);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 459 to 471
	value = ((uint32_t)(input[57] & 0x1f) << 27) |
			((uint32_t)input[58] << 19);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 472 to 484
	value = ((uint32_t)input[59] << 24) |
			((uint32_t)(input[60] & 0xf8) << 16);
	output[36] = (double)(value >> 19);
}

static void unpack_37_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 481 bits of data.
	// Input must be at least 61 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 382 to 394
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xe0) << 14);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 395 to 407
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 408 to 420
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xf8) << 16);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 421 to 433
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xc0) << 13);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 434 to 446
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xfe) << 18);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 447 to 459
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xf0) << 15);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 460 to 472
	value = ((uint32_t)(input[57] & 0x0f) << 28) |
			((uint32_t)input[58] << 20) |
			((uint32_t)(input[59] & 0x80) << 12);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 473 to 485
	value = ((uint32_t)(input[59] & 0x7f) << 25) |
			((uint32_t)(input[60] & 0xfc) << 17);
	output[36] = (double)(value >> 19);
}

static void unpack_37_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 481 bits of data.
	// Input must be at least 61 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 382 to 394
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xe0) << 14);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 395 to 407
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 408 to 420
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xf8) << 16);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 421 to 433
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xc0) << 13);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 434 to 446
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xfe) << 18);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 447 to 459
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xf0) << 15);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 460 to 472
	value = ((uint32_t)(input[57] & 0x0f) << 28) |
			((uint32_t)input[58] << 20) |
			((uint32_t)(input[59] & 0x80) << 12);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 473 to 485
	value = ((uint32_t)(input[59] & 0x7f) << 25) |
			((uint32_t)(input[60] & 0xfc) << 17);
	output[36] = (double)(value >> 19);
}

static void unpack_37_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 481 bits of data.
	// Input must be at least 61 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 383 to 395
	value = ((uint32_t)(input[47] & 0x01) << 31) |
			((uint32_t)input[48] << 23) |
			((uint32_t)(input[49] & 0xf0) << 15);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 396 to 408
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0x80) << 12);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 409 to 421
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xfc) << 17);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 422 to 434
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xe0) << 14);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 435 to 447
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 448 to 460
	value = ((uint32_t)input[56] << 24) |
			((uint32_t)(input[57] & 0xf8) << 16);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 461 to 473
	value = ((uint32_t)(input[57] & 0x07) << 29) |
			((uint32_t)input[58] << 21) |
			((uint32_t)(input[59] & 0xc0) << 13);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 474 to 486
	value = ((uint32_t)(input[59] & 0x3f) << 26) |
			((uint32_t)(input[60] & 0xfe) << 18);
	output[36] = (double)(value >> 19);
}

static void unpack_37_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 481 bits of data.
	// Input must be at least 61 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 383 to 395
	value = ((uint32_t)(input[47] & 0x01) << 31) |
			((uint32_t)input[48] << 23) |
			((uint32_t)(input[49] & 0xf0) << 15);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 396 to 408
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0x80) << 12);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 409 to 421
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xfc) << 17);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 422 to 434
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xe0) << 14);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 435 to 447
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 448 to 460
	value = ((uint32_t)input[56] << 24) |
			((uint32_t)(input[57] & 0xf8) << 16);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 461 to 473
	value = ((uint32_t)(input[57] & 0x07) << 29) |
			((uint32_t)input[58] << 21) |
			((uint32_t)(input[59] & 0xc0) << 13);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 474 to 486
	value = ((uint32_t)(input[59] & 0x3f) << 26) |
			((uint32_t)(input[60] & 0xfe) << 18);
	output[36] = (double)(value >> 19);
}

static void unpack_37_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 481 bits of data.
	// Input must be at least 61 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 384 to 396
	value = ((uint32_t)input[48] << 24) |
			((uint32_t)(input[49] & 0xf8) << 16);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 397 to 409
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xc0) << 13);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 410 to 422
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xfe) << 18);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 423 to 435
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xf0) << 15);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 436 to 448
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0x80) << 12);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 449 to 461
	value = ((uint32_t)(input[56] & 0x7f) << 25) |
			((uint32_t)(input[57] & 0xfc) << 17);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 462 to 474
	value = ((uint32_t)(input[57] & 0x03) << 30) |
			((uint32_t)input[58] << 22) |
			((uint32_t)(input[59] & 0xe0) << 14);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 475 to 487
	value = ((uint32_t)(input[59] & 0x1f) << 27) |
			((uint32_t)input[60] << 19);
	output[36] = (double)(value >> 19);
}

static void unpack_37_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 37 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 481 bits of data.
	// Input must be at least 61 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 384 to 396
	value = ((uint32_t)input[48] << 24) |
			((uint32_t)(input[49] & 0xf8) << 16);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 397 to 409
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xc0) << 13);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 410 to 422
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xfe) << 18);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 423 to 435
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xf0) << 15);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 436 to 448
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0x80) << 12);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 449 to 461
	value = ((uint32_t)(input[56] & 0x7f) << 25) |
			((uint32_t)(input[57] & 0xfc) << 17);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 462 to 474
	value = ((uint32_t)(input[57] & 0x03) << 30) |
			((uint32_t)input[58] << 22) |
			((uint32_t)(input[59] & 0xe0) << 14);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 475 to 487
	value = ((uint32_t)(input[59] & 0x1f) << 27) |
			((uint32_t)input[60] << 19);
	output[36] = (double)(value >> 19);
}

static void unpack_38_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 494 bits of data.
	// Input must be at least 62 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 377 to 389
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xfc) << 17);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 390 to 402
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xe0) << 14);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 403 to 415
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)input[51] << 19);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 416 to 428
	value = ((uint32_t)input[52] << 24) |
			((uint32_t)(input[53] & 0xf8) << 16);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 429 to 441
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21) |
			((uint32_t)(input[55] & 0xc0) << 13);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 442 to 454
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xfe) << 18);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 455 to 467
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xf0) << 15);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 468 to 480
	value = ((uint32_t)(input[58] & 0x0f) << 28) |
			((uint32_t)input[59] << 20) |
			((uint32_t)(input[60] & 0x80) << 12);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 481 to 493
	value = ((uint32_t)(input[60] & 0x7f) << 25) |
			((uint32_t)(input[61] & 0xfc) << 17);
	output[37] = (double)(value >> 19);
}

static void unpack_38_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 494 bits of data.
	// Input must be at least 62 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 377 to 389
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xfc) << 17);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 390 to 402
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xe0) << 14);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 403 to 415
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)input[51] << 19);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 416 to 428
	value = ((uint32_t)input[52] << 24) |
			((uint32_t)(input[53] & 0xf8) << 16);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 429 to 441
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21) |
			((uint32_t)(input[55] & 0xc0) << 13);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 442 to 454
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xfe) << 18);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 455 to 467
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xf0) << 15);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 468 to 480
	value = ((uint32_t)(input[58] & 0x0f) << 28) |
			((uint32_t)input[59] << 20) |
			((uint32_t)(input[60] & 0x80) << 12);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 481 to 493
	value = ((uint32_t)(input[60] & 0x7f) << 25) |
			((uint32_t)(input[61] & 0xfc) << 17);
	output[37] = (double)(value >> 19);
}

static void unpack_38_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 494 bits of data.
	// Input must be at least 62 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 378 to 390
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xfe) << 18);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 391 to 403
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xf0) << 15);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 404 to 416
	value = ((uint32_t)(input[50] & 0x0f) << 28) |
			((uint32_t)input[51] << 20) |
			((uint32_t)(input[52] & 0x80) << 12);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 417 to 429
	value = ((uint32_t)(input[52] & 0x7f) << 25) |
			((uint32_t)(input[53] & 0xfc) << 17);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 430 to 442
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0xe0) << 14);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 443 to 455
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)input[56] << 19);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 456 to 468
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xf8) << 16);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 469 to 481
	value = ((uint32_t)(input[58] & 0x07) << 29) |
			((uint32_t)input[59] << 21) |
			((uint32_t)(input[60] & 0xc0) << 13);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 482 to 494
	value = ((uint32_t)(input[60] & 0x3f) << 26) |
			((uint32_t)(input[61] & 0xfe) << 18);
	output[37] = (double)(value >> 19);
}

static void unpack_38_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 494 bits of data.
	// Input must be at least 62 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 378 to 390
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xfe) << 18);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 391 to 403
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xf0) << 15);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 404 to 416
	value = ((uint32_t)(input[50] & 0x0f) << 28) |
			((uint32_t)input[51] << 20) |
			((uint32_t)(input[52] & 0x80) << 12);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 417 to 429
	value = ((uint32_t)(input[52] & 0x7f) << 25) |
			((uint32_t)(input[53] & 0xfc) << 17);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 430 to 442
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0xe0) << 14);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 443 to 455
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)input[56] << 19);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 456 to 468
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xf8) << 16);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 469 to 481
	value = ((uint32_t)(input[58] & 0x07) << 29) |
			((uint32_t)input[59] << 21) |
			((uint32_t)(input[60] & 0xc0) << 13);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 482 to 494
	value = ((uint32_t)(input[60] & 0x3f) << 26) |
			((uint32_t)(input[61] & 0xfe) << 18);
	output[37] = (double)(value >> 19);
}

static void unpack_38_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 494 bits of data.
	// Input must be at least 62 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 379 to 391
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 392 to 404
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xf8) << 16);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 405 to 417
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xc0) << 13);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 418 to 430
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xfe) << 18);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 431 to 443
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xf0) << 15);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 444 to 456
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)input[56] << 20) |
			((uint32_t)(input[57] & 0x80) << 12);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 457 to 469
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)(input[58] & 0xfc) << 17);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 470 to 482
	value = ((uint32_t)(input[58] & 0x03) << 30) |
			((uint32_t)input[59] << 22) |
			((uint32_t)(input[60] & 0xe0) << 14);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 483 to 495
	value = ((uint32_t)(input[60] & 0x1f) << 27) |
			((uint32_t)input[61] << 19);
	output[37] = (double)(value >> 19);
}

static void unpack_38_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 494 bits of data.
	// Input must be at least 62 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 379 to 391
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 392 to 404
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xf8) << 16);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 405 to 417
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xc0) << 13);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 418 to 430
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xfe) << 18);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 431 to 443
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xf0) << 15);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 444 to 456
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)input[56] << 20) |
			((uint32_t)(input[57] & 0x80) << 12);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 457 to 469
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)(input[58] & 0xfc) << 17);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 470 to 482
	value = ((uint32_t)(input[58] & 0x03) << 30) |
			((uint32_t)input[59] << 22) |
			((uint32_t)(input[60] & 0xe0) << 14);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 483 to 495
	value = ((uint32_t)(input[60] & 0x1f) << 27) |
			((uint32_t)input[61] << 19);
	output[37] = (double)(value >> 19);
}

static void unpack_38_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 494 bits of data.
	// Input must be at least 63 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 380 to 392
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0x80) << 12);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 393 to 405
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)(input[50] & 0xfc) << 17);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 406 to 418
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xe0) << 14);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 419 to 431
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)input[53] << 19);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 432 to 444
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xf8) << 16);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 445 to 457
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21) |
			((uint32_t)(input[57] & 0xc0) << 13);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 458 to 470
	value = ((uint32_t)(input[57] & 0x3f) << 26) |
			((uint32_t)(input[58] & 0xfe) << 18);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 471 to 483
	value = ((uint32_t)(input[58] & 0x01) << 31) |
			((uint32_t)input[59] << 23) |
			((uint32_t)(input[60] & 0xf0) << 15);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 484 to 496
	value = ((uint32_t)(input[60] & 0x0f) << 28) |
			((uint32_t)input[61] << 20) |
			((uint32_t)(input[62] & 0x80) << 12);
	output[37] = (double)(value >> 19);
}

static void unpack_38_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 494 bits of data.
	// Input must be at least 63 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 380 to 392
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0x80) << 12);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 393 to 405
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)(input[50] & 0xfc) << 17);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 406 to 418
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xe0) << 14);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 419 to 431
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)input[53] << 19);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 432 to 444
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xf8) << 16);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 445 to 457
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21) |
			((uint32_t)(input[57] & 0xc0) << 13);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 458 to 470
	value = ((uint32_t)(input[57] & 0x3f) << 26) |
			((uint32_t)(input[58] & 0xfe) << 18);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 471 to 483
	value = ((uint32_t)(input[58] & 0x01) << 31) |
			((uint32_t)input[59] << 23) |
			((uint32_t)(input[60] & 0xf0) << 15);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 484 to 496
	value = ((uint32_t)(input[60] & 0x0f) << 28) |
			((uint32_t)input[61] << 20) |
			((uint32_t)(input[62] & 0x80) << 12);
	output[37] = (double)(value >> 19);
}

static void unpack_38_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 494 bits of data.
	// Input must be at least 63 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 381 to 393
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xc0) << 13);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 394 to 406
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)(input[50] & 0xfe) << 18);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 407 to 419
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xf0) << 15);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 420 to 432
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0x80) << 12);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 433 to 445
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xfc) << 17);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 446 to 458
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0xe0) << 14);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 459 to 471
	value = ((uint32_t)(input[57] & 0x1f) << 27) |
			((uint32_t)input[58] << 19);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 472 to 484
	value = ((uint32_t)input[59] << 24) |
			((uint32_t)(input[60] & 0xf8) << 16);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 485 to 497
	value = ((uint32_t)(input[60] & 0x07) << 29) |
			((uint32_t)input[61] << 21) |
			((uint32_t)(input[62] & 0xc0) << 13);
	output[37] = (double)(value >> 19);
}

static void unpack_38_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 494 bits of data.
	// Input must be at least 63 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 381 to 393
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xc0) << 13);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 394 to 406
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)(input[50] & 0xfe) << 18);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 407 to 419
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xf0) << 15);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 420 to 432
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0x80) << 12);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 433 to 445
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xfc) << 17);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 446 to 458
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0xe0) << 14);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 459 to 471
	value = ((uint32_t)(input[57] & 0x1f) << 27) |
			((uint32_t)input[58] << 19);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 472 to 484
	value = ((uint32_t)input[59] << 24) |
			((uint32_t)(input[60] & 0xf8) << 16);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 485 to 497
	value = ((uint32_t)(input[60] & 0x07) << 29) |
			((uint32_t)input[61] << 21) |
			((uint32_t)(input[62] & 0xc0) << 13);
	output[37] = (double)(value >> 19);
}

static void unpack_38_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 494 bits of data.
	// Input must be at least 63 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 382 to 394
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xe0) << 14);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 395 to 407
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 408 to 420
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xf8) << 16);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 421 to 433
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xc0) << 13);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 434 to 446
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xfe) << 18);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 447 to 459
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xf0) << 15);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 460 to 472
	value = ((uint32_t)(input[57] & 0x0f) << 28) |
			((uint32_t)input[58] << 20) |
			((uint32_t)(input[59] & 0x80) << 12);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 473 to 485
	value = ((uint32_t)(input[59] & 0x7f) << 25) |
			((uint32_t)(input[60] & 0xfc) << 17);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 486 to 498
	value = ((uint32_t)(input[60] & 0x03) << 30) |
			((uint32_t)input[61] << 22) |
			((uint32_t)(input[62] & 0xe0) << 14);
	output[37] = (double)(value >> 19);
}

static void unpack_38_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 494 bits of data.
	// Input must be at least 63 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 382 to 394
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xe0) << 14);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 395 to 407
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 408 to 420
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xf8) << 16);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 421 to 433
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xc0) << 13);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 434 to 446
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xfe) << 18);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 447 to 459
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xf0) << 15);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 460 to 472
	value = ((uint32_t)(input[57] & 0x0f) << 28) |
			((uint32_t)input[58] << 20) |
			((uint32_t)(input[59] & 0x80) << 12);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 473 to 485
	value = ((uint32_t)(input[59] & 0x7f) << 25) |
			((uint32_t)(input[60] & 0xfc) << 17);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 486 to 498
	value = ((uint32_t)(input[60] & 0x03) << 30) |
			((uint32_t)input[61] << 22) |
			((uint32_t)(input[62] & 0xe0) << 14);
	output[37] = (double)(value >> 19);
}

static void unpack_38_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 494 bits of data.
	// Input must be at least 63 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 383 to 395
	value = ((uint32_t)(input[47] & 0x01) << 31) |
			((uint32_t)input[48] << 23) |
			((uint32_t)(input[49] & 0xf0) << 15);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 396 to 408
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0x80) << 12);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 409 to 421
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xfc) << 17);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 422 to 434
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xe0) << 14);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 435 to 447
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 448 to 460
	value = ((uint32_t)input[56] << 24) |
			((uint32_t)(input[57] & 0xf8) << 16);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 461 to 473
	value = ((uint32_t)(input[57] & 0x07) << 29) |
			((uint32_t)input[58] << 21) |
			((uint32_t)(input[59] & 0xc0) << 13);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 474 to 486
	value = ((uint32_t)(input[59] & 0x3f) << 26) |
			((uint32_t)(input[60] & 0xfe) << 18);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 487 to 499
	value = ((uint32_t)(input[60] & 0x01) << 31) |
			((uint32_t)input[61] << 23) |
			((uint32_t)(input[62] & 0xf0) << 15);
	output[37] = (double)(value >> 19);
}

static void unpack_38_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 494 bits of data.
	// Input must be at least 63 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 383 to 395
	value = ((uint32_t)(input[47] & 0x01) << 31) |
			((uint32_t)input[48] << 23) |
			((uint32_t)(input[49] & 0xf0) << 15);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 396 to 408
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0x80) << 12);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 409 to 421
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xfc) << 17);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 422 to 434
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xe0) << 14);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 435 to 447
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 448 to 460
	value = ((uint32_t)input[56] << 24) |
			((uint32_t)(input[57] & 0xf8) << 16);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 461 to 473
	value = ((uint32_t)(input[57] & 0x07) << 29) |
			((uint32_t)input[58] << 21) |
			((uint32_t)(input[59] & 0xc0) << 13);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 474 to 486
	value = ((uint32_t)(input[59] & 0x3f) << 26) |
			((uint32_t)(input[60] & 0xfe) << 18);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 487 to 499
	value = ((uint32_t)(input[60] & 0x01) << 31) |
			((uint32_t)input[61] << 23) |
			((uint32_t)(input[62] & 0xf0) << 15);
	output[37] = (double)(value >> 19);
}

static void unpack_38_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 494 bits of data.
	// Input must be at least 63 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 384 to 396
	value = ((uint32_t)input[48] << 24) |
			((uint32_t)(input[49] & 0xf8) << 16);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 397 to 409
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xc0) << 13);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 410 to 422
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xfe) << 18);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 423 to 435
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xf0) << 15);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 436 to 448
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0x80) << 12);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 449 to 461
	value = ((uint32_t)(input[56] & 0x7f) << 25) |
			((uint32_t)(input[57] & 0xfc) << 17);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 462 to 474
	value = ((uint32_t)(input[57] & 0x03) << 30) |
			((uint32_t)input[58] << 22) |
			((uint32_t)(input[59] & 0xe0) << 14);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 475 to 487
	value = ((uint32_t)(input[59] & 0x1f) << 27) |
			((uint32_t)input[60] << 19);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 488 to 500
	value = ((uint32_t)input[61] << 24) |
			((uint32_t)(input[62] & 0xf8) << 16);
	output[37] = (double)(value >> 19);
}

static void unpack_38_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 38 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 494 bits of data.
	// Input must be at least 63 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 384 to 396
	value = ((uint32_t)input[48] << 24) |
			((uint32_t)(input[49] & 0xf8) << 16);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 397 to 409
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xc0) << 13);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 410 to 422
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xfe) << 18);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 423 to 435
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xf0) << 15);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 436 to 448
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0x80) << 12);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 449 to 461
	value = ((uint32_t)(input[56] & 0x7f) << 25) |
			((uint32_t)(input[57] & 0xfc) << 17);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 462 to 474
	value = ((uint32_t)(input[57] & 0x03) << 30) |
			((uint32_t)input[58] << 22) |
			((uint32_t)(input[59] & 0xe0) << 14);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 475 to 487
	value = ((uint32_t)(input[59] & 0x1f) << 27) |
			((uint32_t)input[60] << 19);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 488 to 500
	value = ((uint32_t)input[61] << 24) |
			((uint32_t)(input[62] & 0xf8) << 16);
	output[37] = (double)(value >> 19);
}

static void unpack_39_13bit_signed_0off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 13-bit signed
	// Skip 0 bits, read 507 bits of data.
	// Input must be at least 64 bytes long

	int32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 377 to 389
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xfc) << 17);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 390 to 402
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xe0) << 14);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 403 to 415
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)input[51] << 19);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 416 to 428
	value = ((uint32_t)input[52] << 24) |
			((uint32_t)(input[53] & 0xf8) << 16);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 429 to 441
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21) |
			((uint32_t)(input[55] & 0xc0) << 13);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 442 to 454
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xfe) << 18);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 455 to 467
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xf0) << 15);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 468 to 480
	value = ((uint32_t)(input[58] & 0x0f) << 28) |
			((uint32_t)input[59] << 20) |
			((uint32_t)(input[60] & 0x80) << 12);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 481 to 493
	value = ((uint32_t)(input[60] & 0x7f) << 25) |
			((uint32_t)(input[61] & 0xfc) << 17);
	output[37] = (double)(value >> 19);

	// read output[38]: from bit 494 to 506
	value = ((uint32_t)(input[61] & 0x03) << 30) |
			((uint32_t)input[62] << 22) |
			((uint32_t)(input[63] & 0xe0) << 14);
	output[38] = (double)(value >> 19);
}

static void unpack_39_13bit_unsigned_0off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 13-bit unsigned
	// Skip 0 bits, read 507 bits of data.
	// Input must be at least 64 bytes long

	uint32_t value;

	// read output[0]: from bit 0 to 12
	value = ((uint32_t)input[0] << 24) |
			((uint32_t)(input[1] & 0xf8) << 16);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 13 to 25
	value = ((uint32_t)(input[1] & 0x07) << 29) |
			((uint32_t)input[2] << 21) |
			((uint32_t)(input[3] & 0xc0) << 13);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 26 to 38
	value = ((uint32_t)(input[3] & 0x3f) << 26) |
			((uint32_t)(input[4] & 0xfe) << 18);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 39 to 51
	value = ((uint32_t)(input[4] & 0x01) << 31) |
			((uint32_t)input[5] << 23) |
			((uint32_t)(input[6] & 0xf0) << 15);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 52 to 64
	value = ((uint32_t)(input[6] & 0x0f) << 28) |
			((uint32_t)input[7] << 20) |
			((uint32_t)(input[8] & 0x80) << 12);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 65 to 77
	value = ((uint32_t)(input[8] & 0x7f) << 25) |
			((uint32_t)(input[9] & 0xfc) << 17);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 78 to 90
	value = ((uint32_t)(input[9] & 0x03) << 30) |
			((uint32_t)input[10] << 22) |
			((uint32_t)(input[11] & 0xe0) << 14);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 91 to 103
	value = ((uint32_t)(input[11] & 0x1f) << 27) |
			((uint32_t)input[12] << 19);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 104 to 116
	value = ((uint32_t)input[13] << 24) |
			((uint32_t)(input[14] & 0xf8) << 16);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 117 to 129
	value = ((uint32_t)(input[14] & 0x07) << 29) |
			((uint32_t)input[15] << 21) |
			((uint32_t)(input[16] & 0xc0) << 13);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 130 to 142
	value = ((uint32_t)(input[16] & 0x3f) << 26) |
			((uint32_t)(input[17] & 0xfe) << 18);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 143 to 155
	value = ((uint32_t)(input[17] & 0x01) << 31) |
			((uint32_t)input[18] << 23) |
			((uint32_t)(input[19] & 0xf0) << 15);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 156 to 168
	value = ((uint32_t)(input[19] & 0x0f) << 28) |
			((uint32_t)input[20] << 20) |
			((uint32_t)(input[21] & 0x80) << 12);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 169 to 181
	value = ((uint32_t)(input[21] & 0x7f) << 25) |
			((uint32_t)(input[22] & 0xfc) << 17);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 182 to 194
	value = ((uint32_t)(input[22] & 0x03) << 30) |
			((uint32_t)input[23] << 22) |
			((uint32_t)(input[24] & 0xe0) << 14);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 195 to 207
	value = ((uint32_t)(input[24] & 0x1f) << 27) |
			((uint32_t)input[25] << 19);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 208 to 220
	value = ((uint32_t)input[26] << 24) |
			((uint32_t)(input[27] & 0xf8) << 16);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 221 to 233
	value = ((uint32_t)(input[27] & 0x07) << 29) |
			((uint32_t)input[28] << 21) |
			((uint32_t)(input[29] & 0xc0) << 13);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 234 to 246
	value = ((uint32_t)(input[29] & 0x3f) << 26) |
			((uint32_t)(input[30] & 0xfe) << 18);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 247 to 259
	value = ((uint32_t)(input[30] & 0x01) << 31) |
			((uint32_t)input[31] << 23) |
			((uint32_t)(input[32] & 0xf0) << 15);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 260 to 272
	value = ((uint32_t)(input[32] & 0x0f) << 28) |
			((uint32_t)input[33] << 20) |
			((uint32_t)(input[34] & 0x80) << 12);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 273 to 285
	value = ((uint32_t)(input[34] & 0x7f) << 25) |
			((uint32_t)(input[35] & 0xfc) << 17);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 286 to 298
	value = ((uint32_t)(input[35] & 0x03) << 30) |
			((uint32_t)input[36] << 22) |
			((uint32_t)(input[37] & 0xe0) << 14);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 299 to 311
	value = ((uint32_t)(input[37] & 0x1f) << 27) |
			((uint32_t)input[38] << 19);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 312 to 324
	value = ((uint32_t)input[39] << 24) |
			((uint32_t)(input[40] & 0xf8) << 16);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 325 to 337
	value = ((uint32_t)(input[40] & 0x07) << 29) |
			((uint32_t)input[41] << 21) |
			((uint32_t)(input[42] & 0xc0) << 13);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 338 to 350
	value = ((uint32_t)(input[42] & 0x3f) << 26) |
			((uint32_t)(input[43] & 0xfe) << 18);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 351 to 363
	value = ((uint32_t)(input[43] & 0x01) << 31) |
			((uint32_t)input[44] << 23) |
			((uint32_t)(input[45] & 0xf0) << 15);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 364 to 376
	value = ((uint32_t)(input[45] & 0x0f) << 28) |
			((uint32_t)input[46] << 20) |
			((uint32_t)(input[47] & 0x80) << 12);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 377 to 389
	value = ((uint32_t)(input[47] & 0x7f) << 25) |
			((uint32_t)(input[48] & 0xfc) << 17);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 390 to 402
	value = ((uint32_t)(input[48] & 0x03) << 30) |
			((uint32_t)input[49] << 22) |
			((uint32_t)(input[50] & 0xe0) << 14);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 403 to 415
	value = ((uint32_t)(input[50] & 0x1f) << 27) |
			((uint32_t)input[51] << 19);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 416 to 428
	value = ((uint32_t)input[52] << 24) |
			((uint32_t)(input[53] & 0xf8) << 16);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 429 to 441
	value = ((uint32_t)(input[53] & 0x07) << 29) |
			((uint32_t)input[54] << 21) |
			((uint32_t)(input[55] & 0xc0) << 13);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 442 to 454
	value = ((uint32_t)(input[55] & 0x3f) << 26) |
			((uint32_t)(input[56] & 0xfe) << 18);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 455 to 467
	value = ((uint32_t)(input[56] & 0x01) << 31) |
			((uint32_t)input[57] << 23) |
			((uint32_t)(input[58] & 0xf0) << 15);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 468 to 480
	value = ((uint32_t)(input[58] & 0x0f) << 28) |
			((uint32_t)input[59] << 20) |
			((uint32_t)(input[60] & 0x80) << 12);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 481 to 493
	value = ((uint32_t)(input[60] & 0x7f) << 25) |
			((uint32_t)(input[61] & 0xfc) << 17);
	output[37] = (double)(value >> 19);

	// read output[38]: from bit 494 to 506
	value = ((uint32_t)(input[61] & 0x03) << 30) |
			((uint32_t)input[62] << 22) |
			((uint32_t)(input[63] & 0xe0) << 14);
	output[38] = (double)(value >> 19);
}

static void unpack_39_13bit_signed_1off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 13-bit signed
	// Skip 1 bits, read 507 bits of data.
	// Input must be at least 64 bytes long

	int32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 378 to 390
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xfe) << 18);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 391 to 403
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xf0) << 15);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 404 to 416
	value = ((uint32_t)(input[50] & 0x0f) << 28) |
			((uint32_t)input[51] << 20) |
			((uint32_t)(input[52] & 0x80) << 12);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 417 to 429
	value = ((uint32_t)(input[52] & 0x7f) << 25) |
			((uint32_t)(input[53] & 0xfc) << 17);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 430 to 442
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0xe0) << 14);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 443 to 455
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)input[56] << 19);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 456 to 468
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xf8) << 16);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 469 to 481
	value = ((uint32_t)(input[58] & 0x07) << 29) |
			((uint32_t)input[59] << 21) |
			((uint32_t)(input[60] & 0xc0) << 13);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 482 to 494
	value = ((uint32_t)(input[60] & 0x3f) << 26) |
			((uint32_t)(input[61] & 0xfe) << 18);
	output[37] = (double)(value >> 19);

	// read output[38]: from bit 495 to 507
	value = ((uint32_t)(input[61] & 0x01) << 31) |
			((uint32_t)input[62] << 23) |
			((uint32_t)(input[63] & 0xf0) << 15);
	output[38] = (double)(value >> 19);
}

static void unpack_39_13bit_unsigned_1off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 13-bit unsigned
	// Skip 1 bits, read 507 bits of data.
	// Input must be at least 64 bytes long

	uint32_t value;

	// read output[0]: from bit 1 to 13
	value = ((uint32_t)(input[0] & 0x7f) << 25) |
			((uint32_t)(input[1] & 0xfc) << 17);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 14 to 26
	value = ((uint32_t)(input[1] & 0x03) << 30) |
			((uint32_t)input[2] << 22) |
			((uint32_t)(input[3] & 0xe0) << 14);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 27 to 39
	value = ((uint32_t)(input[3] & 0x1f) << 27) |
			((uint32_t)input[4] << 19);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 40 to 52
	value = ((uint32_t)input[5] << 24) |
			((uint32_t)(input[6] & 0xf8) << 16);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 53 to 65
	value = ((uint32_t)(input[6] & 0x07) << 29) |
			((uint32_t)input[7] << 21) |
			((uint32_t)(input[8] & 0xc0) << 13);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 66 to 78
	value = ((uint32_t)(input[8] & 0x3f) << 26) |
			((uint32_t)(input[9] & 0xfe) << 18);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 79 to 91
	value = ((uint32_t)(input[9] & 0x01) << 31) |
			((uint32_t)input[10] << 23) |
			((uint32_t)(input[11] & 0xf0) << 15);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 92 to 104
	value = ((uint32_t)(input[11] & 0x0f) << 28) |
			((uint32_t)input[12] << 20) |
			((uint32_t)(input[13] & 0x80) << 12);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 105 to 117
	value = ((uint32_t)(input[13] & 0x7f) << 25) |
			((uint32_t)(input[14] & 0xfc) << 17);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 118 to 130
	value = ((uint32_t)(input[14] & 0x03) << 30) |
			((uint32_t)input[15] << 22) |
			((uint32_t)(input[16] & 0xe0) << 14);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 131 to 143
	value = ((uint32_t)(input[16] & 0x1f) << 27) |
			((uint32_t)input[17] << 19);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 144 to 156
	value = ((uint32_t)input[18] << 24) |
			((uint32_t)(input[19] & 0xf8) << 16);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 157 to 169
	value = ((uint32_t)(input[19] & 0x07) << 29) |
			((uint32_t)input[20] << 21) |
			((uint32_t)(input[21] & 0xc0) << 13);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 170 to 182
	value = ((uint32_t)(input[21] & 0x3f) << 26) |
			((uint32_t)(input[22] & 0xfe) << 18);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 183 to 195
	value = ((uint32_t)(input[22] & 0x01) << 31) |
			((uint32_t)input[23] << 23) |
			((uint32_t)(input[24] & 0xf0) << 15);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 196 to 208
	value = ((uint32_t)(input[24] & 0x0f) << 28) |
			((uint32_t)input[25] << 20) |
			((uint32_t)(input[26] & 0x80) << 12);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 209 to 221
	value = ((uint32_t)(input[26] & 0x7f) << 25) |
			((uint32_t)(input[27] & 0xfc) << 17);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 222 to 234
	value = ((uint32_t)(input[27] & 0x03) << 30) |
			((uint32_t)input[28] << 22) |
			((uint32_t)(input[29] & 0xe0) << 14);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 235 to 247
	value = ((uint32_t)(input[29] & 0x1f) << 27) |
			((uint32_t)input[30] << 19);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 248 to 260
	value = ((uint32_t)input[31] << 24) |
			((uint32_t)(input[32] & 0xf8) << 16);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 261 to 273
	value = ((uint32_t)(input[32] & 0x07) << 29) |
			((uint32_t)input[33] << 21) |
			((uint32_t)(input[34] & 0xc0) << 13);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 274 to 286
	value = ((uint32_t)(input[34] & 0x3f) << 26) |
			((uint32_t)(input[35] & 0xfe) << 18);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 287 to 299
	value = ((uint32_t)(input[35] & 0x01) << 31) |
			((uint32_t)input[36] << 23) |
			((uint32_t)(input[37] & 0xf0) << 15);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 300 to 312
	value = ((uint32_t)(input[37] & 0x0f) << 28) |
			((uint32_t)input[38] << 20) |
			((uint32_t)(input[39] & 0x80) << 12);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 313 to 325
	value = ((uint32_t)(input[39] & 0x7f) << 25) |
			((uint32_t)(input[40] & 0xfc) << 17);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 326 to 338
	value = ((uint32_t)(input[40] & 0x03) << 30) |
			((uint32_t)input[41] << 22) |
			((uint32_t)(input[42] & 0xe0) << 14);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 339 to 351
	value = ((uint32_t)(input[42] & 0x1f) << 27) |
			((uint32_t)input[43] << 19);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 352 to 364
	value = ((uint32_t)input[44] << 24) |
			((uint32_t)(input[45] & 0xf8) << 16);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 365 to 377
	value = ((uint32_t)(input[45] & 0x07) << 29) |
			((uint32_t)input[46] << 21) |
			((uint32_t)(input[47] & 0xc0) << 13);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 378 to 390
	value = ((uint32_t)(input[47] & 0x3f) << 26) |
			((uint32_t)(input[48] & 0xfe) << 18);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 391 to 403
	value = ((uint32_t)(input[48] & 0x01) << 31) |
			((uint32_t)input[49] << 23) |
			((uint32_t)(input[50] & 0xf0) << 15);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 404 to 416
	value = ((uint32_t)(input[50] & 0x0f) << 28) |
			((uint32_t)input[51] << 20) |
			((uint32_t)(input[52] & 0x80) << 12);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 417 to 429
	value = ((uint32_t)(input[52] & 0x7f) << 25) |
			((uint32_t)(input[53] & 0xfc) << 17);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 430 to 442
	value = ((uint32_t)(input[53] & 0x03) << 30) |
			((uint32_t)input[54] << 22) |
			((uint32_t)(input[55] & 0xe0) << 14);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 443 to 455
	value = ((uint32_t)(input[55] & 0x1f) << 27) |
			((uint32_t)input[56] << 19);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 456 to 468
	value = ((uint32_t)input[57] << 24) |
			((uint32_t)(input[58] & 0xf8) << 16);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 469 to 481
	value = ((uint32_t)(input[58] & 0x07) << 29) |
			((uint32_t)input[59] << 21) |
			((uint32_t)(input[60] & 0xc0) << 13);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 482 to 494
	value = ((uint32_t)(input[60] & 0x3f) << 26) |
			((uint32_t)(input[61] & 0xfe) << 18);
	output[37] = (double)(value >> 19);

	// read output[38]: from bit 495 to 507
	value = ((uint32_t)(input[61] & 0x01) << 31) |
			((uint32_t)input[62] << 23) |
			((uint32_t)(input[63] & 0xf0) << 15);
	output[38] = (double)(value >> 19);
}

static void unpack_39_13bit_signed_2off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 13-bit signed
	// Skip 2 bits, read 507 bits of data.
	// Input must be at least 64 bytes long

	int32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 379 to 391
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 392 to 404
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xf8) << 16);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 405 to 417
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xc0) << 13);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 418 to 430
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xfe) << 18);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 431 to 443
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xf0) << 15);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 444 to 456
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)input[56] << 20) |
			((uint32_t)(input[57] & 0x80) << 12);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 457 to 469
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)(input[58] & 0xfc) << 17);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 470 to 482
	value = ((uint32_t)(input[58] & 0x03) << 30) |
			((uint32_t)input[59] << 22) |
			((uint32_t)(input[60] & 0xe0) << 14);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 483 to 495
	value = ((uint32_t)(input[60] & 0x1f) << 27) |
			((uint32_t)input[61] << 19);
	output[37] = (double)(value >> 19);

	// read output[38]: from bit 496 to 508
	value = ((uint32_t)input[62] << 24) |
			((uint32_t)(input[63] & 0xf8) << 16);
	output[38] = (double)(value >> 19);
}

static void unpack_39_13bit_unsigned_2off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 13-bit unsigned
	// Skip 2 bits, read 507 bits of data.
	// Input must be at least 64 bytes long

	uint32_t value;

	// read output[0]: from bit 2 to 14
	value = ((uint32_t)(input[0] & 0x3f) << 26) |
			((uint32_t)(input[1] & 0xfe) << 18);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 15 to 27
	value = ((uint32_t)(input[1] & 0x01) << 31) |
			((uint32_t)input[2] << 23) |
			((uint32_t)(input[3] & 0xf0) << 15);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 28 to 40
	value = ((uint32_t)(input[3] & 0x0f) << 28) |
			((uint32_t)input[4] << 20) |
			((uint32_t)(input[5] & 0x80) << 12);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 41 to 53
	value = ((uint32_t)(input[5] & 0x7f) << 25) |
			((uint32_t)(input[6] & 0xfc) << 17);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 54 to 66
	value = ((uint32_t)(input[6] & 0x03) << 30) |
			((uint32_t)input[7] << 22) |
			((uint32_t)(input[8] & 0xe0) << 14);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 67 to 79
	value = ((uint32_t)(input[8] & 0x1f) << 27) |
			((uint32_t)input[9] << 19);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 80 to 92
	value = ((uint32_t)input[10] << 24) |
			((uint32_t)(input[11] & 0xf8) << 16);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 93 to 105
	value = ((uint32_t)(input[11] & 0x07) << 29) |
			((uint32_t)input[12] << 21) |
			((uint32_t)(input[13] & 0xc0) << 13);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 106 to 118
	value = ((uint32_t)(input[13] & 0x3f) << 26) |
			((uint32_t)(input[14] & 0xfe) << 18);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 119 to 131
	value = ((uint32_t)(input[14] & 0x01) << 31) |
			((uint32_t)input[15] << 23) |
			((uint32_t)(input[16] & 0xf0) << 15);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 132 to 144
	value = ((uint32_t)(input[16] & 0x0f) << 28) |
			((uint32_t)input[17] << 20) |
			((uint32_t)(input[18] & 0x80) << 12);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 145 to 157
	value = ((uint32_t)(input[18] & 0x7f) << 25) |
			((uint32_t)(input[19] & 0xfc) << 17);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 158 to 170
	value = ((uint32_t)(input[19] & 0x03) << 30) |
			((uint32_t)input[20] << 22) |
			((uint32_t)(input[21] & 0xe0) << 14);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 171 to 183
	value = ((uint32_t)(input[21] & 0x1f) << 27) |
			((uint32_t)input[22] << 19);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 184 to 196
	value = ((uint32_t)input[23] << 24) |
			((uint32_t)(input[24] & 0xf8) << 16);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 197 to 209
	value = ((uint32_t)(input[24] & 0x07) << 29) |
			((uint32_t)input[25] << 21) |
			((uint32_t)(input[26] & 0xc0) << 13);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 210 to 222
	value = ((uint32_t)(input[26] & 0x3f) << 26) |
			((uint32_t)(input[27] & 0xfe) << 18);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 223 to 235
	value = ((uint32_t)(input[27] & 0x01) << 31) |
			((uint32_t)input[28] << 23) |
			((uint32_t)(input[29] & 0xf0) << 15);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 236 to 248
	value = ((uint32_t)(input[29] & 0x0f) << 28) |
			((uint32_t)input[30] << 20) |
			((uint32_t)(input[31] & 0x80) << 12);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 249 to 261
	value = ((uint32_t)(input[31] & 0x7f) << 25) |
			((uint32_t)(input[32] & 0xfc) << 17);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 262 to 274
	value = ((uint32_t)(input[32] & 0x03) << 30) |
			((uint32_t)input[33] << 22) |
			((uint32_t)(input[34] & 0xe0) << 14);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 275 to 287
	value = ((uint32_t)(input[34] & 0x1f) << 27) |
			((uint32_t)input[35] << 19);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 288 to 300
	value = ((uint32_t)input[36] << 24) |
			((uint32_t)(input[37] & 0xf8) << 16);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 301 to 313
	value = ((uint32_t)(input[37] & 0x07) << 29) |
			((uint32_t)input[38] << 21) |
			((uint32_t)(input[39] & 0xc0) << 13);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 314 to 326
	value = ((uint32_t)(input[39] & 0x3f) << 26) |
			((uint32_t)(input[40] & 0xfe) << 18);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 327 to 339
	value = ((uint32_t)(input[40] & 0x01) << 31) |
			((uint32_t)input[41] << 23) |
			((uint32_t)(input[42] & 0xf0) << 15);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 340 to 352
	value = ((uint32_t)(input[42] & 0x0f) << 28) |
			((uint32_t)input[43] << 20) |
			((uint32_t)(input[44] & 0x80) << 12);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 353 to 365
	value = ((uint32_t)(input[44] & 0x7f) << 25) |
			((uint32_t)(input[45] & 0xfc) << 17);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 366 to 378
	value = ((uint32_t)(input[45] & 0x03) << 30) |
			((uint32_t)input[46] << 22) |
			((uint32_t)(input[47] & 0xe0) << 14);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 379 to 391
	value = ((uint32_t)(input[47] & 0x1f) << 27) |
			((uint32_t)input[48] << 19);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 392 to 404
	value = ((uint32_t)input[49] << 24) |
			((uint32_t)(input[50] & 0xf8) << 16);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 405 to 417
	value = ((uint32_t)(input[50] & 0x07) << 29) |
			((uint32_t)input[51] << 21) |
			((uint32_t)(input[52] & 0xc0) << 13);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 418 to 430
	value = ((uint32_t)(input[52] & 0x3f) << 26) |
			((uint32_t)(input[53] & 0xfe) << 18);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 431 to 443
	value = ((uint32_t)(input[53] & 0x01) << 31) |
			((uint32_t)input[54] << 23) |
			((uint32_t)(input[55] & 0xf0) << 15);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 444 to 456
	value = ((uint32_t)(input[55] & 0x0f) << 28) |
			((uint32_t)input[56] << 20) |
			((uint32_t)(input[57] & 0x80) << 12);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 457 to 469
	value = ((uint32_t)(input[57] & 0x7f) << 25) |
			((uint32_t)(input[58] & 0xfc) << 17);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 470 to 482
	value = ((uint32_t)(input[58] & 0x03) << 30) |
			((uint32_t)input[59] << 22) |
			((uint32_t)(input[60] & 0xe0) << 14);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 483 to 495
	value = ((uint32_t)(input[60] & 0x1f) << 27) |
			((uint32_t)input[61] << 19);
	output[37] = (double)(value >> 19);

	// read output[38]: from bit 496 to 508
	value = ((uint32_t)input[62] << 24) |
			((uint32_t)(input[63] & 0xf8) << 16);
	output[38] = (double)(value >> 19);
}

static void unpack_39_13bit_signed_3off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 13-bit signed
	// Skip 3 bits, read 507 bits of data.
	// Input must be at least 64 bytes long

	int32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 380 to 392
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0x80) << 12);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 393 to 405
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)(input[50] & 0xfc) << 17);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 406 to 418
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xe0) << 14);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 419 to 431
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)input[53] << 19);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 432 to 444
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xf8) << 16);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 445 to 457
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21) |
			((uint32_t)(input[57] & 0xc0) << 13);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 458 to 470
	value = ((uint32_t)(input[57] & 0x3f) << 26) |
			((uint32_t)(input[58] & 0xfe) << 18);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 471 to 483
	value = ((uint32_t)(input[58] & 0x01) << 31) |
			((uint32_t)input[59] << 23) |
			((uint32_t)(input[60] & 0xf0) << 15);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 484 to 496
	value = ((uint32_t)(input[60] & 0x0f) << 28) |
			((uint32_t)input[61] << 20) |
			((uint32_t)(input[62] & 0x80) << 12);
	output[37] = (double)(value >> 19);

	// read output[38]: from bit 497 to 509
	value = ((uint32_t)(input[62] & 0x7f) << 25) |
			((uint32_t)(input[63] & 0xfc) << 17);
	output[38] = (double)(value >> 19);
}

static void unpack_39_13bit_unsigned_3off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 13-bit unsigned
	// Skip 3 bits, read 507 bits of data.
	// Input must be at least 64 bytes long

	uint32_t value;

	// read output[0]: from bit 3 to 15
	value = ((uint32_t)(input[0] & 0x1f) << 27) |
			((uint32_t)input[1] << 19);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 16 to 28
	value = ((uint32_t)input[2] << 24) |
			((uint32_t)(input[3] & 0xf8) << 16);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 29 to 41
	value = ((uint32_t)(input[3] & 0x07) << 29) |
			((uint32_t)input[4] << 21) |
			((uint32_t)(input[5] & 0xc0) << 13);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 42 to 54
	value = ((uint32_t)(input[5] & 0x3f) << 26) |
			((uint32_t)(input[6] & 0xfe) << 18);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 55 to 67
	value = ((uint32_t)(input[6] & 0x01) << 31) |
			((uint32_t)input[7] << 23) |
			((uint32_t)(input[8] & 0xf0) << 15);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 68 to 80
	value = ((uint32_t)(input[8] & 0x0f) << 28) |
			((uint32_t)input[9] << 20) |
			((uint32_t)(input[10] & 0x80) << 12);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 81 to 93
	value = ((uint32_t)(input[10] & 0x7f) << 25) |
			((uint32_t)(input[11] & 0xfc) << 17);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 94 to 106
	value = ((uint32_t)(input[11] & 0x03) << 30) |
			((uint32_t)input[12] << 22) |
			((uint32_t)(input[13] & 0xe0) << 14);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 107 to 119
	value = ((uint32_t)(input[13] & 0x1f) << 27) |
			((uint32_t)input[14] << 19);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 120 to 132
	value = ((uint32_t)input[15] << 24) |
			((uint32_t)(input[16] & 0xf8) << 16);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 133 to 145
	value = ((uint32_t)(input[16] & 0x07) << 29) |
			((uint32_t)input[17] << 21) |
			((uint32_t)(input[18] & 0xc0) << 13);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 146 to 158
	value = ((uint32_t)(input[18] & 0x3f) << 26) |
			((uint32_t)(input[19] & 0xfe) << 18);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 159 to 171
	value = ((uint32_t)(input[19] & 0x01) << 31) |
			((uint32_t)input[20] << 23) |
			((uint32_t)(input[21] & 0xf0) << 15);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 172 to 184
	value = ((uint32_t)(input[21] & 0x0f) << 28) |
			((uint32_t)input[22] << 20) |
			((uint32_t)(input[23] & 0x80) << 12);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 185 to 197
	value = ((uint32_t)(input[23] & 0x7f) << 25) |
			((uint32_t)(input[24] & 0xfc) << 17);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 198 to 210
	value = ((uint32_t)(input[24] & 0x03) << 30) |
			((uint32_t)input[25] << 22) |
			((uint32_t)(input[26] & 0xe0) << 14);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 211 to 223
	value = ((uint32_t)(input[26] & 0x1f) << 27) |
			((uint32_t)input[27] << 19);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 224 to 236
	value = ((uint32_t)input[28] << 24) |
			((uint32_t)(input[29] & 0xf8) << 16);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 237 to 249
	value = ((uint32_t)(input[29] & 0x07) << 29) |
			((uint32_t)input[30] << 21) |
			((uint32_t)(input[31] & 0xc0) << 13);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 250 to 262
	value = ((uint32_t)(input[31] & 0x3f) << 26) |
			((uint32_t)(input[32] & 0xfe) << 18);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 263 to 275
	value = ((uint32_t)(input[32] & 0x01) << 31) |
			((uint32_t)input[33] << 23) |
			((uint32_t)(input[34] & 0xf0) << 15);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 276 to 288
	value = ((uint32_t)(input[34] & 0x0f) << 28) |
			((uint32_t)input[35] << 20) |
			((uint32_t)(input[36] & 0x80) << 12);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 289 to 301
	value = ((uint32_t)(input[36] & 0x7f) << 25) |
			((uint32_t)(input[37] & 0xfc) << 17);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 302 to 314
	value = ((uint32_t)(input[37] & 0x03) << 30) |
			((uint32_t)input[38] << 22) |
			((uint32_t)(input[39] & 0xe0) << 14);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 315 to 327
	value = ((uint32_t)(input[39] & 0x1f) << 27) |
			((uint32_t)input[40] << 19);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 328 to 340
	value = ((uint32_t)input[41] << 24) |
			((uint32_t)(input[42] & 0xf8) << 16);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 341 to 353
	value = ((uint32_t)(input[42] & 0x07) << 29) |
			((uint32_t)input[43] << 21) |
			((uint32_t)(input[44] & 0xc0) << 13);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 354 to 366
	value = ((uint32_t)(input[44] & 0x3f) << 26) |
			((uint32_t)(input[45] & 0xfe) << 18);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 367 to 379
	value = ((uint32_t)(input[45] & 0x01) << 31) |
			((uint32_t)input[46] << 23) |
			((uint32_t)(input[47] & 0xf0) << 15);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 380 to 392
	value = ((uint32_t)(input[47] & 0x0f) << 28) |
			((uint32_t)input[48] << 20) |
			((uint32_t)(input[49] & 0x80) << 12);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 393 to 405
	value = ((uint32_t)(input[49] & 0x7f) << 25) |
			((uint32_t)(input[50] & 0xfc) << 17);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 406 to 418
	value = ((uint32_t)(input[50] & 0x03) << 30) |
			((uint32_t)input[51] << 22) |
			((uint32_t)(input[52] & 0xe0) << 14);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 419 to 431
	value = ((uint32_t)(input[52] & 0x1f) << 27) |
			((uint32_t)input[53] << 19);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 432 to 444
	value = ((uint32_t)input[54] << 24) |
			((uint32_t)(input[55] & 0xf8) << 16);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 445 to 457
	value = ((uint32_t)(input[55] & 0x07) << 29) |
			((uint32_t)input[56] << 21) |
			((uint32_t)(input[57] & 0xc0) << 13);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 458 to 470
	value = ((uint32_t)(input[57] & 0x3f) << 26) |
			((uint32_t)(input[58] & 0xfe) << 18);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 471 to 483
	value = ((uint32_t)(input[58] & 0x01) << 31) |
			((uint32_t)input[59] << 23) |
			((uint32_t)(input[60] & 0xf0) << 15);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 484 to 496
	value = ((uint32_t)(input[60] & 0x0f) << 28) |
			((uint32_t)input[61] << 20) |
			((uint32_t)(input[62] & 0x80) << 12);
	output[37] = (double)(value >> 19);

	// read output[38]: from bit 497 to 509
	value = ((uint32_t)(input[62] & 0x7f) << 25) |
			((uint32_t)(input[63] & 0xfc) << 17);
	output[38] = (double)(value >> 19);
}

static void unpack_39_13bit_signed_4off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 13-bit signed
	// Skip 4 bits, read 507 bits of data.
	// Input must be at least 64 bytes long

	int32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 381 to 393
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xc0) << 13);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 394 to 406
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)(input[50] & 0xfe) << 18);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 407 to 419
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xf0) << 15);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 420 to 432
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0x80) << 12);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 433 to 445
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xfc) << 17);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 446 to 458
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0xe0) << 14);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 459 to 471
	value = ((uint32_t)(input[57] & 0x1f) << 27) |
			((uint32_t)input[58] << 19);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 472 to 484
	value = ((uint32_t)input[59] << 24) |
			((uint32_t)(input[60] & 0xf8) << 16);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 485 to 497
	value = ((uint32_t)(input[60] & 0x07) << 29) |
			((uint32_t)input[61] << 21) |
			((uint32_t)(input[62] & 0xc0) << 13);
	output[37] = (double)(value >> 19);

	// read output[38]: from bit 498 to 510
	value = ((uint32_t)(input[62] & 0x3f) << 26) |
			((uint32_t)(input[63] & 0xfe) << 18);
	output[38] = (double)(value >> 19);
}

static void unpack_39_13bit_unsigned_4off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 13-bit unsigned
	// Skip 4 bits, read 507 bits of data.
	// Input must be at least 64 bytes long

	uint32_t value;

	// read output[0]: from bit 4 to 16
	value = ((uint32_t)(input[0] & 0x0f) << 28) |
			((uint32_t)input[1] << 20) |
			((uint32_t)(input[2] & 0x80) << 12);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 17 to 29
	value = ((uint32_t)(input[2] & 0x7f) << 25) |
			((uint32_t)(input[3] & 0xfc) << 17);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 30 to 42
	value = ((uint32_t)(input[3] & 0x03) << 30) |
			((uint32_t)input[4] << 22) |
			((uint32_t)(input[5] & 0xe0) << 14);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 43 to 55
	value = ((uint32_t)(input[5] & 0x1f) << 27) |
			((uint32_t)input[6] << 19);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 56 to 68
	value = ((uint32_t)input[7] << 24) |
			((uint32_t)(input[8] & 0xf8) << 16);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 69 to 81
	value = ((uint32_t)(input[8] & 0x07) << 29) |
			((uint32_t)input[9] << 21) |
			((uint32_t)(input[10] & 0xc0) << 13);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 82 to 94
	value = ((uint32_t)(input[10] & 0x3f) << 26) |
			((uint32_t)(input[11] & 0xfe) << 18);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 95 to 107
	value = ((uint32_t)(input[11] & 0x01) << 31) |
			((uint32_t)input[12] << 23) |
			((uint32_t)(input[13] & 0xf0) << 15);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 108 to 120
	value = ((uint32_t)(input[13] & 0x0f) << 28) |
			((uint32_t)input[14] << 20) |
			((uint32_t)(input[15] & 0x80) << 12);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 121 to 133
	value = ((uint32_t)(input[15] & 0x7f) << 25) |
			((uint32_t)(input[16] & 0xfc) << 17);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 134 to 146
	value = ((uint32_t)(input[16] & 0x03) << 30) |
			((uint32_t)input[17] << 22) |
			((uint32_t)(input[18] & 0xe0) << 14);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 147 to 159
	value = ((uint32_t)(input[18] & 0x1f) << 27) |
			((uint32_t)input[19] << 19);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 160 to 172
	value = ((uint32_t)input[20] << 24) |
			((uint32_t)(input[21] & 0xf8) << 16);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 173 to 185
	value = ((uint32_t)(input[21] & 0x07) << 29) |
			((uint32_t)input[22] << 21) |
			((uint32_t)(input[23] & 0xc0) << 13);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 186 to 198
	value = ((uint32_t)(input[23] & 0x3f) << 26) |
			((uint32_t)(input[24] & 0xfe) << 18);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 199 to 211
	value = ((uint32_t)(input[24] & 0x01) << 31) |
			((uint32_t)input[25] << 23) |
			((uint32_t)(input[26] & 0xf0) << 15);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 212 to 224
	value = ((uint32_t)(input[26] & 0x0f) << 28) |
			((uint32_t)input[27] << 20) |
			((uint32_t)(input[28] & 0x80) << 12);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 225 to 237
	value = ((uint32_t)(input[28] & 0x7f) << 25) |
			((uint32_t)(input[29] & 0xfc) << 17);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 238 to 250
	value = ((uint32_t)(input[29] & 0x03) << 30) |
			((uint32_t)input[30] << 22) |
			((uint32_t)(input[31] & 0xe0) << 14);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 251 to 263
	value = ((uint32_t)(input[31] & 0x1f) << 27) |
			((uint32_t)input[32] << 19);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 264 to 276
	value = ((uint32_t)input[33] << 24) |
			((uint32_t)(input[34] & 0xf8) << 16);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 277 to 289
	value = ((uint32_t)(input[34] & 0x07) << 29) |
			((uint32_t)input[35] << 21) |
			((uint32_t)(input[36] & 0xc0) << 13);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 290 to 302
	value = ((uint32_t)(input[36] & 0x3f) << 26) |
			((uint32_t)(input[37] & 0xfe) << 18);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 303 to 315
	value = ((uint32_t)(input[37] & 0x01) << 31) |
			((uint32_t)input[38] << 23) |
			((uint32_t)(input[39] & 0xf0) << 15);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 316 to 328
	value = ((uint32_t)(input[39] & 0x0f) << 28) |
			((uint32_t)input[40] << 20) |
			((uint32_t)(input[41] & 0x80) << 12);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 329 to 341
	value = ((uint32_t)(input[41] & 0x7f) << 25) |
			((uint32_t)(input[42] & 0xfc) << 17);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 342 to 354
	value = ((uint32_t)(input[42] & 0x03) << 30) |
			((uint32_t)input[43] << 22) |
			((uint32_t)(input[44] & 0xe0) << 14);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 355 to 367
	value = ((uint32_t)(input[44] & 0x1f) << 27) |
			((uint32_t)input[45] << 19);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 368 to 380
	value = ((uint32_t)input[46] << 24) |
			((uint32_t)(input[47] & 0xf8) << 16);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 381 to 393
	value = ((uint32_t)(input[47] & 0x07) << 29) |
			((uint32_t)input[48] << 21) |
			((uint32_t)(input[49] & 0xc0) << 13);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 394 to 406
	value = ((uint32_t)(input[49] & 0x3f) << 26) |
			((uint32_t)(input[50] & 0xfe) << 18);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 407 to 419
	value = ((uint32_t)(input[50] & 0x01) << 31) |
			((uint32_t)input[51] << 23) |
			((uint32_t)(input[52] & 0xf0) << 15);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 420 to 432
	value = ((uint32_t)(input[52] & 0x0f) << 28) |
			((uint32_t)input[53] << 20) |
			((uint32_t)(input[54] & 0x80) << 12);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 433 to 445
	value = ((uint32_t)(input[54] & 0x7f) << 25) |
			((uint32_t)(input[55] & 0xfc) << 17);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 446 to 458
	value = ((uint32_t)(input[55] & 0x03) << 30) |
			((uint32_t)input[56] << 22) |
			((uint32_t)(input[57] & 0xe0) << 14);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 459 to 471
	value = ((uint32_t)(input[57] & 0x1f) << 27) |
			((uint32_t)input[58] << 19);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 472 to 484
	value = ((uint32_t)input[59] << 24) |
			((uint32_t)(input[60] & 0xf8) << 16);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 485 to 497
	value = ((uint32_t)(input[60] & 0x07) << 29) |
			((uint32_t)input[61] << 21) |
			((uint32_t)(input[62] & 0xc0) << 13);
	output[37] = (double)(value >> 19);

	// read output[38]: from bit 498 to 510
	value = ((uint32_t)(input[62] & 0x3f) << 26) |
			((uint32_t)(input[63] & 0xfe) << 18);
	output[38] = (double)(value >> 19);
}

static void unpack_39_13bit_signed_5off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 13-bit signed
	// Skip 5 bits, read 507 bits of data.
	// Input must be at least 64 bytes long

	int32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 382 to 394
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xe0) << 14);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 395 to 407
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 408 to 420
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xf8) << 16);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 421 to 433
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xc0) << 13);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 434 to 446
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xfe) << 18);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 447 to 459
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xf0) << 15);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 460 to 472
	value = ((uint32_t)(input[57] & 0x0f) << 28) |
			((uint32_t)input[58] << 20) |
			((uint32_t)(input[59] & 0x80) << 12);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 473 to 485
	value = ((uint32_t)(input[59] & 0x7f) << 25) |
			((uint32_t)(input[60] & 0xfc) << 17);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 486 to 498
	value = ((uint32_t)(input[60] & 0x03) << 30) |
			((uint32_t)input[61] << 22) |
			((uint32_t)(input[62] & 0xe0) << 14);
	output[37] = (double)(value >> 19);

	// read output[38]: from bit 499 to 511
	value = ((uint32_t)(input[62] & 0x1f) << 27) |
			((uint32_t)input[63] << 19);
	output[38] = (double)(value >> 19);
}

static void unpack_39_13bit_unsigned_5off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 13-bit unsigned
	// Skip 5 bits, read 507 bits of data.
	// Input must be at least 64 bytes long

	uint32_t value;

	// read output[0]: from bit 5 to 17
	value = ((uint32_t)(input[0] & 0x07) << 29) |
			((uint32_t)input[1] << 21) |
			((uint32_t)(input[2] & 0xc0) << 13);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 18 to 30
	value = ((uint32_t)(input[2] & 0x3f) << 26) |
			((uint32_t)(input[3] & 0xfe) << 18);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 31 to 43
	value = ((uint32_t)(input[3] & 0x01) << 31) |
			((uint32_t)input[4] << 23) |
			((uint32_t)(input[5] & 0xf0) << 15);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 44 to 56
	value = ((uint32_t)(input[5] & 0x0f) << 28) |
			((uint32_t)input[6] << 20) |
			((uint32_t)(input[7] & 0x80) << 12);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 57 to 69
	value = ((uint32_t)(input[7] & 0x7f) << 25) |
			((uint32_t)(input[8] & 0xfc) << 17);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 70 to 82
	value = ((uint32_t)(input[8] & 0x03) << 30) |
			((uint32_t)input[9] << 22) |
			((uint32_t)(input[10] & 0xe0) << 14);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 83 to 95
	value = ((uint32_t)(input[10] & 0x1f) << 27) |
			((uint32_t)input[11] << 19);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 96 to 108
	value = ((uint32_t)input[12] << 24) |
			((uint32_t)(input[13] & 0xf8) << 16);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 109 to 121
	value = ((uint32_t)(input[13] & 0x07) << 29) |
			((uint32_t)input[14] << 21) |
			((uint32_t)(input[15] & 0xc0) << 13);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 122 to 134
	value = ((uint32_t)(input[15] & 0x3f) << 26) |
			((uint32_t)(input[16] & 0xfe) << 18);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 135 to 147
	value = ((uint32_t)(input[16] & 0x01) << 31) |
			((uint32_t)input[17] << 23) |
			((uint32_t)(input[18] & 0xf0) << 15);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 148 to 160
	value = ((uint32_t)(input[18] & 0x0f) << 28) |
			((uint32_t)input[19] << 20) |
			((uint32_t)(input[20] & 0x80) << 12);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 161 to 173
	value = ((uint32_t)(input[20] & 0x7f) << 25) |
			((uint32_t)(input[21] & 0xfc) << 17);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 174 to 186
	value = ((uint32_t)(input[21] & 0x03) << 30) |
			((uint32_t)input[22] << 22) |
			((uint32_t)(input[23] & 0xe0) << 14);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 187 to 199
	value = ((uint32_t)(input[23] & 0x1f) << 27) |
			((uint32_t)input[24] << 19);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 200 to 212
	value = ((uint32_t)input[25] << 24) |
			((uint32_t)(input[26] & 0xf8) << 16);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 213 to 225
	value = ((uint32_t)(input[26] & 0x07) << 29) |
			((uint32_t)input[27] << 21) |
			((uint32_t)(input[28] & 0xc0) << 13);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 226 to 238
	value = ((uint32_t)(input[28] & 0x3f) << 26) |
			((uint32_t)(input[29] & 0xfe) << 18);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 239 to 251
	value = ((uint32_t)(input[29] & 0x01) << 31) |
			((uint32_t)input[30] << 23) |
			((uint32_t)(input[31] & 0xf0) << 15);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 252 to 264
	value = ((uint32_t)(input[31] & 0x0f) << 28) |
			((uint32_t)input[32] << 20) |
			((uint32_t)(input[33] & 0x80) << 12);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 265 to 277
	value = ((uint32_t)(input[33] & 0x7f) << 25) |
			((uint32_t)(input[34] & 0xfc) << 17);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 278 to 290
	value = ((uint32_t)(input[34] & 0x03) << 30) |
			((uint32_t)input[35] << 22) |
			((uint32_t)(input[36] & 0xe0) << 14);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 291 to 303
	value = ((uint32_t)(input[36] & 0x1f) << 27) |
			((uint32_t)input[37] << 19);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 304 to 316
	value = ((uint32_t)input[38] << 24) |
			((uint32_t)(input[39] & 0xf8) << 16);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 317 to 329
	value = ((uint32_t)(input[39] & 0x07) << 29) |
			((uint32_t)input[40] << 21) |
			((uint32_t)(input[41] & 0xc0) << 13);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 330 to 342
	value = ((uint32_t)(input[41] & 0x3f) << 26) |
			((uint32_t)(input[42] & 0xfe) << 18);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 343 to 355
	value = ((uint32_t)(input[42] & 0x01) << 31) |
			((uint32_t)input[43] << 23) |
			((uint32_t)(input[44] & 0xf0) << 15);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 356 to 368
	value = ((uint32_t)(input[44] & 0x0f) << 28) |
			((uint32_t)input[45] << 20) |
			((uint32_t)(input[46] & 0x80) << 12);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 369 to 381
	value = ((uint32_t)(input[46] & 0x7f) << 25) |
			((uint32_t)(input[47] & 0xfc) << 17);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 382 to 394
	value = ((uint32_t)(input[47] & 0x03) << 30) |
			((uint32_t)input[48] << 22) |
			((uint32_t)(input[49] & 0xe0) << 14);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 395 to 407
	value = ((uint32_t)(input[49] & 0x1f) << 27) |
			((uint32_t)input[50] << 19);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 408 to 420
	value = ((uint32_t)input[51] << 24) |
			((uint32_t)(input[52] & 0xf8) << 16);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 421 to 433
	value = ((uint32_t)(input[52] & 0x07) << 29) |
			((uint32_t)input[53] << 21) |
			((uint32_t)(input[54] & 0xc0) << 13);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 434 to 446
	value = ((uint32_t)(input[54] & 0x3f) << 26) |
			((uint32_t)(input[55] & 0xfe) << 18);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 447 to 459
	value = ((uint32_t)(input[55] & 0x01) << 31) |
			((uint32_t)input[56] << 23) |
			((uint32_t)(input[57] & 0xf0) << 15);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 460 to 472
	value = ((uint32_t)(input[57] & 0x0f) << 28) |
			((uint32_t)input[58] << 20) |
			((uint32_t)(input[59] & 0x80) << 12);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 473 to 485
	value = ((uint32_t)(input[59] & 0x7f) << 25) |
			((uint32_t)(input[60] & 0xfc) << 17);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 486 to 498
	value = ((uint32_t)(input[60] & 0x03) << 30) |
			((uint32_t)input[61] << 22) |
			((uint32_t)(input[62] & 0xe0) << 14);
	output[37] = (double)(value >> 19);

	// read output[38]: from bit 499 to 511
	value = ((uint32_t)(input[62] & 0x1f) << 27) |
			((uint32_t)input[63] << 19);
	output[38] = (double)(value >> 19);
}

static void unpack_39_13bit_signed_6off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 13-bit signed
	// Skip 6 bits, read 507 bits of data.
	// Input must be at least 65 bytes long

	int32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 383 to 395
	value = ((uint32_t)(input[47] & 0x01) << 31) |
			((uint32_t)input[48] << 23) |
			((uint32_t)(input[49] & 0xf0) << 15);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 396 to 408
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0x80) << 12);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 409 to 421
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xfc) << 17);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 422 to 434
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xe0) << 14);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 435 to 447
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 448 to 460
	value = ((uint32_t)input[56] << 24) |
			((uint32_t)(input[57] & 0xf8) << 16);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 461 to 473
	value = ((uint32_t)(input[57] & 0x07) << 29) |
			((uint32_t)input[58] << 21) |
			((uint32_t)(input[59] & 0xc0) << 13);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 474 to 486
	value = ((uint32_t)(input[59] & 0x3f) << 26) |
			((uint32_t)(input[60] & 0xfe) << 18);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 487 to 499
	value = ((uint32_t)(input[60] & 0x01) << 31) |
			((uint32_t)input[61] << 23) |
			((uint32_t)(input[62] & 0xf0) << 15);
	output[37] = (double)(value >> 19);

	// read output[38]: from bit 500 to 512
	value = ((uint32_t)(input[62] & 0x0f) << 28) |
			((uint32_t)input[63] << 20) |
			((uint32_t)(input[64] & 0x80) << 12);
	output[38] = (double)(value >> 19);
}

static void unpack_39_13bit_unsigned_6off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 13-bit unsigned
	// Skip 6 bits, read 507 bits of data.
	// Input must be at least 65 bytes long

	uint32_t value;

	// read output[0]: from bit 6 to 18
	value = ((uint32_t)(input[0] & 0x03) << 30) |
			((uint32_t)input[1] << 22) |
			((uint32_t)(input[2] & 0xe0) << 14);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 19 to 31
	value = ((uint32_t)(input[2] & 0x1f) << 27) |
			((uint32_t)input[3] << 19);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 32 to 44
	value = ((uint32_t)input[4] << 24) |
			((uint32_t)(input[5] & 0xf8) << 16);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 45 to 57
	value = ((uint32_t)(input[5] & 0x07) << 29) |
			((uint32_t)input[6] << 21) |
			((uint32_t)(input[7] & 0xc0) << 13);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 58 to 70
	value = ((uint32_t)(input[7] & 0x3f) << 26) |
			((uint32_t)(input[8] & 0xfe) << 18);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 71 to 83
	value = ((uint32_t)(input[8] & 0x01) << 31) |
			((uint32_t)input[9] << 23) |
			((uint32_t)(input[10] & 0xf0) << 15);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 84 to 96
	value = ((uint32_t)(input[10] & 0x0f) << 28) |
			((uint32_t)input[11] << 20) |
			((uint32_t)(input[12] & 0x80) << 12);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 97 to 109
	value = ((uint32_t)(input[12] & 0x7f) << 25) |
			((uint32_t)(input[13] & 0xfc) << 17);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 110 to 122
	value = ((uint32_t)(input[13] & 0x03) << 30) |
			((uint32_t)input[14] << 22) |
			((uint32_t)(input[15] & 0xe0) << 14);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 123 to 135
	value = ((uint32_t)(input[15] & 0x1f) << 27) |
			((uint32_t)input[16] << 19);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 136 to 148
	value = ((uint32_t)input[17] << 24) |
			((uint32_t)(input[18] & 0xf8) << 16);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 149 to 161
	value = ((uint32_t)(input[18] & 0x07) << 29) |
			((uint32_t)input[19] << 21) |
			((uint32_t)(input[20] & 0xc0) << 13);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 162 to 174
	value = ((uint32_t)(input[20] & 0x3f) << 26) |
			((uint32_t)(input[21] & 0xfe) << 18);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 175 to 187
	value = ((uint32_t)(input[21] & 0x01) << 31) |
			((uint32_t)input[22] << 23) |
			((uint32_t)(input[23] & 0xf0) << 15);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 188 to 200
	value = ((uint32_t)(input[23] & 0x0f) << 28) |
			((uint32_t)input[24] << 20) |
			((uint32_t)(input[25] & 0x80) << 12);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 201 to 213
	value = ((uint32_t)(input[25] & 0x7f) << 25) |
			((uint32_t)(input[26] & 0xfc) << 17);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 214 to 226
	value = ((uint32_t)(input[26] & 0x03) << 30) |
			((uint32_t)input[27] << 22) |
			((uint32_t)(input[28] & 0xe0) << 14);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 227 to 239
	value = ((uint32_t)(input[28] & 0x1f) << 27) |
			((uint32_t)input[29] << 19);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 240 to 252
	value = ((uint32_t)input[30] << 24) |
			((uint32_t)(input[31] & 0xf8) << 16);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 253 to 265
	value = ((uint32_t)(input[31] & 0x07) << 29) |
			((uint32_t)input[32] << 21) |
			((uint32_t)(input[33] & 0xc0) << 13);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 266 to 278
	value = ((uint32_t)(input[33] & 0x3f) << 26) |
			((uint32_t)(input[34] & 0xfe) << 18);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 279 to 291
	value = ((uint32_t)(input[34] & 0x01) << 31) |
			((uint32_t)input[35] << 23) |
			((uint32_t)(input[36] & 0xf0) << 15);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 292 to 304
	value = ((uint32_t)(input[36] & 0x0f) << 28) |
			((uint32_t)input[37] << 20) |
			((uint32_t)(input[38] & 0x80) << 12);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 305 to 317
	value = ((uint32_t)(input[38] & 0x7f) << 25) |
			((uint32_t)(input[39] & 0xfc) << 17);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 318 to 330
	value = ((uint32_t)(input[39] & 0x03) << 30) |
			((uint32_t)input[40] << 22) |
			((uint32_t)(input[41] & 0xe0) << 14);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 331 to 343
	value = ((uint32_t)(input[41] & 0x1f) << 27) |
			((uint32_t)input[42] << 19);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 344 to 356
	value = ((uint32_t)input[43] << 24) |
			((uint32_t)(input[44] & 0xf8) << 16);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 357 to 369
	value = ((uint32_t)(input[44] & 0x07) << 29) |
			((uint32_t)input[45] << 21) |
			((uint32_t)(input[46] & 0xc0) << 13);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 370 to 382
	value = ((uint32_t)(input[46] & 0x3f) << 26) |
			((uint32_t)(input[47] & 0xfe) << 18);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 383 to 395
	value = ((uint32_t)(input[47] & 0x01) << 31) |
			((uint32_t)input[48] << 23) |
			((uint32_t)(input[49] & 0xf0) << 15);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 396 to 408
	value = ((uint32_t)(input[49] & 0x0f) << 28) |
			((uint32_t)input[50] << 20) |
			((uint32_t)(input[51] & 0x80) << 12);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 409 to 421
	value = ((uint32_t)(input[51] & 0x7f) << 25) |
			((uint32_t)(input[52] & 0xfc) << 17);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 422 to 434
	value = ((uint32_t)(input[52] & 0x03) << 30) |
			((uint32_t)input[53] << 22) |
			((uint32_t)(input[54] & 0xe0) << 14);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 435 to 447
	value = ((uint32_t)(input[54] & 0x1f) << 27) |
			((uint32_t)input[55] << 19);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 448 to 460
	value = ((uint32_t)input[56] << 24) |
			((uint32_t)(input[57] & 0xf8) << 16);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 461 to 473
	value = ((uint32_t)(input[57] & 0x07) << 29) |
			((uint32_t)input[58] << 21) |
			((uint32_t)(input[59] & 0xc0) << 13);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 474 to 486
	value = ((uint32_t)(input[59] & 0x3f) << 26) |
			((uint32_t)(input[60] & 0xfe) << 18);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 487 to 499
	value = ((uint32_t)(input[60] & 0x01) << 31) |
			((uint32_t)input[61] << 23) |
			((uint32_t)(input[62] & 0xf0) << 15);
	output[37] = (double)(value >> 19);

	// read output[38]: from bit 500 to 512
	value = ((uint32_t)(input[62] & 0x0f) << 28) |
			((uint32_t)input[63] << 20) |
			((uint32_t)(input[64] & 0x80) << 12);
	output[38] = (double)(value >> 19);
}

static void unpack_39_13bit_signed_7off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 13-bit signed
	// Skip 7 bits, read 507 bits of data.
	// Input must be at least 65 bytes long

	int32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 384 to 396
	value = ((uint32_t)input[48] << 24) |
			((uint32_t)(input[49] & 0xf8) << 16);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 397 to 409
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xc0) << 13);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 410 to 422
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xfe) << 18);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 423 to 435
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xf0) << 15);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 436 to 448
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0x80) << 12);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 449 to 461
	value = ((uint32_t)(input[56] & 0x7f) << 25) |
			((uint32_t)(input[57] & 0xfc) << 17);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 462 to 474
	value = ((uint32_t)(input[57] & 0x03) << 30) |
			((uint32_t)input[58] << 22) |
			((uint32_t)(input[59] & 0xe0) << 14);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 475 to 487
	value = ((uint32_t)(input[59] & 0x1f) << 27) |
			((uint32_t)input[60] << 19);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 488 to 500
	value = ((uint32_t)input[61] << 24) |
			((uint32_t)(input[62] & 0xf8) << 16);
	output[37] = (double)(value >> 19);

	// read output[38]: from bit 501 to 513
	value = ((uint32_t)(input[62] & 0x07) << 29) |
			((uint32_t)input[63] << 21) |
			((uint32_t)(input[64] & 0xc0) << 13);
	output[38] = (double)(value >> 19);
}

static void unpack_39_13bit_unsigned_7off(const uint8_t * input, double * output) {
	// Unpack 39 values into output
	// Values are 13-bit unsigned
	// Skip 7 bits, read 507 bits of data.
	// Input must be at least 65 bytes long

	uint32_t value;

	// read output[0]: from bit 7 to 19
	value = ((uint32_t)(input[0] & 0x01) << 31) |
			((uint32_t)input[1] << 23) |
			((uint32_t)(input[2] & 0xf0) << 15);
	output[0] = (double)(value >> 19);

	// read output[1]: from bit 20 to 32
	value = ((uint32_t)(input[2] & 0x0f) << 28) |
			((uint32_t)input[3] << 20) |
			((uint32_t)(input[4] & 0x80) << 12);
	output[1] = (double)(value >> 19);

	// read output[2]: from bit 33 to 45
	value = ((uint32_t)(input[4] & 0x7f) << 25) |
			((uint32_t)(input[5] & 0xfc) << 17);
	output[2] = (double)(value >> 19);

	// read output[3]: from bit 46 to 58
	value = ((uint32_t)(input[5] & 0x03) << 30) |
			((uint32_t)input[6] << 22) |
			((uint32_t)(input[7] & 0xe0) << 14);
	output[3] = (double)(value >> 19);

	// read output[4]: from bit 59 to 71
	value = ((uint32_t)(input[7] & 0x1f) << 27) |
			((uint32_t)input[8] << 19);
	output[4] = (double)(value >> 19);

	// read output[5]: from bit 72 to 84
	value = ((uint32_t)input[9] << 24) |
			((uint32_t)(input[10] & 0xf8) << 16);
	output[5] = (double)(value >> 19);

	// read output[6]: from bit 85 to 97
	value = ((uint32_t)(input[10] & 0x07) << 29) |
			((uint32_t)input[11] << 21) |
			((uint32_t)(input[12] & 0xc0) << 13);
	output[6] = (double)(value >> 19);

	// read output[7]: from bit 98 to 110
	value = ((uint32_t)(input[12] & 0x3f) << 26) |
			((uint32_t)(input[13] & 0xfe) << 18);
	output[7] = (double)(value >> 19);

	// read output[8]: from bit 111 to 123
	value = ((uint32_t)(input[13] & 0x01) << 31) |
			((uint32_t)input[14] << 23) |
			((uint32_t)(input[15] & 0xf0) << 15);
	output[8] = (double)(value >> 19);

	// read output[9]: from bit 124 to 136
	value = ((uint32_t)(input[15] & 0x0f) << 28) |
			((uint32_t)input[16] << 20) |
			((uint32_t)(input[17] & 0x80) << 12);
	output[9] = (double)(value >> 19);

	// read output[10]: from bit 137 to 149
	value = ((uint32_t)(input[17] & 0x7f) << 25) |
			((uint32_t)(input[18] & 0xfc) << 17);
	output[10] = (double)(value >> 19);

	// read output[11]: from bit 150 to 162
	value = ((uint32_t)(input[18] & 0x03) << 30) |
			((uint32_t)input[19] << 22) |
			((uint32_t)(input[20] & 0xe0) << 14);
	output[11] = (double)(value >> 19);

	// read output[12]: from bit 163 to 175
	value = ((uint32_t)(input[20] & 0x1f) << 27) |
			((uint32_t)input[21] << 19);
	output[12] = (double)(value >> 19);

	// read output[13]: from bit 176 to 188
	value = ((uint32_t)input[22] << 24) |
			((uint32_t)(input[23] & 0xf8) << 16);
	output[13] = (double)(value >> 19);

	// read output[14]: from bit 189 to 201
	value = ((uint32_t)(input[23] & 0x07) << 29) |
			((uint32_t)input[24] << 21) |
			((uint32_t)(input[25] & 0xc0) << 13);
	output[14] = (double)(value >> 19);

	// read output[15]: from bit 202 to 214
	value = ((uint32_t)(input[25] & 0x3f) << 26) |
			((uint32_t)(input[26] & 0xfe) << 18);
	output[15] = (double)(value >> 19);

	// read output[16]: from bit 215 to 227
	value = ((uint32_t)(input[26] & 0x01) << 31) |
			((uint32_t)input[27] << 23) |
			((uint32_t)(input[28] & 0xf0) << 15);
	output[16] = (double)(value >> 19);

	// read output[17]: from bit 228 to 240
	value = ((uint32_t)(input[28] & 0x0f) << 28) |
			((uint32_t)input[29] << 20) |
			((uint32_t)(input[30] & 0x80) << 12);
	output[17] = (double)(value >> 19);

	// read output[18]: from bit 241 to 253
	value = ((uint32_t)(input[30] & 0x7f) << 25) |
			((uint32_t)(input[31] & 0xfc) << 17);
	output[18] = (double)(value >> 19);

	// read output[19]: from bit 254 to 266
	value = ((uint32_t)(input[31] & 0x03) << 30) |
			((uint32_t)input[32] << 22) |
			((uint32_t)(input[33] & 0xe0) << 14);
	output[19] = (double)(value >> 19);

	// read output[20]: from bit 267 to 279
	value = ((uint32_t)(input[33] & 0x1f) << 27) |
			((uint32_t)input[34] << 19);
	output[20] = (double)(value >> 19);

	// read output[21]: from bit 280 to 292
	value = ((uint32_t)input[35] << 24) |
			((uint32_t)(input[36] & 0xf8) << 16);
	output[21] = (double)(value >> 19);

	// read output[22]: from bit 293 to 305
	value = ((uint32_t)(input[36] & 0x07) << 29) |
			((uint32_t)input[37] << 21) |
			((uint32_t)(input[38] & 0xc0) << 13);
	output[22] = (double)(value >> 19);

	// read output[23]: from bit 306 to 318
	value = ((uint32_t)(input[38] & 0x3f) << 26) |
			((uint32_t)(input[39] & 0xfe) << 18);
	output[23] = (double)(value >> 19);

	// read output[24]: from bit 319 to 331
	value = ((uint32_t)(input[39] & 0x01) << 31) |
			((uint32_t)input[40] << 23) |
			((uint32_t)(input[41] & 0xf0) << 15);
	output[24] = (double)(value >> 19);

	// read output[25]: from bit 332 to 344
	value = ((uint32_t)(input[41] & 0x0f) << 28) |
			((uint32_t)input[42] << 20) |
			((uint32_t)(input[43] & 0x80) << 12);
	output[25] = (double)(value >> 19);

	// read output[26]: from bit 345 to 357
	value = ((uint32_t)(input[43] & 0x7f) << 25) |
			((uint32_t)(input[44] & 0xfc) << 17);
	output[26] = (double)(value >> 19);

	// read output[27]: from bit 358 to 370
	value = ((uint32_t)(input[44] & 0x03) << 30) |
			((uint32_t)input[45] << 22) |
			((uint32_t)(input[46] & 0xe0) << 14);
	output[27] = (double)(value >> 19);

	// read output[28]: from bit 371 to 383
	value = ((uint32_t)(input[46] & 0x1f) << 27) |
			((uint32_t)input[47] << 19);
	output[28] = (double)(value >> 19);

	// read output[29]: from bit 384 to 396
	value = ((uint32_t)input[48] << 24) |
			((uint32_t)(input[49] & 0xf8) << 16);
	output[29] = (double)(value >> 19);

	// read output[30]: from bit 397 to 409
	value = ((uint32_t)(input[49] & 0x07) << 29) |
			((uint32_t)input[50] << 21) |
			((uint32_t)(input[51] & 0xc0) << 13);
	output[30] = (double)(value >> 19);

	// read output[31]: from bit 410 to 422
	value = ((uint32_t)(input[51] & 0x3f) << 26) |
			((uint32_t)(input[52] & 0xfe) << 18);
	output[31] = (double)(value >> 19);

	// read output[32]: from bit 423 to 435
	value = ((uint32_t)(input[52] & 0x01) << 31) |
			((uint32_t)input[53] << 23) |
			((uint32_t)(input[54] & 0xf0) << 15);
	output[32] = (double)(value >> 19);

	// read output[33]: from bit 436 to 448
	value = ((uint32_t)(input[54] & 0x0f) << 28) |
			((uint32_t)input[55] << 20) |
			((uint32_t)(input[56] & 0x80) << 12);
	output[33] = (double)(value >> 19);

	// read output[34]: from bit 449 to 461
	value = ((uint32_t)(input[56] & 0x7f) << 25) |
			((uint32_t)(input[57] & 0xfc) << 17);
	output[34] = (double)(value >> 19);

	// read output[35]: from bit 462 to 474
	value = ((uint32_t)(input[57] & 0x03) << 30) |
			((uint32_t)input[58] << 22) |
			((uint32_t)(input[59] & 0xe0) << 14);
	output[35] = (double)(value >> 19);

	// read output[36]: from bit 475 to 487
	value = ((uint32_t)(input[59] & 0x1f) << 27) |
			((uint32_t)input[60] << 19);
	output[36] = (double)(value >> 19);

	// read output[37]: from bit 488 to 500
	value = ((uint32_t)input[61] << 24) |
			((uint32_t)(input[62] & 0xf8) << 16);
	output[37] = (double)(value >> 19);

	// read output[38]: from bit 501 to 513
	value = ((uint32_t)(input[62] & 0x07) << 29) |
			((uint32_t)input[63] << 21) |
			((uint32_t)(input[64] & 0xc0) << 13);
	output[38] = (double)(value >> 19);
}


unpack_func_t unpack_13bit[UNPACK_13BIT_MAX_COUNT][8][2] = {
	{
		{
			unpack_1_13bit_unsigned_0off,
			unpack_1_13bit_signed_0off,
		},
		{
			unpack_1_13bit_unsigned_1off,
			unpack_1_13bit_signed_1off,
		},
		{
			unpack_1_13bit_unsigned_2off,
			unpack_1_13bit_signed_2off,
		},
		{
			unpack_1_13bit_unsigned_3off,
			unpack_1_13bit_signed_3off,
		},
		{
			unpack_1_13bit_unsigned_4off,
			unpack_1_13bit_signed_4off,
		},
		{
			unpack_1_13bit_unsigned_5off,
			unpack_1_13bit_signed_5off,
		},
		{
			unpack_1_13bit_unsigned_6off,
			unpack_1_13bit_signed_6off,
		},
		{
			unpack_1_13bit_unsigned_7off,
			unpack_1_13bit_signed_7off,
		},
	},
	{
		{
			unpack_2_13bit_unsigned_0off,
			unpack_2_13bit_signed_0off,
		},
		{
			unpack_2_13bit_unsigned_1off,
			unpack_2_13bit_signed_1off,
		},
		{
			unpack_2_13bit_unsigned_2off,
			unpack_2_13bit_signed_2off,
		},
		{
			unpack_2_13bit_unsigned_3off,
			unpack_2_13bit_signed_3off,
		},
		{
			unpack_2_13bit_unsigned_4off,
			unpack_2_13bit_signed_4off,
		},
		{
			unpack_2_13bit_unsigned_5off,
			unpack_2_13bit_signed_5off,
		},
		{
			unpack_2_13bit_unsigned_6off,
			unpack_2_13bit_signed_6off,
		},
		{
			unpack_2_13bit_unsigned_7off,
			unpack_2_13bit_signed_7off,
		},
	},
	{
		{
			unpack_3_13bit_unsigned_0off,
			unpack_3_13bit_signed_0off,
		},
		{
			unpack_3_13bit_unsigned_1off,
			unpack_3_13bit_signed_1off,
		},
		{
			unpack_3_13bit_unsigned_2off,
			unpack_3_13bit_signed_2off,
		},
		{
			unpack_3_13bit_unsigned_3off,
			unpack_3_13bit_signed_3off,
		},
		{
			unpack_3_13bit_unsigned_4off,
			unpack_3_13bit_signed_4off,
		},
		{
			unpack_3_13bit_unsigned_5off,
			unpack_3_13bit_signed_5off,
		},
		{
			unpack_3_13bit_unsigned_6off,
			unpack_3_13bit_signed_6off,
		},
		{
			unpack_3_13bit_unsigned_7off,
			unpack_3_13bit_signed_7off,
		},
	},
	{
		{
			unpack_4_13bit_unsigned_0off,
			unpack_4_13bit_signed_0off,
		},
		{
			unpack_4_13bit_unsigned_1off,
			unpack_4_13bit_signed_1off,
		},
		{
			unpack_4_13bit_unsigned_2off,
			unpack_4_13bit_signed_2off,
		},
		{
			unpack_4_13bit_unsigned_3off,
			unpack_4_13bit_signed_3off,
		},
		{
			unpack_4_13bit_unsigned_4off,
			unpack_4_13bit_signed_4off,
		},
		{
			unpack_4_13bit_unsigned_5off,
			unpack_4_13bit_signed_5off,
		},
		{
			unpack_4_13bit_unsigned_6off,
			unpack_4_13bit_signed_6off,
		},
		{
			unpack_4_13bit_unsigned_7off,
			unpack_4_13bit_signed_7off,
		},
	},
	{
		{
			unpack_5_13bit_unsigned_0off,
			unpack_5_13bit_signed_0off,
		},
		{
			unpack_5_13bit_unsigned_1off,
			unpack_5_13bit_signed_1off,
		},
		{
			unpack_5_13bit_unsigned_2off,
			unpack_5_13bit_signed_2off,
		},
		{
			unpack_5_13bit_unsigned_3off,
			unpack_5_13bit_signed_3off,
		},
		{
			unpack_5_13bit_unsigned_4off,
			unpack_5_13bit_signed_4off,
		},
		{
			unpack_5_13bit_unsigned_5off,
			unpack_5_13bit_signed_5off,
		},
		{
			unpack_5_13bit_unsigned_6off,
			unpack_5_13bit_signed_6off,
		},
		{
			unpack_5_13bit_unsigned_7off,
			unpack_5_13bit_signed_7off,
		},
	},
	{
		{
			unpack_6_13bit_unsigned_0off,
			unpack_6_13bit_signed_0off,
		},
		{
			unpack_6_13bit_unsigned_1off,
			unpack_6_13bit_signed_1off,
		},
		{
			unpack_6_13bit_unsigned_2off,
			unpack_6_13bit_signed_2off,
		},
		{
			unpack_6_13bit_unsigned_3off,
			unpack_6_13bit_signed_3off,
		},
		{
			unpack_6_13bit_unsigned_4off,
			unpack_6_13bit_signed_4off,
		},
		{
			unpack_6_13bit_unsigned_5off,
			unpack_6_13bit_signed_5off,
		},
		{
			unpack_6_13bit_unsigned_6off,
			unpack_6_13bit_signed_6off,
		},
		{
			unpack_6_13bit_unsigned_7off,
			unpack_6_13bit_signed_7off,
		},
	},
	{
		{
			unpack_7_13bit_unsigned_0off,
			unpack_7_13bit_signed_0off,
		},
		{
			unpack_7_13bit_unsigned_1off,
			unpack_7_13bit_signed_1off,
		},
		{
			unpack_7_13bit_unsigned_2off,
			unpack_7_13bit_signed_2off,
		},
		{
			unpack_7_13bit_unsigned_3off,
			unpack_7_13bit_signed_3off,
		},
		{
			unpack_7_13bit_unsigned_4off,
			unpack_7_13bit_signed_4off,
		},
		{
			unpack_7_13bit_unsigned_5off,
			unpack_7_13bit_signed_5off,
		},
		{
			unpack_7_13bit_unsigned_6off,
			unpack_7_13bit_signed_6off,
		},
		{
			unpack_7_13bit_unsigned_7off,
			unpack_7_13bit_signed_7off,
		},
	},
	{
		{
			unpack_8_13bit_unsigned_0off,
			unpack_8_13bit_signed_0off,
		},
		{
			unpack_8_13bit_unsigned_1off,
			unpack_8_13bit_signed_1off,
		},
		{
			unpack_8_13bit_unsigned_2off,
			unpack_8_13bit_signed_2off,
		},
		{
			unpack_8_13bit_unsigned_3off,
			unpack_8_13bit_signed_3off,
		},
		{
			unpack_8_13bit_unsigned_4off,
			unpack_8_13bit_signed_4off,
		},
		{
			unpack_8_13bit_unsigned_5off,
			unpack_8_13bit_signed_5off,
		},
		{
			unpack_8_13bit_unsigned_6off,
			unpack_8_13bit_signed_6off,
		},
		{
			unpack_8_13bit_unsigned_7off,
			unpack_8_13bit_signed_7off,
		},
	},
	{
		{
			unpack_9_13bit_unsigned_0off,
			unpack_9_13bit_signed_0off,
		},
		{
			unpack_9_13bit_unsigned_1off,
			unpack_9_13bit_signed_1off,
		},
		{
			unpack_9_13bit_unsigned_2off,
			unpack_9_13bit_signed_2off,
		},
		{
			unpack_9_13bit_unsigned_3off,
			unpack_9_13bit_signed_3off,
		},
		{
			unpack_9_13bit_unsigned_4off,
			unpack_9_13bit_signed_4off,
		},
		{
			unpack_9_13bit_unsigned_5off,
			unpack_9_13bit_signed_5off,
		},
		{
			unpack_9_13bit_unsigned_6off,
			unpack_9_13bit_signed_6off,
		},
		{
			unpack_9_13bit_unsigned_7off,
			unpack_9_13bit_signed_7off,
		},
	},
	{
		{
			unpack_10_13bit_unsigned_0off,
			unpack_10_13bit_signed_0off,
		},
		{
			unpack_10_13bit_unsigned_1off,
			unpack_10_13bit_signed_1off,
		},
		{
			unpack_10_13bit_unsigned_2off,
			unpack_10_13bit_signed_2off,
		},
		{
			unpack_10_13bit_unsigned_3off,
			unpack_10_13bit_signed_3off,
		},
		{
			unpack_10_13bit_unsigned_4off,
			unpack_10_13bit_signed_4off,
		},
		{
			unpack_10_13bit_unsigned_5off,
			unpack_10_13bit_signed_5off,
		},
		{
			unpack_10_13bit_unsigned_6off,
			unpack_10_13bit_signed_6off,
		},
		{
			unpack_10_13bit_unsigned_7off,
			unpack_10_13bit_signed_7off,
		},
	},
	{
		{
			unpack_11_13bit_unsigned_0off,
			unpack_11_13bit_signed_0off,
		},
		{
			unpack_11_13bit_unsigned_1off,
			unpack_11_13bit_signed_1off,
		},
		{
			unpack_11_13bit_unsigned_2off,
			unpack_11_13bit_signed_2off,
		},
		{
			unpack_11_13bit_unsigned_3off,
			unpack_11_13bit_signed_3off,
		},
		{
			unpack_11_13bit_unsigned_4off,
			unpack_11_13bit_signed_4off,
		},
		{
			unpack_11_13bit_unsigned_5off,
			unpack_11_13bit_signed_5off,
		},
		{
			unpack_11_13bit_unsigned_6off,
			unpack_11_13bit_signed_6off,
		},
		{
			unpack_11_13bit_unsigned_7off,
			unpack_11_13bit_signed_7off,
		},
	},
	{
		{
			unpack_12_13bit_unsigned_0off,
			unpack_12_13bit_signed_0off,
		},
		{
			unpack_12_13bit_unsigned_1off,
			unpack_12_13bit_signed_1off,
		},
		{
			unpack_12_13bit_unsigned_2off,
			unpack_12_13bit_signed_2off,
		},
		{
			unpack_12_13bit_unsigned_3off,
			unpack_12_13bit_signed_3off,
		},
		{
			unpack_12_13bit_unsigned_4off,
			unpack_12_13bit_signed_4off,
		},
		{
			unpack_12_13bit_unsigned_5off,
			unpack_12_13bit_signed_5off,
		},
		{
			unpack_12_13bit_unsigned_6off,
			unpack_12_13bit_signed_6off,
		},
		{
			unpack_12_13bit_unsigned_7off,
			unpack_12_13bit_signed_7off,
		},
	},
	{
		{
			unpack_13_13bit_unsigned_0off,
			unpack_13_13bit_signed_0off,
		},
		{
			unpack_13_13bit_unsigned_1off,
			unpack_13_13bit_signed_1off,
		},
		{
			unpack_13_13bit_unsigned_2off,
			unpack_13_13bit_signed_2off,
		},
		{
			unpack_13_13bit_unsigned_3off,
			unpack_13_13bit_signed_3off,
		},
		{
			unpack_13_13bit_unsigned_4off,
			unpack_13_13bit_signed_4off,
		},
		{
			unpack_13_13bit_unsigned_5off,
			unpack_13_13bit_signed_5off,
		},
		{
			unpack_13_13bit_unsigned_6off,
			unpack_13_13bit_signed_6off,
		},
		{
			unpack_13_13bit_unsigned_7off,
			unpack_13_13bit_signed_7off,
		},
	},
	{
		{
			unpack_14_13bit_unsigned_0off,
			unpack_14_13bit_signed_0off,
		},
		{
			unpack_14_13bit_unsigned_1off,
			unpack_14_13bit_signed_1off,
		},
		{
			unpack_14_13bit_unsigned_2off,
			unpack_14_13bit_signed_2off,
		},
		{
			unpack_14_13bit_unsigned_3off,
			unpack_14_13bit_signed_3off,
		},
		{
			unpack_14_13bit_unsigned_4off,
			unpack_14_13bit_signed_4off,
		},
		{
			unpack_14_13bit_unsigned_5off,
			unpack_14_13bit_signed_5off,
		},
		{
			unpack_14_13bit_unsigned_6off,
			unpack_14_13bit_signed_6off,
		},
		{
			unpack_14_13bit_unsigned_7off,
			unpack_14_13bit_signed_7off,
		},
	},
	{
		{
			unpack_15_13bit_unsigned_0off,
			unpack_15_13bit_signed_0off,
		},
		{
			unpack_15_13bit_unsigned_1off,
			unpack_15_13bit_signed_1off,
		},
		{
			unpack_15_13bit_unsigned_2off,
			unpack_15_13bit_signed_2off,
		},
		{
			unpack_15_13bit_unsigned_3off,
			unpack_15_13bit_signed_3off,
		},
		{
			unpack_15_13bit_unsigned_4off,
			unpack_15_13bit_signed_4off,
		},
		{
			unpack_15_13bit_unsigned_5off,
			unpack_15_13bit_signed_5off,
		},
		{
			unpack_15_13bit_unsigned_6off,
			unpack_15_13bit_signed_6off,
		},
		{
			unpack_15_13bit_unsigned_7off,
			unpack_15_13bit_signed_7off,
		},
	},
	{
		{
			unpack_16_13bit_unsigned_0off,
			unpack_16_13bit_signed_0off,
		},
		{
			unpack_16_13bit_unsigned_1off,
			unpack_16_13bit_signed_1off,
		},
		{
			unpack_16_13bit_unsigned_2off,
			unpack_16_13bit_signed_2off,
		},
		{
			unpack_16_13bit_unsigned_3off,
			unpack_16_13bit_signed_3off,
		},
		{
			unpack_16_13bit_unsigned_4off,
			unpack_16_13bit_signed_4off,
		},
		{
			unpack_16_13bit_unsigned_5off,
			unpack_16_13bit_signed_5off,
		},
		{
			unpack_16_13bit_unsigned_6off,
			unpack_16_13bit_signed_6off,
		},
		{
			unpack_16_13bit_unsigned_7off,
			unpack_16_13bit_signed_7off,
		},
	},
	{
		{
			unpack_17_13bit_unsigned_0off,
			unpack_17_13bit_signed_0off,
		},
		{
			unpack_17_13bit_unsigned_1off,
			unpack_17_13bit_signed_1off,
		},
		{
			unpack_17_13bit_unsigned_2off,
			unpack_17_13bit_signed_2off,
		},
		{
			unpack_17_13bit_unsigned_3off,
			unpack_17_13bit_signed_3off,
		},
		{
			unpack_17_13bit_unsigned_4off,
			unpack_17_13bit_signed_4off,
		},
		{
			unpack_17_13bit_unsigned_5off,
			unpack_17_13bit_signed_5off,
		},
		{
			unpack_17_13bit_unsigned_6off,
			unpack_17_13bit_signed_6off,
		},
		{
			unpack_17_13bit_unsigned_7off,
			unpack_17_13bit_signed_7off,
		},
	},
	{
		{
			unpack_18_13bit_unsigned_0off,
			unpack_18_13bit_signed_0off,
		},
		{
			unpack_18_13bit_unsigned_1off,
			unpack_18_13bit_signed_1off,
		},
		{
			unpack_18_13bit_unsigned_2off,
			unpack_18_13bit_signed_2off,
		},
		{
			unpack_18_13bit_unsigned_3off,
			unpack_18_13bit_signed_3off,
		},
		{
			unpack_18_13bit_unsigned_4off,
			unpack_18_13bit_signed_4off,
		},
		{
			unpack_18_13bit_unsigned_5off,
			unpack_18_13bit_signed_5off,
		},
		{
			unpack_18_13bit_unsigned_6off,
			unpack_18_13bit_signed_6off,
		},
		{
			unpack_18_13bit_unsigned_7off,
			unpack_18_13bit_signed_7off,
		},
	},
	{
		{
			unpack_19_13bit_unsigned_0off,
			unpack_19_13bit_signed_0off,
		},
		{
			unpack_19_13bit_unsigned_1off,
			unpack_19_13bit_signed_1off,
		},
		{
			unpack_19_13bit_unsigned_2off,
			unpack_19_13bit_signed_2off,
		},
		{
			unpack_19_13bit_unsigned_3off,
			unpack_19_13bit_signed_3off,
		},
		{
			unpack_19_13bit_unsigned_4off,
			unpack_19_13bit_signed_4off,
		},
		{
			unpack_19_13bit_unsigned_5off,
			unpack_19_13bit_signed_5off,
		},
		{
			unpack_19_13bit_unsigned_6off,
			unpack_19_13bit_signed_6off,
		},
		{
			unpack_19_13bit_unsigned_7off,
			unpack_19_13bit_signed_7off,
		},
	},
	{
		{
			unpack_20_13bit_unsigned_0off,
			unpack_20_13bit_signed_0off,
		},
		{
			unpack_20_13bit_unsigned_1off,
			unpack_20_13bit_signed_1off,
		},
		{
			unpack_20_13bit_unsigned_2off,
			unpack_20_13bit_signed_2off,
		},
		{
			unpack_20_13bit_unsigned_3off,
			unpack_20_13bit_signed_3off,
		},
		{
			unpack_20_13bit_unsigned_4off,
			unpack_20_13bit_signed_4off,
		},
		{
			unpack_20_13bit_unsigned_5off,
			unpack_20_13bit_signed_5off,
		},
		{
			unpack_20_13bit_unsigned_6off,
			unpack_20_13bit_signed_6off,
		},
		{
			unpack_20_13bit_unsigned_7off,
			unpack_20_13bit_signed_7off,
		},
	},
	{
		{
			unpack_21_13bit_unsigned_0off,
			unpack_21_13bit_signed_0off,
		},
		{
			unpack_21_13bit_unsigned_1off,
			unpack_21_13bit_signed_1off,
		},
		{
			unpack_21_13bit_unsigned_2off,
			unpack_21_13bit_signed_2off,
		},
		{
			unpack_21_13bit_unsigned_3off,
			unpack_21_13bit_signed_3off,
		},
		{
			unpack_21_13bit_unsigned_4off,
			unpack_21_13bit_signed_4off,
		},
		{
			unpack_21_13bit_unsigned_5off,
			unpack_21_13bit_signed_5off,
		},
		{
			unpack_21_13bit_unsigned_6off,
			unpack_21_13bit_signed_6off,
		},
		{
			unpack_21_13bit_unsigned_7off,
			unpack_21_13bit_signed_7off,
		},
	},
	{
		{
			unpack_22_13bit_unsigned_0off,
			unpack_22_13bit_signed_0off,
		},
		{
			unpack_22_13bit_unsigned_1off,
			unpack_22_13bit_signed_1off,
		},
		{
			unpack_22_13bit_unsigned_2off,
			unpack_22_13bit_signed_2off,
		},
		{
			unpack_22_13bit_unsigned_3off,
			unpack_22_13bit_signed_3off,
		},
		{
			unpack_22_13bit_unsigned_4off,
			unpack_22_13bit_signed_4off,
		},
		{
			unpack_22_13bit_unsigned_5off,
			unpack_22_13bit_signed_5off,
		},
		{
			unpack_22_13bit_unsigned_6off,
			unpack_22_13bit_signed_6off,
		},
		{
			unpack_22_13bit_unsigned_7off,
			unpack_22_13bit_signed_7off,
		},
	},
	{
		{
			unpack_23_13bit_unsigned_0off,
			unpack_23_13bit_signed_0off,
		},
		{
			unpack_23_13bit_unsigned_1off,
			unpack_23_13bit_signed_1off,
		},
		{
			unpack_23_13bit_unsigned_2off,
			unpack_23_13bit_signed_2off,
		},
		{
			unpack_23_13bit_unsigned_3off,
			unpack_23_13bit_signed_3off,
		},
		{
			unpack_23_13bit_unsigned_4off,
			unpack_23_13bit_signed_4off,
		},
		{
			unpack_23_13bit_unsigned_5off,
			unpack_23_13bit_signed_5off,
		},
		{
			unpack_23_13bit_unsigned_6off,
			unpack_23_13bit_signed_6off,
		},
		{
			unpack_23_13bit_unsigned_7off,
			unpack_23_13bit_signed_7off,
		},
	},
	{
		{
			unpack_24_13bit_unsigned_0off,
			unpack_24_13bit_signed_0off,
		},
		{
			unpack_24_13bit_unsigned_1off,
			unpack_24_13bit_signed_1off,
		},
		{
			unpack_24_13bit_unsigned_2off,
			unpack_24_13bit_signed_2off,
		},
		{
			unpack_24_13bit_unsigned_3off,
			unpack_24_13bit_signed_3off,
		},
		{
			unpack_24_13bit_unsigned_4off,
			unpack_24_13bit_signed_4off,
		},
		{
			unpack_24_13bit_unsigned_5off,
			unpack_24_13bit_signed_5off,
		},
		{
			unpack_24_13bit_unsigned_6off,
			unpack_24_13bit_signed_6off,
		},
		{
			unpack_24_13bit_unsigned_7off,
			unpack_24_13bit_signed_7off,
		},
	},
	{
		{
			unpack_25_13bit_unsigned_0off,
			unpack_25_13bit_signed_0off,
		},
		{
			unpack_25_13bit_unsigned_1off,
			unpack_25_13bit_signed_1off,
		},
		{
			unpack_25_13bit_unsigned_2off,
			unpack_25_13bit_signed_2off,
		},
		{
			unpack_25_13bit_unsigned_3off,
			unpack_25_13bit_signed_3off,
		},
		{
			unpack_25_13bit_unsigned_4off,
			unpack_25_13bit_signed_4off,
		},
		{
			unpack_25_13bit_unsigned_5off,
			unpack_25_13bit_signed_5off,
		},
		{
			unpack_25_13bit_unsigned_6off,
			unpack_25_13bit_signed_6off,
		},
		{
			unpack_25_13bit_unsigned_7off,
			unpack_25_13bit_signed_7off,
		},
	},
	{
		{
			unpack_26_13bit_unsigned_0off,
			unpack_26_13bit_signed_0off,
		},
		{
			unpack_26_13bit_unsigned_1off,
			unpack_26_13bit_signed_1off,
		},
		{
			unpack_26_13bit_unsigned_2off,
			unpack_26_13bit_signed_2off,
		},
		{
			unpack_26_13bit_unsigned_3off,
			unpack_26_13bit_signed_3off,
		},
		{
			unpack_26_13bit_unsigned_4off,
			unpack_26_13bit_signed_4off,
		},
		{
			unpack_26_13bit_unsigned_5off,
			unpack_26_13bit_signed_5off,
		},
		{
			unpack_26_13bit_unsigned_6off,
			unpack_26_13bit_signed_6off,
		},
		{
			unpack_26_13bit_unsigned_7off,
			unpack_26_13bit_signed_7off,
		},
	},
	{
		{
			unpack_27_13bit_unsigned_0off,
			unpack_27_13bit_signed_0off,
		},
		{
			unpack_27_13bit_unsigned_1off,
			unpack_27_13bit_signed_1off,
		},
		{
			unpack_27_13bit_unsigned_2off,
			unpack_27_13bit_signed_2off,
		},
		{
			unpack_27_13bit_unsigned_3off,
			unpack_27_13bit_signed_3off,
		},
		{
			unpack_27_13bit_unsigned_4off,
			unpack_27_13bit_signed_4off,
		},
		{
			unpack_27_13bit_unsigned_5off,
			unpack_27_13bit_signed_5off,
		},
		{
			unpack_27_13bit_unsigned_6off,
			unpack_27_13bit_signed_6off,
		},
		{
			unpack_27_13bit_unsigned_7off,
			unpack_27_13bit_signed_7off,
		},
	},
	{
		{
			unpack_28_13bit_unsigned_0off,
			unpack_28_13bit_signed_0off,
		},
		{
			unpack_28_13bit_unsigned_1off,
			unpack_28_13bit_signed_1off,
		},
		{
			unpack_28_13bit_unsigned_2off,
			unpack_28_13bit_signed_2off,
		},
		{
			unpack_28_13bit_unsigned_3off,
			unpack_28_13bit_signed_3off,
		},
		{
			unpack_28_13bit_unsigned_4off,
			unpack_28_13bit_signed_4off,
		},
		{
			unpack_28_13bit_unsigned_5off,
			unpack_28_13bit_signed_5off,
		},
		{
			unpack_28_13bit_unsigned_6off,
			unpack_28_13bit_signed_6off,
		},
		{
			unpack_28_13bit_unsigned_7off,
			unpack_28_13bit_signed_7off,
		},
	},
	{
		{
			unpack_29_13bit_unsigned_0off,
			unpack_29_13bit_signed_0off,
		},
		{
			unpack_29_13bit_unsigned_1off,
			unpack_29_13bit_signed_1off,
		},
		{
			unpack_29_13bit_unsigned_2off,
			unpack_29_13bit_signed_2off,
		},
		{
			unpack_29_13bit_unsigned_3off,
			unpack_29_13bit_signed_3off,
		},
		{
			unpack_29_13bit_unsigned_4off,
			unpack_29_13bit_signed_4off,
		},
		{
			unpack_29_13bit_unsigned_5off,
			unpack_29_13bit_signed_5off,
		},
		{
			unpack_29_13bit_unsigned_6off,
			unpack_29_13bit_signed_6off,
		},
		{
			unpack_29_13bit_unsigned_7off,
			unpack_29_13bit_signed_7off,
		},
	},
	{
		{
			unpack_30_13bit_unsigned_0off,
			unpack_30_13bit_signed_0off,
		},
		{
			unpack_30_13bit_unsigned_1off,
			unpack_30_13bit_signed_1off,
		},
		{
			unpack_30_13bit_unsigned_2off,
			unpack_30_13bit_signed_2off,
		},
		{
			unpack_30_13bit_unsigned_3off,
			unpack_30_13bit_signed_3off,
		},
		{
			unpack_30_13bit_unsigned_4off,
			unpack_30_13bit_signed_4off,
		},
		{
			unpack_30_13bit_unsigned_5off,
			unpack_30_13bit_signed_5off,
		},
		{
			unpack_30_13bit_unsigned_6off,
			unpack_30_13bit_signed_6off,
		},
		{
			unpack_30_13bit_unsigned_7off,
			unpack_30_13bit_signed_7off,
		},
	},
	{
		{
			unpack_31_13bit_unsigned_0off,
			unpack_31_13bit_signed_0off,
		},
		{
			unpack_31_13bit_unsigned_1off,
			unpack_31_13bit_signed_1off,
		},
		{
			unpack_31_13bit_unsigned_2off,
			unpack_31_13bit_signed_2off,
		},
		{
			unpack_31_13bit_unsigned_3off,
			unpack_31_13bit_signed_3off,
		},
		{
			unpack_31_13bit_unsigned_4off,
			unpack_31_13bit_signed_4off,
		},
		{
			unpack_31_13bit_unsigned_5off,
			unpack_31_13bit_signed_5off,
		},
		{
			unpack_31_13bit_unsigned_6off,
			unpack_31_13bit_signed_6off,
		},
		{
			unpack_31_13bit_unsigned_7off,
			unpack_31_13bit_signed_7off,
		},
	},
	{
		{
			unpack_32_13bit_unsigned_0off,
			unpack_32_13bit_signed_0off,
		},
		{
			unpack_32_13bit_unsigned_1off,
			unpack_32_13bit_signed_1off,
		},
		{
			unpack_32_13bit_unsigned_2off,
			unpack_32_13bit_signed_2off,
		},
		{
			unpack_32_13bit_unsigned_3off,
			unpack_32_13bit_signed_3off,
		},
		{
			unpack_32_13bit_unsigned_4off,
			unpack_32_13bit_signed_4off,
		},
		{
			unpack_32_13bit_unsigned_5off,
			unpack_32_13bit_signed_5off,
		},
		{
			unpack_32_13bit_unsigned_6off,
			unpack_32_13bit_signed_6off,
		},
		{
			unpack_32_13bit_unsigned_7off,
			unpack_32_13bit_signed_7off,
		},
	},
	{
		{
			unpack_33_13bit_unsigned_0off,
			unpack_33_13bit_signed_0off,
		},
		{
			unpack_33_13bit_unsigned_1off,
			unpack_33_13bit_signed_1off,
		},
		{
			unpack_33_13bit_unsigned_2off,
			unpack_33_13bit_signed_2off,
		},
		{
			unpack_33_13bit_unsigned_3off,
			unpack_33_13bit_signed_3off,
		},
		{
			unpack_33_13bit_unsigned_4off,
			unpack_33_13bit_signed_4off,
		},
		{
			unpack_33_13bit_unsigned_5off,
			unpack_33_13bit_signed_5off,
		},
		{
			unpack_33_13bit_unsigned_6off,
			unpack_33_13bit_signed_6off,
		},
		{
			unpack_33_13bit_unsigned_7off,
			unpack_33_13bit_signed_7off,
		},
	},
	{
		{
			unpack_34_13bit_unsigned_0off,
			unpack_34_13bit_signed_0off,
		},
		{
			unpack_34_13bit_unsigned_1off,
			unpack_34_13bit_signed_1off,
		},
		{
			unpack_34_13bit_unsigned_2off,
			unpack_34_13bit_signed_2off,
		},
		{
			unpack_34_13bit_unsigned_3off,
			unpack_34_13bit_signed_3off,
		},
		{
			unpack_34_13bit_unsigned_4off,
			unpack_34_13bit_signed_4off,
		},
		{
			unpack_34_13bit_unsigned_5off,
			unpack_34_13bit_signed_5off,
		},
		{
			unpack_34_13bit_unsigned_6off,
			unpack_34_13bit_signed_6off,
		},
		{
			unpack_34_13bit_unsigned_7off,
			unpack_34_13bit_signed_7off,
		},
	},
	{
		{
			unpack_35_13bit_unsigned_0off,
			unpack_35_13bit_signed_0off,
		},
		{
			unpack_35_13bit_unsigned_1off,
			unpack_35_13bit_signed_1off,
		},
		{
			unpack_35_13bit_unsigned_2off,
			unpack_35_13bit_signed_2off,
		},
		{
			unpack_35_13bit_unsigned_3off,
			unpack_35_13bit_signed_3off,
		},
		{
			unpack_35_13bit_unsigned_4off,
			unpack_35_13bit_signed_4off,
		},
		{
			unpack_35_13bit_unsigned_5off,
			unpack_35_13bit_signed_5off,
		},
		{
			unpack_35_13bit_unsigned_6off,
			unpack_35_13bit_signed_6off,
		},
		{
			unpack_35_13bit_unsigned_7off,
			unpack_35_13bit_signed_7off,
		},
	},
	{
		{
			unpack_36_13bit_unsigned_0off,
			unpack_36_13bit_signed_0off,
		},
		{
			unpack_36_13bit_unsigned_1off,
			unpack_36_13bit_signed_1off,
		},
		{
			unpack_36_13bit_unsigned_2off,
			unpack_36_13bit_signed_2off,
		},
		{
			unpack_36_13bit_unsigned_3off,
			unpack_36_13bit_signed_3off,
		},
		{
			unpack_36_13bit_unsigned_4off,
			unpack_36_13bit_signed_4off,
		},
		{
			unpack_36_13bit_unsigned_5off,
			unpack_36_13bit_signed_5off,
		},
		{
			unpack_36_13bit_unsigned_6off,
			unpack_36_13bit_signed_6off,
		},
		{
			unpack_36_13bit_unsigned_7off,
			unpack_36_13bit_signed_7off,
		},
	},
	{
		{
			unpack_37_13bit_unsigned_0off,
			unpack_37_13bit_signed_0off,
		},
		{
			unpack_37_13bit_unsigned_1off,
			unpack_37_13bit_signed_1off,
		},
		{
			unpack_37_13bit_unsigned_2off,
			unpack_37_13bit_signed_2off,
		},
		{
			unpack_37_13bit_unsigned_3off,
			unpack_37_13bit_signed_3off,
		},
		{
			unpack_37_13bit_unsigned_4off,
			unpack_37_13bit_signed_4off,
		},
		{
			unpack_37_13bit_unsigned_5off,
			unpack_37_13bit_signed_5off,
		},
		{
			unpack_37_13bit_unsigned_6off,
			unpack_37_13bit_signed_6off,
		},
		{
			unpack_37_13bit_unsigned_7off,
			unpack_37_13bit_signed_7off,
		},
	},
	{
		{
			unpack_38_13bit_unsigned_0off,
			unpack_38_13bit_signed_0off,
		},
		{
			unpack_38_13bit_unsigned_1off,
			unpack_38_13bit_signed_1off,
		},
		{
			unpack_38_13bit_unsigned_2off,
			unpack_38_13bit_signed_2off,
		},
		{
			unpack_38_13bit_unsigned_3off,
			unpack_38_13bit_signed_3off,
		},
		{
			unpack_38_13bit_unsigned_4off,
			unpack_38_13bit_signed_4off,
		},
		{
			unpack_38_13bit_unsigned_5off,
			unpack_38_13bit_signed_5off,
		},
		{
			unpack_38_13bit_unsigned_6off,
			unpack_38_13bit_signed_6off,
		},
		{
			unpack_38_13bit_unsigned_7off,
			unpack_38_13bit_signed_7off,
		},
	},
	{
		{
			unpack_39_13bit_unsigned_0off,
			unpack_39_13bit_signed_0off,
		},
		{
			unpack_39_13bit_unsigned_1off,
			unpack_39_13bit_signed_1off,
		},
		{
			unpack_39_13bit_unsigned_2off,
			unpack_39_13bit_signed_2off,
		},
		{
			unpack_39_13bit_unsigned_3off,
			unpack_39_13bit_signed_3off,
		},
		{
			unpack_39_13bit_unsigned_4off,
			unpack_39_13bit_signed_4off,
		},
		{
			unpack_39_13bit_unsigned_5off,
			unpack_39_13bit_signed_5off,
		},
		{
			unpack_39_13bit_unsigned_6off,
			unpack_39_13bit_signed_6off,
		},
		{
			unpack_39_13bit_unsigned_7off,
			unpack_39_13bit_signed_7off,
		},
	},
};
